;; @(#) tinybl.el -- (B)uffer (L)ist handling functions aka C-x C-b

;; @(#) $Id: tinybl.el,v 1.15 1997/05/31 11:40:59 jaalto NotReleased $
;; @(#) $Keywords: tools, buffer list, sort $
;; $KnownCompatibility: 19.28 , XEmacs 19.14 $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Feb 14 1996
;;
;; To get information on this program use ident(1) or do M-x tibl-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinybl|Jari Aalto|jari.aalto@poboxes.com|
;; (B)uffer (L)ist handling functions, Sorting by name, mode, ext...|
;; 31-May-1997|$Revision: 1.15 $|~/misc/tinybl.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;;   Put this file on your Emacs-Lisp load path, add following into your
;;   ~/.emacs startup file
;;   Code can be easily extracted with tinylib.el / ti::m-pkg-rip-magic
;;
;;* _
;;* 	(require 'tinybl)
;;
;;   Or use autoload, which is prefered. Your emacs starts up faster
;;
;;* _
;;* 	(autoload 'tibl-bindings-on      "tinybl" t t)
;;* 	(add-hook 'buffer-menu-mode-hook 'tibl-bindings-on)
;;
;;
;;  If you don't want the default bindings, put this to your .emacs too
;;  And define your own bindings.
;;
;;* _
;;*     (add-hook 'tibl-load-hook 'tibl-bindings-off)
;;
;; If you have any questions, use this function
;;
;;      M-x tibl-submit-bug-report

;;}}}
;;{{{ docs

;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tibl-" in front of
;;   every function & variable. It stands for '(ti)ny (B)uffer (l)ist
;;   function collection.
;; - variable names contain letters 'tibl-:'

;;  Preface, Feb 1996
;;
;;	It struck me, that the C-x C-b listing was utterly useless most of
;;	the time when I tried to one buffer among 50 others.  What I needed
;;	was a simple way to organize buffer menu with few keystrokes. And
;;	that's all this package is about. This package provides some
;;	common sorting ways after M-x list-buffer.
;;
;;  Additional bindings available when you load this package
;;
;;	    C-c n  tibl-sort-by-name
;;	    C-c e  tibl-sort-by-ext
;;	    C-c t  tibl-sort-by-type
;;	    C-c s  tibl-sort-by-size


;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May	31	19967	[jari]		19.28	v1.15		NotReleased
;; - Checkdoc 1.29 clear. Defcustom support. Internal chnges due to
;;   lib functions.
;;
;; Apr	18	19967	[jari]		19.28	v1.12-1.14	NotReleased
;; - Added lib m require due to macros.
;;
;; Aug	25	1996	[jari]		19.28	v1.10		NotReleased
;; - Rearranged loading. Now uses autoloads. Changed all '(lambda forms
;;   to use (function (lambda
;;
;; Aug	20	1996	[jari]		19.28	v1.7-1.8	NotReleased
;; - Slight adjustments. Removed all byte compiler errors found
;;   while using XEmacs 19.14. More robust auto-install.
;;
;; May	16	1996	[jari]		19.28	v1.6		NotReleased
;; - Corrected the autoload instructions. Added some "accident" preventing
;;   when called from M-x tibl-some-function.
;;   tibl-submit-bug-report		:+
;;   tibl-do-it-p			:+
;;   tibl-do				:+
;;
;; Feb	29	1996	[jari]		19.28	v1.5		NotReleased
;; - sasdjb@unx.sas.com (David Biesack) pointed out that the colors weren't
;;   preserved. Now corrected the main routine to include text properties.
;;
;; Feb	21	1996	[jari]		19.28	v1.4		NotReleased
;; - Extensions! I wanted extension sort too...now it's there.
;;
;; Feb	18	1996	[jari]		19.28	v1.3		NotReleased
;; - Small error in 'provide' statement. Minor adj. to kyes off func.
;; - Added C-u to each function for reverse sorting.
;;
;; Feb	15	1996	[jari]		19.28	v1.2		NotReleased
;; - ByteComp errors corrected and changed some function names.
;;
;; Feb	15	1996	[jari]		19.28	v1.1		Created
;; - Made complete package out of my private functions.


;;}}}

;;; Code:

;;{{{ setup: require

(require 'tinylibm)
(eval-when-compile (use-dynamic-compilation))


(defgroup-tiny TinyBl tibl-: tools
    "This package provides some common sorting ways after C-x C-b.
See default keybinding in C-c map.")

;;}}}
;;{{{ setup: variables

;;; .......................................................... &v-bind ...

(defun tibl-bindings-on ()
  "Bind sorting fuctions to C - c [stno] in Buffer list buffer."
  (interactive)
  (cond
   ((not (boundp 'Buffer-menu-mode-map))
    (load "buff-menu")))

  (define-key Buffer-menu-mode-map "\C-cn"   'tibl-sort-by-name)
  (define-key Buffer-menu-mode-map "\C-ce"   'tibl-sort-by-ext)
  (define-key Buffer-menu-mode-map "\C-ct"   'tibl-sort-by-type)
  (define-key Buffer-menu-mode-map "\C-cs"   'tibl-sort-by-size)

  ;;  Original buffer list sort order
  ;;
  (define-key Buffer-menu-mode-map "\C-co"
    (function
     (lambda ()
       "original order"
       (interactive)
       (list-buffers) (switch-to-buffer "*Buffer List*"))))

  )

(defun tibl-bindings-off ()
  "Remove default bindings."
  (interactive)
  (remove-hook 'buffer-menu-mode-hook 'tibl-bindings-on)

  (cond
   ((and (boundp 'Buffer-menu-mode-map)
	 (eq (lookup-key Buffer-menu-mode-map "\C-cn")
	     'tibl-sort-by-name))	;just check this, if ok, then..

    (define-key Buffer-menu-mode-map "\C-cn"   'nil)
    (define-key Buffer-menu-mode-map "\C-ce"   'nil)
    (define-key Buffer-menu-mode-map "\C-ct"   'nil)
    (define-key Buffer-menu-mode-map "\C-cs"   'nil)
    (define-key Buffer-menu-mode-map "\C-co"   'nil)
    ))
  )

;;; ......................................................... &v-hooks ...

(defcustom tibl-load-hook nil
  "*Hook run when file is loaded."
  :type 'hook :group 'TinyBl
  )

;;; .......................................................... version ...


(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinybl.el"
 "tibl"
 tibl-:version-id
 "$Id: tinybl.el,v 1.15 1997/05/31 11:40:59 jaalto NotReleased $"
 '(tibl-:version-id
   )))


;;}}}

;;; ########################################################### &Funcs ###

;;{{{ code: macros

;;; ----------------------------------------------------------------------
;;;
(defsubst tibl-do-it-p  ()
  "Check if current buffer is generated by list-buffer."
  (string-match "Buffer[ \t]+List" (buffer-name)))


;;; ----------------------------------------------------------------------
;;;
(defmacro tibl-do (&rest body)
  "Execute BODY only in Buffer List"
  (` (if (not (tibl-do-it-p))
	 (message "Trouble, not in buffer 'Buffer list' ?")
       (,@ body))
   ))

;;}}}
;;{{{ code: sort engine

;;; ----------------------------------------------------------------------
;;;
(defun tibl-sort (level re &optional numeric reverse case)
  "Sort buffer list.

Input:

  LEVEL	    subexpression
  RE	    regexp
  NUMERIC   numeric sort
  REVERSE   reverse order
  CASE	    case sensitive."
  (let* ((bfn	"*Buffer List*")	;buffer name
	 buffer-read-only
	 list
	 buffer
	 )

    (if (setq buffer (get-buffer bfn))	;make sure it esists
	(save-excursion
	  (set-buffer buffer)
	  ;;  get all lines, text properties are included so that
	  ;;  coloring is preserved.
	  ;;
	  (setq list (ti::b-grep-lines "." nil nil 'prop))

	  (setq
	   list
	   (ti::m-sort-regexp-fields
	    list level re numeric reverse case))

	  (erase-buffer)		;replace contents
	  (list-print list)
	  (set-buffer-modified-p nil)	;not modified
	  (PMIN)
	  ))))


;;; ----------------------------------------------------------------------
;;; #todo: doesn't quite do the right thing..
;;; I'd like to have EXT + NAME comparision
;;;
(defun tibl-sort-ext-subr (&optional reverse)
  "Sort by extension. Optionally REVERSE."
  (let* (buffer-read-only
	 )
    (message "Sorting...")

    (PMIN) (forward-line 2)		;Skip over the headers

    (sort-subr
     reverse 'forward-line 'end-of-line
     (function
      (lambda ()			; start key FUNC
	(let* ((re-ext	".*/[a-z0-9].*+\\.\\([^.\n]+\\)")
	       )
	  (beginning-of-line)
	  (cond
	   ((not (looking-at re-ext))
		 ;;  Sort by Type if no extension found
		 (re-search-forward "[0-9][ \t]+")
		 )
	    ((looking-at ".*/\\.\\([^.\n]+\\)")	; no real name here , /.netrc
	     ;;  Put these first
	     (goto-char (match-beginning 1)))

	    (t
	     ;; And then files with Extensions
	     (goto-char (match-beginning 1))))

;;;	(d! (buffer-substring (point) (EOLP)))
	  nil
	  )
	nil
	)))
    (message "Sorting...Done")
    ))

;;}}}
;;{{{ code: interactive

;;; ----------------------------------------------------------------------
;;;
(defun tibl-sort-by-name (&optional arg)
  "Sort `list-buffers' output by name. Prefix ARG reverses the sort."
  (interactive "P")
  (tibl-do
   (tibl-sort  1 "^[\t .*D%>]+ \\([^\t ]+\\)"  nil arg)
   ))


;;; ----------------------------------------------------------------------
;;;
(defun tibl-sort-by-ext (&optional arg)
  "Sort `list-buffers' output by extension. Prefix ARG reverses the sort.
If there is no file name, the sort is done according to mode name.
Files like ~/.netrc aren't considered to have extensions."
  (interactive "P")
  (tibl-do
   (tibl-sort-ext-subr arg)
   ))

;;; ----------------------------------------------------------------------
;;;
(defun tibl-sort-by-type (&optional arg)
  "Sort `list-buffers' output by type.  Prefix ARG reverses the sort."
  (interactive "P")
  (tibl-do
   (tibl-sort  1 "^.*[0-9][ \t]+\\([^\t ]+\\)"  nil arg)
  ))

;;; ----------------------------------------------------------------------
;;;
(defun tibl-sort-by-size (&optional arg)
  "Sort `list-buffers' output by size. Prefix ARG reverses the sort."
  (interactive "P")
  (tibl-do
   (tibl-sort 1 "[ \t]+\\([0-9]+\\)[\t ]+[A-Z]" 'numeric arg)
  ))

;;}}}

;; Install
;;
(add-hook 'buffer-menu-mode-hook 'tibl-bindings-on) ;delayed install
(when (boundp  'Buffer-menu-mode-map)		;already exists, install now
  (tibl-bindings-on))

(provide   'tinybl)
(run-hooks 'tibl-load-hook)

;;; tinybl.el ends here
