;; @(#) tinybufid.el -- Identifying buffer regardless of mode

;; @(#) $Id: tinybufid.el,v 1.21 1997/05/14 07:59:25 jaalto NotReleased $
;; @(#) $Keywords: library, buffer identification $
;;
;; $KnownCompatibility: 19.28+, XEmacs 19.14/20+ $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Feb 13 1995
;;
;; To get information on this program use ident(1) or do M-x tib-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinybufid|Jari Aalto|jari.aalto@poboxes.com|
;; Identifying buffers regardless of mode with various ways|
;; 14-May-1997|$Revision: 1.21 $|~/misc/tinybufid.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ........................................................ &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file
;;
;;	(require 'tinybufid)
;;
;; Or if you want it to be loaded only when used for the first time:
;;
;;	(autolad 'tib-info "tinybufid" "Buffer info" t)
;;
;; Function to call to get buffer identity:
;;
;;	tib-info
;;
;; If you have any questions, always use function
;;
;;      M-x tib-submit-bug-report
;;
;; Request:
;;      Please send any example file or mode that I'm not aware of,
;;      I'll support any mode when it is needed to.

;;}}}
;;{{{ Documentation

;; .................................................... &t-commentary ...
;;; Commentary:

;; RESERVERD prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tib-" in front of
;;   every function & variable. It stands for '(ti)ny (b)uffer identification.
;; - variable names contain 'tib-:'

;; Preface, feb 1995
;;
;;	Seems quite uninteresting package to you? I bet it does, unless you're
;;	a lisp programmer who has infite time to twiddle his c++-mode-hook +
;;	20 others hooks and have all the time develop nice little funcs to make
;;	living in emacs easier :-/
;;
;;	While someone may think that all users stick to one mode eg. when they
;;	are programming C++, that's not obvious. For example I move constantly
;;	between all kind of modes during programming and the trouble is, that
;;	while the buffer LOGICAL content remains the same, the hooks knows
;;	nothing about it. Hooks are just dummies that gets called whenever
;;	you turn some mode on, try C++ mode over nroff code and you'll be
;;	suprised...[not a serious example of cource]
;;
;;	Now let's see one example: like I coded this package:
;;
;;	.   write list		     ;lisp-mode + folding-mode
;;	.   hmm, write some comments ;tii-mode,  no folding
;;	.   hmm, need center-command ;text-mode
;;	.   code again		     ;lisp + ..
;;	.   adjust defconst var pos. ;tii-mode (tinyindent.el)
;;	.   code again		     ;lisp-mode
;;
;;	I have bound all common modes into keys that I can access very fast
;;	so that changing modes is no problem. What is the problem, is that
;;	when you turn off the CODE-mode, all info about comment-start etc.
;;	vanishes. The tii-mode and TIC (tinycom.el) fortunately doesn't
;;	care about the modes, because they have their own heuristics to
;;	determine suitable comments etc.
;;
;; Overview of features
;;
;;	o   This is LIBRARY package
;;	o   Tries to identify the buffer with various different criterias.
;;	o   Usefull for checking what kind of file is in buffer and making
;;	    decisions based on that. Very usefull in hooks or so.
;;
;;  Imenu example
;;
;;	If your're using imenu.el to generate buffer jump points, it
;;	is very likely that the imenu command won't generate right jump
;;	points if you're in wrong mode.
;;
;;	let's use imenu example. This is how I used to have it configured:
;;
;;* ;; ** NO,NO **
;;* _
;;* (defun my-imenu-mouse (event)
;;*   (interactive "e")
;;*   (my-imenu))
;;* _
;;* (defun my-imenu (&optional arg)
;;*   "Sets parameters to imenu."
;;*   (let* (raise
;;* 	 )
;;*     (setq imenu-max-items 20
;;* 	  imenu-sort-function nil)
;;*     (cond
;;*     ((memq major-mode '(lisp-mode emacs-lisp-mode lisp-interaction-mode))
;;*       (setq imenu-create-index-function 'imenu-example--create-lisp-index
;;* 	    imenu-sort-function         'imenu--sort-by-name
;;* 	    raise t
;;* 	    ))
;;*      ((memq major-mode '(c++-mode))
;;*       (setq imenu-create-index-function 'imenu-example--create-c++-index
;;* 	    imenu-sort-function         'imenu--sort-by-name
;;* 	    raise t
;;* 	    ))
;;*      ((memq major-mode '(c-mode))
;;*       (setq imenu-create-index-function 'imenu-example--create-c-index
;;* 	    imenu-sort-function         'imenu--sort-by-name
;;* 	    raise t
;;* 	    ))
;;*      )
;;*     (if raise (imenu))
;;*     ))
;;* ;; End
;;
;;	Code seemed pretty obvious don't' you think ? It sniffs around to
;;	see if we're on some mode and then configures imenu according to
;;	it.
;;
;;	Here is much better and more robust way. It'll let you be in any
;;	mode while retaining right imenu. [The example can be
;;	automatically extracted with function ti::m-pkg-rip-magic from
;;	tinylib.el, follow the instructions there.]
;;
;;* ;; ** MORE ROBUST **
;;* _
;;* (require 'imenu)
;;* _
;;* ;;  Separate functions for keyboard and mouse.
;;* (defun my-imenu-mouse (event &optional arg)
;;*   (interactive "e\nP")
;;*   (my-imenu arg))
;;* _
;;* _
;;* (defun my-imenu (&optional arg)
;;*   "Sets parameters to imenu. If called with arg, the output is
;;* unsorted.
;;* "
;;*   (interactive "P")
;;*   (let* (
;;* 	 (s-func (if arg nil 'imenu--sort-by-name))
;;* 	 (table
;;* 	  '((lisp-mode
;;* 	     imenu-example--create-lisp-index)
;;* 	    (emacs-lisp-mode
;;* 	     imenu-example--create-lisp-index)
;;* 	    (c++-mode
;;* 	     imenu-example--create-c++-index)
;;* 	    (c-mode
;;* 	     imenu-example--create-c-index)
;;* 	    ))
;;*      ;;  So, in what mode were really?
;;* 	 (mode (or (tib-info t) major-mode))
;;* 	 (el (assoc mode table))
;;* 	 )
;;*     (if (null el)
;;* 	(message "Sorry, no imenu for this buffer.")
;;*       (setq imenu-max-items		20
;;* 	    imenu-sort-function		s-func
;;* 	    imenu-create-index-function	(nth 1 el))
;;*       (imenu)				;launch it !
;;*       )
;;*   ))
;;* ;; End
;;
;;  Your mode is not supported ?
;;
;;	I'm sure that this file cannot identify all modes out there,
;;	since I'm only aware of these included in this package. Please
;;	send me any example file that is edited in particular mode, I'd
;;	be glad to support more modes.
;;
;;  What buffers to identify ?
;;
;;	The aim of this package is to be 'aid'; offer some help for
;;	user's that need to set up their hooks or use commands that are
;;	based on buffer content. This is *not* the same as buffer
;;	recognition, and this package will not be one. To recognize all
;;	buffers and their contents would be waste of time.
;;
;;	Summarum: let's be reasonable and _think_ what buffers we need to
;;	identify and add support to those.

;;}}}
;;{{{ History

;;; Change Log:
;; ........................................................ &t-history ...
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May	14	1997	[jaalto]	19.28	 v1.20		NotReleased
;; - Changed all non standard boolean tests to use when and unless
;;
;; Apr	18	1997	[jaalto]	19.28	 v1.18-1.21	NotReleased
;; - Updated java comment syntax. Added hugs-mode support
;; - Added hugs mode.
;;
;; Jan	15	1997	[jaalto]	19.28	 v1.17		NotReleased
;; - text changes
;;
;; Dec	17	1996	[jaalto]	19.28	 v1.14-1.16	NotReleased
;; - Rearranged loading. Now uses autoload as much as possible.
;; - Added lib m require due to macros. Added text-texinfo.
;;
;; Sep	11	1996	[jaalto]	19.28	 v1.13		NotReleased
;; - Added povray-mode.
;;
;; Aug	22	1996	[jaalto]	19.28	 v1.10-12	NotReleased
;; - Corrected all XEmacs 19.14 compiler errors. Now works in XEmacs.
;; - Added tib-cnv-txt2comment, and comment start/end to variables
;; - Added "resource-code-shell" comment table settings.
;;
;; Jun	17	1996	[jaalto]	19.28	 v1.9		NotReleased
;; - Added fvwm-mode support. Added tib-:info variable for fast lookup,
;;   now the tib-info accepts parameter VARIABLE.
;;
;; May	21	1996	[jaalto]	19.28	 v1.9		NotReleased
;; - Added idl-mode.
;;
;; Feb	5	1996	[jaalto]	19.28	 v1.8		NotReleased
;; - Small code changes. Uses safe-widen in places where needed.
;;
;; Feb	5	1996	[jaalto]	19.28	 v1.7		NotReleased
;; - lisp-mnt.el's lm-verify checked. Rewrote Feedback functions.
;;
;; Oct	4	1995	[jaalto]	19.28	 v1.5-6		NotReleased
;; - Added support for cobol mode (not for the new object cobol)
;; - Made lisp regexp more accurate, it gace false hits...
;;
;; Sep	28	1995	[jaalto]	19.28	 v1.4		NotReleased
;; - Added support for java code (sun OS)
;;
;; Jun	16	1995	[jaalto]	19.28	 v1.3		NotReleased
;; - Added fortran mode support. Added imenu example. In general,
;;   added more documentation. Harmonized code structure. Corrected
;;   minor mistakes in code. The main routine rewritten to handle
;;   symbols too.
;;
;; May	20	1995	[jaalto]	19.28	 v1.2		NotReleased
;; - Working quite well, not heavily tested, since I have to convert
;;   my hook to use this package. You can tell I'm lazy about touching
;;   my system when it's tuned and works ok :-/
;; - This file seems really promising. Added the imenu example. The
;;   time gap to 1.1 is due to more urgent tasks and packages I
;;   developed in the mean while.
;; - Added -*-Emacs-lisp-*- first line handling too. I think this is
;;   seldom used, because emacs can now identify file by its
;;   extension. I have never used this contruction, because the
;;   file-name-alist is more in hand..
;;
;; Feb	13	1995	[jaalto]	19.28	 v1.1		Created

;; People who have contributed.
;;
;; Sep 11 1996 Chris Van Humbeeck
;; - vhumbeec@pluto.cs.kuleuven.ac.be (Chris Van Humbeeck), author of
;;   hugs-mode.el sent me hugs example file.
;;
;; Sep 11 1996 Kevin K. Lewis <lewikk@rockdal.aud.alcatel.com>
;; - Sent povray-mode mode file (also author of that mode)
;;
;; May 31 1996  Rob Walker <rwalker@cisco.com>
;; - Sent me some sample fvwm files.
;;
;; May 17 1996  Scott Hassan <hassan@cs.stanford.edu>
;; - Created idl-mode to edit CORBA idl files and send me example file.
;;
;; Oct 4, 1995  Caleb Deupree <cdeupree@trex.cincom.com>
;; - Posted cobol-mode into the help group [not in ohio back then]
;; - Sent good example codes of cobol and offered full expalnation, wow.
;;
;; Sep 28, 1995  Patrick Taylor <patrick.taylor@template.com>
;; - Kindly sent me java code example file
;;
;; May 20, 1995 ttateyam@defmacro.cs.utah.edu (Terry T Tateyama)
;; - Adviced how vhdl mode is detected.
;;
;; Jun 9, 1995 jcupak@isd99.sanders.lockheed.com (John J. Cupak)
;; - Adviced what are the ada mode extensions
;;
;; Jun 16, 1995 emwson@emw.ericsson.se (Martin Lorentzson)
;; - Kindly sent fortran example file and told what extension were used.
;;
;; Jun 16, 1995 bwarsaw@CNRI.Reston.VA.US (Barry A. Warsaw)
;; - Kindly adviced how python files are detected
;;
;; To do list:
;;
;; - Please send me all usefull RE-strings and modes you're using
;;   that I'm not familiar with.



;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinylib)
(eval-when-compile (use-dynamic-compilation))

;;}}}
;;{{{ setup: hooks

;;; ......................................................... &v-hooks ...

(defvar tib-:load-hook nil
  "*Hook run when file has been loaded.")

;;}}}
;;{{{ setup: private

;;; ................................................ &v-buffer-private ...

(defvar tib-:info  nil
  "Buffer local variable.This value is updated every time
function tib-info called. For faster responses, you may wan to write your
code like this:

	(setq info tib-info nil 'var)

Because peeking the variable is 40x times faster.
")
(make-variable-buffer-local 'tib-:info)

;;; ....................................................... &v-private ...

(defconst tib-:default-falist
  '(;;  This first line -*- test should represent exact mode, we trust
    ;;  to it blindly. If the content is not what this mode says, it's
    ;;  user's own mistake.
    tib-tre1st-e

    tib-tre1st
    tib-tbfn
    tib-text
    tib-tbn
    tib-trebuf
    )
  "Default order of evaluation to determine buffer contents.
\(FUNC FUNC FUNC\). The calling stops immediately when some function
returns non-nil.
")


;; Globals set by functions.
;; - These are heavily used. User may check these too.
;; - They are Set after the buffer is studied.

(defconst tib-:bn nil    "global: set by study func, buffer name")
(defconst tib-:bfn nil   "global: set by study func, buffer file name")
(defconst tib-:bext nil  "global: set by study func, buffer fn ext.")
(defconst tib-:1st nil   "global: set by study func, 1st line of buffer")

;;}}}
;;{{{ setup: user config

;;; ........................................................ &v-public ...
;;; it is INTENTIONAL that the variables are defconst, change these
;;; with tib-:load-hook


(defvar tib-:file-ext-re "[a-zA-Z0-9]\\(\\.\\)[a-zA-Z0-9]+$"
  "A regexp that says what files can have extension. Everything after the
DOT is considered to form extension. Files like ~/.cshrc are not normally
considered to have estension.

Reference:
  See function tib-get-file-ext
")



(defconst tib-:1st-line-re-alist
 '(
   ("^#.*perl"		"code-perl")
   ("^#.*python"	"code-python")
   ("^#.*scm"	        "code-scheme")
   ("^#.*tcl"	        "code-tcl")

   ;;    Of course this is not bullet proof, but many lisp code package
   ;;    has first line describing the package.
   ("^;;[ \t]+.*\.el "  "code-lisp")


   ("^#.*awk"		"code-awk")

   ;; It's custom to start the file with ':' no-op, if it's
   ;; sh-coded, since it prevents accidental # for the first line
   ;; --> intepreted as csh code by shell if it sees # as first char
   ;;
   ("^#.*\/sh\\|^[ \t]*:[ \t]*$"	"code-shell-sh")
   ("^#.*\/csh"		"code-shell-csh")
   ("^#.*\/tcsh"	"code-shell-tcsh")
   ("^#.*\/ksh"		"code-shell-ksh")
   ("^#.*\/zsh"		"code-shell-zsh")
   ("^#.*\/bash"	"code-shell-bash")

   ;;  This is just rough guess, propably false hit, but vhdl files
   ;;  uses comments like '--'.
   ("^--"		"code-vhdl")

   ;;  Fortran code uses comments beginning with "c", we assume that
   ;;  there must be at least TWO spaces after initial comment
   ("^c  +"		"code-fortran")

   ("^[ \t]*!"		"resource-x")		;.Xinitrc ot the like comment

   ;; It's custom to start an nroff man page with a comment that
   ;; holds version control Id string. Comment is  = .\"
   ;;
   ("^\.\\\""	"text-nroff")
   )
 "*list of \(REGEXP str) where RE is tried upon 1 line match, normally
a bang-slash or emacs --** notation")


;;  - Remember, first one that macthes id used! Put most restrictive at
;;    the beginning.
;;  - The regexp scanning should be the last resort, because its potential
;;    mishits.
;;  - There is no need to add regexp here if buffer can be identified by other
;;    means easily ie. all WWW files have universal .html extension.
;;
(defconst tib-:buf-re-alist
  (list

   (list
    (concat
     ;;   interface CServicePreferences;  /* Forward references */
     ;;
     "interface [0-9A-Za-z]+[ \t]+[0-9A-Za-z]+[ \t]*;"

     ;; exception InvalidRequest{TString aReason;};
     ;;
     "\\|exception[ \t]+[0-9A-Za-z]+{.*;"

     ;; typedef sequence<CosPropertyService::PropertyNames> PropertyNamesList;
     ;;
     "\\|typedef[ \t]+[0-9A-Za-z]+<"
     )
    "code-idl")


   ;; *FvwmIdentBack MidnightBlue
   ;; *FvwmIdentFore Yellow
   ;; Style "FvwmButtons" CirculateSkip
   ;;
   '("Style[ \t]+\"FvwmButtons\"[ \t]+[A-Z]\\|^[*]Fvwm"
     "resource-code-fvwm")

   ;;
   ;;
   '("^@c "
     "text-texinfo")

   ;;  #declare Purple_Clouds = pigment {
   ;;  #include "woodmaps.inc"
   ;;  ...
   ;;  Paraboloid_Y
   ;;  scale <30.0, 10.0, 40.0>
   ;;  rotate 180.0*z
   ;;  translate <40.0, 14.0, 50.0>
   ;;
   (list
    (concat
     "^#include[ \t]+.*.inc\""
     "\\|^#declare[ \t]+[^ \t]+[ \t]*="
     "\\|\\(scale\\|translate\\)[ \t]<[ \t]*[0-9.][ \t]*,.*>"
     )
     "code-povray")


   (list
    (concat
     "[(][ \t]*\\(defmacro\\|defvar\\|defconst\||defun\\|setq"
     "\\|add-hook\\|remove-hook\\|let[*]"
     "\\)"
     )
    "code-lisp")

   '("entity[ \t]+[a-z_A-Z]+[ \t]+is"		       "code-vhdl")


   (list
    (concat
     ;; 01  WORK-AREA.
     ;;     02 PI               PIC S9V9(14).
     ;;
     "01[ \t]+WORK-AREA\\.\\|01[ \t]+CONSTANTS\\."

     ;;		ACUCOBOL-85
     "\\|\\(working-storage\\|portability\\)[ \t]+section\\."
     "\\|perform[ \t]+initialize-environment\\."
     ;;		display "F4 = Exit Demonstration", line 11, column 8.
     "\\|display [\"].*,[ \t]*column[ \t]+[0-9]+\\."
     )
    "code-cobol")

   ;;  it's "write(*,*)" , and definitely fortran
   '("write[(][*],[*][)]"                               "code-fortran")


   ;;
   (list
    (concat
     "class.*\\(extends\\implements\\)"

     "\\|\\(protected\\|public\\)[ \t]+"
     "\\(synchronized[ \t]+\\)?\\(Object\\)"
     )
    "code-java")

   ;;  Function definition
   ;;    def add_doc(self, document, keyword_list):
   ;;    def __init__(self):
   '("^[ \t]+def[ \t]+[a-zA-Z_]+[(].*:"                 "code-python")

   ;;  Oracle sql
   ;;  select * from p_msc_rej where measurement_time = '1995061914124675';
   '("select.*from.*where.*=.*;"			      "code-sql")

   '("^[.]TH"					       "text-nroff")
   '("^Newsgroup:"                                      "text-news")
   '("^To:.*@"                                          "text-mail")


   (list
    ;;  conjug     :: Words -> Words -> String
    ;;  netails    :: [a] -> [[a]]
    ;;  replies    :: Words -> Words -> [String]
    ;;
    "conjug[ \t]+::.*->\\|netails[ \t]+::.*->\\|replies::[ \t]+.*->"
    "code-hugs"
    )


   )
  "*List of \(REGEXP str\) where RE is tried upon whole file.
First one matched is used to determine file type, so put most restrictive
REs first.")



(defconst tib-:file-ext-re-alist
 '(
   (".a"     . "code-ada")		;Ada 83/87
   (".ada"   . "code-ada")		;Ada 83/87
   (".ads"   . "code-ada")		;ada 95
   (".adb"   . "code-ada")		;Ada 95 body/implementation
   (".awk"   . "code-awk")
   (".c"     . "code-c")
   (".cbl"   . "code-cobol")		;this is the standard hdr & strc
   (".cc"    . "code-c++")
   (".cxx"   . "code-c++")
   (".cob"   . "code-cobol")		;non-standard unix
   (".cpp"   . "code-c++")
   (".C"     . "code-c++")
   (".el"    . "code-lisp")
   (".f"     . "code-fortran")
   (".F"     . "code-fortran")
   (".for"   . "code-fortran")
   (".fvwmrc". "resource-code-fvwm")
   (".h"     . "code-c-header")
   (".hh"    . "code-c++-header")
   (".hs"    . "code-hugs")
   (".i"     . "code-cobol-header")	;non-standard unix
   (".idl"   . "code-idl")		;CORBA idl, hassan@cs.stanford.edu
   (".html"  . "text-html")
   (".java"  . "code-java")
   (".m"     . "code-objective-c")
   (".mod"   . "code-objective-c")
   (".md"    . "code-modula-header")	;at least modula-2
   (".mi"    . "code-modula")		;implementation modula-2
   (".pas"   . "code-pascal")
   (".pl"    . "code-perl-library")
   (".pls"   . "code-perl-shell")
   (".pm"    . "code-perl")

   ;;  also uses .inc and .map but I hesitate to add those extension,
   ;;  because some other may use .inc or .map too for other purposes.
   ;;
   (".pov"   . "code-povray")

   (".py"    . "code-python")
   (".sh"    . "code-shell")		;might be csh/ksh/csh
   (".sql"   . "code-sql")
   (".tex"   . "text-tex")
   (".texi"  . "text-tex-info")
   (".vhd"   . "code-vhdl")
   (".vhdl"  . "code-vhdl")
   )
 "*List of (ASSOC-KEY STR) where KEY is tried upon
buffer-file-name's extension.")


;;  If the file cannot be identified by extension...
(defconst tib-:file-fn-re-alist
  '(
    ("\\.ema"        "code-lisp")	;.emacs , .emacs.dired
    ("\/\\.[Xx]"     "resource-x")	;.Xdefauls, .xinirc
    ("\/\\.t?cshrc"  "resource-code-shell-csh")	;alike csh = tcsh
    ("\/\\.bashrc"   "resource-code-shell-sh")  ;alike bash = sh
    ("\\.csh"        "shell-csh")	;like .cshrc or myScript.csh
    )
  "*List of (REGEXP STR) where RE is tried upon _whole_ buffer-file-name")


;; - Buffers that do not have buffer-file-name property at all.
;; - Only put 'trusted' buffer names that are known to all here.
(defconst tib-:bn-re-alist
 '(
   ("[*]info"       "text-manual-info")
   ("[*]man"        "text-manual-shell")
   ("[*]shell"      "process-shell")
   ("[*]ftp"        "process-ftp")
   ("[*]Article"    "text-news")
   ("[*]Summary"    "text-news")
   )
 "*List of (REGEXP STR) where RE is tried upon buffer-name")


(defconst tib-:falist 'tib-:default-falist
"*Variable SYMBOL holding functions to call when buffer type is
determined. Notice, that this is also the order of evaluation.
")


(defconst tib-:txt2mode
  '(
    ("ada"                 ada-mode     "--")
    ("awk"		   awk-mode	"#")
    ("code-c$"		   c-mode	"/*" "*/")
    ("code-c++"		   c++-mode	"//")
    ("code-cobol"	   cobol-mode	)
    ("code-fortran"	   fortran-mode "C")
    ("code-fvwm"	   fvwm-mode	)
    ("code-hugs"	   hugs-mode	"--" )
    ("code-idl"		   idl-mode	)
    ("code-java"	   java-mode	"/*" "*/")
    ("code-objective-c"    c-mode	)
    ("code-pascal"	   pascal-mode	)

    ("html"		   html-mode	"<!---" "-->")

    ("code-python"	   python-mode	)
    ("code-scheme"	   scheme-mode	)
    ("code-sql"		   sql-mode	"REM")
    ("code-tcl"		   tcl-mode)

    ("pascal"		   pascal-mode  "{" "}")
    ("perl"		   perl-mode	"#")
    ("code-povray"	   povray-mode	"/*" "*/")


    ("lisp\\|emacs-lisp"   lisp-mode	";")

    ("text-tex"		   tex-mode     "%")
    ("text-texinfo"        texinfo-mode "@c")
    ("text-tex-info"	   texinfo-mode "%")
    ("text-mail"	   mail-mode)
    ("text-news"	   mail-mode)

    ("shell-sh"		   sh-mode	"#")
    ("shell-csh"	   csh-mode	"#")
    ("shell-ksh"	   ksh-mode	"#")
    ("shell-zsh"	   zsh-mode	"#")

    ("resource-code-shell-csh" csh-mode	"#")
    ("resource-code-shell-sh"   sh-mode	"#")

    ("tex$"		   tex-mode	"%")
    )
  "*List of
'((REGEXP MODE-NAME-SYMBOL [COMMENT-START COMMENT-END]) (
  (R M C C)
  ..
  )
where RE represent match against string that describes the buffer
contents. The comment-start and end fields are optional.")




;;}}}
;;{{{ version

;;; ....................................................... &v-version ...


(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinybufid.el"
 "tib"
 tib-:version-id
 "$Id: tinybufid.el,v 1.21 1997/05/14 07:59:25 jaalto NotReleased $"
 '(tib-:load-hook
   tib-:default-falist
   tib-:falist

   tib-:bn
   tib-:bfn
   tib-:bext
   tib-:1st

   tib-:file-ext-re
   tib-:1st-line-re-alist
   tib-:buf-re-alist
   tib-:file-ext-re-alist
   tib-:file-fn-re-alist
   tib-:bn-re-alist

   tib-:txt2mode
   )))


;;}}}

;;; ########################################################### &funcs ###

;;{{{ misc

;;; ............................................................ &misc ...

;;; ----------------------------------------------------------------------
;;;
(defmacro tib-get-falist ()
  "*Return functions list stored in tib-:falist"
  (list 'eval 'tib-:falist))


;;; ----------------------------------------------------------------------
;;;
(defun tib-cnv-txt2mode (txt)
  "This is kinda fake function, it returns the original MODE based
on the text that represents the buffer contents. This functions purpose
is solely to return you a _symbol_ that is more commonly known to all, than
the _string_ representing a mode.

NOTE:
 Symbol returned does not necessary representy any mode you can turn on.
 Use 'fboundp' test to be sure the symbol is callable function.
"
  (let ((table tib-:txt2mode)
	el re mode
	ret
	)
    (list-loop table el
      (if ret (setq table nil))		;Stop the loop
      (setq re (nth 0 el)  mode (nth 1 el))
      (if (string-match re txt)
	  (setq ret mode))
      )
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tib-cnv-txt2comment (txt)
  "Returns (COMMENT-START . COMMENT-END) variables for text representing
the buffer contents. Notice that comment-end may be nil meaning it
is not needed for mode."
  (let ((table tib-:txt2mode)
	com-s com-e
	elt re
	)
    (list-loop table elt
      (setq re (nth 0 elt))

      (if (> (length elt) 2)
	  (setq com-s (nth 2 elt)))

      (if (> (length elt) 3)
	  (setq com-s (nth 3 elt)))

      (if (null (string-match re txt))
	  (setq com-s nil   com-e nil)
	(setq table nil))	;stop there

;;;      (d! re txt )

      )
    (if com-s
	(cons com-s com-e))
    ))

;;}}}
;;{{{ id

;;; ############################################################## &id ###


;;; ----------------------------------------------------------------------
;;;
(defun tib-get-file-ext (fn)
  "Return file extension.

References:
  See variable tib-:file-ext-re how file extension is determined.
"
  (let* ((re tib-:file-ext-re)
	 ret
	 p
	 )
    (cond
     (fn				;doesn't have filename at all *temp*
      (if (null (string-match re fn)) nil      ; no ext
	(setq p (match-beginning 1))	;dot position
	(setq ret (substring fn p)))))
    ret
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tib-get-id-line ()
  "Return first ID line of the file. Empty lines are skipped."
  (let* ((comment-beg	(regexp-quote (or comment-start " ")))
	 (empty-re	(concat "[ \t]*" comment-beg "[ \t]*$"))
	 ret
	 )
    (save-excursion
      (safe-widen
       (PMIN)
       (while (and (not (eobp))		;search first sensible line
		   (looking-at empty-re))
	 (forward-line 1))

       (cond
	((not (eobp))
	 (setq ret (read-full-line))))
       ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tib-match-emacs-mode (str)
  "Emacs supports special first line syntax eg. -*-Emacs-Lisp-*-,
to turn on mode when file loads. We try to find function <text>-mode
from the internal symbol list of emacs if line contains -*- marks.

Return:
  symbol        ,real mode function name found from emacs.
                 Btw, emacs barks you automatically if functions given
                 in line doesn't exist when file is loaded.
  nil
"
  (let* (ret
	 mode
	 sym
	 )
    (cond
     ((setq mode (strmatget "-[*]-\\(.*\\)-[*]-" 1 str))
      ;;  let's make symbol out of it
      (setq mode (concat (downcase mode) "-mode"))
      (if (null (setq sym (intern-soft mode)))
	  (progn
	    ;;  too bad, such mode not loaded into emacs, well if person
	    ;;  has loaded file, emacs had barfed already about this unknown
	    ;;  mode: "file mode specification error, void function, <mode>"
	    nil
	    )
	(if (fboundp sym)		;let's make sure sym is func ...
	    (setq ret sym))		;it's valid mode
	)))
    ret
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tib-match (str l)
  "Matches str against LIST el 1, return LIST elt 2"
  (let* (el
	 ret re
	 )
    (list-loop l el
      (setq re (nth 0 el)  )
      (if (string-match re str)
	  (setq ret (nth 1 el)   l nil))
      )
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tib-buf-match (&optional point)
  "Tries match on buffer. Starts searching from point-min or from
optional POINT."
  (let ((loop	t)
	(p	(or point (point-min)))
	(l	tib-:buf-re-alist)
	ret el re
	)
    (save-excursion
      (safe-widen
       (goto-char p)			;start here
       (while (and loop l)
	 (setq el (car l)
	       re (nth 0 el))

	 (cond ((re-search-forward re nil t)
		(setq ret    (nth 1 el)
		      loop   nil)))
	 (NEXTP l)
	 ))
      ret
      )))

;;}}}

;;{{{ study

;;; ########################################################### &study ###

;;; ----------------------------------------------------------------------
;;;
(defun tib-study-reset ()
  "Resets some globals."
  (setq tib-:bfn nil
	tib-:bext nil
	tib-:1st nil
	))


;;; ----------------------------------------------------------------------
;;;
(defun tib-study ()
  "Sets some globals, so that they can be used by all funcs. This
reduces overhead of getting these variables multiple times."
  (let* (
	 (bp  (current-buffer))
	 (bn  (buffer-name))
	 (bfn (buffer-file-name bp))
	 (ext (tib-get-file-ext bn))
	 (id  (tib-get-id-line))
	 )
    (tib-study-reset)
    (setq tib-:bfn   bfn
	  tib-:bext  ext
	  tib-:1st   id
	  tib-:bn    bn
	  )
    nil					;so that can be hook func
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tib-eval (type)
  "Chew buffer contents. Be sure to run tib-study first so that
global variables get set.

Return:
  string     ,type string
  symbol     ,if real mode found in first line -*- ..-*-
  nil
"
  (let* (
	 (l-ext tib-:file-ext-re-alist)	;list ...
	 (l-bfn tib-:file-fn-re-alist)
	 (l-bn  tib-:bn-re-alist)
	 (l-1st tib-:1st-line-re-alist)

	 (id  tib-:1st)		;these are already set
	 (ext tib-:bext)
;;;	 (fn  tib-:bfn)
	 (bn  tib-:bn)
	 el ret
	 )
    (cond
     ((eq type 'ext)
;;;      (d! "ext")
      (if (setq el (assoc ext l-ext))
	  (setq ret (cdr el))))

     ((eq type 'bfn)			;buffer name test
;;;      (d! "bfn")
      (setq ret (tib-match bn l-bfn)))	;whole file match

     ((eq type 'bn)			;buffer name test
      (setq ret (tib-match bn l-bn))
      )

     ((and (eq type '1st-emacs)		;special -*-Emacs-Lisp-*-
	   (stringp id))
;;;      (d! "1st-emacs")
      (setq ret (tib-match-emacs-mode id)))

     ((and (eq type '1st-re)
	   (stringp id))
;;;      (d! "1st")
      (setq ret (tib-match id l-1st)))

     ((eq type 'buf-re)			;whole buffer is searched
;;;      (d! "buf-re")
      (setq ret (tib-buf-match)))
     )
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - testing/evaluation  functions , begin with letters 'tib-t' ,
;;;   eg. tib-tre = regexp test
;;; - Pre-argument calls to tib-eval function

(defun tib-text		() "" (tib-eval 'ext))
(defun tib-tbfn		() "" (tib-eval 'bfn))
(defun tib-tbn		() "" (tib-eval 'bn))
(defun tib-tre1st	() "" (tib-eval '1st-re))
(defun tib-tre1st-e	() "" (tib-eval '1st-emacs))
(defun tib-trebuf	() "" (tib-eval 'buf-re))

;;}}}
;;{{{ main

;;; ############################################################ &main ###

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tib-info (&optional mode variable-lookup verb)
  "Tries to identify buffer type by guessing. It doesn't rely on mode,
because that doesn't necessarily tell what the buffer holds. Many users still
program their shell scripts with fundamental and so on.

Mode check isn't included, because you can check it yourself easily:
if this function returns nil, _then_ it's propably the time to check
the mode :-/

The normal order of evaluation is as follows:
-  First line in the file
-  Whole filename including path = buffer-file-name
-  File name extension
-  buffer-name is checked. [tmp buffers has no file name]
-  Whole buffer is searched for RE texts

References:
   tib-func-alist  ,order of evaluation.

Return values:
  when optional MODE = nil
  Some appropriate _string_ that represents the content. notice that this
  string is usually generalized description, _but_ it the file has special
  1st line in form of -*-..-*- the string is direct mode name string.

  when optional MODE = non-nil
  Return possible mode name as _symbol_

  when VARIABLE is non-nil, the variable tib-:info is read instead.
  If it has non-nil value, the value is returned, otw full buffer
  is parsed again and variable's value is updated.

References:
  tib-:info	,buffer local variable updated during every call.
"
  (interactive)
  (let* ((verb  (or verb (interactive-p)))
	 (funcs (tib-get-falist))
	 ret
	 func
	 doit
	 )

    ;; .................................................... do lookup? ...
    (setq ret tib-:info)

    (cond
     ((null variable-lookup)
      (setq doit t))

     ((and variable-lookup (null tib-:info)) ;no value stored
      (setq doit t))

     ((and variable-lookup		;must same type
	   (null mode)			;string request
	   (not (stringp tib-:info)))
      (setq doit t))

     ((and variable-lookup		;must same type
	   mode				;symbol request
	   (not (symbolp tib-:info)))
      (setq doit t)
      ))

    ;; .................................................... do the job ...
    (when doit
      (tib-study)			;prepare globals to avoid overhead
      (while (and funcs
		  (setq func (car funcs))
		  (null (setq ret (funcall func))))
	(NEXTP funcs)
	)

      ;;  how the results should be returned ?
      (when ret				;found anything?
	(if mode
	    (if (symbolp ret)		;return symbol
		ret			;it's real mode name
	      (setq ret (tib-cnv-txt2mode ret))) ;return possible mode name
	  (if (symbolp ret)
	      (setq ret (symbol-name ret)))
	  )
	(if verb (message (prin1-to-string ret)))
	)

      ;; Update the buffer local variable
      (setq tib-:info ret)
      )


    ret
    ))

;;}}}

(provide   'tinybufid)
(run-hooks 'tib-:load-hook)

;;; tinybufid.el ends here
