;; @(#) tinyck.el -- (C)onvert (k)eybindings for XEmacs or Emacs

;; @(#) $Id: tinyck.el,v 1.26 1997/07/04 16:03:48 jaalto NotReleased $
;; @(#) $Keywords: tools keybindings conversion $
;; $KnownCompatibility: 19.28+, XEmacs 19.14+ $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $


;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 13 1996
;;
;; To get information on this program use ident(1) or do M-x tinyck-version
;; Look at the code with folding.el, tinybm.el

;; LCD Archive Entry:
;; tinyck|Jari Aalto|jari.aalto@poboxes.com|
;; (C)onvert (k)eybindings. You can use both XEmacs or Emacs styled bindings|
;; 04-Jul-1997|$Revision: 1.26 $|~/misc/tinyck.el.Z|

;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;;  Put this file on your Emacs-Lisp load path, add following into your
;;  ~/.emacs startup file. This must be the very first entry before
;;  any keybindings take in effect.
;;
;;      (require 'tinyck)
;;
;;  You can also use the preferred way: autoload
;;
;;      (autoload 'tinyck-advice-control "tinyck")
;;
;;  And when you need conversion you wrap the code with calls:
;;
;;      (tinyck-advice-control)           ;; ON
;;      <key definitions>
;;      (tinyck-advice-control 'disable)  ;; OFF
;;
;;  Remember that you DON'T LEAVE THIS PACKAGE ON. Make sure the 'disable
;;  is the last thing you do. It disables the package and makes sure your
;;  other emacs packages work properly
;;
;;  If you have any questions, use this function
;;
;;       M-x tinyck-submit-feedback


;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:
;;
;;  Preface
;;
;;	This file tries to overcome differencies between Emacs and XEmacs
;;	keybinding. I developed this at the time when I used 19.28 and
;;	I wanted to use my other packages with any emacs version in
;;	spite of key definition differencies. This file is in fact
;;	"library" and propably interests only lisp programmers that want
;;	to make some old package, that has Emacs specific bindings,
;;	to work in XEmacs (or vice versa).
;;
;;  Emacs 19.30+ note
;;
;;      Newer Emacs release now supports XEmacs styled bindings.
;;      You can write
;;
;;          (local-set-key [(control meta up)] 'ignore)
;;
;;      and it should work both in XEmacs and Emacs. If all your keybindins
;;      are like that and you don't use Emacs lower than 19.30, then you
;;      don't need this package.
;;
;;  Putting your key definitions to separate file
;;
;;      You should separate all you keybindings to _one_ file,
;;      do not stuff all your emacs definitions in one huge ~/.emacs,
;;      but instead use some basic structure like this:
;;
;;          ~/.emacs            -- the main
;;          ~/.emacs.kbd.el     -- All the keybindinds
;;          ~/.emacs.x.el       -- X settings and X-packages loads
;;          ~/.emacs.vc.el      -- vc settings and modifications
;;          ~/.emacs.bup.el     -- Backup settings and redefinitions
;;          ~/.emacs.gnus.el    -- gnus customization
;;          ..
;;
;;      You can load your other initialize files from .emacs with `load'
;;	command. Suppose you have Emacs keybinding startup file; which you
;;	want to make compatible with XEmacs too.
;;
;;	    ;; Begin of startup file loader
;;	    ;;
;;          (autoload 'tinyck-advice-control "tinyck")
;;
;;          (if (xemacs-p)               ;; From tinylibm.el , the 'M' lib
;;              (tinyck-advice-control)) ;; We need conversion in XEmacs
;;
;;          ;; This file has Emacs styled key settings
;;          (load "~/.emacs.kbd")
;;
;;          (if (xemacs-p)
;;              (tinyck-advice-control 'disable)) ;; Turn conversion off
;;
;;
;;          (load "~/.emasc.x)
;;          ...
;;          ;; End of startup loader
;;
;;
;;  Lowlevel explanation for function tick-convert
;;
;;      If you're in Emacs, you use X-event bindings like this
;;
;;          (global-set-key [C-up] 'ignore)
;;
;;      Unfortunately, this does not work in XEmacs, but using the
;;      conversion function before the definition, it does.
;;
;;          (global-set-key (tinyck [C-up]) 'ignore)
;;
;;      Now the current Emacs version gets the right keybinding,
;;
;;          for Emacs  it returns       --> [C-up]
;;          for XEmacs it returns       --> '(control up)
;;
;;      You can also use the XEmacs keybinding, since the conversion goes
;;      both ways. Having the following setting:
;;
;;          (global-set-key (tinyck '(control up)) 'ignore)
;;
;;      it converts this to suitable form depending on the current Emacs
;;      in use.
;;
;;  About advices
;;
;;      So that you don't have to go and add that 'tinyck' call for every
;;      keybinding, I have provided advices that hook themself to
;;      appropriate keybinding functions.  The conversion is then done
;;      transparently and you don't have to change a thing in you
;;      keybinding settings.
;;
;;  About debugging
;;
;;      If you suspect any weird behavior in your emacs while
;;      this package is loaded, you should check that the `tinyck-:debug'
;;      is turned on. (`M-x' `tinyck-debug')
;;
;;      The buffer `tinyck-:debug-buffer' constantly records any conversion
;;      actions and you can find the problems quickly. Please send the
;;      supicious/false conversion lines to the maintainer of this package
;;      and if possible, tell how the conversion should go in your opinion.
;;
;;      I'd recommend that you keep the debugging permanently on, because
;;      if problems arise afterwards and if the debug were off, there is
;;      no way to tell what went wrong in what command.
;;
;;      Important; when you have problems, increase
;;
;;          tinyck-:debug-buffer-size
;;
;;      immediately to some arbitrary big value so that you get all the
;;      conversions recorded.
;;
;;  Known limitations
;;
;;      This package tries to do its best to make the conversion, but
;;      sometimes it is just impossible. For example the following
;;      case is beyond of this package. In Emacs you can define
;;
;;          (define-key xxx-mode-map [?\C-`]            'xxx-tab-backward)
;;                                   ^^^^^^^
;;
;;      But when your're in XEmacs and you try to do the same, it gives
;;      error although tinyck is currently active. The reason is that lisp
;;      intepreter never actually passes string "[?\C-`]" to `define-key'
;;      but it actually evaluates the vector in place to integer value and
;;      sends that to `define-key' function . The call actually is seen in
;;      Emacs like this:
;;
;;          (define-key xxx-mode-map [4194400]          'xxx-tab-backward)
;;                                   ^^^^^^^^^in my HP-UX 9.05
;;
;;      And in XEmacs it is evaluates to this:
;;
;;          (define-key xxx-mode-map [0]                'xxx-tab-backward)
;;
;;      The code "0" appears, because XEmacs doesn't know Emacs "?\C-`".
;;      You should write [(control ?\`)] for XEmacs and it would work ok.
;;      Be aware of this limitation if you plan to use same key commands
;;      for both Emacs and search for another solution. In these cases you
;;      are adviced to do the following instead:
;;
;;          (require 'tinyck)                   ;Keybind converter.
;;          (tinyck-advice-control)             ;turn it on
;;
;;          (define-key tit-mode-map [(control ?\`)]  'tit-tab-backward)
;;          ;; And other similar keybindings ...
;;          ;; ..
;;
;;          (tinyck-advice-control 'disable)    ;don't let it on
;;
;;      The presented code will now work for any Emacs and XEmacs.
;;      In general, it seems to be more safer to write keybindings
;;      in XEmacs style than in old Emacs style.
;;
;;  Thank you
;;
;;      Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;      Presented initial idea of the conversion process.
;;      Commented how the conversion should go in XEmacs.
;;
;;      Stephen Eglen  <stephene@cogs.susx.ac.uk>
;;      Stephen had the patience to send bug reports from XEmacs 19.12 and
;;      test new versions of tinyck.el

;;}}}
;;{{{ History

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jul	4	1997	[jari]		19.28	v1.26		NotReleased
;; - Forget to handle down-mouse-1 --> button1up conversion. Now ok.
;; - False conversion from XEmacs (control button3up) to Emacs. Now ok.
;;
;; Jun	19	1997	[jari]		19.28	v1.24-1.25	NotReleased
;; - A-kp-tab should be read as 'Alt kp_tab' not 'Alt kp-tab'. Corrected.
;;
;; Apr	18	1997	[jari]		19.28	v1.22-1.23	NotReleased
;; - Some textual changes. Added handling of symbol whose name has two
;;   parts: kp-tab is kp-tab, not 'kp' and 'tab'
;;
;; Mar	5	1997	[jari]		19.28	v1.22		NotReleased
;; - I haven't changed a thing in this pacakge for a very long
;;   time at it has been working very well. I turned off the debug
;;   because I assume that there won't be no major problems ahead.
;; - Corrected some text typos. Renames all varaibles to use -: syntax
;;
;; Jan  15      1997    [jari]          19.28   v1.20-1.21      Beta
;; - Corrected bug that prevented turning on debug.
;; - Clarified the installation instructions.
;;
;; Sep  25      1996    [jari]          19.28   v1.19           Beta
;; - Just some cosmetic changes.
;;
;; Aug  20      1996    [jari]          19.28   v1.15-18        Beta
;; - Removed all XEmacs 19.14 byteCompiler warnings. And corrected one error
;; - Corrected XEmacs byteCompiler errors.
;;
;; Aug  16      1996    [jari]          19.28   v1.14           Beta
;; - Some more problems, eg. ESC key code ?\e handling for XEmacs
;;   corrected. Wrote new section 'Known limitations'.
;;
;; May  17      1996    [jari]          19.28   v1.12-13        Beta
;; - Still small error, pointed by Vladimir. In XEmacs you can write
;;   [f1 a], where a is a symbol meaning "a" character. This would be
;;   an error in Emacs.
;; - Also corrected the xe2gnu, gnu2xe vector calls in 'tinyck'.
;;
;; May  16      1996    [jari]          19.28   v1.11           Beta
;; - This is MAJOR REWRITE.
;; - All functions rewritten and the whole conversion is now done
;;   totally differently. Thanks to Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;   who send define-key examples how the conversion should go.
;;
;; May  13      1996    [jari]          19.28   v1.10           Beta
;; - Added 1000 line size limit for the debug buffer. Renamed all
;;   variables. Separated the installation part. Added 19.30 and
;;   debug text sections.
;;
;; May  13      1996    [jari]          19.28   v1.9            Beta
;; - Added debugging capabilities. This should make troubleshooting
;;   easier.
;;
;; May  13      1996    [jari]          19.28   v1.8            Beta
;; - Vladimir showed that the conversion function didn't accept
;;   more complex structures like [f1 a o]
;; - corrected that problem, now [?\C-z mouse-1] is turned into
;;   '(26 button1) in XEmacs

;;}}}


;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'advice)

(eval-and-compile
  ;;  Don't require lib package unnecessarily
  (autoload 'ti::m-pkg-version-info    "tinylib")
  (autoload 'ti::m-pkg-submit-feedback "tinylib")
  )

;;}}}

;;{{{ setup: -- private

;;; ......................................................... &v-hooks ...


(defvar tinyck-:load-hook '(tinyck-advice-control)
  "*Hook run when file has been loaded.")

;;; ....................................................... &v-private ...
;;; Private varaibles


(defconst tinyck-:xemacs-flag  (string-match "XEmacs" (emacs-version))
  "Automatic flag that is set if XEmacs is detected.")

(defconst tinyck-:emacs-minor
  (if (boundp 'emacs-minor-version)
      emacs-minor-version 0)
  "Emacs minor version or 0 if cannot detect one.")


(defconst tinyck-:advice-re  "^tinyck-keybind"
  "Advice REGEXP.")

(defvar tinyck-:this-command  nil
  "Private. Current advice command.")


;;  To prevent buffer growing too much
;;
(defvar tinyck-:debug-buffer-size 500
  "Clear the `tinyck-:debug-buffer' if line count exceed this value.")


(defvar tinyck-:debug-buffer "*tinyck-debug*"
  "Debug buffer for keybinding commands.")

;;}}}
;;{{{ setup: -- user

;;; ........................................................ &v-public ...
;;; User configurable, but in general you don't need to touch this
;;; section.

(defvar tinyck-:debug nil
  "*Turn on/off key conversion debugging.")

(defvar tinyck-:keep-next-symbol-together
  '("kp"
    )
  "*Keeep SYMBOL and next keybind definition together.
When this string (or symbol) is found from keybinding definition, it is
not a standalone event name, but only part of it. After reading the next
binding name, the X-event has been read in full.

Eg. 'kp' is a prefix for keypad X-event symbols, so we actually mean
one key when we say 'kp-tab' and not two separate keys like 'kp' and 'tab'.


Format:
 '(STRING-SYMBOL
   S-S
   ..
   )

")

(defconst tinyck-:key-table
  '(
    (A          . alt)
    (C          . control)
    (H          . hyper)
    (S          . shift)
    (s          . super)
    (M          . meta)
    (mouse-1    . button1)
    (mouse-2    . button2)
    (mouse-3    . button3)
    (down-mouse-1    . button1up)
    (down-mouse-2    . button2up)
    (down-mouse-3    . button3up)
    )
  "*Keybind modifier mappings from Emacs to XEmacas.
Format:
'((EMACS-MODIFIER . XEMACS-MODIFIER) (E . XEmacs) ..)
")

;;}}}
;;{{{ setup: -- version

;;; ......................................................... &version ...

(defvar tinyck-:version-id
  "$Id: tinyck.el,v 1.26 1997/07/04 16:03:48 jaalto NotReleased $"
  "Full program version ID string.")

;;; ----------------------------------------------------------------------
;;;
(defun tinyck-version (&optional arg)
  "version information."
  (interactive "P")
  (ti::m-pkg-version-info "tinyck.el" arg))

;;; ----------------------------------------------------------------------
;;;
(defun tinyck-submit-feedback ()
  "Submit suggestions, error corrections, impressions, anything..."
  (interactive)
  (ti::m-pkg-submit-feedback "tinyck.el"))

;;}}}

;;; ########################################################### &funcs ###

;;{{{ misc, debug

;;; ----------------------------------------------------------------------
;;;
(defmacro tinyck-do-p (arg)
  "Check if conversion is needed."
  (` (not (stringp (, arg)))            ;pass "" string bindings as is
   ))


;;; ----------------------------------------------------------------------
;;; - Just for load hook
;;;
(defun tinyck-debug-on ()
  "Turn on debug."
  (interactive)
  (tinyck-debug 1))

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tinyck-debug  (&optional arg)
  "Turn on tinyck debug on or off. See buffer `tinyck-:debug-buffer'."
  (interactive)
  (cond
   ((eq 1 arg)
    (setq tinyck-:debug t))
   ((memq arg '(0 -1))
    (setq tinyck-:debug nil))
   (t
    (setq tinyck-:debug (not tinyck-:debug))
    ))

  (if (interactive-p)
      (message (concat "Debug " (if tinyck-:debug "on" "off"))))
  )


;;; ----------------------------------------------------------------------
;;;
(defun tinyck-debug-write (str)
  "Record debug information."
  (let* ((buffer (get-buffer-create tinyck-:debug-buffer))
         )
    (save-excursion
      (set-buffer buffer)

      (if (> (count-lines (point-min) (point-max))
             tinyck-:debug-buffer-size)
          (erase-buffer))

      (goto-char (point-max))
      (insert str)
      )))

;;}}}
;;{{{ advice

;;; .......................................................... &advice ...

;;; ----------------------------------------------------------------------
;;;
(defun tinyck-advice-control (&optional disable verb)
  "Install advices or optionally DISABLE them."
  (interactive "P")
  (let* ((funcs
          '(global-set-key
            local-set-key
            define-key
            ))
         (re   tinyck-:advice-re)
         (verb (or verb (interactive-p)))
         func
         )
    (while funcs
      (setq func (car funcs))
      (condition-case nil
          (progn
            (if disable
                (ad-disable-advice  func 'any re)    ;;clear flag
              (ad-enable-advice     func 'any re))
            (ad-activate func)                              ;;change state
            ))
       (setq funcs (cdr funcs))
       )
    (if verb
        (if disable
            (message "tinyck Advices disabled.")
          (message "tinyck Advices activated.")))
    ))


;;; ----------------------------------------------------------------------
;;;
(defadvice global-set-key  (before tinyck-keybind-converter  dis)
  "XEmacs and Emacs emulation. See function `tinyck' for full story."
  (setq tinyck-:this-command 'global-set-key)
  (if (tinyck-do-p (ad-get-arg 0))
      (ad-set-arg 0 (tinyck (ad-get-arg 0)))))

;;; ----------------------------------------------------------------------
;;;
(defadvice local-set-key  (before tinyck-keybind-converter  dis)
  "XEmacs and Emacs emulation. See function `tinyck' for full story."
  (setq tinyck-:this-command 'local-set-key)
  (if (tinyck-do-p (ad-get-arg 0))
      (ad-set-arg 0 (tinyck (ad-get-arg 0)))))

;;; ----------------------------------------------------------------------
;;;
(defadvice define-key  (before tinyck-keybind-converter  dis)
  "XEmacs and Emacs emulation. See function`tinyck' for full story."
  (setq tinyck-:this-command 'define-key)
  (let* ((arg (ad-get-arg 1))
         )
    (cond
     ((tinyck-do-p arg)
      (if (and (vectorp arg)
              (length arg)              ; "[0]"
              (eq 0 (elt arg 0))
              )
         (error                         ; otw user doesn't know what
          (concat                       ; going on.. barf immediately
           "define-key/tinyck.el: "
           "Vector contains zero. Did you use Emacs styled \"[?\\C-`]\" "
           "Wich can't be converted? Use equivalent [(control ?\\`)] "
           "instead which works for both XEmacs and Emacs. "
           "See comments in tinyck.el for more."
           )))
      (ad-set-arg 1 (tinyck (ad-get-arg 1)))
      ))
    ))


;;; ----------------------------------------------------------------------
;;; (ad-unadvise 'tinyck)
;;;
(defadvice tinyck  (around tinyck-debug act)
  "Debug filter. Record command, input/output values."
  (cond
   ((eq nil tinyck-:debug)
    ad-do-it
    )
   (t
    (tinyck-debug-write
     (format
      "\n%-15s %-25s >> "
      (or (prin1-to-string tinyck-:this-command) "")
      (or (prin1-to-string (ad-get-args 0))     "")
      ))

    ad-do-it

    (tinyck-debug-write
     (concat
      (prin1-to-string ad-return-value)
      ))
    )))

;;}}}
;;{{{ conversions

;;; ----------------------------------------------------------------------
;;;
(defun tinyck-get-key-code-string  (str)
  "Convert A-C-k --> ?\\A-\\C-k"
  (let* ((ret   "?\\")
         (len   (length str))
         (i     0)
         ch
         )
    (while (< i len)
      (setq ch (aref str i))
      (setq ret
            (concat
             ret
             (if (eq ch ?-)
                 "-\\"
               (char-to-string ch))))
      (setq i (1+ i))
      )
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - This is kinda faking Emacs, but since there is _no_ other way to
;;;   find the integer code for vector [?\A-a], we have to make Emacs
;;;   tell it to us.
;;;
(defun tinyck-get-key-code (simple-key-sequence)
  "Find out the integer value for SIMPLE-KEY-SEQUENCE, like S-a"
  (let* (lisp-mode-hook                 ;don't run any hooks while here
         (buffer (get-buffer-create "*tmp*"))
         (modes  '(lisp-mode
                   emacs-lisp-mode
                   lisp-interaction-mode
                   ))
         ret
         )

    ;; XEmacs doens't have this variable, Quiet ByteCompiler warning.
    ;; This is no-op
    ;;
    (if lisp-mode-hook
        (setq lisp-mode-hook nil))

    (setq simple-key-sequence
          (tinyck-get-key-code-string simple-key-sequence))

    (save-excursion
      (set-buffer buffer)
      (erase-buffer)

      ;;  Don't just always execute (lisp-mode), since
      ;;  setting up major mode may be time consuming.
      ;;
      (if (not (memq major-mode modes))
          (lisp-mode))

      (insert "[" simple-key-sequence "]")

      ;;  This spits out the integer number
      (eval-last-sexp 1)
      (beginning-of-line)
      (cond
       ((looking-at ".*\\[\\([0-9]+\\)")
        (setq ret
              (string-to-int
               (buffer-substring (match-beginning 1) (match-end 1))))
        ))
      ret
      )))


;;; ----------------------------------------------------------------------
;;;
;;; (tinyck-gnu2xe-vector [C-kp-tab])
;;;
(defun tinyck-gnu2xe-vector  (vec)
  "Convert Emacs vector bindings to XEmacs style.
[C-kp-tab]   --> [(control kp-tab)]
[M-f1 C-f2]  --> [(meta f1) (control f2)]
[?\e delete] --> [(meta delete)]  , we suppose meta is same as ESC."
  (let* ((table     tinyck-:key-table)
	 (keep-list tinyck-:keep-next-symbol-together)
         (i     0)
         len
         x
         elt
         str
         pos
         aset-pos
         list
         new-vec
	 gather-flag
	 gather-str
         )
    (setq len (length vec)  elt (elt vec 0))
    (setq i        0
          aset-pos 0
          )
    (setq new-vec (make-vector len nil)) ;put results here

    ;;  This is for due to ESC key in commands like:
    ;;  Emacs [?\e ?k] --> XEmacs [(meta k)]
    ;;
    (cond
     ((and (eq 27 elt)                  ;first element is ?\e
           (symbolp (elt vec 1)))
      ;;  Put elements 0 and 1 together "?\e delete" --> "M-delete"
      (setq str (concat "M-" (symbol-name (elt vec 1))))
      (aset vec 1 (intern str))
      (setq i 1)                        ;start here, skip item 0
      (setq new-vec (make-vector (1- len) nil))
      )

     ((and (eq 27 elt)                  ;first element is ?\e
           (eq 2 len))
      (setq str (concat "M-" (char-to-string (elt vec 1))))
      (aset vec 1 (intern str))
      (setq i 1)                ;start here, skip item 0
      (setq new-vec (make-vector (1- len) nil))
      ))




    (while (< i len)
      (setq x (elt vec i))

      (cond
       ;;  [?\C-x mouse-1] ==> [(control x) (button1)]
       ((and (integerp x)
             (< x 27))
        (setq x
              (list
               'control
               (intern (char-to-string (+ 96 x))) ; ?\C-a -- "a"
               ))
        )

       ((integerp x)                    ; other [?\C-z ...]
        nil)

       ((symbolp x)
        (setq str (symbol-name x));; [C-up] => "C-up"
        (setq list nil)


        (while str
          (if (or (string-match "^\\(down-\\)?mouse-[1-3]" str)
                  (string-match "^[^-]+" str))
              (progn
                (setq pos (match-end 0))
                (setq elt (substring str 0 pos))

                (if (< pos (length str))
                        (setq str (substring str (1+ pos)))
                      (setq str nil))
		)

	    ;;  No more "-" characters in string
	    ;;
	    (setq elt str   str nil)
	    )


	  ;;  There are certain X symbols that should be kept together
	  ;;  [C-kp-tab]  --> (control kp_tab) and not (control kp tab)
	  ;;
	  ;;
	  (cond
	   ((member elt keep-list)
	    (setq gather-str elt  gather-flag 0   elt nil))

	   ((stringp gather-str)
	    (setq gather-flag (1+ gather-flag))
	    (if (eq 1 gather-flag)
		(setq elt (concat gather-str "_" elt)
		      gather-str  nil
		      gather-flag nil
		      )
	      (setq elt gather-str   gather-flag nil   gather-str nil))
	    ))

;;;	  (d! elt str gather-flag gather-str list)
	  (cond
	   (elt
	    (setq elt (intern elt))
	    (if (setq x (assq elt table))
		(setq elt (cdr x)))
	    (setq list (append list (list elt)))
	    )))


        (setq x list)
        ))

      (aset new-vec aset-pos x)
      (setq i (1+ i)   aset-pos (1+ aset-pos))
      )
    new-vec
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tinyck-xe2gnu-vector  (vec)
  "Convert [(meta f1) (control f2)] --> [M-f1 C-f2]"
  (let* ((i     0)
         len
         sym
         x
         new-vec
         )
    (setq len (length vec))
    (setq new-vec (make-vector len nil))

    (while (< i len)
      (setq x (elt vec i))

      (cond
       ((integerp x)                    ;[?\C-z ...]
        nil)                            ;as is

       ((and (symbolp x)
             (setq sym (symbol-name x))
             (eq 1 (length sym)))       ;one character
        ;; In XEmacs, it's valid to have [f1 a], where 'a' means character
        ;; a. In Emacs you'd need ?a for that.
        ;;
        (setq x (string-to-char sym)) ;as char
        )


       ((listp x)
        (setq x (tinyck-xe2gnu-list x))
        ))

      (aset new-vec i x)
      (setq i (1+ i))
      )
    new-vec
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tinyck-xe2gnu-list  (list)
  "(meta f1) --> M-f1 symbol, or '(alt a) --> 120345 some keycode integer."
  (let* ((table tinyck-:key-table)
         item elt
         str padd
         ret
         )
    (setq str "")
    (while list
      (setq elt (car list))
;;;      (d! (car list) (rassq elt table) list)

      (setq padd (if (cdr list)
                     "-"
                   ""))

      (cond
       ((setq item (rassq elt table))
        (setq elt (symbol-name (car item)))
        )
       ((integerp elt)
        (setq elt (char-to-string elt)))

       ((and (stringp str)
             (symbolp elt))             ;keep it as string, see concat
        (setq elt (symbol-name elt)))
       )

      (setq str (concat str elt padd))
      (setq list (cdr list))
      )

;;;    (d! str)

    (cond
     ((string= "" str)
      nil)
     ((and (not (string-match "mouse" str))
	   (string-match "-.$\\|^[?][\\]?" str) ; "A-a"  "A-C-k" "?\C-`"
	   )
      (setq ret (tinyck-get-key-code str)))
     (t
      (setq ret (intern str))
      ))
    ret
    ))

;;}}}
;;{{{ main

;;; ----------------------------------------------------------------------
;;; - 20 Apr 1996, Idea by Vladimir Alexiev <vladimir@cs.ualberta.ca>
;;; - 22 Apr 1996, Reprogrammed by Jari Aalto <jaalto@tre.tele.nokia.fi>
;;;
;;;###autoload
(defun tinyck (key &optional xe)
  "Transform key binding to XEmacs or Emacs in current envinronment.
on current emacs. This enables you to have same keybinding file
for both emacs versions. You can write keybindings either in XEmacs
or Emacs style.

    In Emacs :  (tinyck '(meta up)) --> [M-up]
    In Xemacs:  (tinyck [M-up])     --> '(meta up)

This function does the conversion only if it needs to, and returns
immediately if no conversion is needed. This should minimize performance
penalty.

Input:
  key   ,key sequence
  xe    ,flag. If this is nil, then Emacs env. is assumed. However
         `tinyck-:xemacs-flag' is obeyed if it is non-nil.
        ,if non-nil, then XEmacs env. is assumed and conversion to
         XEmacs like bindings are done."
  (let (
        ;;      For greater speed this is read from variable
        ;;      and not dynamically for every call.
        (xe     (or xe tinyck-:xemacs-flag))
        ret
        vec
        D                               ;debug
        )

    ;; ................................................... vector2list ...
    (cond
     ((and (not xe)                     ; in Emacs
           (vectorp key))               ; [C-up]
      (cond
       ((and (listp (elt key 0))
             (< tinyck-:emacs-minor 30)) ;19.30 supports [(control up)]
        (setq D "1 xe2gnu-vector")
        (setq ret (tinyck-xe2gnu-vector key))
        )
       (t
        (setq D "1 as is")
        (setq ret key))))               ; return "as is"


     ((and (not xe)                     ; '(control f1) --> C-fi
           (listp key))
      (setq D "2 tinyck-xe2gnu-list")
      (setq vec (make-vector 1 nil))
      (setq ret (tinyck-xe2gnu-list key))
      (aset vec 0 ret)
      (setq ret vec)
      )


     ((and xe
           (or (listp key)              ; '(control up) in XEmacs
               (symbolp key)            ; 'button2
               (and (vectorp key)       ; [(button2]) case...
                    (listp (elt key 0)))
               ))
      (setq D "3")
      (setq ret key))                   ; return "as is"

     ((and xe
           (vectorp key))               ; [C-up] to XEmacs
      (setq D "4  gnu2xe-vector")
      (setq ret (tinyck-gnu2xe-vector  key))
      ))

    ;; Quiet XEmacs 19.14 ByteCompiler, This is no-op.
    (if D (setq D D))
;;; (d! D)


    ret
    ))

;;}}}

(provide   'tinyck)
(run-hooks 'tinyck-:load-hook)

;;; tinyck.el ends here
