;; @(#) tinydiff.el -- Diff output minor mode. Browsing, patching.

;; @(#) $Id: tinydiff.el,v 1.94 1997/07/02 08:54:00 jaalto NotReleased $
;; @(#) $Keywords: tools $
;;
;; $KnownCompatibility: 19.28+ XEmacs19.14+ $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Jan 21 1996
;;
;; To get information on this program use ident(1) or do M-x tdi-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinydiff|Jari Aalto|jari.aalto@poboxes.com|
;; Diff minor mode. Diff browsing, applying email patches easily.|
;; 02-Jul-1997|$Revision: 1.94 $|~/misc/tinydiff.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;;   Put this file on your Emacs-Lisp load path, add following into your
;;   ~/.emacs startup file
;;   Code can be easily extracted with tinylib.el / ti::m-pkg-rip-magic
;;
;;* _
;;* 	(setq tdi-:diff-program  "gdiff")      ;; Use GNU if possible
;;* 	(setq tdi-:patch-program "gdpatch -t")
;;* _
;;*     (require 'tinydiff)
;;
;;   OR use this; your .emacs will load much  quicker
;;
;;* _
;;*     (autoload 'tdi-mode             "tinydiff" t t)
;;*     (autoload 'tdi-diff             "tinydiff" t t)
;;*     (autoload 'tdi-diff-show        "tinydiff" t t)
;;*     (autoload 'tdi-diff-show-noask   "tinydiff" t t)
;;
;;   I also suggest that you bind those commands into keys like:
;;
;;* _
;;*     ;; Personally I bind all (m)inor modes under keys C-c "m"
;;*     (global-set-key "\C-cmd         'tdi-mode)
;;*     (global-set-key "\C-cd"         'tdi-diff-show)
;;*     (global-set-key "\C-cp"         'tdi-patch)
;;* _
;;
;;   If you have any questions, always use function
;;
;;	M-x tdi-debug-toggle		turn on debug before you send bugs
;;      M-x tdi-submit-bug-report       send bug report


;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tdi-" in front of
;;   every function & variable. It stands for '(ti)ny (di)ff minor mode.
;; - variable names contain letters 'tdi-:'

;; Preface, jan 1996
;;
;;      Long ago I made simple function that would generate instant diff
;;      for the file I was editing before I would check it in RCS. I
;;      didn't use vc.el, since it was not in the emacs distribution back then
;;
;;      I started wondering why I used "goto-line" command in other buffer
;;      every time I wanted to jump to diff point... and it gave me idea
;;      to make separate diff mode: it would handle jumping for me.
;;
;;  Overview of features
;;
;;      o   Buffer based simple diff/rcsdiff/patch package.
;;      o   You cab diff current buffer against last saved backup copy.
;;      o   Give and manipulate diff command in echo-area: file name
;;	    completion; switching between rcsdiff/diff, guess rcs version
;;	    for buffer ...
;;      o   When you have diff output in buffer, turning on `tdi-mode'
;;          allows you to browse source buffer by jumping diff blocks fwd/back
;;          and showing the source location.
;;      o   Supported: normal diff, context diff, gnu diff -u, gnu diff -n
;;      o   Easy patching. Finds file to patch (along user defined paths)
;;          and applies the diff. You can receive patches by email
;;          and apply them with one or two keystrokes.
;;      o   loads patch rejection file, if patch didn't succeed 100%
;;
;;  Genrating diff -- parsing diff
;;
;;      Be in buffer where you have diff file and just turn on the
;;
;;          tdi-mode
;;
;;      Then take a look at the bindings you have available:
;;
;;          C-x b
;;
;;      If you want to generate [rcs]diff for current buffer, call function
;;
;;          tdi-diff-show
;;
;;      And it shows diff and puts you in tdi-mode. X window users and
;;      those that have the highlighting capabitities can enjoy more about
;;      this mode, because it marks line numbers in buffer with
;;      mouse-face. You just click the point to jump to diff position
;;
;;  Doing diffs
;;
;;      The main purpose of this module is to help you taking "diff shots",
;;      inside emacs. This means that the file must be loaded into
;;      emacas, before you execute
;;
;;          M-x tdi-diff-show
;;
;;      o   If the file is not rcs controlled you're offered regular diff
;;      o   if file is rcs controlled, your're offered rcsdiff prompt
;;      o   if the buffer has changed, you're offered to diff against
;;          last saved file to see recent changes you have done since you
;;          saved the buffer.
;;
;;  Doing diffs: the Prereq tag
;;
;;	It is important that when you send diff, it is diff between two
;;	rcs versions if possible (if you're author of program). In those
;;	cases where revision information can be found, the diff data
;;	is preceeded with this line:
;;
;;	    Prereq: N.NN	eg. Prereq: 1.76
;;
;;	If the receiving end has GNU patch, the patch program first checks
;;	if the version that person has is exactly N.NN and aborts if
;;	he had some other version. This prevent applying diffs that
;;	are meant to other versions. Regular Unix *patch* program
;;	does not notice the *Prereq:* tag, so consider getting more
;;	safer GNU version as soon as possible.
;;
;;  Command prompt
;;
;;      The help key is on C-h, press it to get summary of command while
;;      you're in minibuffer prompt. The command prompt in minibuffer looks
;;      like this for rcs controlled file.
;;
;;          > cd /users/foo/dir1/dir2; rcsdiff -c -r1.21 test.txt
;;
;;      You can edit this command as much as you like, but please leave 'cd
;;      XXX' alone since the minibuffer commands expect it it be
;;      present.
;;
;;  Command prompt: rcsdiff and diff toggle
;;
;;      To conveniently construct diff command against another
;;      file, say test2.txt, you can hit key
;;
;;          C-z
;;
;;      to chage the command immediately to
;;
;;          > cd /users/foo/dir1/dir2; diff -c test.txt
;;
;;      And add the test2.txt to the end of line. If you want to restore
;;      the previous rcsdiff form, just hit C-z again. This C-z facility
;;      works only if the initial command was rcsdiff.
;;
;;  Command prompt: tab completes file name
;;
;;      While your're editing the command you can also use the TAB key to
;;      complete filename in the 'cd' command directory. If you specify any
;;      directories for the file, the directory's files are completed.
;;      That feature should allow you to get filenames into the prompt
;;      easily.
;;
;;  Command prompt: diffing between two RCS revisions
;;
;;      There is also more commands, like C-x which changes
;;
;;          > cd /users/foo/dir1/dir2; rcsdiff -c -r1.21 test.txt
;;
;;      command so that it has two -r commands. You can now take diffs
;;      between versions two easily. The C-x key is a toggle.
;;
;;          > cd /users/foo/dir1/dir2; rcsdiff -c -r1.21 -r1.21 test.txt
;;
;;  Command prompt: autosave and backup file diff
;;
;;      Otherhelpfull commands insert he #autosaved# and backup~ filenames
;;      into the current point. Remember to put the # or ~ file to
;;      the left and not to the right. you do want to diff current file
;;      against the saved one; right? The first one is original
;;      prompt. That second is after C-x and latter after C-v
;;
;;          > cd /users/foo/dir1/dir2; diff -c test.txt
;;                                            * point here
;;
;;          > cd /users/foo/dir1/dir2; diff -c #test.txt# test.txt
;;          > cd /users/foo/dir1/dir2; diff -c ~/backup/test.txt~ test.txt
;;
;;      Notice that your backup file may not reside int he same directory.
;;      The backupfilename is returned by function make-backup-file.
;;
;;  Patching
;;
;;      There is also included little patching function.
;;
;;          M-x tdi-patch          ,non verbose
;;          C-u M-x tdi-patch      ,verbose
;;
;;      For elisp (.el) files the load-path is automatically searched
;;      for possible destination of the patch. You can set variable
;;
;;          tdi-:patch-list
;;
;;      To match files and their associated patch directories if you
;;      receive patches for other files regularly. This function is
;;      most usefull for RCS diffs, because they can be easily
;;      detected and the file information is also included in the
;;      diff.
;;
;;  Patch: general notes
;;
;;      Note: normally when 'patch' program is called it always makes
;;      backup with the suffix .orig. So if you have applied a patch,
;;      then there is two file in the directory.
;;
;;          FILE.txt        -- patched file
;;          FILE.txt.orig   -- original file, before the patch
;;
;;      It also creates rejections file if all dind't go as planned.
;;
;;          FILE.txt.rej
;;
;;  Patch: success or failure
;;
;;      When the patch has been applied, This package checks if all went
;;      well. If rejection file was created; then the patch process's
;;      output is shown and the rejection file is loaded so that you can
;;      see what possibly went wrong and if you should be concerned.
;;
;;      If you get this rejection file, then there propably is
;;      potential trouble. Please
;;      contact the sender of patch immediately and tell about your
;;      troubles. There are few common reasons why patch failure happened.
;;
;;      o   Sender forgot -kk switch when he run rcsdiff to the file
;;          that was not owned by him (See RCS for details about -kk)
;;      o   Too few context, Sender should try increasing context with
;;          -C switch (like -C7)
;;      o   The patch got modified during email transition. Ask
;;          to send the patch in UU, PGP encrypted or PGP base64
;;          (clearsig off) signed format.
;;
;;  Patch: what happens after success
;;
;;      When the patch succeeds, there is a bit special handling for Emacs
;;      elisp packages. Say we recieve correction to the following module
;;      and you have it loaded in emacs: (feature 'foo) returns true.
;;
;;          foo.el
;;
;;      After patch is applied, you're asked if you want to reload the
;;      new release of *foo* module (just patched). You should answer
;;      'Yes' to get the newest one running in your Emacs immediately.
;;
;;  Patch: after success, returning to original version
;;
;;      If the patched version, which is usually new version of the progrmam
;;      doesn't work as it is supposed to, you can go back to the original
;;      version by appluing the same patch again. You should report what
;;      problems you had to the maintainer and inform that you wnet back
;;      to previous version.
;;
;;      *IMPORTANT* If you did get the rejection file, you can't use that
;;      patch to go back to original!! See next chapter how to go to
;;      original version in that case
;;
;;  Patch: rejection file created -- what to do?
;;
;;      If you want to go back to original version, apply the same diff
;;      again; this reverses just applied patch. Just call =M-x=
;;      =tdi-patch= in the buffer where you have the diff.
;;
;;      When you do that, the function detects that there is already a
;;      .orig file and prompts you to choose an appropriate action.
;;      Here is the explanation what they do and what should you choose
;;
;;      *o*
;;
;;      Go back to (o)riginal. This copies the FILE.txt.orig over the
;;      FILE.txt and deletes FILE.txt.orig and doesn't do _anything_
;;      else (stops the patching process). You're back to starting
;;      point as if you never patched anything.
;;
;;      *r*
;;
;;      (R)etry means that the FILE.txt.orig is copied over FILE.txt and the
;;      pach is tried again for FILE.txt. You may have asked the author to
;;      send more context with using the -C10 switch and after you received
;;      this new patch you want to try if it now goes ok. The FILE.txt.orig
;;      still remains as a backup
;;
;;      *g*
;;
;;      (G)o says that we should apply the diff again to FILE.txt. Do this
;;      _ONLY_ if you did not get rejections last time. The intention
;;      is that you apply the patch again, and this reverses the situation.
;;      I mean 1) you patch; you get new version 2) you patch again: you
;;      degrade to the version before patch (original file before patch)
;;
;;  Development note
;;
;;      I know about ediff.el, which is much more complete package
;;      than this is. My aim was to develop a simple but handy
;;      package for everyday diff'ing and easy package patching.
;;
;;  Bugs
;;
;;      The "f" key, which shows the function identifier, can handle
;;      buffers which are narrowed, but if the buffer is using folding.el
;;      or similar package where goto-line does not work properly, the
;;      returned message shown to user is not correct.
;;
;;      Please unfold the buffer and you get the correct result.
;;
;;  Sending good bug reports
;;
;;      If you find anything funny happening in the command line
;;      prompt while you use the tdi minibuffer commands. Immediately
;;      do following.
;;
;;      o   Turn on debug: `M-x' `tdi-debug-toggle'
;;      o   Turn on emacs debug: (setq debug-on-error t)
;;      o   Clear the debug buffer *tdi-debug* if it exists
;;      o   Start from original situation
;;      o   Do what you did and when the weird condition is met
;;          immediately go to *tdi-debug* buffer and save the
;;          content and send it to the maintainer.
;;

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jul	2	1997	[jari]		19.28	v1.95		NotReleased
;; - Problem in finding the filename of diff. Corrected.
;;
;; Jun	6	1997	[jari]		19.28	v1.93-1.94	NotReleased
;; - Chekdoc 1.42 clear. Added defcustom support; renamed some variables.
;; - Better warning if nox1 gnu diff found.
;; - New program find installation.
;;
;; May	14	1997	[jari]		19.28	v1.88-1.90	NotReleased
;; - Minibuffer keys didn't install when package got loaded. Corrected.
;; - Changed all non standard boolean tests to use when and unless
;;
;; Apr	21	1997	[jari]		19.28	v1.85-1.87	NotReleased
;; - now uses minor mode wizard.
;;
;; Mar	24	1997	[jari]		19.28	v1.76-1.84	NotReleased
;; - XEmacs20 CHAR handling corrections.
;; - now uses library function's file name completion in command prompt.
;;   This gives much better visual completion. Removed old completion
;;   code from this package.
;;
;; Mar	6	1997	[jari]		19.28	v1.75		NotReleased
;; - Moved diff type detection function to mail lib because it is used in
;;   other packages too. It was a general function.
;;
;; Mar	5	1997	[jari]		19.28	v1.74		NotReleased
;; - Corrected function tdi-patch-check-if-load function: it didn't reload
;;   the .el if it was running in the emacs. Also put together .rej and .orig
;;   messages so that no two separate messages are displayed. This saves
;;   some thime when calling the tdi-patch.
;;
;; Mar	4	1997	[jari]		19.28	v1.73		NotReleased
;; - Corrected autosave ## insert command.
;; - Corrected error which happend if file has no .el extentension
;;   (after aptch, checked emacs eliep file)
;;
;; Mar	1	1997	[jari]		19.28	v1.69-1.72	NotReleased
;; - corrected the "after patch" functions that check patch success status
;;   and offer loading files to emacs.
;; - Improved the tdi-patch fucntion. Now it offers
;;   3 choices if there is .orig file already for the file beeing patched.
;; - Reorganized the documentation and explained more about patching and
;;   the added features.
;; - quoted the '#autosave#' file.
;;
;; Feb  26      1997    [jari]          19.28   v1.65-1.68      NotReleased
;; - Corrected the file buffer detect function (from diff listing)
;; - Rewrote the highlighting to use text properties instead, because
;;   XEmacs support for overlays isn't possible.
;; - tinymy's autoloding of  mview function corrected.
;;
;; Feb  26      1997    [jari]          19.28   v1.61-1.64      NotReleased
;; - Added view mode setup to diff buffer.
;; - Added automatic FILE.el file reload to emacs if the package
;;   is currently running inside Emacs. This helps getting new
;;   release to Emacs very easily.
;; - Wrote more to the Documentation section: about the patches
;; - Adde new C-h HELP binding in minibuffer prompt.
;; - mode install fmacro call corrected.
;;
;; Feb  22      1997    [jari]          19.28   v1.58-1.60      NotReleased
;; - Renamed all variables to have -: including the hooks.
;; - Radical change in mode functions: they are now created by macro
;;   from the 'm' library. Makes code reuse much stronger.
;; - Now when calling tdi-diff-show, it warns about existing autosaved
;;   file that may be left when you killed emacs.
;;
;; Feb  18      1997    [jari]          19.28   v1.57           NotReleased
;; - Rewrote the debug. now there is user function tdi-debug.
;; - New user function C-c tdi-cl-insert-file-autosave and
;;   C-v tdi-cl-insert-file-backup. Very usefull when taking diffs
;;   against emacs saved files.
;;
;; Feb  14      1997    [jari]          19.28   v1.56           NotReleased
;; - Fixed C-x command line function. Didn't give two -r prompts.
;;
;; Jan  24      1997    [jari]          19.28   v1.52-1.55      NotReleased
;; - now all autoloads are in the 'm' lib.
;; - Removed the special diff program call. Now the diff prompt
;;   is given immediately.
;;
;; Nov  8       1996    [jari]          19.28   v1.50-1.51      NotReleased
;; - If there is .orig file, program offers to use it as destination
;;   to patch. This is usefull is a) patch failed b) user requested
;;   new patch c) now he want to try again.
;;
;;   answering "yes" does the following
;;
;;      file.txt.orig       --> delete file.txt
;;                              cp file.txt.orig file.txt
;;
;; Nov  7       1996    [jari]          19.28   v1.49           NotReleased
;; - Added "Prereq:" tag to version diffs, that is a signal to
;;   patch in receiving end.
;; - Corrected rejection file loading: now it loads rejects [bug]
;; - Now allows spaced at the beg of "*** "---", because gnu diff
;;   has switch --initial-tab.
;; - Main code modified so that it writes whole file out, and
;;   doesn't try to search the diff begin point any more. The patch
;;   can find the proper place, so we don't want to do extra work.
;; - Added confirmation is the file beeing patched already exist in
;;   emacs and it is not saved.
;;
;;
;; Nov  7       1996    [jari]          19.28   v1.45-48        NotReleased
;; - Small changes. Code layout changed a bit.
;; - Error in command-generate corrected.
;; - Error indetecting the diff file name, now anchors with "^"
;; - Corrected getting command prompt: user function was not called.
;; - Now offers loading the file into emacs if there was buffer
;;   with thast name or if the patched file was RCS controlled when
;;   patch took place.
;;   tdi-get-dir-from-cmd               :+
;;   tdi-patch-check-if-load                  :+
;;
;; Nov  1       1996    [jari]          19.28   v1.44           NotReleased
;; - Moved code around a bit and changed the tdi-diff-command-generate,
;;   so that the prompt is set in one place.
;;
;; Oct  31      1996    [jari]          19.28   v1.43           NotReleased
;; - Minor bug in main function: if the name of file was not found,
;;   then gave "region" error. Fixed.
;; - Now loads the rejection file automaticlly if needed.
;;
;; Sep  25      1996    [jari]          19.28   v1.41-1.42      NotReleased
;; - Rearranged loading, now uses as much as possible autoload to speed
;;   up package loading.
;;
;; Sep  19      1996    [jari]          19.28   v1.40           NotReleased
;; - Bug was sneaking in the tdi-patch. There may have been false
;;   (old) region active and the function didn't notice that. Now
;;   it makes sure that the min point of region is bigger or equal to
;;   diff start point. User sees message "Please draw region over diff..."
;;
;; Aug  30      1996    [jari]          19.28   v1.39           NotReleased
;; - Forgot to add diffing in RCS buffers too: now asks if you
;;   want to diff agains file on disk if buffer has changed.
;; - Added feature to check if underlying file has changed --> asks
;;   if you want to diff buffer against it.
;;
;; Aug  26      1996    [jari]          19.28   v1.38           NotReleased
;; - Added safe-widen, so that whole buffer is always saved when
;;   comparing buffer against buffer-file. Added "modtime" test to
;;   give "compare buffer" prompt if file has been changed on disk.
;;
;; Aug  23      1996    [jari]          19.28   v1.37           NotReleased
;; - Made XEmacs 19.14 compatible and cleared all ByteComp errors.
;;
;; Aug  14      1996    [jari]          19.28   v1.36           NotReleased
;; - Now it offers diff prompt even if called from buffer that has
;;   no associated file.
;; - For non-rcs files: if buffer is modified it asks if user wants to
;;   diff buffer content against file on the disk (most recent changes
;;   to last save command)
;;   tdi-diff-show                      :!
;;   tdi-diff-command-generate          :!
;;
;; Jun  28      1996    [jari]          19.28   v1.35           NotReleased
;; - XEmacs compatibility: used ti::xe-mark-active-p instead
;;
;; Jun  25      1996    [jari]          19.28   v1.34           NotReleased
;; - Renamed functions so that they do not clash with XEmacs 19.14
;;
;; Jun  20      1995    [jari]          19.28   v1.33           NotReleased
;; - There was a problem in detecting gnu unified diff's filename.
;;   now corrected.
;; - Added some tests that the diff is readable after it si written
;;   to file. This was needed becasue in one host the UMASK setting
;;   put the permission flat "---------" and that "patch < diff"
;;   of course couldn't even open the saves diff! Now the program
;;   changes file permissions if needed.
;;
;; Jun  19      1995    [jari]          19.28   v1.32           NotReleased
;; - Added new command "C-x" to add more -rX.x switched to prompt.
;;   That way yu can easily take diff between two versions.
;;   tdi-cl-rev-add-command             :+ new command
;;
;; Jun  19      1995    [jari]          19.28   v1.31           NotReleased
;; tdi-get-file-name                    :! bug, didn't find rcs file name
;;
;; Jun  11      1995    [jari]          19.28   v1.29-1.30      NotReleased
;; - stephene@cogs.susx.ac.uk (Stephen Eglen) Noticed bug when trying
;;   to get patch from PGP signed message.
;; - Now added the broken dash, "- -", detection and cleanup of PGP
;;   tags before writing the diff out to file.
;;
;; Jun  3       1995    [jari]          19.28   v1.28           NotReleased
;; - Now it doesn't offer saving *tmp* buffers.
;; - Check is C-x ESC ESC passes old command string,
;;   and let it go through.
;;
;; Jun  2       1995    [jari]          19.28   v1.27           NotReleased
;; - Corrected the C-z function, which flips diff commands
;; - rev-- and c++ corrected.
;;
;; May  25      1995    [jari]          19.28   v1.25-1.26      NotReleased
;; - defconst --> defvar. Rewrote  tdi-patch. little XEmacs compatibility.
;; - Got more inspiration to develop the command line interface.
;;   Now it can complete revision numbers.. well actually, ++/--.
;; - rewrote and added many lines of code. Now there is native lisp
;;   support to search previous diff version.
;;   "b" and "y"                        :+ new commands
;;   "[" and "]"                        :+ new commands
;;   tdi-:debug                         :+
;;   tdi-:version-list                  :+
;;   tdi-:version-branch-list           :+
;;   tdi-:diff-prompt                   :+
;;   tdi-:extra-diff-program            :! 'tdi-rcs-diff-between-versions
;;   tdi-rcs-diff-between-versions      :+
;;   tdi-goto-prev-no-update            :+
;;   tdi-goto-next-no-update            :+
;;   tdi-debug                            :+
;;   tdi-kill-revision-list             :+
;;   tdi-update-revision-list           :+
;;   tdi-cl-replace-text                :+
;;   tdi-cl-read-revision               :+
;;   tdi-cl-read-rcs-file-name          :+
;;   tdi-cl-complete-rev++              :+
;;   tdi-cl-complete-rev--              :+
;;   tdi-cl-change-diff-command         :!
;;   tdi-diff-show-noask                :! rewritten
;;   tdi-goto-next                      :! 'no-update' arg
;;
;; Apr  15      1995    [jari]          19.28   v1.24           NotReleased
;;   tdi-func-name-store                :! more accurate
;;   tdi-get-func-name                  :! safeguard against b-of-defun leap
;;
;; Apr  14      1995    [jari]          19.28   v1.23           NotReleased
;; - Changed modeline update command, because XEmacs labels it as obsolete.
;;
;; Apr  3       1995    [jari]          19.28   v1.22           NotReleased
;; - Added section 'editing diff command'.
;; - The diff change command is little smarter and user friendly now.
;;   tdi-:last-data                     :+
;;   tdi-cl-change-diff-command         :! rewritten
;;
;; Mar  14      1995    [jari]          19.28   v1.21           NotReleased
;; - More improvement for detecting the diff file name. Now ClearCase
;;   styled diff is detected.
;;   tdi-patch                     :! tries harder
;;
;; Mar  13      1995    [jari]          19.28   v1.20           NotReleased
;; - The patching function needed lot of corrections. Eg. there was
;;   some very weird bug, that patched my RMAIL file, while reading input
;;   with read-file-name.
;;   tdi-get-file-name                  :! new arg
;;   tdi-patch-with-diff-1                   :! uses minibuffer map now.
;;   tdi-patch                     :! tries harder to find name
;;
;; Mar  13      1995    [jari]          19.28   v1.19           NotReleased
;; - Now the minibuffer keybindings are user configurable
;;   tdi-:minibuffer-map                 :+
;;   'tdi-:mode-define-keys-minibuffer-hook    :+
;;   tdi-mode-define-keys-minibuffer-default       :+
;;   tdi-cl-change-diff-command         :+
;;
;; Mar  12      1995    [jari]          19.28   v1.18           NotReleased
;; - Now the patching is configurable for any file names and using
;;   any path to fincd the file. Some errors corrected when detecting
;;   patch file name from diff.
;;   tdi-:patch-list                    :+
;;   tdi-get-file-name                  :!
;;   tdi-patch-with-diff-1                   :!
;;
;; Mar  11      1995    [jari]          19.28   v1.17           NotReleased
;; - I wanted to edit the last diff command with ESC ESC, so I had to
;;   rewrite whole diff calling. Also I wanted nice fileanme completion
;;   inside minibuffer. Looks good now!
;;   tdi-diff-show                      :! rewitten...
;;   tdi-diff-show-noask                :!
;;   tdi-diff                           :!
;;   tdi-cl-complete-filename           :+ TAB handling
;;   tdi-diff-command-generate          :+
;;
;; Mar  8       1995    [jari]          19.28   v1.16           NotReleased
;; - Changed version variables not to have ":"
;;
;; Feb  18      1996    [jari]          19.28   v1.15           NotReleased
;; - Install 'marks' corrected. tdi-func-name-store, tdi-func-name-store
;;   rewritten
;;   tdi-:patch-program                 :! is now evaled to get the cmd
;;   tdi-func-name-store                :! rewritten
;;   tdd-first-file                     :! rewritten
;;
;; Feb  17      1996    [jari]          19.28   v1.14           NotReleased
;; - Some functions were renamed in tinylib, reflect changes.
;;
;; Feb  15      1996    [jari]          19.28   v1.13           NotReleased
;; - One small error reported by ByteComp corrected.
;; - Again, improved this package a giant leap: now there is function
;;   to apply patch from the buffer. Handy thing if you receive
;;   patches in mail. Detects at least elisp files if they are in
;;   rcsdiff form
;; - added section 'patching'
;;   tdi-:diff-tmp-buffer       :+
;;   tdi-:patch-program         :+
;;   tdi-patch-with-diff-1           :+
;;   tdi-patch             :+
;;   ti::b-diff-type-p            :! modified ret val
;;
;; Feb  15      1996    [jari]          19.28   v1.12           NotReleased
;; - Made the reference function, the  "f" key, user configurable.
;;   tdi-:find-ref-function         :+
;;   tdi-func-name-store        :! Changed to support other ref funcs
;;   tdi-show-function-name     :! new "Can't find ref" message
;;
;; Feb  11      1996    [jari]          19.28   v1.11           NotReleased
;; - Hmm, the bug still was there.. maybe the shell buffer output
;;   is somehow special, because only pop-to-buffer cured the problem.
;; - Now all crucial variables are evaled to get the content. This
;;   allows dynamic use of commnds and changing switches by user functions.
;; - Added 'bugs' section. Added narrowed buffer handling. Now "f"
;;   key can store the function name eg. for changelog entry.  ByteComp ok.
;;   tdi-:function-name-handle-function     :+
;;   tdi-:tdi-function-name-register             :+
;;   tdi-func-name-store                :+
;;
;; Feb  10      1996    [jari]          19.28   v1.10           NotReleased
;; - Corrected annoying bug that didn't put diff buffer's point at
;;   beginning of file after diff run.
;; - Added 'taking diffs' documentation.
;;
;; Feb  4       1996    [jari]          19.28   v1.9            NotReleased
;; - Corrected bug in tdi-diff. Now shows beg of buffer.
;; - Added some docs about taking diffs.
;;
;; Feb  4       1996    [jari]          19.28   v1.8            NotReleased
;; - Added (require 'view), because the precedence of the minor
;;   keymaps is significant. I want that tdi-map overrides the
;;   view mode map's keydefinitions.
;; - Added new function to show function name in the diff line.
;;   See "f" keybinding.
;; - Bug in tdi-parse-buffer, should have called tdi-diff-type
;; - Bug in tdi-diff, if "Id" rcs string was not found, the diff weren't run.
;;
;; Jan  27      1996    [jari]          19.28   v1.7            NotReleased
;; - tdi-version rewritten. Added new function ti::b-diff-type-p
;;
;; Jan  25      1996    [jari]          19.28   v1.6            NotReleased
;; - Added nice feeback and bug reporting functions.
;; - Now conforms lisp-mnt.el standard form. Added some section names
;; - Rewrote the source buffer handling. Now there is function
;;   tdi-set-source-buffer if the buffer cannot be determined automatically
;; - Big code changes.
;;
;; Jan  24      1996    [jari]          19.28   v1.5            NotReleased
;; - Added GNU diff support, -n and -u switches. There seems to be a
;;   bug in 19.28, which relates to recentering the window correcly.
;;   Reported it as a bug: functions (window-start) and (window-end)
;; - Now the diff type information is buffer local variable, so that
;;   program doesn't all the time do lookup.
;; - Now the diff PROGRAMS are user configurable.
;;
;; Jan  22      1996    [jari]          19.28   v1.4            NotReleased
;; - Normal diff form support added.
;;
;; Jan  21      1996    [jari]          19.28   v1.3            NotReleased
;; - The code works now as expected. Added missing next/prev functions
;;   and wote warning in the keymap define section how to deasl with
;;   view.el
;; - Byte Compile errors corrected.
;;
;; Jan  21      1996    [jari]          19.28   v1.2            NotReleased
;; - Code works. Diff function is ok and the minor mode is functional.
;;   Some weird thing cause the minor mode's map to be assigned
;;   globally? I don't understand where the problem is.
;; - There still is missing keyboard functions. Eg nmewt/prev
;;   diff movement
;;
;; Jan  21      1996    [jari]          19.28   v1.1            NotReleased
;; - Renamed many function/var names. Made better key setting function
;;   for user in nonWindowed emacs. Now uses "add-minor-mode"
;;   function from my libs.

;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

;;  make sure this is loaded so that the tdi-mode map can redefine
;;  keys "n" and "p"
;;
(require 'view)

(require 'tinylibm)
(require 'easymenu)


(eval-when-compile
  (autoload 'timy-read-only     "tinymy" t t)
;;;  (autoload 'timy-copy-file  "tinymy" t t)

  (use-dynamic-compilation)
  )


(defgroup-tiny TinyDiff tdi-: tools
  "Take buffer diffs easily, browse diff and apply patch.
  Overview of features.

      o   Buffer based simple diff/rcsdiff/patch package.
      o   You cab diff current buffer against last saved backup copy.
      o   Give and manipulate diff command in echo-area: file name
	    completion; switching between rcsdiff/diff, guess rcs version
	    for buffer ...
      o   When you have diff output in buffer, turning on `tdi-mode'
          allows you to browse source buffer by jumping diff blocks fwd/back
          and showing the source location.
      o   Supported: normal diff, context diff, gnu diff -u, gnu diff -n
      o   Easy patching. Finds file to patch (along user defined paths)
          and applies the diff. You can receive patches by email
          and apply them with one or two keystrokes.
      o   loads patch rejection file, if patch didn't succeed 100%"
  )

;;}}}
;;{{{ setup: variables

;;; .......................................................... &v-bind ...

(defvar tdi-:minibuffer-map nil
  "Mini buffer key map when asked for the right diff command.")

;;; ............................................................ &mode ...
;;;###autoload (autoload 'tdi-install "tinydiff" t t)

(eval-and-compile

(ti::m-fmacro-minor-mode-wizard
 "tdi-" " Tdi" nil " Tdiff" 'TinyDiff "tdi-:" ;1-6

 "Diff browsing minor mode.
Defined keys:
\\{tdi-:mode-map}
"

 "Diff minor mode"
 nil


 "Elp summary sort menu."		;9
 (list
   tdi-:mode-easymenu-name
   ["goto current point"		tdi-goto-kbd		t]
   ["block forward"			tdi-goto-next		t]
   ["block forward, no update"		tdi-goto-next-no-update t]
   ["block backward"			tdi-goto-prev		t]
   ["block backward, no update"		tdi-goto-prev-no-update t]
   "----"
   ["Parse buffer"			tdi-parse-buffer	t]
   ["Set source buffer for diff"	tdi-set-source-buffer	t]
   ["Show function name"		tdi-show-function-name	t]
   )

 (progn
  (tinyck-maybe-activate 'emacs)          ;XEmacs compatibility

  ;;  I first thought to put goto line into " " or "\C-m"
  ;;  That is: SPACE or RETURN, but later realized that they were
  ;;  used in view-mode to scroll buffer up-down.
  ;;
  ;;  Eg. in my setup whenever I turn on the read-only
  ;;  with C-x C-q it also automatically turns on view-mode for
  ;;  easy scrolling...
  ;;

  ;;  This happens to be unsifted in HP-UX, a top-leftmost button.
  ;;  Select something that suit you more...
  ;;
  (define-key   map "`"        'tdi-goto-kbd)

  (define-key   map "~"        'tdi-parse-buffer)
  (define-key   map "!"        'tdi-set-source-buffer)
  (define-key   map "n"        'tdi-goto-next)
  (define-key   map "p"        'tdi-goto-prev)

  ;;  These are borrewd from unix more(1) and less(1)
  ;;
  (define-key   map "y"        'tdi-goto-prev-no-update)
  (define-key   map "b"        'tdi-goto-next-no-update)

  ;;  But perhaps user feelsmore combortable with these.
  ;;
  (define-key   map "P"        'tdi-goto-prev-no-update)
  (define-key   map "B"        'tdi-goto-next-no-update)

  (define-key   map "f"        'tdi-show-function-name)
  (define-key   map [mouse-2]  'tdi-goto-mouse)

  (tinyck-maybe-activate 'emacs 'dis)
  ))

)




;;; ......................................................... &v-hooks ...

(defcustom tdi-:load-hook nil
  "*Hook that is run when package is loaded."
  :type 'hook :group 'TinyDiff
  )

(defcustom tdi-:diff-hook  nil
  "*Hooks that runs after successful diff run."
  :type 'hook :group 'TinyDiff
  )

;;
;; These have to be here, because when someone says
;;   (add-hook 'tdi-:diff-hook 'my-tdi-:diff-hook)
;;
;; The variable gets defined immediately. --> following does nothing...
;;   (defvar tdi-:diff-hook '(tdi-parse-buffer tdi-mode))
;;
(add-hooks 'tdi-:diff-hook
	   '(tdi-parse-buffer
	     tdi-mode
	     tdi-turn-on-view-mode
	     ))

(defcustom tdi-:mode-define-keys-minibuffer-hook
  'tdi-mode-define-keys-minibuffer-default
  "*Function to define the keys for the mini buffer."
  :type 'hook :group 'TinyDiff
  )


;;; ....................................................... &v-private ...

(defvar tdi-:debug  nil
  "Internal. Debug flag. If you have troubles turn on this flag.
Save debug buffer output and contact maintainer.")

(defvar tdi-:diff-type nil
  "Private. Set automatically to type of diff when buffer is parsed.")
(make-variable-buffer-local 'tdi-:diff-type)

(defvar tdi-:diff-source-buffer nil
  "Private. Source buffer for diff. See `tdi-:source-buffer-function'.")
(make-variable-buffer-local 'tdi-:diff-source-buffer)


(defvar tdi-:last-data nil
  "Private. Data storage eg in `tdi-cl-change-diff-command'.")


(defvar tdi-:version-list  nil
  "All Version for file. Updated when diff command is being run.")


(defvar tdi-:version-branch-list  nil
  "Branches. Updated when diff command is being run.")


(defvar tdi-:diff-tmp-buffer "*tdi-tmp*"
  "Temporary work buffer, patch shell results.")

(defvar tdi-:patch-tmp-buffer " *tdi-tmp-patch*"
  "Temporary work buffer, patch.")

(defvar tdi-:package-exist-tinymy  (ti::f-get-load-path "tinymy.el" load-path)
  "Private. Has load path for package tinymy.el if it exist.
It has some functions we may use from there.")


(defvar tdi-:diff-buffer "*diff*"
  "Buffer where diff is inserted.")

;;; ........................................................ &v-public ...
;;; User configurable

(eval-and-compile
(defun tdi-find-program  (program-list default opt seek-option)
  "Try to use GNU program. Return program name.
Input:

  PROGRAM-LIST	    list of program binaries to try (gnu)
  DEFAULT	    if not gnu found, use this binary
  OPT		    additional option for binary
  SEEK-OPTION	    When GNU product, it must know this option."
  (let* ((tmp (use-tmp-buffer "*tmp*" 'clear))
	 (ret "diff")
	 path
	 prg
	 )
    (if path (setq path t))		;XEmacs ByteCom silencer, no-op

    (job-buffer tmp
      ;;   Select gnu if possible
      (list-loop program-list prg
       (when (and (setq path (ti::f-get-load-path prg exec-path))
		  (shell-execute (concat prg " " seek-option)  tmp)
		  (re-search-check seek-option)
		  )
	 (setq default (concat prg " " opt)		;it's gnu ok.
	       program-list nil
	       )))

      (unless ret
	(message "tinydiff: Hm, no GNU %s; Using '%s'" default default)
	(sit-for 1)
	(setq default (concat default " " opt))
	)
      (set-buffer-modified-p nil)
      (kill-buffer tmp)
      default
      )))
)


;;  Different users may want to set the keys differently.
;;  You could say
;;
;;  (setq tdi-:diff-program (progn (my-diff-program-select)))
;;
;;  Which would return appropriate diff program: maybe you want to use
;;  GNU diff for some files and normal diff other times.
;;  (gnu diff offers line exlude options etc that you may want
;;   to need for RCS files.)
;;
;;
(defcustom tdi-:diff-program
  (tdi-find-program '("gdiff" "diff") "diff" "" "--help")
  "*Shell program to generate diff. It should print 'filename: FILE.XXX'.
tag which is read by function `tdi-get-buffer-name'.
This variable is evaled to get the program name, so you can put
lisp form inside it."
  :type '(string :tag "Shell program") :group 'TinyDiff
  )

(defcustom tdi-:rcsdiff-program "rcsdiff"
  "*Shell program to print RCS diff.
It should output the 'filename: FILE' which is read by `tdi-get-buffer-name'.
This variable is evaled to get the program name."
  :type '(string :tag "Shell program") :group 'TinyDiff
  )


;; The diff-switches is defined at least in vc.el
;;
(defcustom tdi-:diff-option
  '(or (and (boundp 'diff-switches)
            (stringp diff-switches)
            diff-switches)
       "-c")
  "*Diff options as STRING.
This variable is evaled to get the options, so it can contain lisp
FORM that returns option string."

  :type '(string :tag "Options") :group 'TinyDiff
  )


(defcustom tdi-:diff-tmp-file
  (if (not (file-exists-p "/tmp"))
      (error "Please set tdi-:diff-tmp-file")
    "/tmp/tdi.diff")
  "*Temporary file where the diff is stored during patching."
  :type 'file :group 'TinyDiff
  )


;; gnu patch : -t, --batch
;; similar to -f, in that it suppresses questions,
;; skip patches for which a file to patch can't be found
;;
;; Note: OLD patch command doesn not know -t switch!
;;
(defcustom tdi-:patch-program
  (tdi-find-program '("gpatch" "patch") "patch" "-t" "--help")
  "*Patch command and its options.
This variable is evaled to get the program name and switches."
    :type '(string :tag "shell command") :group 'TinyDiff
    )


(defcustom tdi-:patch-list
  '(( "[.]el$"  load-path)
    ( "."       '("~/txt" "~/elisp"))
    )
  "*List of item that control how patching is applied.
The list form is:

   '((REGEXP EVAL-FORM) (REGEXP EVAL-FORM) ..)

Where REGEXP is tried against the filename that is found from the patch
itself. EVAL-FORM can be any form that return list of pathnames that can
be searched for the filename. The first file that is found from the path
is used."
  :type  '(repeat
	   (list
	    (string :tag "Regexp")
	    directory
	    ))
  :group 'TinyDiff
  )

;;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. . misc . .

(defcustom tdi-:tdi-function-name-register  ?f
  "*Register used to store the function name.
Only used if `tdi-:function-name-handle-function' is set to
'tdi-func-name-store."
  :type 'character :group 'TinyDiff
  )

;;  The diff data is inserted into register automatically, because
;;  many time the diff data is needed elswhere.
;;
(defcustom tdi-:diff-register ?d
  "*Register name where diff is inserted.
If this variable is nil, then no data is inserted into register.

Variable is evaled to get the register name."
  :type 'character :group 'TinyDiff
  )


;;; ......................................................... &v-funcs ...



(defcustom tdi-:find-ref-function 'beginning-of-defun
  "*Elisp Function to find underlying code's function name around point.
The cursor is positioned in the source buffer and on the referenced
line before calling with no arguments. Function should move the point
in the line below where the associated reference is located.

If no reference is found, function _must_ call 'error'."
  :type 'function :group 'TinyDiff
  )


;;  - It's great to paste the function name into buffer.
;;  - C-x g REG put the name into current buffer...
;;
(defcustom tdi-:function-name-handle-function 'tdi-func-name-store
  "*Function which find the code's function name string.

Input args to function:

  POINT where the `tdi-:find-ref-function' positioned the defun.
  This function should store the function name into register
  `tdi-:tdi-function-name-register'."
  :type 'function :group 'TinyDiff
  )

(defcustom tdi-:source-buffer-function 'tdi-get-buffer-name
  "*Function which return filename for the diff buffer.
You shouldn't touch this function unless you're coping with very
strange diff format. Default function is `tdi-get-buffer-name'."
  :type 'function :group 'TinyDiff
  )


;;}}}
;;{{{ setup: version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinydiff.el"
 "tdi"
 tdi-:version-id
 "$Id: tinydiff.el,v 1.94 1997/07/02 08:54:00 jaalto NotReleased $"
 '(tdi-:version-id
   tdi-:load-hook
   tdi-:mode-hook
   tdi-:diff-hook

   tdi-:mode-name
   tdi-:diff-program
   tdi-:rcsdiff-program
   tdi-:source-buffer-function

   tdi-:function-name-handle-function
   tdi-:tdi-function-name-register
   tdi-:find-ref-function

   tdi-:source-buffer-function
   tdi-:mode-define-keys-hook

   tdi-:diff-register
   tdi-:diff-buffer

   tdi-:diff-option
   tdi-:diff-tmp-file
   tdi-:diff-tmp-buffer
   tdi-:patch-program
   )))



;;}}}

;;; ########################################################### &Funcs ###


;;{{{ code: keymap


;;; ----------------------------------------------------------------------
;;;
(defun tdi-mode-define-keys-minibuffer-default ()
  "This function defines some extra bindings to mini buffer.
Eg. TAB that completes current filename."

  (setq tdi-:minibuffer-map (copy-keymap minibuffer-local-map))

  ;;  Here we define nice tab filename completion inside minibuffer
  ;;  This may be superflous, but what the heck :-)
  ;;
  (define-key tdi-:minibuffer-map "\t"     'tdi-cl-complete-filename)
  (define-key tdi-:minibuffer-map [kp-tab] 'tdi-cl-complete-filename)

  ;;  My HP-UX just has "[" nad "]" nicely available. pick your
  ;;  favourites.
  ;;
  (define-key tdi-:minibuffer-map "["    'tdi-cl-complete-rev--)
  (define-key tdi-:minibuffer-map "]"    'tdi-cl-complete-rev++)

  ;;  Alternate the diff command, The C-z is close to pinky
  ;;  and propably free from emacs commands.
  ;;
  (define-key tdi-:minibuffer-map "\C-z" 'tdi-cl-change-diff-command)
  (define-key tdi-:minibuffer-map "\C-x" 'tdi-cl-rev-add-command)
  (define-key tdi-:minibuffer-map "\C-c" 'tdi-cl-insert-file-autosave)
  (define-key tdi-:minibuffer-map "\C-v" 'tdi-cl-insert-file-backup)
  (define-key tdi-:minibuffer-map "?" 'tdi-cl-minibuffer-help)
  )


;;}}}
;;{{{ code: misc

;;; ............................................................ &misc ...

;;; ----------------------------------------------------------------------
;;;
(defmacro tdi-debug (&rest args)
  "Write debug log if `tdi-:debug' is on with ARGS."
  (`
   (if tdi-:debug
       (let* ((ti:m-debug-buffer "*tdi-debug*")
              )
         (d!! (,@ args) "\n" )))))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-debug-toggle  (&optional arg)
  "Toggle debug to buffer *tdi-debug* with ARG."
  (interactive "P")
  (var-toggle tdi-:debug arg)
  (if (interactive-p)
      (message "tdi: Debug is %s" (if tdi-:debug "on" "off")))
  )

;;; ----------------------------------------------------------------------
;;;
(defsubst tdi-kill-revision-list ()
  "Deletes private version lists."
  (setq tdi-:version-list nil
        tdi-:version-branch-list nil))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-update-revision-list (file &optional version)
  "Read all revision numbers for FILE starting from VERSION."
  (let* ((ver   (or version "1.1"))
         (fid   "tdi-update-revision-list: ")
         (dots  (count-char-in-string ?. ver))
         )


    (when
      (null tdi-:version-list)
      (tdi-debug fid ver dots)
      (setq tdi-:version-list  (ti::m-rcs-all-versions file)))


    (when (or (null tdi-:version-branch-list)
              ;; Chck that there is right brach list
              ;; "1.1"  -- "1.1.1.1" ?
              (not (eq dots
                       (count-char-in-string
                        ?.
                        (car tdi-:version-branch-list))))
              )
          (setq tdi-:version-branch-list
                (ti::m-rcs-get-all-branches
                 ver tdi-:version-list))
          (tdi-debug fid "ver" version)
          )

    tdi-:version-list
    ))


;;; ----------------------------------------------------------------------
;;; - Doing the version getting is SLOW with lisp. Use some external
;;;   shell program to do the job for you. It is MUCH quicker.
;;;
(defun tdi-rcs-diff-between-versions (dir file)
  "Return RCS diff command that diffs current version and previous version.
DIR and FILE is passed to function."
  (interactive)
  (let* ((ver      (ti::m-rcs-buffer-version))
         v-list
         ret prev
         )
    (if (null ver)
        (error "tdi: Cannot find version number."))

    (setq v-list   (tdi-update-revision-list (concat dir file)))
    (setq prev     (ti::m-rcs-previous-version ver v-list))

    (if (null prev)
        (error "tdi: Cannot find previous version number."))

    (setq ret
          (format "cd %s; %s -r%s -r%s %s %s "
                  dir
                  tdi-:rcsdiff-program
                  prev ver
                  (or (eval tdi-:diff-option) "")
                  file
                  ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defsubst tdi-diff-type ()
  "Return type of diff in this buffer by reading setting from variable."
  ;;  If user changes major mode, the buffer local variable gets
  ;;  wiped out --> Set it again with the function call.
  (or tdi-:diff-type
      (setq tdi-:diff-type
            (car (ti::b-diff-type-p)))))


;;; ----------------------------------------------------------------------
;;;
(defsubst tdi-source ()
  "Return source buffer of diff."
  (or tdi-:diff-source-buffer
      (setq tdi-:diff-source-buffer
            (funcall tdi-:source-buffer-function))))



;;; ----------------------------------------------------------------------
;;;
(defun tdi-set-source-buffer (buffer)
  "Set source BUFFER for current diff."
  (interactive "bSource buffer: ")

  (if (not (get-buffer buffer))
      (error "Buffer not exist")
    (setq tdi-:diff-source-buffer buffer)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-shell-exec-nok-p (stat)
  "Check if diff STAT show failure."

  ;; For some reason the rcsdiff gives 1 when ok diff ??
  ;;
  (cond
   ((memq stat '(1))
    nil)
   (t
    (shell-exec-nok-p stat)))
  )



;;; ----------------------------------------------------------------------
;;;
(defun tdi-func-name-store (line)
  "Store found function name from LINE into `tdi-:tdi-function-name-register'.
Currently works well only lisp functions."
  (let* ((reg  tdi-:tdi-function-name-register)
         (mode (symbol-name major-mode))
         txt
         )
    (cond
     ((string-match "lisp" mode)
      ;;  try to extract symbol-name

      ;;  see tdi-get-func-name, because point sits on DEFUN already
      ;;  and the ti::m-find-function-name searches it again..
      (forward-line 1)
      ))

    (setq txt (ti::m-find-function-name))

;;;      (d! (point) txt)

    (cond
     (reg
      (if txt
          (set-register reg txt)
        (set-register reg ""))))                ;empty it
    ))




;;; ----------------------------------------------------------------------
;;;
(defun tdi-get-func-name (buffer line)
  "Return function or variable name at current point.
Switches to BUFFER and go to LINE and calls `beginning-of-defun'"
  (let* ((func          tdi-:function-name-handle-function)
         (find-func     tdi-:find-ref-function)
         (max-leap      100)            ;lines
         point
         ret
         )
    (save-excursion
      (set-buffer buffer)
      (safe-widen
       (goto-line line)
;;;       (d! (current-buffer) (LINE) line (read-full-line))
       (safe-do
        (setq point (point))

        (funcall find-func)             ;exit , if error generated

        ;;  Let's be little intelligent
        ;;
        (if (< (count-lines (point) point) max-leap)
            ;;  Okay, we believe that function was found
            (setq ret (read-full-line))
          ;; Can't be that far away... reset the pointer
          (goto-char point)
          )

        (if func
            (setq ret (funcall func (point))))
        )
       ret
       ))))



;;; ----------------------------------------------------------------------
;;;
(defun tdi-show-function-name (&rest args)
  "Show possible function name in mode line on the current diff point. ARGS."
  (interactive)
  (let* ((line          (tdi-get-line-number))
         (buffer        (tdi-source))
         desc
         )
    (cond
     ((not (and line buffer))
      (message "Sorry, No line and buffer info."))
     (t
      (setq desc  (tdi-get-func-name buffer line))
      (if (stringp desc)
          ;;  Moving mouse wipes this away...
          (message desc)
        (message "Can't find reference."))
      ))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-turn-on-view-mode ()
  "Turn on _good_ view mode if tinymy.el is available.
Call `timy-read-only' to turn on the appropriate view mode. also
detect if you're using tinyezip.el

If there is no tinymy.el available, turn on simple `view-mode'."
  ;;  We have to load the full package if tinyezip is used.
  ;;
  (cond
   ((null tdi-:package-exist-tinymy)
    (view-mode 1))
   (t
    (if (featurep 'tinyezip)
        (require 'tinymy)
      ;;  This allow dynamic byte loading, only this function
      ;;  is loaded if you have byte compiled the package.
      ;;
      (autoload 'timy-read-only "tinymy")
      )
    (set-buffer tdi-:diff-buffer)
    (PMIN)
    (timy-read-only)
    )))



;;}}}
;;{{{ code: command line

;;; ..................................................... &commandLine ...

;;; ----------------------------------------------------------------------
;;; cl = command line
;;;
(defun tdi-cl-replace-text (beg end text)
  "Command line. Replace region between BEG and END with TEXT."
  (save-line-col
    nil nil
    (delete-region beg end)
    (goto-char beg)
    (insert text)
    ))


;;; ----------------------------------------------------------------------
;;; - See tdi-cl-complete-rev++, the side-effect which is explained
;;;   was not intentional. It seems that the skip-function don't do
;;;   their job right
;;; - I just noticed that I liked the side-effect better than moving
;;;   cursor over number all the time.
;;;
(defun tdi-cl-read-revision ()
  "Command line. Read revision.

Return:
 (nbr . pos)    ,where pos is point at line.
 nil"
  (interactive)
  (let* ((fid "tdi-cl-read-revision: ")
         nbr
         pos
         )
    (save-excursion
      ;;  a) in the top of REV number
      ;;  b) pick REV forward
      ;;
      (or (eq 0 (skip-chars-backward "0-9."))
          (eq 0 (skip-chars-forward "^0-9.")))

;; cd /users/jaalto/elisp/; rcsdiff -c -r1.25 -r1.8 tinydiff.el
      (cond
       ((setq nbr (bufmatget "[0-9.]+" 0))
        (setq pos (match-beginning 0)))
       (t
        (beginning-of-line)
        (setq nbr (bufmatget ".*-[rul]\\([0-9.]+\\)" 1))
        (setq pos (match-beginning 1))
        )))

    (tdi-debug fid "col"(current-column) "nbr" nbr "pos" pos )
    (if nbr
        (cons nbr pos))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-directory  ()
  "Return directory name from 'cd' command."
   (save-excursion
     (beginning-of-line)
     (bufmatget "cd +\\([^;]+\\)" 1)))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-read-rcs-file-name (&optional line or-diff-name)
  "Read RCS filename from LINE.
If OR-DIFF-NAME is non-nil, look for 'diff' command instead."
  (let* ((line      (or line (read-full-line)))
         (rcsdiff   tdi-:rcsdiff-program)
         (fid       "tdi-cl-read-rcs-file-name: ")
         (diff      tdi-:diff-program)
         (re        (concat
                     "cd[ \t]+\\([^;]+\\);[ \t]*"
                     (if or-diff-name diff rcsdiff)
                     ".* \\([^ \t]+\\)"
                     ))
         dir file
         ret
         )
    ;;   The directory name is gotten after 'cd' command
    ;;
    (tdi-debug fid line re)

    (cond
     ((and (string-match re line)
           (setq dir  (read-match 1 line))
           (setq file (read-match 2 line)))
      (setq ret (concat dir file))
      ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-rev-add-command  ()
  "Adding two -rX.x string to the command line.
This is only done if there is rcsdiff command and less the 2 -rX.x
switches

Eg.
        rcsdiff -r1.1 file.cc
-->     rcsdiff -r1.1 -r1.1 file.cc

        rcsdiff -r1.1 -r1.1 file.cc
-->     Do nothing, since there is already two -r switches."
  (interactive)
  (let* ((fid       "tdi-cl-rev-add-command: ")
	 (line      (read-full-line))
         (rcsdiff   tdi-:rcsdiff-program)
         (re        (concat "^.*; *" rcsdiff "\\(.*\\)"))
         (i         0)
         prev-list
         r-list
         args list
         elt
         )
    (tdi-debug "\n\n" fid rcsdiff line re)
    (when (string-match re line)
      (setq args (read-match 1))
      (setq list (ti::s-get-blocks line "[ \t]+")
            args list)

      (tdi-debug fid "args" args )

      (list-loop list elt
        (when (string-match "^-r" elt)	;Find this
	  ;;   Here we make list
	  ;;   '((NTH-POS CAR-LIST CDR-LIST) (N CA CD) ..)
	  ;;
	  (nconcc r-list (list i (copy-list prev-list) list))
	  (tdi-debug fid "r-list>>" r-list)
	  )
	(nconcc prev-list elt)
        (inc i))

      (tdi-debug fid "r-list" r-list)

      (cond
       ((eq (length r-list) 0)          ;no -r --> add it
        ;;  looks complicated? The idea to
        ;;  1. get last element, but only if it's NOT an option
        ;;  2. get all, but not the last element.
        ;;
        (setq r-list (reverse args))
        (setq elt (car r-list))
        (tdi-debug fid "elt" elt r-list)

        (if (not (string-match "^-" elt))
            (setq args (reverse (cdr (reverse args))))
          (setq elt nil))               ;it was an option

        (nconcc args "-r" )
        (if elt                         ;file name set ?
            (nconcc args elt ))
        )

       ((eq (length r-list) 1)          ;make copy
        (setq elt (car r-list))		;position
        (setq args (nth 2 elt))         ;cdr-list
        (push (car (nth 2 elt)) args)	;copy
        (setq args                      ;car-list to the beginning
              (list-union (reverse (nth 1 elt)) args))
        ))

      (setq args (list-to-string args))
      (tdi-debug fid "args" args  )
      (delete-region (BOLPP) (EOLPP))
      (insert args)
      )
    (end-of-line)
    ;;  Go after last revision number, so that user can change it easily
    (beginning-of-line)
    (re-search-forward "-r") (skip-chars-forward "^ \t")
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-change-diff-command  ()
  "Change diff command in mini buffer."
  (interactive)
  (let* ((fid      "tdi-cl-change-diff-command: ")
	 (rcsdiff  tdi-:rcsdiff-program)
         (diff     tdi-:diff-program)
         word
         line
         list
         done
	 )

    (tdi-debug fid "in")
    (save-excursion
      (beginning-of-line)

      (cond
       ;;  Make sure it's our command.
       ((and (re-search-forward "; *" nil t)
             (setq word (ti::b-read-word)))

        (setq line (buffer-substring (point) (EOLP)))
        (setq list (ti::s-get-blocks line "[ \t]+"))



        (tdi-debug fid
                 "\n\nLINE " line "\n"
                 "LIST " list "\n"
                 "1 string= word rcsdiff " word rcsdiff "\n"
                 "2 string= word diff "    word diff "\n"
                 "2 tdi-:last-data " tdi-:last-data "\n"
                 "3 " (and (string-match word diff)
                           list
                           (not (stringp tdi-:last-data)))
                 )

        (cond
         ;; (1)
         ;;
         ((string= word rcsdiff)
          (setq tdi-:last-data line)

          ;;  Remove some elements from list
          (setq list (list-find
                      list
                      nil
                      '(not
                        (string-match
                         (concat "^-r\\|" rcsdiff)
                         (car table)))
                      'all
                      ))
          (tdi-debug fid "1 " list )
          (kill-line)
          (insert  diff " " (list-to-string list))
          (setq done t)
          )

         ;;(2)  Did the last command has the same filename ? --> if not
         ;;     then we cannot use C-z
         ;;
         ((and (string-match word diff)
               list
               (stringp tdi-:last-data)
               (string-match-case
                (regexp-quote (nth (1- (length list)) list) )
                tdi-:last-data))
          (kill-line)
          (insert tdi-:last-data)
          (setq done t)
          )

         ;; (3) diff --> rcs diff thing
         ;;
         ((and (string-match word diff)
               list
               (not (stringp tdi-:last-data)))
          (kill-line)
          (insert
           (concat
            rcsdiff
            " -r "
            (list-to-string (cdr list))
            ))
          (setq done t)
          )

         )
        )))
    (cond
     (done
      ;;  Move after the 'cd' and 'diff commands to add/change options easily
      ;;
      (beginning-of-line)
      (re-search-forward "; *" nil t)
      (forward-word 2)
      ))

    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-complete-rev++ ()
  "Complete revision up.
Invoking this function start immediately a process to get all revision
numbers for the file. After that the incrementing of version should
be easy.

Only versions in current branch are stepped, you have to change the
branch manually.

Special side-effect:

   -r1.12  -r1.5
    | |
    | Cursor here increments _this_ number.
    Cursor here increments the _second_ version number"
  (interactive)
  (let* ((file          (tdi-cl-read-rcs-file-name))
         (fid           "tdi-cl-complete-rev++: ")
         (branch-list   tdi-:version-branch-list)
         elt rev
         pos
         )
    (when
      (and file
           (setq elt (tdi-cl-read-revision)))

      (setq rev (car elt)  pos (cdr elt))
      (tdi-debug fid rev pos  )

      (tdi-update-revision-list file rev)
      (when
        ;;   ("1.3" "1.4" ..)
        ;;   ^^^member
        ;;          ^^^cdr
        (setq elt (car-safe (cdr-safe (member rev branch-list))))
        (tdi-cl-replace-text pos (+ pos (length rev)) elt)
        ))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-complete-rev-- ()
  "Complete revision --. See ++ function for explanation."
  (interactive)
  (let* ((file        (tdi-cl-read-rcs-file-name))
         (fid           "tdi-cl-complete-rev--: ")
         (branch-list tdi-:version-branch-list)
         elt rev
         pos
         )
    (when
      (and file
           (setq elt (tdi-cl-read-revision)))

      (setq rev (car elt)  pos (cdr elt))
      (tdi-debug fid rev pos )

      (tdi-update-revision-list file rev)
      (when
        (setq elt (list-previous-elt
                    branch-list rev
                    '(string= arg (car list))
                    ))
        (tdi-cl-replace-text pos (+ pos (length rev)) elt)
        ))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-complete-filename ()
  "Complete filename."
  (interactive)
  (let* ((fid	"tdi-cl-complete-filename")
	 (word  (save-excursion
                  (forward-char -1)
                 (ti::b-read-space-word)))
         (dir   default-directory)
         )

    (tdi-debug fid word dir)
    (unless (var-nil-p word)
      ;;   The directory name we get from the 'CD' prompt if the
      ;;   filename lacks dir part.
      ;;
      (if (not (string-match "/" word))
          (setq dir (tdi-cl-directory)))

      (let ((default-directory (or dir
				   default-directory))
	    )
	(ti::f-complete-file-name-word))

      )))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-insert-file-autosave  (&optional backup)
  "Insert auto-save filename into current point if it exists.
Prefix arg says to insert BACKUP filename instead."
  (interactive)
  (let* ((line      (read-full-line))
         (rcsdiff   tdi-:rcsdiff-program)
         (fid       "tdi-cl-insert-file-autosave: ") ;function id
         file
         file2
         stat
         )

    (setq file (tdi-cl-read-rcs-file-name line 'diff))

    (tdi-debug fid "arg" backup "file" file)

    (cond
     ((string-match rcsdiff line)
      (message
       (substitute-command-keys
        (concat
         "Don't use rcsdiff command; Change command with "
         "\\[tdi-cl-change-diff-command] ")
        )))

     ((not file)
      (message "Can't read file from prompt. Include 'cd'"))

     ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. . process file ..
     (t

      (cond
       (backup
        (setq file2 (make-backup-file-name file))
        (setq stat (file-exists-p file2))
        )
       (t

	;;  The autosave fiel must be surrounded with '' because
	;;  # is shell comment
	;;
        (setq file2 (format
		     "#%s#"
		     (file-name-nondirectory file)))
        (setq stat (file-exists-p (concat (file-name-directory file) file2)))
        ))

      (if (null stat)
          (message "Not found %s " file2 )

	(setq file2 (concat "'" file2 "'"))
        ;; put spaces around filename if needed
        ;;
        (setq file2
              (concat
               (if (space-p (preceding-char)) "" " ")
               file2
               (if (space-p (following-char)) "" " ")
               ))
        (insert file2)
        )))

    ))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-insert-file-backup  ()
  "Command line. Inset backup filename if it exists."
  (interactive)
  (tdi-cl-insert-file-autosave 'backup))



;;; ----------------------------------------------------------------------
;;;
(defun tdi-cl-minibuffer-help ()
  "Show brief help."
  (interactive)
  (with-output-to-temp-buffer "*Help*"
    (princ
     (substitute-command-keys
      "TinyDiff minibuffer command line keys\n\n \\{tdi-:minibuffer-map}"
      )))
  (job-buffer "*Help*"
    (PMIN) (forward-line 4)
    (delete-non-matching-lines "tdi-cl")
    ))

;;}}}

;;{{{ code: diff type, patch

;;; ........................................................... &patch ...


;;; ----------------------------------------------------------------------
;;;
(defsubst tdi-patch-get-dir-from-cmd  (cmd)
  "Return FILE from CMD."
  (when cmd
    (setq cmd (strmatget "cd +\\([^ \t;]+\\)" 1 cmd))
    (ti::s-verify-ends cmd "/")
    ))

;;; ----------------------------------------------------------------------
;;;
(defsubst tdi-patch-get-file-from-cmd  (cmd)
  "Return FILE from CMD."
  (when cmd
    (setq cmd (strmatget
	       (format "cd .*%s +\\([^ \t;]+\\)" tdi-:patch-program)
	       1 cmd))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-patch-cleanup ()
  "Check if we can use the patch as is, PGP change must be restored."
  (let* (
         )
    (save-excursion
      (PMIN)
      ;; PGP breaks the dashed lines:
      ;; - --- 1.2.1.1  1996/06/11 11:36:03
      ;; - --- 212,219 ----
      ;;
      ;; Correct the lines back.
      ;;
      (while (re-search-forward "^\\(- \\)\\(--- [0-9]+.*\\)" nil t)
        (message "Correcting patch: PGP's broken lines...")
        (kill-match 1 (read-match 2))
        )

      (PMIN)
      (when (re-search-forward "^--+BEGIN +PGP +SIGNATURE" nil t)
        (message-quick "Correcting patch: remoming PGP tags...")
        (delete-region (BOLPP) (point-max)))

      (message "Correcting patch: done.")
      )))





;;; ----------------------------------------------------------------------
;;;
(defun tdi-get-file-name (&optional arg)
  "Try to get file name for the diff.
Optionally read line \"RCS file: xxx.el\"

ARG says which to look:

  1 pick line *** file.xx
  2 pick line --- file.xx
  3 pick line +++ file.xx      ,GNU unified diff.

Return list:
  '(file (type pos))
  nil"
 (let* ((stat (ti::b-diff-type-p))
        (re1   "^[ \t]*[*][*][*] +\\([^ \t\n]+\\)")
        (re2   "^[ \t]*--- +\\([^ \t\n]+\\)")
        (re3   "^[ \t]*[+][+][+] +\\([^ \t\n]+\\)")
        (re    re1)                     ;default
        list
        file
        )
   (when stat                           ;only if diff found
     (if arg                            ;set search regexp
         (cond
          ((eq 1 arg) (setq re re1))
          ((eq 2 arg) (setq re re2))
          ((eq 3 arg) (setq re re3))
          ))

     (save-excursion
       (goto-char (cdr stat))

       (cond
        ((save-excursion
           ;;  Ignore path
           (and (or (re-search-backward
                     "RCS +file: +\\([^/]+\\),v" nil t)
                    (re-search-backward
                     "RCS +file: +.*/\\\(.*\\),v" nil t))
                (setq file (read-match 1)))))

        ((save-excursion
            (cond ((re-search-backward "diff -c.*" nil t)
                   (setq list (ti::s-get-blocks (read-full-line) "[ ]+"))
                   (setq file (nth (length list) list)) ;get last element
                  ))))

        ((save-excursion
            (cond ((or (re-search-backward re nil t)
                       (re-search-forward  re nil t))
                   (setq file (read-match 1))
                   ))))
        )))

   (if file                             ;pick the return values
       (list file stat)
     nil)
   ))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-patch-check-if-load  (file buffer)
  "See if we want to load FILE by looking results in BUFFER.
If file was RCS controlled and not in emacs, ask to load it.
If file is active in emacs ad to do `load-file' to refresh current emacs."
  (let* (case-fold-search               ;case sensitive matches
         (fbuffer   (find-buffer-visiting file))
         (modified  (if fbuffer  (buffer-modified-pp fbuffer)))
         (was-rcs   (re-search-check "RCS/"))

         ;; Is package active in emacs? Was patched file .el?
         ;;
	 el-file
	 sym
	 feature
	 no-ask
         )

    (and (setq el-file  (strmatget "\\(.*\\)\\.el$" 1 file))
	 (setq sym	(intern-soft (file-name-nondirectory el-file)))
	 (setq feature	(featurep sym))
	 )

    ;;  Be sure about success
    ;;
    (when (job-buffer buffer (re-search-check "[Hh]unk.*succeed"))
      (cond
       ((and buffer (not modified)
             (or (y-or-n-p "Buffer for the file exist, revert? ")
		 (progn  (setq no-ask t) nil) ;Set flag and Stop case
		 ))
        (pop-to-buffer fbuffer)
        (revert-buffer)
        )

       ((and fbuffer modified)
        (pop-to-buffer buffer)
        (message-quick "Patched file in emacs, but buffer is modified..")
        )

       ((and was-rcs
	     (null no-ask)		;Maybe set in 1st cond case
             (y-or-n-p (format "RCS file patched, find-file %s " file "? ")))
        (find-file file)
        )

      ((and feature
            (y-or-n-p (format "%s is running in your Emacs, reload it?  "
                              file)))
       (load file)
       (message "%s reloaded. Your emacs is now running the latest patch."
                file)
       )))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-patch-check-failure (&optional buffer)
  "Check patch failure messages from BUFFER.
Return:
  filename      rejection file if failure happened.
  '(\"\")       some unknown failure happened; rejection file not available
		String holds matched failure condition.
  nil           Patch succeeded ok."
  (interactive)
  (let* ((re"failed.*saving +rejects +to +\\([^ \t\n]+\\)")
         file
         )

    (job-buffer (or buffer (current-buffer))
      (or
       ;;   1 out of 2 hunks failed--saving rejects to test.el.rej
       ;;
       (setq file (re-search-check re 1 nil 'get-matched))

       ;;  patch: **** this file doesn't appear to be the 1.7 version--aborting
       ;;
       (and (setq file
		  (re-search-check
		   "^patch:.*--aborting" 1 nil 'get-matched-text))
	    (setq file (list file)))

       ;;  Maybe shell error has terminated the command
       ;;  "Unrecognized switch: -t"
       ;;
       (and (not (re-search-check "hunk.*succeed"))
	    (prog1 nil
	      (setq file
		    (list
		     "No 'hunk succeed' message found."))))
       ))
    file
    ))




;;; ----------------------------------------------------------------------
;;;
(defun tdi-patch-check-rejections  (cmd buffer)
  "After CMD, check rejections from BUFFER.
If the the patch command says in this buffer:

  1 out of 1 hunks failed--saving rejects to T2.rej

Then loading the rejection file.

CMD is the original patch command used."
  (let* ((fid      "tdi-patch-check-rejections: ")
         file
         stat
         tmp
         dir
         file-load
         )
    (setq stat (tdi-patch-check-failure buffer))

;;;    (pop-to-buffer (current-buffer)) (d! "rejections" stat cmd)

    (cond
     ((listpp stat)
      (message (car stat)))

     ((stringp stat)
      (setq file stat)
      (if cmd
          (setq dir (tdi-patch-get-dir-from-cmd cmd))
        (setq dir default-directory))

        (setq file-load (concat dir file))

        (tdi-debug fid file-load)

        (cond
         ((file-exists-p file-load)
          (setq tmp (use-tmp-buffer "*tdi-rejections*" 'clear))
          (job-buffer tmp (insert-file-contents file-load))
          (pop-to-buffer tmp)
          t
          )
         (t
          (message-quick "tdi: Can't find DIR for file...")
          (call-interactively 'find-file)
          t
          ))
	))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-patch-with-diff-1 (file beg end &optional interactive)
  "Apply diff to file ie. patch the file.

Input:
  FILE          ,absolute file name where to store diff.
  BEG           ,diff start point in buffer
  END           ,diff end point
  INTERACTIVE   ,User interaction, allow editing the patch command etc."
  (let* ((file          (expand-file-name file))
         (diff-tmp      tdi-:diff-tmp-file)
         (prg           (eval tdi-:patch-program))
         (map           tdi-:minibuffer-map)
         (source-buffer (current-buffer))
         (file-buffer   (find-buffer-visiting file))
         buffer                         ;shell messages
         data-buffer
         ff dir
         cmd
         )
    (when (and
           file-buffer
           (buffer-modified-pp file-buffer)
           interactive
           (not (y-or-n-p
                 (format
                  "buffer %s not saved, continue ?" file-buffer)))
           )
      (error "Aborted."))


    (if (not (file-exists-p file))
        (error "file not found: %s" file ))

    (file-components file dir ff)       ;crack it
    (setq buffer        (use-tmp-buffer tdi-:diff-tmp-buffer 'clear))
    (setq data-buffer   (use-tmp-buffer tdi-:patch-tmp-buffer 'clear))

    (job-buffer data-buffer
      (insert-buffer-substring source-buffer beg end)
      (tdi-patch-cleanup)

;;;      (pop-to-buffer (current-buffer))      (d! "ok")

      ;;  Sometimes user's UMASK is not ok. BE sure that the file is
      ;;  readable after the write.
      ;;
      (if (not (file-writable-p diff-tmp))
          ;;   Make it writable then
          (ti::f-chmod-w-toggle diff-tmp))

      (write-region (point-min) (point-max) diff-tmp)

      (if (not (file-readable-p diff-tmp))
          (set-file-modes diff-tmp 384))       ;; -rw-------

      (setq cmd (concat "cd " dir "; " prg " " ff " < " diff-tmp))

      (if interactive
          (setq cmd (read-from-minibuffer "> " cmd map)))

      (if (var-nil-p cmd)                       ;user cleared the line ?
          (message "Patching cancelled.")
        (shell-execute cmd buffer)
        (when interactive
          (pop-to-buffer buffer)
          (or (tdi-patch-check-rejections cmd buffer)
              (and (stringp file)
                   (tdi-patch-check-if-load file buffer)))
          )

        ))
    ))



;;; ----------------------------------------------------------- &patch ---
;;;
(defun tdi-patch (arg &optional beg end)
  "Try to guess diff type and region in the buffer.
If automatic detection fails user must select diff region by hand.

PREFIX ARG lets user to edit the diff command before executing.
This is enabled by default for interactive calls.

BEG END if diff region; defaults to whole buffer."
  (interactive
   (progn

;;; This is not a good idea if you get many patches; to
;;; ask every time...
;;;
;;;     (if (and tdi-:package-exist-tinymy
;;;           (y-or-n-p "Do you want to make a safe copy? "))
;;;      (call-interactively 'timy-copy-file))

     (cond
      ((ti::xe-mark-active-p)
       (list
        current-prefix-arg
        (region-beginning)              ;avoids region active check
        (region-end)))
      (t
       (list current-prefix-arg nil nil)
       ))))

  (let* ((list          tdi-:patch-list)
         (buffer        (use-tmp-buffer tdi-:diff-tmp-buffer 'clear))
         (verb-arg      (or arg (interactive-p)))
         (go-status     t)              ;should we proceed patching?
	 rej-flag
         char
         ptr re
         file-list
         search-path
         stat
         dir file
         dest-file
         )

    (loop-for 1 4
      (setq stat (tdi-get-file-name COUNTER))

      (when stat
        (setq file (nth 0 stat)
              stat (nth 1 stat))
        (push file file-list)
        (if (string-match "/" file)
            (push (file-name-nondirectory file)  file-list))
        ))
;;;    (d! file-list)

    (cond
     (file-list
      ;; Normally the directory part cannot be used, because people have
      ;; files in different places, search it anyway in case the
      ;; file structure is the same..
      ;;
      ;; 1. "/dir/dir/file.el"
      ;; 2. "file.el"
      ;;
      ;; ClearCase styled diff:
      ;; *** /work/jackr/emacs/folding.el@@/main/5  Wed Mar 13 09:39:36 1996
      ;; --- /work/jackr/emacs/folding.el           Wed Mar 13 13:50:26 1996
      ;;

      ;; ..................................................... find it ...
      ;;
      (list-loop file-list file
        (setq ptr list)                 ;patch paths

        (while (and file
                    (null dest-file)
                    ptr)
          ;;  Get next element from patch table
          ;;
          (setq re              (nth 0 (car ptr))
                search-path     (eval (nth 1 (car ptr))))

          (if (string-match re file)
              (setq dest-file (ti::f-get-load-path file search-path)))
          (NEXTP ptr))
        )
      ))

;;;    (d! beg end)

    ;; ......................................................... patch ...
    ;;

    (cond
     ((stringp dest-file)
      (file-components dest-file dir file))

     ((null dest-file)
      (message-quick "Can't detect file along paths in tdi-:patch-list.")

      ;;  Story of this starange WRAP (save-ex, buffer change ..)
      ;;  - I was receiving diff in RMAIL buffer. When I called this function
      ;;    read-file-name from RMAIL buffer it always gave me RMAIL,
      ;;    not the DIR/FILE combination.
      ;;  - I have no idea why the read-file-name went beserk, but changing
      ;;    the buffer to some *temp* cured the problem
      ;;  - This is highly dangerous, since before I noticed this
      ;;    the DEST-FILE was set to RMAIL, and the patch was done against
      ;;    my mail buffer!  (Hopefully I had it in emacs, so I could wrote
      ;;    the original back...)
      ;;
      ;;
      (save-excursion
        (set-buffer buffer)
        (setq dest-file (read-file-name "Apply diff to: " dir nil t file))
        )))


    (if (or (not (stringp dest-file))
            (not (file-exists-p dest-file)))
        (message-quick "Invalid parameters. Cancelled.")

      (when (file-exists-p (concat dest-file ".rej"))
            (setq rej-flag t))

      (if (not
	   (and
	    (file-exists-p (concat dest-file ".orig"))
	    (prog1 t
	      (setq
	       char
	       (read-char-safe-until
		(format "%s\
.orig found: r = retry patch, o = back to .orig, g = go and patch"
			(if rej-flag "[.rej]" "")
			)
		'(?o ?r ?g    ?\e ?q ?\b ?\C-g))))
	    ))
          ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. ..  no .orig . .
	  (if rej-flag
	      (message-quick "Hmmm... rejection file found."))
        ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .orig exist ..
        (cond
         ((char= char ?r) ;Retry patch to original
          (delete-file dest-file) ;copy-file won't work otw
          (copy-file  (concat dest-file ".orig") dest-file)
          )

         ((char= char ?g) ;Go ahead
          nil)

         ((char= char ?o)
          (delete-file dest-file)
          (copy-file  (concat dest-file ".orig") dest-file)
          (delete-file (concat dest-file ".orig"))
          (setq go-status nil)
          (message "Original file restored: .orig copied over %s" dest-file)
          )

         (t
          (setq go-status nil)
          (message "Cancelled patching.")
          )))

      (if go-status
          (tdi-patch-with-diff-1 dest-file (point-min) (point-max) verb-arg))
      )
    ))


;;}}}
;;{{{ code: command generate

;;; ----------------------------------------------------------------------
;;;
(defun tdi-diff-command-generate  (&optional no-ask)
  "Return diff command as string; optionally NO-ASK."
  (if (null tdi-:minibuffer-map)
      (run-hooks 'tdi-:mode-define-keys-minibuffer-hook ))

  (let* ((map           tdi-:minibuffer-map)
         (tmp-file      tdi-:diff-tmp-file)
         (diff-prg      (eval tdi-:diff-program))
         (riff-prg      (eval tdi-:rcsdiff-program))
         (options       (or (eval tdi-:diff-option) ""))
         (bf            (buffer-file-name))
         (file          "")
         (file2         "")
         (dir           "")
         rev
         prompt
         ans
         )

    (tdi-kill-revision-list)
    (if bf
        (file-components bf dir file))
    (cond
     ;; .............................................. buffer change ...
     ;; - The buffer has changed. Or file has changed.
     ;;
     ((and bf
           (file-exists-p (buffer-file-name))
           (or (buffer-modified-p)
               ;;  someone else edited the same file recently ?
               (null (verify-visited-file-modtime (current-buffer))))
           (y-or-n-p "Diff between buffer and file on disk? ")
           )
        (safe-widen                     ;make sure whole buffer is saved
          (write-region (point-min) (point-max) tmp-file))
        (setq file2 tmp-file)
        )


     ;; ........................................................ rcs ...
     ;; - Buffer is RCS controlled.
     ;;
     ((and bf
           (ti::m-rcs-file-exists-p bf))

      ;; - if the revision information cannot be found, then the
      ;;   '-rX.x' switch is not used.
      ;;
      (setq rev (ti::m-rcs-buffer-version))

      (if (null rev)
          (message-quick "RCS Revision not detected."))

      (setq options
            (format "%s %s"
                    options
                    (if rev
                        (concat "-r" rev " ")
                      "")))
      (setq diff-prg riff-prg)
      )


     ;; .................................................... default ...
     ;; General diff prompt
     ;;
     (t
      (setq dir default-directory)
      ))

    ;; ... ... ... ... ... ... ... ... ... ... command generated . .
    (setq prompt (format  "cd %s; %s %s %s %s "
                          dir
                          diff-prg options
                          file
                          file2
                          ))


    (if no-ask
        (setq ans prompt)
      (setq ans (read-from-minibuffer ">" prompt map)))

    (if (var-nil-p ans)  (setq ans nil))

    ans
    ))

;;}}}
;;{{{ code: generating, parsing diff

;;; .................................................... &diff-parsing ...




;;; ----------------------------------------------------------------------
;;;
(defun tdi-parse-buffer (&optional verb)
  "Prepare diff buffer for `tdi-mode'.  VERB.
Mark diff lines for special handling (try moving mouse over chunk number).

References:
  `tdi-:diff-type'"
  (interactive)
  (let* ((verb          (or verb (interactive-p)))
         (diff-type     (tdi-diff-type))

         ;;  In GNU diff , there is option --initial-tab
         ;;  which adds tab before each diff line to make the
         ;;  text look "as it was originally"
         ;;
         ;;  That's why allowed whitespace at the beginning.
         ;;
         ;;
         (re-c2         "^[ \t]*[*][*][*] \\([0-9]+\\)")
         (re-c3         "^[ \t]*[-][-][-] \\([0-9]+\\)")
         (re-normal     "^\\([0-9]+\\)\\(,[0-9]+\\)?+[acd][0-9]")

         (re-gnu-u      "^[ \t]*@@ [-+]\\([0-9]+\\),[0-9]+[ \t]+[-+]+")
         (re-gnu-n      "^[dac]\\([0-9]+\\) [0-9]+$")

;;;         (reuse-list    '(owner tdi-diff))
         (propl         '(mouse-face    highlight
                          owner         tdi-diff
                          ))
         )

    (save-excursion
      (PMIN)
      ;;  clear our markings first
      (tima-clear-region-properties (point-min) (point-max) '(owner tdi-diff))

      (cond
       ((eq diff-type 'context)
        (tima-re-search re-c2 nil 1 nil propl)
        (PMIN)
        (tima-re-search re-c3 nil 1 nil propl)
        )

       ((eq diff-type 'normal)
        (tima-re-search re-normal nil 1 nil propl)
        )

       ((eq diff-type 'gnu-u)
        (tima-re-search re-gnu-u nil 1 nil propl)
        )

       ((eq diff-type 'gnu-n)
        (tima-re-search re-gnu-n nil 1 nil propl)
       ))

      (if diff-type
          (if verb (message (concat "Diff parsed, type: "
                                    (prin1-to-string diff-type))))
        (if verb (message "Diff not recognized.")))
      )))





;;; ......................................................... &diff-do ...



;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tdi-diff-show (cmd)
  "Generate diff CMD for the buffer and show it in the other window.
Lets user to edit option in the command line."
  (interactive
   (progn
     (if (and
          (buffer-file-name)
          (file-exists-p
           (format
            "%s#%s#"
            (file-name-directory (buffer-file-name))
            (file-name-nondirectory (buffer-file-name))
            )))
          (message-flash
           2 'sit
           (format
            "There is autosave file, use minibuffer %s binding"
            (ti::k-function-bind-info
             'tdi-cl-insert-file-autosave
             tdi-:minibuffer-map
             ))))

     (list
      (tdi-diff-command-generate)
      )))

  (if (and (stringp cmd)
           (buffer-modified-p)
           ;;  If this is in the commend, we're diffing buffer against
           ;;  file on disk
           (not (string-match (regexp-quote tdi-:diff-tmp-file) cmd))
           (buffer-file-name)
           (y-or-n-p "Save buffer before running diff? ")
           )
      (save-buffer))


  (when (stringp cmd)

    (tdi-diff cmd 'show))
  )

;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tdi-diff-show-noask (cmd)
  "Generate diff CMD for the buffer. Guess all parameters."
  (interactive
   (list
    ;;  - The first one runs fine for rcsdiff (ie. buffer is in RCS)
    ;;    but if it fails, we have to ask paramters from user.
    ;;
    (or (tdi-diff-command-generate 'no-ask)
        (tdi-diff-command-generate))
    ))
  (if (buffer-modified-p)
      (and (y-or-n-p "Save buffer before running diff? ")
           (save-buffer)))
  (if (not (var-nil-p cmd))
      (tdi-diff cmd 'show))
  )




;;; ------------------------------------------------------------- &cmd ---
;;; - The diff data is inserted into register automatically, because
;;;   many time the diff data is pasted to somewhere else. Eg. by sending it
;;;   via mail to someone else in projects.
;;;
;;;
;;;###autoload
(defun tdi-diff (cmd &optional show verb)
  "Run diff on buffer, possibly using rcsdiff if file is version controlled.
Inserts contents into register.

The version control is determined by searching RCS strings 'Id' or 'Log'

Input:

  CMD		diff command
  SHOW          show the results
  NO-ASK        run diff without asking any questions.
  VERB          enable verbose messages

References:

  `tdi-:extra-diff-program'
  `tdi-:diff-buffer'
  `tdi-:diff-options'

Return:
 nil            ,the no-ask parameter could not determine right diff.
 buffer         ,the shell output buffer. Note, that the diff may have
                 failed, in that case the buffer does not hold valid output."
  (let* ((bf       (buffer-file-name))
         (buffer   (use-tmp-buffer tdi-:diff-buffer 'clear))
         (verb     (or (interactive-p) verb))
         (reg      (eval tdi-:diff-register)) ;where to put the diff content
         prereq
         stat tmp
         )

    ;;  It the command is rcsdiff, then we must include
    ;;  Prereq: tag to the beginning of diff. Thhat tells patch
    ;;  that when applying the diff, it must find that string
    ;;  from the file before it can prosess applying the patch.
    ;;
    ;;  It prevents patching wrong versions.
    ;;
    ;;      Prereq: 1.10
    ;;
    ;;  Or we use this which is more stricter. It supposes you have
    ;;  rcs 'id' string in a file.
    ;;
    ;;      Prereq: tinylib.el,v 1.10
    ;;
    ;;  Nov 8 1996: Hmm the latter isn't supported, if picks only
    ;;  "tinylib.el,v" and not whole string in the linew
    ;;  I'm going to request imprevement to GNU patch....
    ;;
    (cond
     ((setq tmp (strmatget "-r\\([0-9.]+\\)" 1 cmd))
      ;; Add file name too
      ;;
      (if (or t
              (null (setq tmp2 (tdi-cl-read-rcs-file-name cmd))))
          (setq prereq (format "Prereq: %s\n" tmp))

;;; disabled now.
;;;     (setq tmp2    (file-name-nondirectory tmp2))
;;;     (setq prereq  (format "Prereq: %s,v %s"  tmp2 tmp))
        )))




    (if (and (null cmd)
             (null bf))
        (message-quick "Sorry, this is not a file buffer.")
      (if (null cmd)
          ;;  if the NO-ASK parameter is set, then we can't ask anything
          ;;  from the user. What if the file is not RCS file? Then what we
          ;;  diff against? --> give up and return nil pointer
          (setq buffer nil)

        ;; ... ... ... ... ... ... ... ... ... ... ... ... shell command . .
        (setq stat (shell-execute cmd buffer))
        (job-buffer buffer
          (PMIN)
          (when prereq
            (insert prereq "\n"))
          )

        ;; show the output if
        ;; a) there is an error
        ;; b) user wants to see it anyway
        ;;
        (when (or (shell-exec-nok-p stat) show)
          (setq tmp (current-buffer))
          (display-buffer buffer)
          )

        ;;  Store the diff into register if all went ok
        ;;
        (if (not (tdi-shell-exec-nok-p stat))
            (job-buffer buffer
              (cond
               (reg     ;; save the diff contents
                (set-register reg (buffer-string))
                (if verb
                    (message
                     (concat "Diff in register ..." (char-to-string reg))))
                ))
	      (run-hooks 'tdi-:diff-hook)
	      ))

          ))
    buffer
    ))

;;}}}

;;{{{ code:



;;; ----------------------------------------------------------------------
;;;
(defun tdi-get-buffer-name ()
  "Return buffer name of the current diff."
  (let* (file
         ret
         list
         )
    (save-excursion
      (cond
       ((progn (PMIN)
;;;                    (d! "RCS file")
               (re-search-forward "^RCS file: .*/\\(.*\\),v" nil t))
        ;; RCS file: RCS/folding.el,v
        ;; retrieving revision 1.18
        ;; retrieving revision 1.19
        (setq file (read-match 1)))


      ((progn (PMIN)
;;;           (d! "---")
              (re-search-forward "^---[ \t]+\\([^\t ]+\\)" nil t))
       ;;  --- copy/tinydiff.el Wed Jan 24 16:08:05 1996
       ;;  +++ tinydiff.el      Wed Jan 24 16:29:07 1996
       ;;  @@ -1,6 +1,6 @@
       ;;
       (setq file (read-match 1))
       (or (or (setq ret (and (string-match "/" file)
			      (find-buffer-visiting file)))
               (setq ret (get-buffer (file-name-nondirectory file))))

           ;;  Hmm; the "---" file was not found; try "***" file
           (and
            (progn
              (PMIN)
;;;           (d! "***")
              (re-search-forward "^\\*\\*\\*[ \t]+\\([^\t ]+\\)" nil t)
              ;;  *** copy/tinydiff.el Wed Jan 24 16:08:05 1996
              ;;  --- /tmp/tdi.diff    Wed Jan 24 16:29:07 1996
              ;;
              (setq file (read-match 1)))
            (or (setq ret (and (string-match "/" file)
			       (find-buffer-visiting file)))
                (setq ret (get-buffer (file-name-nondirectory file)))
                ))
           ))



      ((progn (PMIN)
              (re-search-forward "^filename: \\([^\t ]+\\)$"))
       ;;  User tag, eg. output from shell script that generates the
       ;;  'filename' tag + runs the diff program.
       ;;
       (setq file (read-match 1)))


      (t
        ;;  other diffs .. Still open
       ))

      ;; ............................................. examine results ...
      (unless ret                       ;already set?
        (if file                          ;remove directory part
            (setq file   ;; returns nil if no "/" found
                  (file-name-nondirectory file)))

        (cond
         ((get-buffer file)
          ;;  - buffer name == file name
          (setq ret file))

         ((setq list (ti::b-buffer-list file))
          ;;  - Eg name tinydiff.el may be in name tinydiff.el<2>
          ;;  - just pick the first from list
          (setq ret (car list))
          )))

      ret
      )))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-get-line-number ()
  "Return diff line number if line has one."
  (let* ((diff-type     (tdi-diff-type))
         (re-c2         "^[*][*][*] \\([0-9]+\\)")
         (re-c3         "^[-][-][-] \\([0-9]+\\)")
         (re-normal     "^\\([0-9]+\\)\\(,[0-9]+\\)?+[acd][0-9]")

         (re-gnu-u      "^@@ [-+]\\([0-9]+\\),[0-9]+[ \t]+[-+]+")
         (re-gnu-n      "^[dac]\\([0-9]+\\) [0-9]+$")

         ret
         )
    (save-excursion
      (beginning-of-line)
      (cond
       ((eq diff-type 'context)
        (or (setq ret (bufmatget re-c2 1))
            (setq ret (bufmatget re-c3 1)))
        )

       ((eq diff-type 'normal)
        (setq ret (bufmatget re-normal 1)))

       ((eq diff-type 'gnu-n)
        (setq ret (bufmatget re-gnu-n 1)))

       ((eq diff-type 'gnu-u)
        (setq ret (bufmatget re-gnu-u 1)))
        ))


    (if ret
        (setq ret (string-to-int ret)))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-goto (buffer line)
  "Show BUFFER and put cursor at LINE in other window."
  (let* ((ob     (current-buffer))              ;original buffer
         (delay  0.1)
         )
    (switch-to-buffer-other-window buffer)
    (goto-line line)

    ;; Flash the cursor and go back to diff buffer
    (and delay
         (sit-for delay))
    (pop-to-buffer ob)
    ))




;;; ----------------------------------------------------------------------
;;;
(defun tdi-goto-prev ()
  "Search diff position backward."
  (interactive)
  (beginning-of-line)
  (tdi-goto-next 'back 'verb))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-goto-prev-no-update ()
  "Search diff position backward."
  (interactive)
  (beginning-of-line)
  (tdi-goto-next 'back 'verb 'no-update))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-goto-next-no-update ()
  "Search next position."
  (interactive)
  (end-of-line)
  (tdi-goto-next nil 'verb 'no-update))

;;; ----------------------------------------------------------------------
;;;
(defun tdi-goto-next (&optional back verb no-update)
  "Search next position, or  BACKWARD.

Input

  BACK          if non-nil then search backward
  VERB          enable verbose messages
  NO-UPDATE     do not update diff source buffer

Return:
  t             ,if successful
  nil           ,no more hits"
  (interactive)
  (let* ((diff-type     (tdi-diff-type))
         (re-c2         "^[*][*][*] \\([0-9]+\\)")
         (re-c3         "^[-][-][-] \\([0-9]+\\)")
         (re-normal     "^[0-9]+\\(,[0-9]+\\)?+[acd][0-9]")

         (re-gnu-u      "^@@ [-+]\\([0-9]+\\),[0-9]+[ \t]+[-+]+")
         (re-gnu-n      "^[dac]\\([0-9]+\\) [0-9]+$")

         (type-list
          (list
           (cons 'context
                 (concat re-c2 "\\|" re-c3))
           (cons 'normal
                 re-normal)
           (cons 'gnu-u
                 re-gnu-u)
           (cons 'gnu-n
                 re-gnu-n)
           ))
         (func          (if back 're-search-backward 're-search-forward))
         (verb          (or (interactive-p) verb))
         ret
         re
         )
    (setq re (cdr-safe (assoc diff-type type-list)))

    (if (not (stringp re))
        nil
      (if (null (funcall func re nil t))
          (if verb (message "No more hits."))
        ;;   Adjust the display to middle of the screen

        (cond
         ((null no-update)
          (recenter '(4))               ;show it
          (tdi-goto-kbd 'verb)
          (setq ret t)))))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tdi-goto-kbd (&optional verb )
  "Show the diff source in another window.
Input:
  VERB          ,enable verbose messages."
  (interactive)
  (let* ((verb          (or verb (interactive-p)))
         (line          (tdi-get-line-number))
         (buffer        (tdi-source))
         )
    (if (null line)
        ;; nothing to do, not valid line
        (if verb (message "Not sitting on diff source line reference."))
      (if (null buffer)
          (if verb (message "Cannot tell buffer name from diff."))
        (tdi-goto buffer line)
        ))
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tdi-goto-mouse (event)
  "Show current line in other window. Use mouse EVENT.
Activate only if point underneath has 'mouse-property."
  (interactive "e")
  (let* ((buffer        (tdi-source))
         line
         )
    ;;
    (when (tima-get-mouse-property)
      (setq line (remove-properties (ti::b-read-word "[0-9]+")))
      (if (and buffer
               (not (var-nil-p line)))
          (tdi-goto buffer (string-to-int line))
        (message "Sorry, missing Line Nbr or FilenName.")
        ))
    ))


;;}}}
;;{{{ example

;;; ......................................................... &example ...
;; - Code can be easily extracted with function tinylib.el/ti::m-pkg-rip-magic
;; - This hook setup turns on the view mode for easy scrolling
;;   of buffer. See tinymy.el for emacs independent view mode call.
;;
;;* _
;;*   (add-hook 'tdi-:diff-hook  'my-tdi-:diff-hook)
;;* _
;;*   (defun my-tdi-:diff-hook ()
;;*     "Prepares diff buffer for me."
;;*     (autoload 'timy-read-only "tinymy")
;;*     (set-buffer tdi-:diff-buffer)
;;*     (PMIN) (timy-read-only)
;;*     )
;;* _


;;}}}

(add-hook 'tdi-:mode-define-keys-hook  'tdi-mode-define-keys)

(provide   'tinydiff)
(run-hooks 'tdi-:load-hook)

;;; tinydiff.el ends here
