;; @(#) tinyef.el -- Electric file minor mode. Easy filename composing

;; @(#) $Id: tinyef.el,v 1.19 1997/05/31 12:47:15 jaalto NotReleased $
;; @(#) $Keywords: tools $
;;
;; $KnownCompatibility: 19.28,30 , XEmacs 19.14 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Apr 23 1995
;;
;; To get information on this program use ident(1) or do M-x tief-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyef.el|Jari Aalto|jari.aalto@poboxes.com|
;; Electric file minor mode. Minibuffer. Easy filename composing.|
;; 31-May-1997|$Revision: 1.19 $|~/misc/tinyef.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file
;;
;;	(require 'tinyef)
;;
;; Or use this autoload choice and your ~/.emacs is loaded quicker
;;
;;	 (add-hook  'minibuffer-setup-hook  '(lambda () (require 'tinyef)))
;;
;; If you have any questions, use this function
;;
;;	M-x tief-submit-bug-report


;;}}}

;;{{{ Documentation


;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tief-" in front of
;;   every function & variable. It stands for '(ti)ny (e)lecric (f)ile mode.
;; - variable names contain letters 'tief-:', excecpt hook/mode vars.

;;  Preface, Apr 1995
;;
;;      I saw a post in gnu.emacs.sources (what a source of
;;      inspirations), where one of my friend Anders Lindgren
;;      <andersl@csd.uu.se> posted the basic code that allowed electric ~
;;      and electric / character to wipe out full (mini)buffer in certain
;;      cases.
;;
;;      What you see here, is complete rewrite and enchancement of that
;;      code. This is a real must for any minibuffer file handling.
;;
;;  Overview of features
;;
;;	o   Easy filename editing. Deletes directories at time, delete line
;;	    backward, electric tilde, electric slash, electric colon etc..
;;	o   This is a must for minibuffer's C-x C-f command
;;
;;  Description
;;
;;      Believe or not, but for years I haven't understood what an earth
;;      "electric" means. I have heard many e-modes mentioned in the net,
;;      but never much payed attention to them... until now.
;;
;;      Well, this package isn't so "electric", because it only
;;      defines some keys to be "electric" and it needs some other keys
;;      solely to its own use (you can't insert these chars to buffer)
;;
;;      If you're another non-english speaking, as I am, the electric means
;;      that the character you press behaves differently if the pressing
;;      happens around certain other charcters (anyway, some condition is
;;      met which triggers this "other" behavior). Other than that, the
;;      char behaves normally.
;;
;;      Here is smple graph to give you an overview of what this mode
;;      is capable of. In these cases cursor it at the end of line.
;;      Alternatively, just load this file, press C-x C-x and experiment
;;      with keys "[]\/~".
;;
;;	o   b>> means what's on the line before,
;;	o   a>> means what's there after.
;;	o   ""  means what you just pressed
;;	o   []  means which action the character triggered
;;
;;	    b>> http:/www.site.com/~userFoo/dir1/dir2/dir3/ "/" [e-slash]
;;	    a>> http:/
;;	    The e-slash action wiped out the line, because writing
;;	    two slashes normally indicates, that you want to give
;;	    another path
;;
;;	    b>> ~/dir1/dir2/dir3/			"~" [e-tilde]
;;	    a>> ~
;;	    The action wiped the line away, because it assumed
;;	    you want to give "~userFoo" or another "~" relative path
;;
;;
;;	    b>> ~/dir1/dir2/dir3/			"[" [step-delete-back]
;;	    a>> ~/dir1/dir2/
;;	    The action wiped previous directory name or until
;;	    special mark, See code, defaults are  ":/@" (ange-ftp things)
;;
;;
;;	    b>> ~/dir1/dir2/			        "=" [undo]
;;	    a>> ~/dir1/dir2/dir3/
;;	    The action works like normal undo.
;;
;;
;;	    b>> ~/dir1/dir2/				"\" [chunk-delete]
;;	    a>>
;;	    The action deleted whole line. It deletes until special marks
;;	    like "@:". If repeated, it deletes constantly backward
;;
;;  Automatic Intallation
;;
;;	This file includes function
;;
;;	    tief-install
;;
;;	which hooks the mode to the appropriate places. Eg. to your
;;	minibuffer. If you're in trouble, you can always turn this
;;	mode off with the supplied hotkey, which is by default.
;;
;;	    C-c \
;;
;;	You can't "see" whether mode is on or off in minibuffer, since
;;	it doesn't have its own mode line. But calling the hotkey
;;	will tell you the state change.
;;
;;	You can also remove this mode completely from your emacs if you need
;;	to do that in emergencies. just call function
;;
;;	    tief-install
;;
;;	With some prefix argument (like C-u M-x tief-install)


;;}}}
;;{{{ history


;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May	29	1997	[jari]		19.28	v1.18-1.19	NotReleased
;; - Checkdoc 1.29 clear. defcustom support.
;;
;; Apr	18	1997	[jari]		19.28	v1.14-1.17	NotReleased
;; - XEmacs20 character handling. Internal changes
;;
;; Feb 	12	1997	[jari]		19.28	v1.11-1.13	NotReleased
;; - Added autoload installation example. Small changes.
;;
;; Oct	1	1996	[jari]		19.28	v1.9-1.10	NotReleased
;; - Rearranged loading: now uses autoloads more.
;; - Changed all '(lambda forms to use (function (lambda.
;;
;; Aug	20	1996	[jari]		19.28	v1.8		NotReleased
;; - Removed all XEmacs 19.14 byte Compiler errors.
;;
;; Jun	25	1996	[jari]		19.28	v1.7		NotReleased
;; - Renamed functions so that they do not clash with XEmacs 19.14
;;
;; May	3	1995	[jari]		19.28	v1.6		NotReleased
;; - A small adjustent to make sure XEmacs compatibility is preserved.
;;   Now requires tinylilbxe.
;;
;; Apr	30	1995	[jari]		19.28	v1.5		NotReleased
;; - Enchanced the e-tilde action so that "fpt@some.com:pub/dir"
;;   causes "fpt@some.com:~" and another "~" would delete full line.
;;   This was suggested by rwalker@cisco.com (Rob Walker)
;;
;; Apr	29	1995	[jari]		19.28	v1.4		NotReleased
;; - Added e-dollar action, suggested by rwalker@cisco.com (Rob Walker)
;;
;; Apr	23	1995	[jari]		19.28	v1.3		NotReleased
;; - Textual changes. Wrong prefix in feedback functions corrected.
;; - Reprogrammed the keymap keys. Now there is also "move" action
;;   and step-delete-fwd.
;;
;; Apr	23	1995	[jari]		19.28	v1.2		NotReleased
;; - More complete docs and examples. Improved killing inside string,
;;   like if you do C-x ESC ESC to recall the command and fix the path
;;   inside double quotes.
;;
;; Apr	23	1995	[jari]		19.28	v1.1		Created


;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinylibm)

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinyEf tief-: tools
    "Electric file minor mode. Designed for minibuffer file prompt editing.
  Overview of features

	o   Easy filename editing. Deletes directories at time, delete line
	    backward, electric tilde, electric slash, electric colon etc..
	o   This is a must for minibuffer's C-x C-f command
    ")

;;}}}
;;{{{ setup: variables

;;; .......................................................... &v-mode ...

;;;###autoload (autoload 'tief-mode "tinyef" t t)
(eval-and-compile

(ti::m-fmacro-minor-mode-wizard
 "tief-" " Tef" nil nil 'TinyEf "tief-:"		;1-6

  "Electric file name mode.
This mode helps you composing filename more easily. Some keys
are \"electric\", meaning that they have two behavior. By default
~/$ are electric. Some other keys have special meaning and you
cannot insert them into buffer. They do move/delete/undo operations.

Defined keys:
\\{tief-:mode-map}
"

  "Tief"
  nil

  "Electric file mode"
  nil

  nil
  )
)


;;; ......................................................... &v-hooks ...

(defcustom tief-:load-hook 'tief-install
  "*Hook that is run when package is loaded."
  :type 'hook :group 'TinyEf
  )

;;; ....................................................... &v-private ...

(defcustom tief-:mode-defined-maps
  '(global-map
    minibuffer-local-must-match-map	;eg C-x C-f uses this
    read-expression-map
    minibuffer-local-map
    )
  "*Keymaps where to install `tief-mode' hotkey."
  :type '(symbol :tag "Keymap") :group 'TinyEf
  )

;;; ........................................................ &v-public ...
;;; User configurable

;;  note: In my HP-UX the "\" key is at right just above enter, Unshifted.
;;  Please pick your favourite.
;;
(defcustom tief-:mode-key  "\C-c\\"
  "*Key to toggle `tief-mode' on/off in global and minibuffer map."
  :type '(string :tag "Key sequence") :group 'TinyEf
  )


;;  Note: In my HP-UX the "[]\" keys are unshifted and just above ENTER
;;  key. Pick your favourites
;;
;;  Note: it's more confortable to have undo in some non-shift,
;;  non-control key when you're in minibuffer.
;;
(defcustom tief-:mode-key-table
  '(
    (?\[   . step-delete-back)		;KEY -- action symbol
    (?\]   . step-delete-fwd)
    (?\\   . chunk-delete)
    (?\;   . move-back)
    (?\'   . move-fwd)
    (?\~   . e-tilde)			;electric keys
    (?\/   . e-slash)
    (?\$   . e-dollar)
    (?\=   . undo)
    )
  "*Map keys to actions.
Refer source file's default values for action names.
If you change this; you must call function \\[tief-mode-map-define-keys]."
  :type '(repeat
	  (list
	   (character :tag "Electric char")
	   (choice
	    :tag "Action"
	    (const step-delete-back)
	    (const step-delete-fwd)
	    (const chunk-delete)
	    (const move-back)
	    (const move-fwd)
	    (const e-tilde)
	    (const e-slash)
	    (const e-dollar)
	    (const undo)
	    )))
  :group 'TinyEf
  )


(defcustom tief-:step-delete-chars "-./@:"
  "*When using step-delete action, kill untill these chars.
The \"-\" character must be first in the string."
  :type '(string "Charset") :group 'TinyEf
  )



;;}}}
;;{{{ version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyef.el"
 "tief"
 tief-:version-id
 "$Id: tinyef.el,v 1.19 1997/05/31 12:47:15 jaalto NotReleased $"
 '(tief-:version-id
   tief-:load-hook
   tief-:mode-hook
   tief-mode
   tief-:mode-map
   tief-:mode-defined-maps

   tief-:mode-name
   tief-:mode-key-table
   )))


;;}}}

;;{{{ code: misc, keys, install

;;; ########################################################### &Funcs ###

(defsubst tief-key-p (map key)
  "Tests is tief-mode is assigned to KEY in MAP"
  (eq 'tief-mode (lookup-key map key)))

;;; ----------------------------------------------------------------------
;;;
(defsubst tief-act  (char)
  "Return action for CHAR"
  (cdr-safe (char-assq char tief-:mode-key-table)))

;;; ----------------------------------------------------------------------
;;;
(defun tief-install-maps (&optional remove)
  "Define `tief-mode' hotkey to keymaps. Optionally REMOVE."
  (let* ((key	tief-:mode-key)
	 (fun	'tief-mode)
	 (maps	tief-:mode-defined-maps)
	 x
	 )
    (list-loop maps x
      (cond
       (remove
	(if (tief-key-p (eval x) key)	;; eval or symbol-value function
	    (define-key (eval x) key nil)))
       (t
	(define-key (eval x) key fun)
	)))
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tief-mode-map-define-keys  ()
  "Defines `tief-:mode-map' keys.
Always clears the keymap first and reinstalls the minor mode."
  (interactive)
  (let* ((list tief-:mode-key-table)
	 elt
	 )
    (setq tief-:mode-map  (make-sparse-keymap)) ;always refresh

    ;;  Minor modes have copy of the keymap. Get rid of it and
    ;;  replace it with new one.
    ;;
    (ti::k-add-minor-mode	'tief-mode nil nil	'remove)

    (list-loop list elt
      (define-key tief-:mode-map (char-to-string (car elt)) 'tief-char))
    (ti::k-add-minor-mode	'tief-mode 'tief-:mode-name  tief-:mode-map)
    ))



;;; ----------------------------------------------------------------------
;;;
(defun tief-install  (&optional arg)
  "Install package. With oprional ARG uninstall."
  (interactive)
  (cond
   (arg
    (remove-hook	'minibuffer-setup-hook	'tief-minibuffer-setup)
    (remove-hook	'minibuffer-exit-hook	'turn-off-tief-mode)
    (ti::k-add-minor-mode	'tief-mode nil nil	'remove)
    (tief-install-maps  'remove)
    )
   (t
    (add-hook		'minibuffer-setup-hook	'tief-minibuffer-setup)
    (add-hook		'minibuffer-exit-hook	'turn-off-tief-mode)
    (tief-mode-map-define-keys)		;installs also minor-mode
    (tief-install-maps)
    )))



;;}}}
;;{{{ code: minibuffer

;;; ----------------------------------------------------------------------
;;; by Anders Lindgren.
;;;
(defun tief-minibuffer-setup ()
  "Turn on `tief-mode' when entering minibuffer."
  (setq
   tief-mode
   (and (boundp 'minibuffer-completion-table)
	(eq minibuffer-completion-table 'read-file-name-internal))
   ))


;;}}}
;;{{{ code: main


;;; ----------------------------------------------------------------------
;;;
(defun tief-step (&optional back)
  "Position cursor, optionally BACK."
  (let* ((set    tief-:step-delete-chars)
	 (rset   (concat "^" set))	;reverse set
	 (func   (if back 'skip-chars-backward 'skip-chars-forward))
	 (point  (point))
	 limit
	 )
    (if back
	(setq limit (BOLP))
      (setq limit (EOLP)))

    (funcall func rset limit)		;do the movement

    (cond
     ((eq (point) point)		;not moved
      (funcall func set  limit)
      (funcall func rset limit)		;try again
      ))

    (cond
     ((not (eq (point) point))		;moved ok
      (cond
       ((and (null back) (not (eolp)))
	(forward-char 1))		;fix position a little
       )))
    ))




;;; ----------------------------------------------------------------------
;;;
(defun tief-char ()
  "Handle `tief-mode' characters. See `tief-:mode-key-table'.
If there is no action for character insert it as is.

Current keymap:
\\{tief-:mode-map}"
  (interactive)
  (let* ((char		last-command-char) ;char pressed
	 (act           (tief-act char))
	 (re		'(".*@"  ".*:"))
	 (e-list	'(?/  ?@ ?\" ?\'))
	 (pnow          (point))
	 (point		(point))
	 str
	 eolp
	 bolp
	 hits
	 )
    (if (null act)			;no action recognized
	(insert char)

      (save-excursion	(setq bolp (BOLPP)  eolp (EOLPP)))
;;;      (d! char (char-to-string char) act)

      ;; ... ... ... ... ... ... ... ... ... ... ... ...  e-kill-point . .
      ;; find suitable kill point
      ;;
      (save-excursion
	(beginning-of-line)
	(while re
	  (if (and (o-looking-at (car re))
		   (not (eq eolp (match-end 0))))
	      (push (match-end 0)  hits))
	  (NEXTP re))

	(if hits  (setq point (apply 'max hits))) ;;find longest position
	(if (eq point eolp)		          ;;end of line ?
	    (setq point (point)))

	(cond
	 ((eq pnow point)		;no different than current point?
	  (setq str (buffer-substring bolp pnow))

	  ;;  make the end position not to go past string delimiter "
	  ;;
	  (if (not (string-match ".*\"" str))
	      (setq point bolp)
	    (setq point (+ bolp (match-end 0))))

	  ))
	)


      (cond
       ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... undo ..
       ((eq act 'undo)
	(undo)
	)

       ;; ... ... ... ... ... ... ... ... ... ... ... ... ... .. chunk ..
       ((eq act 'chunk-delete)
	;; The kill point is already set
	;;
	(delete-region point (point))
	)

       ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... step ..
       ((memq act '(step-delete-fwd move-fwd))
	(setq point (point))
	(tief-step)

	(if (eq act 'step-delete-fwd)
	    (delete-region point (point)))
	)

       ((memq act '(step-delete-back move-back))
	(setq point (point))
	(tief-step 'back)

	(if (eq act 'step-delete-back)
	    (delete-region point (point)))
	)

       ;; ... ... ... ... ... ... ... ... ... ... ... ... ... electric ..
       ((and (memq act (list 'e-slash))
	     (char-in-list-case (preceding-char) e-list)

	     ;; permit `//hostname/path/to/file'
	     (not (eq (point) (1+ (point-min))))

	     ;; permit `http://url/goes/here'
	     (not (char= ?: (char-after (- (point) 2))))
	     )
	(delete-region point (point))
	(insert char)
	)

       ((memq act '(e-tilde))
	;;  /ftp@some:~  ,pressing "~" now deletes full line
	;;
	(cond
	 ((char= (preceding-char) ?~)
	  (delete-region bolp (point)))
	 ((char= (preceding-char) ?:)
	  ;; /ftp@some:   ,allow adding "~"
	  )
	 (t
	  (delete-region point (point))
	  ))
	(insert char)
	)

       ((memq act '(e-dollar))
	(delete-region bolp (point))
	(insert char)
	)

       (t
	(insert char)
	))
      )))



;;}}}


(provide   'tinyef)
(run-hooks 'tief-:load-hook)

;;; tinyef.el ends here
