;; @(#) tinygflint.el -- Gimpel C/C++ Flint compatible error parser.

;; @(#) $Id: tinygflint.el,v 1.15 1997/06/19 10:51:11 jaalto NotReleased $
;; @(#) $Keywords: language, error parsing $
;; $KnownCompatibility: 19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 2 1995
;;
;; To get information on this program use ident(1) or do M-x tigf-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el


;; LCD Archive Entry:
;; tinygflint|Jari Aalto|jari.aalto@poboxes.com|
;; Gimpel C/C++ Flint compatible error parser.|
;; 19-Jun-1997|$Revision: 1.15 $|~/misc/tinygflint.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ........................................................ &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file
;;
;;     (require 'tinygflint)
;;
;;  or use autoload and your emacs starts up faster.
;;
;;     (defalias 'flint 'tigf)              ;; use better name
;;     (autoload 'flint "tinygflint" t t)
;;     (autoload 'tigf  "tinygflint" t t)
;;
;;  You should also set some variables to configure flint for your
;;  envinronment. See tigf-:locals-hook and the example section at the
;;  end.
;;
;;  Run the Gimpel flint with emacs command
;;
;;	M-x flint
;;
;;  more fancier example at the end of this file.
;;  If you have any questions, use this function
;;
;;      M-x tigf-submit-feedback

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tigf-" in front of
;;   every function & variable. It stands for '(ti)ny (g)impel (f)lint
;;   error parser'.
;; - variable names contain letters 'tigf-:'.

;;  Preface, may 1995
;;
;;	After I had managed to put the TICE together [tinycerr.el], this
;;	was the first parser I needed immediately. The Flint is C++
;;	syntax checker that warns you about code errors. You can't get
;;	getter than that :-/
;;
;; Description
;;
;;	Parser for Gimpel FlexeLint C/C++ syntax checker's output so
;;      that you can use mouse to jump to errors. If you don't
;;      know what fling or prolint is, you don't need to know
;;      about this package.
;;
;;	The ProLint(Gimpel Flint) is marketed by <info@ptsdirect.co.uk>
;;
;; Limitations
;;
;;	In order to this parser to work as expected, make sure you
;;	haven't touched the default output layout of Flint's error
;;	messages. The options -format and -h, that modify the output
;;	should not be touched.
;;
;;	The default error format uses '_' to spot the error:
;;
;;                            _
;;	    insPtr = new wdbmgr_c;
;;	    /users/test/tmp/t.cc  77  Warning 633: Assignment from a
;;          strong type (size_t) in context: arg. no. 1

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jan	19	1997	[jari]		19.28	v1.14-1.15	NotReleased
;; - text changes
;;
;; Sep	25	1996	[jari]		19.28	v1.13		NotReleased
;; - Rearranged loading.
;;
;; Aug	26	1996	[jari]		19.28	v1.12		NotReleased
;; - XE 19.14 ByteComp errors cleared.
;;
;; Jun	12	1996	[jari]		19.28	v1.11		NotReleased
;; - lisp-mnt.el's lm-verify checked. Rewrote feedback funcs. ByteComp ok.
;; - Upgraded to parser 7.00B3, and as it seemed to work fine, I
;;   generalized the support regexp to accept any version of flint.
;;
;; Sep	4	1995	[jari]		19.28	v1.10		NotReleased
;; - Made the support table and regexps more "loose". It seems that
;;   Flint versions 6.00i - 6.00q (which comes with prolint) are
;;   fully compatible. The 6.00i isn't much good, since it always
;;   dumps core, whilst 6.00q seems to be rock solid. Get always newest!
;;
;; Aug	29	1995    [jari]		19.28	v1.9		NotReleased
;; - Added example setup
;; + tigf-get-re-lists, tigf-600i
;;
;; May	24	1995    [jari]		19.28	v1.8		NotReleased
;; - the Special goto function reprogrammed. Now has umbrealla funtion.
;;
;; + tigf-get-compiler-ver, func
;; + tigf-compile-get-string, func
;; + tigf-sg-600i-2line , tigf-special-goto-600i
;;
;; May	5	1995    [jari]		19.28	v1.7		NotReleased
;; + tigf-compile-locals , now defines paragraphs
;; + tigf-locals, separated setting + hook run
;; + tigf-lint-error-msg-cat-word-re: this concerns pargraphs
;; + tigf-lint-error-mark: lint marks the error places
;;
;; May	4	1995    [jari]		19.28	v1.2-1.6	NotReleased
;; - It has been running solid long time now, but It cannot multi line
;;   splitted file & line info. I have been concentration ti TICE
;;   packet, beause it's the key here.
;; + tigf-:locals-hook, so that user can configure his own variables
;;
;; May	2	1995    [jari]		19.28	v1.1		Created

;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinycerr)			;error parser engine

;;}}}
;;{{{ setup: -- hooks

;;; ............................................................ &bind ...

;;; .......................................................... &v-bind ...

;;; ......................................................... &v-hooks ...

(defvar tigf-load-hook nil
  "*Hook run at load time")


(defvar tigf-:locals-hook nil
  "*Function that is run right after the locals are set.")

;;}}}
;;{{{ setup: -- private

;;; ... private variables ................................. &v-private ...

(defvar tigf-:version-re "FlexeLint.*Ver.*"
  "*Regexp that says which line holds version string for compiler.")



;;  - We recently upgraded to 7.00B3 and the same engine in 6.x
;;    works fine with version 7.00.
;;  - Now, I'm assuming here, that all Gimpel flint parsers
;;    produce same erros.. just a rough guess.
;;
(defvar tigf-:support-re "."
  "*Regexp that is tried againt Compiler version string got from
the buffer. ")


(defvar tigf-:support-table
  '(;;  The 6.00 group uses similar error messages
    ("6.0"  tigf-600i tigf-special-goto-600i) ;this is original
    ("."    tigf-600i tigf-special-goto-600i) ;this is a guess
    )
  "List of elements where element 1 is RE that is tried to macth
against compiler's version string. The rest of the list consists of
functions to take care of version dependent actions.")


;;}}}
;;{{{ setup: user conf

;;; ... user configurable .................................. &v-config ...

(defvar tigf-:verbose t
  "*Informational messages option.")

(defvar tigf-:msg-sleep 1
  "*sit-for time for message display.")

(defvar tigf-error-mark ?_
  "*Character that shows the error position.")

(defvar tigf-error-msg-cat-word-re
  (concat
   "^[ \t]*\\(Info\\|Note\\)[ \t]+[0-9]+:"
   )
  "*In order to paragraph to jump to right positions, this regexp
matches those error lines that are CAT'd together without a. empty lines
b. error markers. See the following output.


 _
 };
 /users/jaalto/tmp/t.cc  22  Warning 612: Expected a declarator
 _
 };
 /users/jaalto/tmp/t.cc  28  Warning 612: Expected a declarator
    --- Wrap-up for Module: /users/jaalto/tmp/t.cc

 Info 766: Header file '/usr/include/assert.h' not used in module
     '/users/jaalto/tmp/t.cc'
 Note 964: Header file '/usr/include/memory.h' not directly used in module
     '/users/jaalto/tmp/t.cc'


")



(defvar tigf-:special-goto-table
  '(tigf-sg-600i-hdr			;sg = special goto
    tigf-sg-600i-2line
    )
  "*Function to call in special goto occasions.")




;;}}}
;;{{{ setup: -- version

;;; ... version info ...................................... &v-version ...


(defconst tigf-version-id
  "$Id: tinygflint.el,v 1.15 1997/06/19 10:51:11 jaalto NotReleased $"
  "Latest modification time and version number.")



;;; ----------------------------------------------------------------------
;;;
(defun  tigf-version (arg)
  "version information."
  (interactive "P")
  (ti::m-pkg-version-info "tinygflint.el" arg))

;;; ----------------------------------------------------------------------
;;;
(defun  tigf-submit-feedback ()
  "Submit suggestions, error corrections, impressions, anything..."
  (interactive)
  (ti::m-pkg-submit-feedback "tinygflint.el"))


;;}}}

;;; ########################################################### &Funcs ###



;;{{{ misc

;;; ----------------------------------------------------------------------
;;;
(defun tigf-elt (list sym)
  "Returns SYM element from the 'support table alike LIST'. It is expected
that the _first_ level list is passed to this function."
  (let* ()
    (cond
     ((eq sym 're)  (nth 0 list))
     ((eq sym 'f-gen-re)  (nth 1 list))	;function that generates REGEXPS
     ((eq sym 'f-s-goto)  (nth 2 list))	;.. special goto function
     )))

;;; ----------------------------------------------------------------------
;;;
(defun tigf-msg (msg &optional eflag)
  "Print package's MESSAGE. Show error if EFLAG raised"
  (let* ((verb  tigf-:verbose)
	 (sleep tigf-:msg-sleep)
	 )
    (if (null verb) nil
      (message
       (concat (if eflag "*err "  "")
	       "tigf: " msg))
      ;;  There may be successive messages...
      (and sleep (sit-for sleep)))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tigf-is-supported (str)
  "Return LIST or nil if Flint version is supported. Input is Compiler's
version STRING."
  (let* ((ptr tigf-:support-table)
	 re ret el
	 (loop t)
	 )
    (while (and str loop ptr)
      (setq el (car ptr)  re (car el))
      (if (string-match re str)
	  (setq loop nil  ret el))
      (setq ptr (cdr ptr)))
    ret
    ))

;;}}}
;;{{{ Sepecial goto

;;; .................................................... &special-goto ...

;;; ----------------------------------------------------------------------
;;; "sg" = special goto
;;;
(defun tigf-sg-600i-hdr (point)
  "Handle special error like one below, which is spread on multiple lines.

Info 766: Header file '/usr/include/assert.h' not used in module
     '/users/jaalto/tmp/t.cc'
"
  (let* ((wips  ".*[']\\(.*\\)[']")	;name in 'wips'
	 file				;full dir too
	 ret
	 (match-re ".*: Header file.*not used")
	 re
	 )
    (save-excursion
      (beginning-of-line)
      (cond
       ((looking-at match-re)
	(setq re  (tice-get-c++-include-re wips 1))
	(forward-line 1)
	(setq file (tice-read-match wips 1))

;;;	(d! "~special" file (tice-load-allowed-p file) re)
;;;	(setq R re)

	(cond
	 ((and file re (tice-load-allowed-p file))
	  (if (and (tice-pop-file file 1)
		   (re-search-forward re nil t))
	      nil
	    (tigf-msg (concat "Cant't find: " re) t)
	    )
	  (setq ret t)			;we handled this!
	  ))

	)))
    ret
    ))


;;; ----------------------------------------------------------------------
;;; 2 line spread errors..
;;;
(defun tigf-sg-600i-2line (&optional point)
  "Handles special error: file-line info spread to two lines

Warning 1512: destructor for base class wobinf_c (line 63, file
    /usr/local/NokiaOMC/localenv/include/wobinfmx.h) is not virtual

"
  (let* (
	 file
	 line
	 ret
	 )
    (save-excursion
      (goto-char (if point point (point)))
      (beginning-of-line)
      (setq line (bufmatget ".*[(][ \t]*line[ \t]+\\([0-9]+\\)" 1))

      (forward-line 1)
      (setq file (bufmatget "[ \t]+\\([^ \t]+\\)[)]" 1))
      )

    (cond
     ((and line file)
;;;      (d! "~special" (tice-load-allowed-p file) line file)
      (setq ret t)			;we handled this!

      (if (not (tice-load-allowed-p file))
	  (tigf-msg "Not a project file. Load suppressed for this line.")
	(if (tice-pop-file file 1)
	    (goto-line line)
	  (tigf-msg (concat "Cant't find file" file) t)
	  ))
      ))
    ret
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tigf-special-goto-600i (point)
  "Umbrella function. Calls all special goto functions until
one of them returns t, meaning the line was handled."
  (let* ((fl tigf-:special-goto-table)	;function list
	 func
	 ret
	 )
    (while (and fl (null ret))
      (setq func (car fl))
      (setq ret (funcall func point))
;;      (d! func ret)
      (NEXTP fl)
      )
    ret
    ))

;;}}}

;;{{{ support

;;; ....................................................... &supported ...

;;; ----------------------------------------------------------------------
;;; Parsing output like:
;;;  _
;;;  };
;;; /users/jaalto/t.cc  22  Warning 612:
;;;  Info 754: local structure member X::some (line 16, file /t.cc)
;;;  Note 964: Header file '/usr/include/sys/time.h' not used in module
;;;
(defun tigf-600i ()
  "Flint 6.00i error messages"
  (let* ((err "\\(Info\\|Warning\\|Error\\|Note\\)")
	 (A   "[-a-zA-Z0-9/.]+\.[ch]+")	        ;C/C++ file name
	 (N   "[0-9]+")
	 (ng  (concat "\\(" N "\\)"))		;number group
	 (ag  (concat "\\(" A "\\)"))
	 (w   "[ \t]*")
	 (W   "[ \t]+")
	 (re
	  (concat err W ng ":"))    ;Info 749: local enumeration


	 (f1 (concat "^" w ag))
	 (f2 (concat ".*, file" W "\\([^)]+\\)"  ))
	 (file-re
	  (list (list 1 f1)
		(list 1 f2)))

	 (l1 (concat ".*"  W ng W err ))
	 (l2 (concat ".*[(]line" W ng "," ))
	 (line-re
	  (list (list 1 l1)
		(list 1 l2)))

	 )
    (list re file-re line-re)
    ))



;;; ----------------------------------------------------------------------
;;; Common MAIN function
;;;
(defun tigf-get-re-lists (compiler-string)
  "Returns error match lists by looking at COMPILER-STRING."
  (let* ((ptr  tigf-:support-table)
	 (ver  compiler-string)
	 (loop t)
	 el re func ret
	 )
    (while (and loop ptr ver)
      (setq el   (car ptr)
	    re   (tigf-elt el 're)
	    func (tigf-elt el 'f-gen-re))

      (if (null (string-match re ver))
	  (setq ptr (cdr ptr))
	(setq loop nil
	      ret (funcall func))))
    ret
    ))





;;; ----------------------------------------------------------------------
;;; Common MAIN function
;;;
(defun tigf-special-goto (point)
  "Handles special error lines. Returns t if goto handled. Handling includes
printting error messages too."
  (let* ((ver-str (tice-v 'compiler-str))
	 (list (tigf-is-supported ver-str))
	 func stat
	 )
    (if (null list)
	(progn
;;;	  (d! "~spec-ver" ver-str)
	  (tigf-msg "Special goto not supported for this compiler version")
	  t)				;have to return this
      (setq func (tigf-elt list 'f-s-goto))
      (setq stat (funcall func point))
      ;;  Remember, we must return t if we handled this
      stat)
    ))


;;}}}

;;{{{ Setting compile variables


;;@ (defun tt ()				;temporary quick testing
;;@   (interactive)
;;@   (tigf-compile-locals)
;;@   (tice-emacs-var-set))

;;; ----------------------------------------------------------------------
;;;
(defun tigf-compile-locals ()
  "Sets variables before compile takes effect."
  ;; buffer is in compile already, so here we go!
  (let*  ((errm     (char-to-string tigf-error-mark))
	  (err-re   tigf-error-msg-cat-word-re)
	  (para-s    (concat
		      "^\f"
		      "\\|^[ \t]*$"
		      ;;  These have to be last, because regexp picks
		      ;;  the _first_ OR case
		      "\\|"    err-re
		      "\\|^[ \t]*" errm
		      ))
	  (para-sep "^[ \t\f]*$")
	  )
    ;;  Notice that I set the ERR function too, since flint is known to
    ;;  return NON-zero == abnormal termination after compile...

    (setq-default
     tice-:compile-cmd            "flint"
;;;	tice-:compile-finish-err-func 'my-flint-error
     tice-:compile-finish-ok-func   'tigf-mark-buffer
     tice-:compile-finish-err-func  'tigf-mark-buffer
     tice-:goto-err-func            'tigf-special-goto
     tice-:compiler-support-re      tigf-:support-re
     tice-:compiler-ver-re          tigf-:version-re

     ;;  Use defaults
     tice-:compile-finish-ok-re    "finish"
     tice-:load-allow-re-func      nil

     ;;  make mark/unmark enable/disable goto
     tice-:goto-unmarked           nil
     tice-:max-lines               200
     tice-:paragraph-start         para-s
     tice-:paragraph-separate      para-sep
     )
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tigf-locals ()
  "Sets variables before compile takes effect."
  (tigf-compile-locals)
  (run-hooks 'tigf-:locals-hook)
  )


;;}}}
;;{{{ main

;;; ############################################################ &main ###

;;; ----------------------------------------------------------------------
;;; - RUN THIS FUNCTION if you have errors already in buffer
;;; - the call of function is defined in the locals
;;;
(defun tigf-mark-buffer ()
  "Prepares buffer with flint error messages for mouse click"
  (interactive)
  (let* ((compiler-in-use-str (tice-v 'compiler-str))
	 (el (tigf-get-re-lists compiler-in-use-str))
	 re file-re line-re
	 )
    (if (not (and el
		  (eq (length el) 3)))
	(tigf-msg  "Parsing for this version of flint not supported." t)
      (setq re      (nth 0 el)
	    file-re (nth 1 el)
	    line-re (nth 2 el))
      (goto-char (point-min))
      (tice-parse-entry re file-re line-re)
      )))

;;; ----------------------------------------------------------------------
;;;
(defun tigf (&optional add-opt cmd-set)
  "Runs Gimpel Flint, c/c++ checker over current module."
  (interactive)
  (let* ((tice-:local-var-func	'tigf-locals)	;we change this
	 )
    (tice-compile add-opt cmd-set)
    ))

;;}}}

;;{{{ example

;;; ......................................................... &example ...
;;; - Do not copy this directly.
;;; - It's mere purpose is to give you ideas how to use this .el
;;; - Code can be easily extracted with function tinylib.el/ti::m-pkg-rip-magic

;;* _
;;* _
;;* ;;; ########################################################## x-menu ###
;;* _
;;* (defun my-x-prog-menu (&optional event)
;;*   "Popups an X window of Programming releated stuff."
;;*   (interactive) ;;
;;* _
;;*   ;;  Simulate fake event so that we don't have bind this to mouse key
;;*   ;;  --> now can be used from any key
;;*
;;*   (and (null event)
;;*        (setq event (list (list 353  189) (selected-window))))
;;* _
;;*   (let (item)
;;*     (setq item
;;*           (x-popup-menu
;;*            event
;;*            '("Programming"
;;*              ("Normal"
;;*               ("~~ Tcompile"        . my-hpc++-compile)
;;*               ("~~ Tcompile ++"     . my-hpc++-compile-extra)
;;*               ("~~ Tcompile ++ INC" . my-hpc++-compile-incdirs)
;;*               ("Compile           " . noisy-compile)
;;*               ("Flint Work        " . my-flint-work)
;;*               ("Flint filter std"   . my-flint-filter-1)
;;*               ("Flint filter raw"   . my-flint-filter-2)
;;*               ("Flint std"          . my-flint-std)
;;*               ("Flint raw"          . my-flint-raw)
;;*               ("--"                 . ignore)
;;*               ("wmp"                . my-compile-wmp)
;;*               ("btree"              . my-compile-btree)
;;* _
;;*               )))
;;*     (cond
;;*      ((integerp item)
;;*       (cond
;;*        ((eq item 1)
;;*         t)
;;*        ))
;;*      (t                                 ;direct command
;;*       (if item (call-interactively item))
;;*       ))
;;*     ))
;;* _
;;* ;;; #################################################### TICE general ###
;;* ;;;
;;* _
;;* ;; To allow evaling this buffer multiple times we do this....
;;* (defvar    my-tice:colors nil   "Flag, if my colors are set.")
;;* (defconst  my-project-file-re   "wmp.*mx\\|wig.*mx"
;;*   "*Regexp that matches files that belong to my projects.")
;;* _
;;* ;;    These belong to my PROJECT in my work, I need additional paths...
;;* (defmacro  my-get-inc-dirs ()
;;*   (list 'or (list 'getenv '"_INCDIRS") '""))
;;* _
;;* ;;    Do not put include dirs fo CC, because I use STD libs If I
;;* ;;    compile my own code.
;;* (defconst  my-c++-cmd           "CC -z -g ")
;;* _
;;* (defconst  my-flint-cmd
;;*   (concat
;;*    ;;  This specifies absolute path for my flint prog
;;*    (or (getenv "FLINT") (error "Please set env. variable FLINT"))
;;*    " -I. "
;;*    (my-get-inc-dirs)		;add my include directories
;;*    " "))
;;* _
;;* _
;;* (defun my-tice-colors ()
;;*   (interactive)
;;*   ;; -  Remember to quit all HEAVY applications that use colors, like
;;*   ;;    netscape, otw you get "X server cannot allocate color"
;;*   ;; -  The condition case, prevents susch error to stop you down.
;;* _
;;*   (condition-case nil
;;*       (set-face-foreground 'tice-f-info "wheat")
;;*     (error				;do nothing
;;*      nil))
;;* _
;;*   (condition-case nil
;;*       (set-face-foreground 'tice-f-info "turquoise")
;;*     (error nil))
;;* _
;;*   (condition-case nil
;;*       (set-face-background 'tice-f-note "DeepSkyBlue3")
;;*     (error nil))
;;* _
;;*   (set-face-foreground 'tice-f-note "yellow")
;;* _
;;*   (condition-case nil
;;*       (set-face-background 'tice-f-warn "#3070c0")
;;*     (error nil))
;;* _
;;*   (condition-case nil
;;*       (set-face-foreground 'tice-f-warn "wheat")
;;*     (error nil))
;;* _
;;*   (condition-case nil
;;*       (set-face-foreground 'tice-f-err  "blue3")
;;*     (error nil))
;;*   )
;;* _
;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;;
;;* (defun my-tice-locals-common ()
;;*   "Common settingsfor all compilers."
;;*   (if my-tice:colors nil                        ;already set
;;*     (my-tice-colors)
;;*     (setq my-tice:colors t))
;;* _
;;*   (setq-default
;;*    tice-:load-allow-re-func     'my-tice-load-allow-check
;;*    ;; tice-:compile-finish-err-func   'my-tice-error
;;*    tice-:max-lines               700
;;* _
;;*    ))
;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;;
;;* (defun my-tice-load-allow-check ()
;;*   "Checks for my project files. in the compilation buffer.
;;* If project file is detected, I wan't to see only their files errors,
;;* not libraries.
;;* "
;;*   (let* ((re my-project-file-re)        ;my project files
;;*          )
;;*     (save-excursion                     ;try to spot any file in buffer
;;*       (goto-char (point-min))
;;*       (if (re-search-forward re nil t)  ;allow only load them
;;*           re
;;*         nil)
;;*       )))
;;* _
;;* _
;;* ;;; ################################################### tinygflint.el ###
;;* ;;; Gimpel FLINT 6.00i
;;* _
;;* (setq tigf-:locals-hook  'my-flint-locals)
;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;;
;;* (defun my-flint-locals ()
;;*   "Setting up the compile envinronment."
;;*   (my-tice-locals-common)
;;*   (setq-default tice-:compile-cmd-func          'my-flint-cmd-create)
;;*   )
;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;;
;;* (defun my-flint-cmd-create (file opt mode)
;;*   "Composes my command for flint."
;;*   (let ((cmd my-flint-cmd)              ;this is default
;;*         (def "")                        ;default file
;;*         )
;;* _
;;*     ;;  load default parameters for each module
;;*     (cond
;;*      ((string-match "wmp" file)
;;*       (setq def " -header\\(wmpdatmx.h\\) "))
;;*      )
;;* _
;;*     (cond
;;*      ((eq mode 'raw)                    ;no filterings, no options...
;;*       (setq cmd (concat cmd " +fcp " file))
;;*       )
;;*      ((eq mode 'std)
;;*       (setq cmd (concat cmd "./std.lnt -u " file)) ;with options, no filter
;;*       )
;;*      (t
;;*       ;;  - I run the output through filter first, before I wan't
;;*       ;;    the output to arrive to *compile*.
;;*       (setq cmd (concat cmd
;;*                         " ./std.lnt -u " def
;;*                         file "| flint.pls "
;;*                         (or opt ""))))
;;*      )
;;*     cmd
;;*     ))
;;* _
;;* ;;; ----------------------------------------------------------------------
;;* ;;; Some ready-to-run commands
;;* ;;;
;;* (defun my-flint-work ()
;;*   "As I use it on my work."
;;*   (interactive)
;;*   (flint))
;;* _
;;* (defun my-flint-filter-1 ()
;;*   "filter standard libraries + some flint errors"
;;*   (interactive)
;;*   (flint "-xx"))
;;* _
;;* (defun my-flint-filter-2 ()
;;*   "filter standard libraries "
;;*   (interactive)
;;*   (flint "-xf"))
;;* _
;;* (defun my-flint-std ()
;;*   "Raw lint with options."
;;*   (interactive)
;;*   (flint nil 'std))
;;* _
;;* (defun my-flint-raw ()
;;*   "Raw lint with options."
;;*   (interactive)
;;*   (flint nil 'raw))



;;}}}


(provide   'tinytigf)
(run-hooks 'tigf-load-hook)

;;; tinygflint.el ends here
