;; @(#) tinyindent.el -- like indented-text-mode, but minor-mode, smart indent

;; @(#) $Id: tinyindent.el,v 1.27 1997/06/19 09:42:15 jaalto NotReleased $
;; @(#) $Keywords: editing tab extensions $
;;
;; $KnownCompatibility:  19.28+ XEmacs19.14  $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1994,95,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Sep 18 1994
;;
;; To get information on this program use ident(1) or do M-x tii-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el
;;
;; Original idea by Alan K. Stebbens in autoindent.el


;; LCD Archive Entry:
;; tinyindent|Jari Aalto|jari.aalto@poboxes.com|
;; TAB indentation minor-mode, like indented-text-mode but in minor mode form|
;; 19-Jun-1997|$Revision: 1.27 $|~/misc/tinyindent.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Installation

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;;  ~/.emacs startup file. Rip code with with tinylib.el/ti::m-pkg-rip-magic
;;
;;* _
;;*     (require 'tinyindent)
;;
;; OR use this; your .emacs loads quicker
;;
;;* _
;;*     (autoload 'tii-mode    "tinyindent" t t)
;;*     (autoload 'tii-tt-mode "tinyindent" t t)
;;
;; Suggested keybindings, you're going to use them a lot..
;;
;;* _
;;* 	(global-set-key [C-tab]     'tii-tt-mode) ;; this is toggle
;;* _
;;*	;;;  the first one is for some PC machines (XCeed emulated X)
;;* 	(global-set-key [S-kp-tab]  'tii-mode)
;;* 	(global-set-key [S-backtab] 'tii-mode)    ;; this is on/off mode
;;
;;	For some PC:s in nonWindowed, this is same as S-tab
;;	--> check out with C-h l
;;
;;* _
;;*     (define-key esc-map     "OI"			'tii-mode)
;;
;; If you have any questions, use this function
;;
;;      M-x tii-submit-bug-report

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVERD prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tii-" in front of
;;   every function & variable. It stands for '(ti)ny (i)indent
;; - variable names contain letters 'tii-:', excecpt version/hook/mode vars.

;;  Preface, 1994
;;
;;	The original auto-indent-mode from autoindent.el was very short
;;	and limited, so I thought I extend it a little...here is the
;;	result.  Thank you Alan for giving me a push to extend your code
;;	into new directions.  When I spoke with Alan and he gave me free
;;	hands, because he hadn't used the .el for quite a long time.
;;
;;	I wasn't satisfied with the  indent-relative function, so
;;	I coded a preprocessor for it. Now the cursor won't jump
;;	all over the line if the previous one was empty. Just
;;	try original  M-x indent-relative when there is empty line above
;;	and you'll see what I mean.
;;
;;	And where this module really shines: Has it ever been easier to line
;;	up variables according to '=' or in within lisp 'let*', or writing
;;	mail messages while this mode is turned on...
;;
;;  Overview of features
;;
;;	o   General block editing or indentation MINOR mode. Replacement for
;;	    indented-text-mode.
;;	o   Takes over the TAB and BACKSPACE key.
;;	o   Looks back to guess right indentation. and uses relative indent
;;	    when not at BOL.
;;	o   Special indentation is suggested if cursor is at BOL and
;;	    user defined regexp is matched in line above. (like adding
;;	    multiple c++ comments)
;;	o   Extra tii-tt-mode for writing descriptions within comments. This
;;	    allows user to choose when to use HARD tab or SOFT tab = relative
;;	    to the text above. TAB TAB inserts hard tab, TAB SPC inserts soft
;;	    tab.
;;
;;  Word about defining keys to maps
;;
;;	Do not ever use local-set-key inside some my-XXX-mode-hook,
;;	because you may loose these keybindings when minor modes are turned on.
;;	Always use (define-key XXX-mode-map <key> <func>) inside hooks
;;	and your keybindings will not be affected.
;;
;;	Simple example for lisp mode:
;;
;; (add-hook 'lisp-mode-hook 'my-lisp-hook)     ;; *better version*
;; (add-hook 'emacs-lisp-mode-hook 'my-lisp-hook)
;;
;; (defun my-lisp-hook ()
;;   "My lisp setings"
;;   (define-key shared-lisp-mode-map		;; <<<<
;;     "\M-e"
;;     '(lambda () (interactive) "Eval buffer"
;;        (eval-current-buffer) (message "eval ok"))))
;; ;; End
;;
;; (defun my-lisp-hook ()			;; *not recommmended version*
;;   "My lisp setings"
;;   (local-set-key				;; <<<<
;;     "\M-e"
;;     '(lambda () (interactive) "Eval buffer"
;;        (eval-current-buffer) (message "eval ok"))))
;; ;; End
;;
;;
;;	In the latter case, turning on the tii<or any> minor mode would
;;	have caused losting your ESC-e keybinding.

;;}}}
;;{{{ History

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun	19	1997	[jari]		19.28	v1.27		NotReleased
;; - text changes. Checkdoc 1.42 clear.
;;
;; May	14	1997	[jari]		19.28	v1.26		NotReleased
;; - Internal changes
;; - Changed all non standard boolean tests to use when and unless
;;
;; Mar  19	1997	[jari]		19.28	v1.21-1.24	NotReleased
;; - Code rewritten a bit. XE20 character handling support added.
;;
;; Jan  15	1997	[jari]		19.28	v1.17-20	NotReleased
;; - Rearranged loading. Now uses autoloads as much as possible.
;; - Added lib m require due to macros.
;;
;; Sep  10	1996	[jari]		19.28	v1.16		NotReleased
;; - The modename appearing fixed.
;;
;; Aug  29	1996	[jari]		19.28	v1.14-15	NotReleased
;; - Modernized the code a bit. Removed old functions that are now in libs
;;   rewrote more cleaner installation functions.
;; - Fixed mode-line display problem.
;;
;; Sep  2	1995	[jari]		19.28	v1.13		NotReleased
;; - lisp-mnt.el's lm-verify checked. ByteComp ok. Rewrote feedback funcs.
;;
;; Sep  2	1995	[jari]		19.28	v1.12		Release_3
;; - Added ###autoload directives. Added tii-:bol, so that user can decide
;;   if he wants special BOLP handling or not.
;;
;; Aug  31	1995	[jari]		19.28	v1.11		NotReleased
;; - Thanks for Anders Lindgren, andersl@csd.uu.se (author of follow-mode)
;;   and his explanation about using minor modes right I was able to solve
;;   the hangup. I also added note about how the keybinding should
;;   be accoplished withing mode hooks.
;;
;; Aug  28	1995	[jari]		19.28	v1.10		NotReleased
;; - Total hangup with previous version, now I'm using standard minor
;;   map approach, but tii-mode keymap overrides definitions on the
;;   current map. Should investigate why ?
;; - Needs little adjustments for indenting from the BOL.
;;
;; Aug  28	1995	[jari]		19.28	v1.9		NotReleased
;; - Finally removed that horrible local-set-key "\t" .. hacks and started
;;   using 19.28 emacs minor modes. Now the Tab key behaves lot more
;;   expectedly.
;; - Made variables buffer local and removed buffer local
;;   variable create function. Much cleaner now.
;; - Cleaned the whole tii-relative function. Uses now event if possible,
;;   so that it won't choke to events as previously used read-char .
;;
;; Mar  22	1995	[jari]		19.28	v1.8		Release_2
;; - The modeline didn't show the "mode" , this was due to lacking
;;   local variable definitions.
;;
;; Feb  9	1995	[jari]		19.28	v1.2-1.7       	Release_1
;; - Development versions.
;;
;; Sep 18	1994	[jari]		18.57	v1.1		first code

;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinylibm)
(eval-when-compile (use-dynamic-compilation))

;;}}}
;;{{{ setup: all

;;; .......................................................... &v-bind ...

(defvar tii-:mode-map nil
  "*Minor keymap, only for TAB change. Copy of `current-local-map'.")

;;; ......................................................... &v-hooks ...

(defvar tii-:mode-load-hook nil
  "*Hook run when file is loaded.")

(defvar tii-:mode-hook nil
  "*Hook run when `tii-mode' turned on.")

(defvar tii-:mode-define-keys-hook 'tii-:mode-map-define-keys
  "*Hook to define keys for mode.")


;;; .......................................................... &v-mode ...

(defvar tii-mode nil
  "If set, indicates that auto-indent mode is active.
This variable isautomatically set by invoking \\[tii-mode].")
(make-variable-buffer-local 'tii-mode)

(defvar tii-tt-mode nil
  "*Hard tab submode.")
(make-variable-buffer-local 'tii-tt-mode)

;;; ....................................................... &v-private ...


(defvar tii-:RET nil
  "Global return value used, when multiple values are neede.
Shouldn't interest regular user.")
(make-variable-buffer-local 'tii-:RET)


(defvar tii-:cp 0  "Internal. Current point.")
(make-variable-buffer-local 'tii-:cp)

(defvar tii-:cl 0  "Internal. Current line.")
(make-variable-buffer-local 'tii-:cl)


;;; ........................................................ &v-public ...
;;; User configurable

;;; - The BOL is special, because when you write code, the crucial
;;;   point is line start: you decide indentation or cursor positioning with
;;;   that first keystroke.
(defvar tii-:bol t
  "*Flag that determines if beg. of line should be treated differently.")


(defvar tii-:special-re
  (concat
   "^[ \t]*\\(//\\|\#\\|!\\|REM\\)[ \t]*"
   ;;   don't put ;;+, since someone may draw ;;;;;;;;;;...
   "\\|^[ \t]*;;:?;?[ \t]*"
   )
  "*Special indent at the beginning of line.
Sometimes single indent isn't enough. For example it would be convenient
to write long C++ comments by hitting the TAB on the next line. Original
RE handles considers these as special cases.
!          .Xdefauls or X-related files
#          Perl, awk, shell
//         C++
;;;        Lisp
REM        Oracle Sqlplus, SQL files in general")


(defvar tii-:mode-str-orig " Tii"
  "*String to be displayed in mode line.")

(defvar tii-:tt-mode-str-orig " TiiT"
  "*String to be displayed in mode line.")


;;  This is no a user option
;;
(defvar tii-:mode-name tii-:mode-str-orig
  "Current minor mode status displayed. Changed dynamically.")
(make-variable-buffer-local ' tii-:mode-name)


;;; ......................................................... &version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyindent.el"
 "tii"
 tii-:version-id
 "$Id: tinyindent.el,v 1.27 1997/06/19 09:42:15 jaalto NotReleased $"
 '(tii-:version-id
   tii-:mode-map
   tii-:mode-load-hook
   tii-:mode-hook
   tii-:mode-define-keys-hook
   tii-:special-re
   tii-:mode-str-orig
   tii-:mode-str
   )))


;;}}}

;;; ########################################################### &Funcs ###

;;{{{ code: misc


;;; ----------------------------------------------------------------------
;;;
(defun tii-:mode-map-define-keys ()
  "Defines keybindings to tii-:mode-map."

  (define-key  tii-:mode-map "\t"        'tii-relative)

  ;;  eg. lisp-mode uses backward-delete-char-untabify which is
  ;;  uncomfortable in editing.
  ;;
  ;;  The 2nd bind works in X env only
  (define-key  tii-:mode-map "\177"        'delete-backward-char)
  )

;;; ----------------------------------------------------------------------
;;;
;;;### (autoload 'tii-install-mode "tinyindent" t t)
(ti::m-fmacro-minor-mode-install
 tii-install-mode
 tii-mode
 tii-:mode-map
 tii-:mode-name
 tii-:mode-define-keys-hook
 )

;;; ----------------------------------------------------------------------
;;;
(defun tii-confirm (msg)
  "Confirms action with MSG.
RET/SPC = ok. The real character pressed is available
thru global variable tii-:RET."
  (setq tii-:RET (read-char-safe msg))
  (if (or (char= tii-:RET ?\C-m)
	  (char= tii-:RET ?\ ))		; RET/SPC
      t
    nil
    ))

;;}}}
;;{{{ engine

;;; .......................................................... &engine ...

;;; ----------------------------------------------------------------------
;;;
(defun tii-special-handle ()
  "Handle some special lines -- like `gin-mode',  but simpler.
Supposes that point is at the beginning of investigated line.
Moves point 1 line forward af ter done.

Returns:
  filling pattern to use at front of line or nil"
  ;;  Look for some special lines, like C++
  (let* ((s-re	   tii-:special-re)
	 fill
	 line
	 )
    (when (looking-at s-re)
      ;;  back to original line
      ;;
      (forward-line 1)		;otherwise visible to user when asking
      (when (tii-confirm "indent special? ")
	(setq line
	      (save-excursion
		(goto-char  (- (point) 1))
		(read-full-line)))
	(string-match s-re line)
	(setq fill
	      (substring line (match-beginning 0) (match-end 0)))
	))
    fill
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tii-relative ()
  "Almost like `indent-relative', but handles some special cases.
- if the above line if NOT empty, then we indent relatively automatically
- if above line IS empty, then ask if normal TAB/relative indent.

References:
  tii-:special-re"
  (interactive)
  (let* (
	 (bolp-flag     tii-:bol)
	 (p		(point))
	 (cur-col	(current-column))
	 (imode	        t)
         (SPC		(char-to-int ?\ ))

	 prev-empty
	 prev-col
	 bp ep				;BEG END point
	 fill
	 line
	 ch
	 skip
	 )

    (catch 'cancel
      (save-excursion
	(save-excursion
	  (setq bp (BOLPP))
	  (setq ep (EOLPP))

	  (forward-line -1)
	  (setq prev-empty (looking-at "[ \t]*$"))
	  (end-of-line)
	  (setq prev-col (current-column))
	  )

	;;  make sure these are NOT nil
	;;
	(if (null tii-:cp) (setq tii-:cp 0))
	(if (null tii-:cl) (setq tii-:cl 0))

	;;  Count lines has A BUG! , If I'm at the beg of line
        ;;  or 1 char forward it gives different values!
	;;
	(setq line (count-lines 1 p))
	(if (or (eq p bp) (eobp))
	    (setq line (1+ line)))	;BEG of line error

	;;   - the user has answered to question, we are on the same line
	;;   - if he is at the beginning, then ALWAYS ask (forced ask)
	;;
	(if prev-empty
	    (if (and			;already asked ?
		 (>= tii-:cp bp)
		 (<= tii-:cp ep)
		 )
		(setq skip 1))
	  (if (null (bolp)) (setq skip 2))          ;BOL ?
	  (if (< prev-col cur-col) (setq skip 3))   ;previous line is shorter
	  )

;;;  (d! skip "POINT" p  " " tii-:cl line " bp ep  " bp ep tii-:cp)

	(if skip  (throw 'cancel t))	;we were on this line already

	(setq tii-:cl line)		;update line number
	(setq tii-:cp p)		;current point position

	;;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
	;;  The real engine
	(setq tii-:RET nil)
	(cond
	 ((bobp)
	  (tab-to-tab-stop))

	 ((bolp)
	  (forward-line -1)		;Check previous line
	  (if (setq fill (tii-special-handle))
	      (progn
		(throw 'cancel t))
	    (forward-line 1)
	    (when tii-tt-mode
	      (if (null bolp-flag)
		  (setq imode t)
		(setq imode (tii-confirm "indent relative?")))
	      ))
	  (setq ch tii-:RET)		;this was pressed
	  ))

	))

    ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ catch end ^^^
    (if fill
	(insert fill)			;see save-excursion, fill is set
      (cond
       (tii-tt-mode
	(setq ch (or ch (read-char-safe)))

	(cond
	 ((eq ch SPC)
	  (indent-relative))
	((char= ch ?\t)
	 (tab-to-tab-stop)
	 (setq ch nil)			;tab stop already does this
	 )
	(t
	 (indent-relative)))
	)
       (t
	(cond
	 (imode
	  (indent-relative)		;other char follows
	  (if (eq ch SPC)		;kill space, because it means YES
	      (setq ch nil))
	  )
	 (t
	  (tab-to-tab-stop)		;use hard tab
	  (setq ch nil)			;kill the TAB char
	  ))
	))

      ;; (d! imode ch (print-p ch)))
      ;;  the TAB char automatically moves to tab-to-tab-stop
      ;;  if it's inserted
      ;;
      (if (and (print-p ch)
	       (not (eq ch SPC)	))	;this is already handled
	  (insert ch))			;add the character, don't loose it

      )))

;;}}}
;;{{{ modes

;;; ........................................................... &modes ...


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tii-tt-mode (&optional arg)
  "Toggle `tii-tt-mode' with ARG. See description in `tii-mode'."
  (interactive "P")
  (var-toggle tii-tt-mode arg)             ;toggle mode variable
  (cond
   (tii-tt-mode
    (unless tii-mode			;turn on the major mode tii-mode
      (tii-mode)			;turn it on
      (setq tii-tt-mode t)

      )
    (setq tii-:mode-name tii-:tt-mode-str-orig)
    )
   (t
    (setq tii-:mode-name tii-:mode-str-orig)
    ))

  (ti::xe-modeline-update)
  )


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tii-mode (&optional arg)
  "Toggle relative indentation mode with ARG.

Indentation is determined according to previous lines. Special
indent happens only at the beginning of line, where user is asked if
he wants to have relative or \"hard\" indentation.

>> submode `tii-tt-mode' <<
This isn't really mode. It just turns one flag on in `tii-mode', so that
it behaves a little differently. If the `tii-mode' is not running, it
wiil be turned on. turning off `tii-tt-mode' _does_not_ end `tii-mode'.

Sometimes you want to control between 'hard' tab and 'soft' tab, ie.
relative indent. This mode causes second character to be read after
tab key is hit. The following happens:

TAB TAB     ,inserts hard tab
TAB SPC     ,indent relative without inserting space char.
TAB x       ,indents relative and inserting character x

\\{tii-:mode-map}"
  (interactive "P")

  (if (null (assq 'tii-mode minor-mode-alist))
      (tii-install-mode))

  (var-toggle tii-mode arg)             ;toggle mode variable

  ;; .......................................................... on ...
  (cond
   (tii-mode
    (if tii-tt-mode
	nil				;Hard tab mode already set
      (setq tii-:mode-name tii-:mode-str-orig))
    (run-hooks 'tii-:mode-hook)
    )
   (t
    ;; ........................................................... off ...
    ;;  restore the values
    (setq tii-tt-mode nil)
    ))
  (ti::xe-modeline-update)
  )

;;}}}

(provide   'tinyindent)
(run-hooks 'tii-:mode-load-hook)

;;; tinyindent.el ends here
