;; @(#) tinyliby.el -- library of functions related to Emacs s(y)stem

;; @(#) $Id: tinyliby.el,v 1.28 1997/06/18 07:48:56 jaalto NotReleased $
;; @(#) $Keywords: library, system functions $
;; $KnownCompatibility: 19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1995,96,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      May 20 1995
;;
;; To get information on this program use ident(1) or do M-x tiliby-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el


;; LCD Archive Entry:
;; tinyliby|Jari Aalto|jari.aalto@poboxes.com|
;; library of emacs sYstem related functions|
;; 18-Jun-1997|$Revision: 1.28 $|~/misc/tinyliby.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ........................................................ &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file
;;
;;     (require 'tinyliby)
;;
;; If you are developing software that uses this library you should use
;; Following to ensure that you get right version of library loaded.
;;
;;	(require 'tinyliby)
;;	(tinyliby-require "1.1")         ;or any other version+

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; Briefly:
;; o    This is library, package itself does nothing.
;; o    Collection of emacs system functions.

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - The name convention of the TINY LIB functions are described and defined
;;   in main module: tinylib.el
;; - To avoid collisions to other modules "ti::y-" is used in front of
;;   every function. It stands for (ti)ny library for s(y)stem funcs.
;; - variable names contain letters 'ti:y-' , except the version
;;   variables.


;;  Preface, 1995
;;
;;	Some emacs system related functions are collected hre, that
;;	I used all the time. I was working with tinydgboot.el, which let
;;	me boot Ding or GNUS configuration without leaving emacs. I had
;;	to introduce myself to totally new areas in emacs, also the
;;	tinypatch.el taught lot more about emacs.
;;
;;  Code policy
;;
;;	All functions are written by [jari], unless otherwise notified.
;;	If I have found interesting function from some other .el I have
;;	written a reference to that .el. If the code is directly taken
;;	'almost as is' all the credit belongs to the Author of the
;;	original code. Cheers for him, to write so recpectable code, that
;;	it can be used right from the box!
;;
;;	If someone contributes good, general purpose function, all the
;;	credit belongs to him automatically. Persons name and date and so
;;	on is included.

;;}}}
;;{{{ History

;; ........................................................ &t-history ...
;;; Change Log:
;;
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun  18	1997	[jari]		19.28	v1.26-1.28	NotReleased
;; - version-require removed.  ti::y-describe-symbols corrected.
;;
;; May	27	1997	[jari]		19.28	v1.24-1.25	NotReleased
;; - Changed all non standard boolean tests to use when and unless.
;; - checkdoc 1.42 clean.
;;
;; Apr  18	1997	[jari]		19.28	v1.22-1.23	NotReleased
;; - improved the describe symbols function (now uses i-args call) and
;;   improved ti::y-get-file-documentation interactive call.
;;
;; Feb	27	1997	[jari]		19.28	v1.20-1-21	NotReleased
;; - ti:y-tmp-buffer now uses it's own buffer and not shared *tmp* which
;;   may be wiped away any time.
;; - ti::y-unload-feature is now interactive. Funny thing happened,
;;   you you 1)laod pacakge 2)call this function tounload it
;;   3) reload the pacakge.... Suprise the defavars don't define the
;;   variables any more! (19.29)
;;
;; Feb	22	1997	[jari]		19.28	v1.19		NotReleased
;; - One library func call renamed.
;;
;; Nov	1	1996	[jari]		19.28	v1.15-1.18	NotReleased
;; - Rearranged loading. now uses autoloads.
;; - Changed all '(lambda forms to use (function (lambda
;; - Added lib m require due to macros.
;;
;; Jun	15	1996	[jari]		19.28	v1.14		NotReleased
;; - Cleared all XE byteCompiler errors.
;;
;; Jun	14	1996	[jari]		19.28	v1.13		NotReleased
;; - Modified describe symbols, now detects macros and prints info,
;;   where the function was loaded (the package name)
;;
;; Jun	3	1996	[jari]		19.28	v1.12		NotReleased
;; - Added dynamic byte compile support. Enchanced the describe-symbols.
;;
;; May	29	1996	[jari]		19.28	v1.11		NotReleased
;; - Made the describe function more better. Now it shows function
;;   calling args + keys to access the function.
;;
;; Feb	21	1996	[jari]		19.28	v1.10		NotReleased
;; - Serious error in 'M'. It redefined append-to-buffer. Renamed in 'M'
;; - Added value printting in describe-symbols. Rewrote symbol-summary.
;;
;; Feb	5	1996	[jari]		19.28	v1.9		NotReleased
;; - Byte compiler errors corrected. Slight error in get-file-docum
;;   function.. It couldn't recognice my-func( , whenre paren was sticked
;;   to func name.
;;   ti::y-get-file-documentation :! corrected
;;
;; Feb	5	1996	[jari]		19.28	v1.8		NotReleased
;; - lisp-mnt.el's lm-verify tested. New feedbak interface.
;;   ti::y-get-file-documentation :+ new function to get all docs
;;   ti::y-describe-symbols	  :! new parameter, exclude-re
;;
;; Nov	12	1995	[jari]		19.28	v1.7		NotReleased
;; - Small bug found: when giving nil arg, the funcs were dropped.
;;   ti::y-describe-symbols	:! corrected
;;
;; Oct	8	1995	[jari]		19.28	v1.6		NotReleased
;; - Hey, the describe function it getting more cool all the time...
;;   Now I added fancy inyteractive to tell you what we are searching.
;;   At least I can't remember all prefix arg meanings, so the prompt
;;   must remind me about the choice. Also added nice command history.
;;   ti::y-describe-symbols	:! More attractive interactive
;;
;; Oct	8	1995	[jari]		19.28	v1.5		NotReleased
;; - Major rewrite of describe-symvols, now it accepts arguments
;;   ti::y-describe-symbols	:! More easy locate what you're seeking
;;
;; Oct	3	1995	[jari]		19.28	v1.4		NotReleased
;; - Added handy symbol gathering function and some variables.
;;   tinylib-load-hook		:+ load hook
;;   ti:y-tmp-buffer		:+ tmp buf
;;   ti:y-desc-buffer		:+ tmp buf
;;   ti::y-symbol-summary	:+ handy information picker
;;   ti::y-describe-symbols	:+ better than emacs's appropos
;;
;; Sep	2	1995	[jari]		19.28	v1.3		NotReleased
;; - Added ti::y-unload-symbols, ti::y-remove-from-hooks
;; - Changed parameters for ti::y-unload, now they are symbols
;; - Changed ti::y-get-symbols, now accepts TEST-FORM arg
;;
;; Jun	22	1995	[jari]		19.28	v1.2		NotReleased
;; - Corrected byte comiler errors
;;
;; May	20	1995	[jari]		19.28	v1.1		Created


;;}}}

;;; Code:
;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinylibm)

(eval-and-compile
  (autoload 'adelete				    "assoc")
  )

(eval-when-compile
  (require 'advice)
  (use-dynamic-compilation)
  )

;;}}}

;;{{{ setup: -- variables

;;; ....................................................... &v-private ...


(defconst ti:y-package "ti::y"
  "Librarys identifier. For error messages etc...")

(defsubst ti::y  ()
  "Return package Id string."
  ti:y-package)

(defvar ti:y-ret nil
  "Global return value of package.")

(defvar ti:y-describe-symbols-history nil
  "History of regexps.")

;;; ........................................................ &v-public ...

(defvar tinylib-load-hook nil
  "Hook run when file is loaded.")

(defvar ti:y-tmp-buffer "*ti::y-tmp*"
  "*Temporary buffer name")


(defvar ti:y-desc-buffer "*desc*"
  "*Describe buffer")


;;}}}
;;{{{ setup: -- version

;;; ....................................................... &v-version ...
;;; These are not library funcs, so they have normal 'tinyliby-' prefix

(defconst tinyliby-version
  (substring "$Revision: 1.28 $"11 15)
  "Latest version number.")


(defconst tinyliby-version-id
  "$Id: tinyliby.el,v 1.28 1997/06/18 07:48:56 jaalto NotReleased $"
  "Latest modification time and version number.")


;;; ----------------------------------------------------------------------
;;;
(defun tinyliby-version (&optional arg)
  "Show version information. ARG will instruct to print message to echo area."
  (interactive "P")
  (ti::m-pkg-version-info "tinyliby.el" arg))

;;; ----------------------------------------------------------------------
;;;
(defun tinyliby-submit-feedback ()
  "Submit suggestions, error corrections, impressions, anything..."
  (interactive)
  (ti::m-pkg-submit-feedback "tinyliby.el"))


;;}}}

;;; ########################################################### &Funcs ###

;;{{{ features, load list


;;; ......................................................... &loading ...


;;; ----------------------------------------------------------------------
;;;
(defun ti::y-load-cleanup (ELT)
  "Remove ELT from `after-load-alist' by replacing entry with nil."
(let* ((ptr after-load-alist)
	 forms
	 frm
	 elt
	 )

    (list-loop ptr elt
      (setq forms (cdr elt))
      (list-loop forms frm
	;; change form to nil
	(if (equal frm ELT)
	    (setcar forms nil))
	))
    ))

;;; ........................................................ &features ...

(defun ti::y-lh-where (sym)
  "Looks `load-history' to find SYM. The sym may be function or variable name.

Return:
  list       ,feature's load history entry where variable were found.
  nil        ,no information in `load-history' about this variable."
  (let* ((ptr  load-history)
	 ret entry
	 )
    (list-loop ptr entry
      (setq entry (car ptr))		;point to functions
      (if (memq sym entry)
	  (setq ret (car ptr)  ptr nil))
      )
    ret
    ))


;;; ----------------------------------------------------------------------
;;; - Does little garbage collect...but what the heck!
;;; - lh = load-history
;;;
(defun ti::y-lh-get (sym)
  "Return variables and functins defined by feature SYM.
The symbols are tested to be [f]boundp, so the list consists of
those elements only that actually exist in emacs.

Return:
  ((variable-list ..) (func-list ..))
   nil     ,if none found"
  (let* ((name  (symbol-name sym))
	 (list  (cdr (assoc name load-history)))
	 vl fl el ptr
	 )

    (if (null list) nil
      ;;  Search the variables/funcs start position in list
      (while (and list
		  (listp (car list)))
;;;	(d! (listp (car list)) (car list) )
	(setq list (cdr list))
	)

      (setq ptr list)
      (while ptr
	(setq el (car ptr))
;;;	(d! (listp el) (boundp el)(fboundp el) el)
	(if (listp el)
	    nil
;;;	  (if (eq 'gnus-article-buffer el)
;;;	      (d! ">>" (boundp 'gnus-article-buffer)))
	  (if (boundp el)
	    (setq vl (append vl (list el))))
	   (if (fboundp el)
	       (setq fl (append fl (list el))))
	   )
	(setq  ptr (cdr ptr)))
;;;      (d! "3" vl fl)
      )

    ;;
    (if (or vl fl)
	(list vl fl)
      nil)
    ))



;;; ----------------------------------------------------------------------
;;;  - Be sure what your're doing if using this...
;;;
(defun ti::y-feature-kill (sym)
  "Kill feature SYM and its `load-history' information permanently."
  (let* ((name (symbol-name sym))
	 )
    ;;  Load history , dependencies remove
    (if (assoc name load-history)
	(setq load-history (adelete 'load-history name)))

    ;;  Kill the symbol from feature list
    (if (featurep sym)
	(setq features (delete sym features)))

    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::y-unload-symbols (list)
  "Unload all variables/functions in LIST of symbols."
  (mapcar
   (function
    (lambda (x)
      (cond
       ((fboundp x)
	(fmakunbound x))
       ((boundp x)
	(makunbound x))
       )))
   list))


;;; ----------------------------------------------------------------------
;;;
(defun ti::y-unload (mode list)
  "According to MODE, unload all variables/features/functions in LIST.

MODE can be
'var      ,list of variables
'func     ,list of functions
'feature  ,list of features  , caution !! Be sure to get
           features variables/funcs before you use this, since it'll
           delete all information that `unload-feature' cmd needs.

           The `unload-feature' is not always good cmd, because it checks
           dependencies and may not allow you to delete a feature."
  (let* (var
	 test-func
	 kill-func
	 )
    (cond
     ((eq 'var mode)
      (setq  test-func 'boundp  kill-func 'makunbound))
     ((eq  'func mode)
      (setq  test-func 'fboundp  kill-func 'fmakunbound))
     ((eq 'feature mode)
      ;;  - Emacs don't let us remove a feature if it contains some
      ;;    require statement. Be sure to get the information
      ;;    about the variables and func first before killing feature,
      ;;    since we destroy load-history information also!!
      (setq  test-func 'featurep  kill-func 'unload-feature)
      )
     (t
      (error "unknown mode" mode)))

;;;    (d! kill-func list)
    (while list
      (setq var (car list))


      ;; Test if exist
      (if (null (funcall test-func var)) nil

;;;	(d! kill-func var)
	;; Existed, so how to remove ?
	(cond
	 ((eq kill-func 'unload-feature)
	  ;;  Feature kill is special
;;;	  (d! "F-kill" var)
	  (ti::y-feature-kill var)
	  )
	 ((eq kill-func 'fmakunbound)
	  ;;  Okay, I know, I'm shooting with rocks here by using
	  ;;  advise remove to all funcs, but it's safe this way...
	  (ad-unadvise var)
	  (funcall kill-func var)
	  )
	 (t
	  (funcall kill-func var)))
	)
      (setq list (cdr list))
      )
    ))



;;; ----------------------------------------------------------------------
;;;
(defun ti::y-unload-feature (sym &optional verb)
  "Unload feature SYM, by cleaning `load-history' an undefining symbols. VERB.
This is not the same funcktion as `unload-feature'.

Return:
  t          ,if feature existed _and_ removed.
  nil        ,feature does not exist."
  (interactive
   (list
    (intern-soft
     (completing-read
      "Complete feature to unload: "
      (list-to-assoc-menu (mapcar 'prin1-to-string features))
      nil 'must-match))))

  (let* (l
	 )
    (VERB)
    (when sym
      ;;   First we got to save the vars/funcs
      ;;   Then we can kill load-history
      (setq l  (ti::y-lh-get sym)) ;get (\var func\) list
      (if (null l)
	  nil				;no bound funcs/vars
;;;	(d! "~len" sym (length l)
;;;	    (length (nth 0 l)) (length (nth 1 l))  )
	(ti::y-unload 'feature (list sym))	;feature + load-history clean
	(ti::y-unload 'var     (nth 0 l) )
	(ti::y-unload 'func    (nth 1 l) )
	))
    (if verb (message "Feature now completely unloaded."))
    ))



;;; ----------------------------------------------------------------------
;;;
(defun ti::y-feature-list-unload (list)
  "Remove feature LIST and their vars/funcs.
Input is list of features. Does not check any dependencies between features."
  (while list
;;;    (d!"~batch" (car list))
    (ti::y-unload-feature (car list));feature and it's vars/funcs
    (NEXTP list)			;next feature..
    ))




;;; ----------------------------------------------------------------------
;;;
(defun ti::y-remove-from-hooks (symlist re)
  "Look hook SYMLIST and remove all symbols matching RE.

If hook element is in form of  'lambda' instead of callable function symbol,
this element is ignored. This function cannot remove lambda functions
from hook, because match is done against `symbol-name'."
  (mapcar
   (function
    (lambda (hook)			;one hook at the time
     (if (null (boundp hook))		;is list element variable ?
	 nil				;cannot handle it
       (cond

        ;;  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ add-hook form ^^^
	((listp (eval hook))		;is hook in '(...) form ?
;;;	 (d! "list" hook)
	 (mapcar			;step functions in list
	  (lambda (el)
	    (if (and (not (eq el 'lambda)) ;skip lambda notation
		     (symbolp el)
		     (string-match re (symbol-name el)))
		(remove-hook hook el))
	    )
	  (eval hook)))
        ;;  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ setq form ^^^
	((and (symbolp (eval hook)))
	 (if (string-match re (symbol-name hook))
	     (set hook nil)))		;clear he hook totally
	))))
   symlist))

;;}}}
;;{{{ internal Symbols

;;; ######################################################### &symbols ###

;;; ----------------------------------------------------------------------
;;;
(defun ti::y-get-symbols (re &optional test-form)
  "Return list of symbols that match RE.
The function 'mapatom' will return ALL symbols, no matter if they don't
even exist any more [fboundp, boundp].

You can supply your own TEST-FORM to cause it drop away certain atoms.
the current atom is stored in variable 'sym'.

Eg. test-form = '(or (fboundp sym) (boundp sym))"
  (let* (list
	 (test (or test-form t))
	 )
    (mapatoms
     (function
      (lambda (sym)
	(if (and (eval test)
		 (string-match re (symbol-name sym)))
	    (setq list (cons sym list))))))
    list
    ))





;;; ----------------------------------------------------------------------
;;; - There is another possibility, step through `load-history', but
;;;   since it's not in all emacs and it's buggy (at least in 19.28)
;;;   we don't use it here...
;;;
(defun ti::y-get-file-documentation (file &optional verb)
  "Gather all documentation from symbols in FILE.
You have to load the file into emacs first (eval it), because this
function reads the documentation properties from memory.

Input:
  FILE       absolute file name
  VERB       if non-nil, verbose messages are printed and
             the buffer is displayed when function finishes.

Return:
  buffer     pointer where documentation is stored."
  (interactive
   (let* (file
	  feature
	  )

     (setq file
	   (call-interactively
	    '(lambda (f)
	       (interactive "FDocs from lisp package file: ") f)))

     ;;  We must find the FILE.el name
     ;;
     (or (setq feature (strmatget ".*/\\(.*\\)\\.el$" 1 file))
	 (error "Can't read .el filename. %s " file))


     ;; there must be 'FILE feature
     ;;
     (or (and (intern-soft feature)
	      (setq feature (intern-soft feature)))
	 (y-or-n-p (format "\
No '%s feature found, are you absolutely sure you have loaded the file? "
			   feature
			   ))
	 (error "Abort."))

     (list file)
     ))

  (let* ((tmp-buffer	(get-buffer-create ti:y-tmp-buffer))
	 (file-buffer	(file-pure-find file))
	 (re		(concat "^[(]\\([ \t]*"
				"defsubst\\|defvar\\|defconst"
				"\\|defmacro\\|defun"
				"\\)[ \t]*\\([^ \t\n\f(]+\\)"
				))
	 (func-re       "defsubst\\|defmacro\\|defun")
	 (verb          (or verb (interactive-p)))
	 (count		0)
	 ok-flag
	 doc
	 type
	 sym-name
	 sym

	 )
    (unwind-protect
	(save-excursion

	  (save-excursion		; make preparations
	    (set-buffer tmp-buffer) (erase-buffer))

	  (set-buffer file-buffer)
	  (PMIN)
	  (while (re-search-forward re nil t)

	    (setq type      (read-match 1)
		  sym-name  (read-match 2))

	    (setq sym (intern-soft sym-name))
	    (inc count)


	    ;;  print messages for every 10th only, it's too fast to
	    ;;  show every symbol...
	    (if (and verb
		     (= 0 (% count 10)))
		(message (concat count ": " sym-name)))

            ;; ... ... ... ... ... ... ... ... ... ... ... ... .. func ...
	    (cond
	     ((string-match func-re type)
	      (if (or (null sym)
		      (not (fboundp sym)))
		  (error (concat "Can't find function: " sym-name))

		(setq doc
		      (format
		       "%-40s%s\n%s\n\n"
		       sym-name
		       "Function: "
		       (or (documentation  sym)
			   "not documented")))
		))
             ;; ... ... ... ... ... ... ... ... ... ... ... ... .. var  ..
	     (t
	      (if (or (null sym)
		      (not (boundp sym)))
		  (error (concat "Can't find variable: " sym-name))
		(setq sym (intern-soft sym-name))
		(setq doc
		      (format "%-40s%s\n%s\n\n"
			      sym-name
			      (if (user-variable-p sym)
				  "Option: " "Variable: ")
			      (or (documentation-property
				   sym 'variable-documentation)
				  "not documented"))))
	      ))

	    (append-to-buffer-string tmp-buffer doc)
	    )
	  (setq ok-flag t)		;all completed
	  )
      ;; ... ... ... ... ... ... ... ... ... ... ... ... ... . cleanup . .
      ;; - Recover from Ctrl-g, remove the loaded file.
      ;;
      (kill-buffer file-buffer)
      )
    (if (and verb ok-flag)
	(pop-to-buffer tmp-buffer))	;show contents
    (if verb (message ""))		;clear the echo area

    tmp-buffer
    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::y-describe-symbols-i-args (&optional arg)
  "Ask interactive args for `ti::y-describe-symbols'. ARG is prefix arg."
  (let* ((prompt
	  (cond
	   ((equal arg '(4))
	    "Describe <vars all> matching: ")
	   ((equal arg '(16))
	    "Describe <var options> matching: ")
	   ((equal arg '(64))
	    "Describe <var non-options> matching: ")

	   ((equal arg 0)
	    "Describe <funcs interactive> matching: ")
	   ((equal arg 9)
	    "Describe <funcs non-interactive> matching: ")
	   ((integerp arg)
	    "Describe <funcs all> matching: ")

	   (t
	    "Describe <all> symbols matching: "
	    )))
	 ans
	 )
    (list
     (read-from-minibuffer		;ARG 1
      prompt nil
      nil nil
      'ti:y-describe-symbols-history
      )

     current-prefix-arg			;ARG 2

     ;;  Now handle exclude regexp       ;ARG 3
     (if (var-nil-p (setq ans (read-from-minibuffer "exclude: ")))
	 nil
       ans)

					;ARG 4
     (if (not (listpp arg))
	 (y-or-n-p "Try to find bind info too; may take longer... ? "))

     nil				;ARG 5
     )))



;;; ----------------------------------------------------------------------
;; - This originates from the elisp manual pages somewhere,
;;   but I have made major additions and modifications to it.
;; - Actually this is massive add-on to the original one eg.  it can look
;;   behind aliased functions  (fset, defalias) and has nice
;;   interactive interface.
;;
;; - I suggest that you add this to your .emacs, since
;;   this function is utterly usefull for locating anything.
;;* (autoload 'describe-symbols  "tinyliby" t t)
;;* (if (not (fboundp 'describe-symbols))
;;*     (defalias 'describe-symbols 'ti::y-describe-symbols))
;;
;;
(defun ti::y-describe-symbols
  (pattern &optional mode exclude-re bind-info out-buffer)
  "Describe the Emacs Lisp symbols matching PATTERN.
All symbols that have PATTERN in their name are described.

MODE can be

  nil	     return everything

  list 4     return variables		prefix arg \\[universal-argument]
  list 16    return only options,	prefix arg \\[universal-argument] \\[universal-argument]
  list 64    return only non-options,	prefix arg \\[universal-argument] \\[universal-argument] \\[universal-argument]

  nbr        return only functions
  nbr 0      return only interactive funcs
  nbr 9      return only non-interactive funcs

EXCLUDE-RE

  Excludes matches.

BIND-INFO

   If non-nil, then try to find binding info too. Note: if this flag
   is on, the time function executes decreases dramatically.

OUT-BUFFER

   Where to print the info.

References:

  `ti:y-desc-buffer'"
  ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ interactive ^^^
  (interactive (ti::y-describe-symbols-i-args current-prefix-arg))

  ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ interactive end ^^^
  (let* ((buffer (or out-buffer ti:y-desc-buffer))
	 subrp-test
	 MF MFI MFF			;mode func
	 MV MVO MVV			;mode var
	 sym-list
	 ALIAS
	 FUNC
	 (DEF "")
	 tmp

	 ;;  Build up the function cell into variable 'describe-func'
         (describe-func
          (function
           (lambda (s)			;; <-- symbol IN
             ;; ............................................. function ...
             ;; Print description of symbol.

             (cond
	      ((and MF (fboundp s))
	       (setq ALIAS (defalias-p s))
	       (setq FUNC (or  ALIAS s))

	       (cond			   ;; what is the main class ?
		((and MFI (commandp FUNC)) ;; means interactive
		 (setq DEF "Command: "))

		((and MFF)
		 (setq DEF "Function: ")))

	       (if ALIAS
		   (setq DEF (concat DEF "Alias: " (symbol-name ALIAS))))
	       (if (autoload-p FUNC)
		   (setq DEF (concat DEF " Autoload: ")))


	       (princ
		(format
		 "%-40s %s\n%s%s%s%s\n\n"
		 s
		 DEF

		 (if (setq tmp (function-args-p FUNC))
		     (progn
;;;		       (d! FUNC "ARGS" tmp (symbol-function FUNC))

		       ;; in xe, this doesn't print functions arguments,
		       ;; but the pacakge load information
		       ;; '(from "ange-ftp.elc")', but that's good to
		       ;; know too.
		       ;;
		       (concat tmp "\n")
		       )
		   "<Can't read func arglist>")

                 ;; .................................... function info ...
                 ;;
		 (when (or MF MFI MFF)
		   (concat
		    (cond
		     ((setq subrp-test (subrp (symbol-function s)))
		      "<Built-in-Lisp-primitive>\n")
		     ((string-match "^#" (prin1-to-string (symbol-function s)))
		      ;; didn't work ??
		      ;; (byte-code-function-p s)
		      "<Byte-compiled> ")
		    ((func-car-test s 'macro)
		     "<Macro> ")
		    (t
		     ""
		     ))

		    (if subrp-test
			""
		      (concat
		       "<Package: "
		       (or (car-safe (ti::y-lh-where s))
			   "unknown")
		       ">"))
		     ))


		 (if (and
		      bind-info
		      (and (or MF MFI MFF)
			   (setq tmp (ti::k-function-bind-info s))))
		     (concat "\t" tmp "\n")
		     "\n")

		 (or (condition-case ()
			 (documentation  FUNC)
		       (error "<Function does not exist; not defined>"))
		     "not documented")

		 ))

	       )

	      ;; ............................................. variable ...
	      ((and MV (boundp s))
;;;	       (d! s MV MVO MVV)
	       (cond
		((and MVO (user-variable-p s))		;; option var
		 (princ
		  (format "%-40s %-9s%s\n%s\n\n"
			  s
			  "Option: "
			  (prin1-to-string (eval s))
			  (or (documentation-property
			       s 'variable-documentation)
			      "not documented"))))
		((and MVV )
		 (princ
		  (format "%-40s %-9s%s\n%s\n\n"
			  s
			  "Variable: "
			  (prin1-to-string (eval s))
			  (or (documentation-property
			       s 'variable-documentation)
			      "not documented")
			  )))
		))
	      ))))
	 )

    ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ let end ^^^
    (cond
     ((and (not (null mode))
	   (listp mode))
      (setq MV t MVO t MVV t)
      (cond
       ((equal mode '(16))
	(setq MVV nil ))
       ((equal mode '(64))
	(setq MVO nil ))
       ))

     ((integerp mode)
      (setq MF t MFI t MFF t)
      (cond
       ((= 0 mode)
	(setq MFF nil))
       ((= 9 mode)
	(setq MFI nil))
       ))
     (t
      (setq MV t MVO t MVV t MF t MFI t MFF t))
     )
    ;; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ options end ^^^


    ;; Build a list of symbols that match pattern.
    (mapatoms (function
	       (lambda (sym)
		 (if (and (string-match pattern (symbol-name sym))
			  (or (null exclude-re)
			      (and (stringp exclude-re)
				   (not
				    (string-match exclude-re
						   (symbol-name sym))))))
		     (setq sym-list (cons sym sym-list))))))

    ;; Display the data.
    (if (null sym-list)
	(message "Describe symbols: No matches for given criterias.")
      (with-output-to-temp-buffer buffer
	(mapcar describe-func (sort sym-list 'string<))
	(print-help-return-message)
	))

    ))


;;; ----------------------------------------------------------------------
;;;
(defun ti::y-symbol-summary (re &optional verb)
  "Make elisp script out of variables and functions that match RE. VERB.
Supposes that point is on buffer that is produced by
`ti::y-describe-symbols'

Return:
  buffer	,where is ready output"
  (interactive "sRe: ")
  (let* ((out-buffer	(use-tmp-buffer ti:y-tmp-buffer 'clear))
	 (verb		(or verb (interactive-p)))
	 list
	 words line var
	 vlist flist
	 )
    (setq list
	  (ti::b-grep-lines
	   (concat (or re "")
		   ".*\\(command\\|variable\\|option\\|function\\):")))

    (save-excursion
      (set-buffer out-buffer)
      ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
      (while list
	(setq line	(car list)
	      words	(ti::s-get-blocks line "[ \t]+")
	      var	(nth 0 words)
	      )

	(cond
	 ((string-match "Variable\\|option" line)
	  (push var vlist))

	 ((string-match "Command\\|Function" line)
	  (push var flist))

	 (t
	  (insert (concat "#" line "\n")) ;problem with line ?
	  ))

	(NEXTP list)
	)
      ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
      (lisp-mode)
      (insert "(defconst vlist\n  '(\n")

      (setq vlist (reverse vlist))
      (while vlist
	(insert (concat (car vlist) "\n"))
	(NEXTP vlist))

      (insert ")\n \"Variables\")\n\n")

      (insert "(defconst flist\n  '(\n")
      (setq flist (reverse flist))
      (while flist
	(insert (concat (car flist) "\n"))
	(NEXTP flist))

      (insert ")\n \"Functions\")\n\n")
      (indent-region (point-min) (point-max) nil)

      )

    (if verb (pop-to-buffer out-buffer))
    out-buffer
    ))



;;}}}
;;{{{ examples

;;; ........................................................ &examples ...
;;; - If you're MH, VM user, don't get upset on this example.
;;;   I use RMAIL, because I have hacked it a lot to my taste, but one
;;;   day I accidentally started VM...and my whole system went crazy..what
;;;   a day...
;;; - To prevent accidents I started using this little function. Beware,
;;;   it takes a while to execute, but after it there is no trace of MH,VM
;;;   anywhere in my Emacs.

;;* _
;;* (defun my-vm-mh-kill ()
;;*   "Removes VM, MH permanently"
;;*   (require 'tinyliby)
;;* _
;;*   (let (list)
;;*     (setq list (ti::y-get-symbols "^vm-\\|^vm$"))
;;*     (ti::y-unload-symbols list)
;;* _
;;*     (setq list (ti::y-get-symbols "^mh-\\|^mh$"))
;;*     (ti::y-unload-symbols list)
;;* _
;;*     (setq list (ti::y-get-symbols "hook"))
;;*     (ti::y-remove-from-hooks list "^vm\\|mh")
;;*     ))



;;}}}



;;{{{ provide
;;; ......................................................... &provide ...

(provide    'tinyliby)
(run-hooks  'tinyliby-load-hook)

;;}}}

;;; tinyliby.el ends here
