;; @(#) tinymenu.el -- Simple echo-area menus for less used functions.

;; @(#) $Id: tinymenu.el,v 1.31 1997/06/26 07:30:01 jaalto NotReleased $
;; @(#) $Keywords: tools, menu $
;; $KnownCompatibility: 19.28+ XEmacs 19.14+ $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Feb 1 1996
;;
;; To get information on this program use ident(1) or do M-x timu-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinymenu|Jari Aalto|jari.aalto@poboxes.com|
;; Simple echo-area menus for calling less used functions easily.|
;; 26-Jun-1997|$Revision: 1.31 $|~/misc/tinymenu.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file
;;
;;      (require 'tinymenu)
;;
;;
;;  Or use autoload, which is prefered. Your ~/.emacs loads faster
;;
;;      (autoload 'timu-:menu-default "tinymenu" t t)
;;
;;  To bring up the menu (or menus), bind the main function into some key:
;;
;;      (global-set-key "\C-cM"   'timu-:menu-default)
;;
;;  If you want to access some special menus, you can use this:
;;
;;      (global-set-key "\C-M0"    'timu-:menu-default)  ;; the main menu
;;      (global-set-key "\C-M1"
;;         '(lambda (&optional arg) (interactive "P")
;;                  (timu-:menu my-menu1 arg)))
;;
;;      (global-set-key "\C-M2"
;;         '(lambda (&optional arg) (interactive "P")
;;                  (timu-:menu my-menu2 arg)))
;;
;;
;;  Just make sure you have defined the variables my-menu1 and my-menu2
;;  which hold the menu information.
;;
;;  If you have any questions, use these functions
;;
;;      M-x timu-submit-bug-report

;;}}}
;;{{{ docs

;;; Commentary:

;;  Preface, Feb 1996
;;
;;      Oh Boy, my ~/.emacs.keys.el file started looking like a
;;      mess.. There were hundreds of bindings that were impossible to
;;      remember any more and file size currently shows 74K. Of course it
;;      holds hook settings too, so it's not all key bindings.
;;
;;      Many times I was staring at the buffer and wondering
;;
;;        ..I'm sure I have bound function that deletes blank lines
;;        somewhere..but I can't remember where did I bind it.
;;
;;      Normally I'd reach for the M-x COMMAND to try to remember the
;;      function name, but other times I'd hit C-x b to bring up all
;;      bindings. Then I would start tedious search with C-s to find the
;;      function I was longing for.
;;
;;      I thought that there definitely had to be some way to construct
;;      menu that would use echo area. Instead of remembering that I had
;;      done these bindings
;;
;;          "C-cmf"             , m = maps, f = folding-minor-mode
;;          "C-cmF"             , ... auto-fill minor mode
;;          ...
;;
;;      Whn I have tinymenu, I can call, say F1, to bring up menu which
;;      assists me to find proper function. I don't have to use M-x COMMAND
;;      almost any more, because all seldom used functions are behind this
;;      menu.
;;
;;  Overview of features
;;
;;	o   This package can be used as lisp library; mode's
;;	    keybindings can be put behind echo area menu. (similar to
;;	    menu bar)
;;	o   Regular Emacs user can also less used binding to more memorable
;;	    (guided) echo menu by just defining couple of 'my' variables.
;;
;;  Customizing ready menus
;;
;;      If some pacakge defines good menus and you want to change them a
;;      little and not to copy the whole 'defvar MENU' to your .emacs, you
;;      can use following commands to manipulate menu items
;;
;;          timu-add
;;          timu-set-doc-string
;;
;;      Say, you have following menu
;;
;;          (defconst timu-:menu-sample
;;          '(
;;            "?=help, 1=test1, 2=test2"
;;            ((?1 . (  (timu-test1 1 2 3)))
;;             (?2 . (  (timu-test2 1 2 3)))
;;             )))
;;
;;      and you don't like keybinding '?2'. You first delete the menu item,
;;      then add yours and lastly you update the doc string that is printed
;;	in echo area. Here is how you do all these
;;
;;          (timu-add 'timu-:menu-sample ?2  nil 'delete)
;;          (timu-add 'timu-:menu-sample ?t '( (my-test 1 2 3)))
;;          (timu-set-doc-string
;;              'timu-:menu-sample
;;              "?=help, 1=test1, t=myTest")
;;
;;      And the modified menu looks like this
;;
;;          (defconst timu-:menu-sample
;;          '(
;;            "?=help, 1=test1, t=myTest"
;;            ((?1 . (  (timu-test1 1 2 3)))
;;             (?t . (  (my-test 1 2 3)))
;;             )))
;;
;;      If you want to replace many commands from the menu, it is lot
;;      easier if you copy the defvar and make direct changes there. Here
;;      is one more example to replace 2 choices at time
;;
;;          (mapcar
;;            '(lambda (x)
;;               (let ((key (car x))
;;                     )
;;                 (timu-add 'timu-:menu-sample nil 'delete) ;; Do chnage
;;                 (timu-add 'timu-:menu-sample key (cdr x))
;;                 ))
;;          '((?1 . ( (my-1 1 2 3)))     ;; lost of changes to map over
;;             (?2 . ( (my-2 1 2 3)))
;;             ))
;;          ;; End
;;
;;          (timu-set-doc-string 'timu-:menu-sample "?=help, 1=my1 2=my2")
;;
;;  Having a test run
;;
;;      The easiest way to introduce the menu is that you try it. Follow
;;      these steps. Then you're ready to make your own menus.
;;
;;      .   Load this file. M-x load-file ~/elisp/tinymenu.el
;;      .   Start emnu. M-x timu-:menu-default
;;      .   Press key "?" to get help string.
;;      .   Try other choices
;;

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; May	14	1997	[jari]		19.28	v1.28-1.31	NotReleased
;; - Internal changes due to library functions.
;; - Changed all non standard boolean tests to use when and unless
;; - Docuementation changes. Checkdoc 1.42 ok. timu-add corrected.
;;
;; Mar	19	1997	[jari]		19.28	v1.19-1.27	NotReleased
;; - Renamed all varaibles to use -: convention.
;; - XEmacs20 character handling corrected.
;;
;; Feb	24	1997	[jari]		19.28	v1.17-1.18	NotReleased
;; - If menu was called in dedicated frame that popped up lisp error.
;;   Fixed. Now selects some free frame and returns back after help
;;   is shown.
;; - Added new menu manipulation function: timu-add, timu-set-doc-string
;;   and wrote description to the documentation section.
;;
;; Jan  15      1997    [jari]          19.28   v1.12-1.16      NotReleased
;; - Rearranged loading. Now uses autoloads.
;; - Added lib m require due to macros. Added ESC key to terminate menu
;; - now the pfx-arg is not mandatory any more, but optional.
;;
;; Jun  1       1996    [jari]          19.28   v1.11           NotReleased
;; - Removed duplicate functions, that are already implemented in my
;;   libs. Adjusted lookuout to reflect my standard package template.
;;
;; Feb  18      1996    [jari]          19.28   v1.10           NotReleased
;; - Made some minor adjustments regading the prefix arg display as
;;   Chris suggested. Added better documentation to menu structure var.
;; - Added "flash" effect if pressed key was not a menu item. The solid menu
;;   is little too tired.
;; - Corrected minor bugs in help. Now there is echo area help.
;;
;; Feb  17      1996    [jari]          19.28   v1.9            NotReleased
;; - Corrected few mispellings and added variable timu-:menu which
;;   holds defaults menu structure. Rewrote func doc of timu-:menu.
;; - "Christopher J. Madsen" <ac608@yfn.ysu.edu> sent me valuable
;;   feedback and ready functions. Now the menu can handle prefix
;;   args.
;;
;; Feb  3       1996    [jari]          19.28   v1.8            NotReleased
;; - There was call to read-char-safe, when it should have called
;;   this library's function timu-read-char-safe.
;;   Thanks to Holger Wenzel <hw@rebecca.itm.rwth-aachen.de> who found
;;   the bug.
;; - Added timu-help function, and command '?' which will show
;;   variables's documentation string as help.
;; - Corrected serious bug. The eval-form wasn't cleared before every loop
;;   cycle --> led to multiple calls of previous form.
;;
;; Feb  1       1996    [jari]          19.28   v1.7            Release_1
;; - Okay. Put this to ohio. Small text changes only.
;;
;; Feb  1       1996    [jari]          19.28   v1.6            NotReleased
;; - Still bugs in the FLAG handling. Maybe this time...
;;
;; Feb  1       1996    [jari]          19.28   v1.5            NotReleased
;; - Oops, I didn't realize that I had used English "time" for my
;;   library abbreviation (ti)ny (me)nu --> changed all names to timu
;;   (ti)ny (m)en(u)
;;
;; Feb  1       1996    [jari]          19.28   v1.4            NotReleased
;; - Added new parameter FLAG to the list. Now it's possible return
;;   back to list after calling a function or evaling a form. Nice.
;;
;; Feb  1       1996    [jari]          19.28   v1.3            NotReleased
;; - Some finishing touches.
;;
;; Feb  1       1996    [jari]          19.28   v1.2            NotReleased
;; - Rewrote the character reading routines. Now they are available
;;   in tinylibm.el too.
;;
;; Feb  1       1996    [jari]          19.28   v1.1            NotReleased
;; - Started. Works nicely.

;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'tinylibm)

(eval-when-compile (use-dynamic-compilation))


;;}}}
;;{{{ setup: variables

;;; .......................................................... &v-bind ...

;;; ......................................................... &v-hooks ...

(defvar timu-:load-hook nil
  "*Hook that is run when package has been loaded.")


;;; ....................................................... &v-private ...


(defvar timu-:prefix-arg  nil
  "Prefix arg when menu is called.")


;;;  This is just an example, not a user variable.
;;;  This is how you use the package
;;;  NOTE: put the help into the documentation string. Like
;;;        in variable timu-:menu-mode.
;;;
(defconst timu-:menu-sample
  '(
    "?=help, 1=test1, 2=test2, m=mode, u=undefined , e=eval. q=quit"
    ((?1 . (  (timu-test1 1 2 3)))      ;this does not have FLAG
     (?2 . (t (timu-test2)))            ;FLAG used.
     (?m . timu-:menu-mode)
     (?u . timu-:menu-not-exist)        ;this variable does not exist :-)
     (?e . (t (progn
                (message "menu item evaled. Pfx: '%s' "
                         (prin1-to-string timu-:prefix-arg)
                         )
                (sleep-for 1)
                )))
    ))
  "*This is documentation string of variable `timu-:menu-sample'.
Themenu help is put here.

Reserved menu keys

    Characters 'q' and 'Q' are reserved for quitting the menu prompt,
    and '?' is reserved for help, so they cannot be used for keys.

Menu structure is as follows

    FLAG is optional. It says if the menu should be shown after
    function has completed. If FLAG is missing, the menu is not displayed
    after the function call.

    The DISPLAYED-MENU-STRING is evaled, so it can contain any lisp expression
    yielding a string.

    (defconst my-meny
     '(
      DISPLAYED-MENU-STRING
      ((CHARTER-KEY    . ANOTHER-MENU-VARIABLE-SYMBOL)
       (CHARACTER-KEY  . ([FLAG] (FUNCTION-NAME PARAMETER PARAMETER...)))
       (CHARACTER-KEY  . ([FLAG] (FORM-TO-EVAL)))
       ..
       ))
    \" MENU HELP RESIDES IN THE DOCUMENTATION STRING\"
)")

;;; This is just an example how you could utilize the prefix arguments.
;;; - Example kindly send by "Christopher J. Madsen" <madsen@computek.net>
;;;
;;;(defconst timu-:menu-mail
;;;  '((if current-prefix-arg
;;;        "View mailbox read-only:  E)macs M)ailbox P)erl   R)ead/write"
;;;      "View mailbox:  E)macs M)ailbox P)erl   R)ead-only")
;;;    ((?e . ( (vm-visit-folder "~/mail/emacs" current-prefix-arg)))
;;;     (?m . ( (call-interactively 'vm)))
;;;     (?p . ( (vm-visit-folder "~/mail/perl" current-prefix-arg)))
;;;     (?r . (t(setq current-prefix-arg (if current-prefix-arg nil '(4)))))
;;;     ))
;;;  "Select a mailbox to visit")


;;; This is just an example, not a user variable.
;;;
(defconst timu-:menu-mode
  '(
    "Press ?/ cC l tT f p FO "
    ((?c . ( (c-mode)))
     (?C . ( (cc-mode)))
     (?l . ( (lisp-mode)))
     (?t . ( (text-mode)))
     (?T . ( (indented-text-mode)))
     (?f . ( (fundamental-mode)))
     (?p . (  (picture--mode)))

     (?F . (t (auto-fill-mode)))
     (?O . (t (font-lock-mode)))
     (?/ . timu-:menu-sample)           ;back to ROOT menu
     ))
  "*Mode menu help.
Major modes
  c = turn on `c-mode'
  C = turn on C++ mode
  l = turn on lisp mode
  t = turn on text mode
  T = turn on indented text mode
  f = turn on fundamental mode
  p = turn on picture mode

Minor modes
  F = turn on auto fill mode
  O = turn on f(o)nt lock mode

Special keys
  / = Return to root menu")

;;; ........................................................ &v-config ...

(defvar timu-:menu 'timu-:menu-sample
  "*Variable holding the default root menu.")

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinymenu.el"
 "timu"
 tili-:version-id
 "$Id: tinymenu.el,v 1.31 1997/06/26 07:30:01 jaalto NotReleased $"
 '(timu-:version-id
   )))




;;}}}

;;; ########################################################### &Funcs ###

;;{{{ code: test funcs

;;; ....................................................... test funcs ...

(defun timu-test1 (&optional arg1 arg2 arg3)
  "Sample Menu test function with ARG1 ARG2 ARG3."
  (message (format "function 1 called with args: %s %s %s" arg1 arg2 arg3)))

(defun timu-test2 ()
  "Sample Menu test function."
  (message (format "function 2 called"))
  (sleep-for 1)
  )

;;}}}
;;{{{  menu item add, delete

;;; ------------------------------------------------------------- &add ---
;;;
;;;###autoload
(defun timu-add (menu-symbol ch cell &optional delete)
  "Add or delete (CH . CELL) from menu-symbol.

Example:

  (timu-add 'timu-:menu-sample ?2  nil 'delete)
  (timu-add 'timu-:menu-sample ?t '( (my-test 1 2 3)))

Return:
  nil       no add done due to existing CELL
            no remove due to non-existing CELL
  cell"
  (let* ((menu  (symbol-value menu-symbol))
	 (doc	(nth 0 menu))
	 (list  (nth 1 menu))
         elt
         ret
         )
    (or (characterp ch)
	(error "CH must be character"))
    (setq elt (char-assq ch list))

    (cond
     (delete
      (when elt
	(setq ret elt)
	(setq list (delete elt list))
	(set menu-symbol (list doc list))
	))

     ((and (null delete)
           (not elt))                   ;not already exist?
      (setq ret (cons ch cell))
      (push ret list)
      (set menu-symbol (list doc list))
      ))
    ret
    ))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun timu-set-doc-string (menu-symbol doc-string)
  "Use  MENU-SYMBOL and set its DOC-STRING.

Example

  (timu-set-doc-string 'timu-:menu-sample"?=help, 1=test1, t=myTest")"
  (let* ((menu (symbol-value menu-symbol))
	 )
    ;;  It's better to check that the arg is right; setcar won't
    ;;  do that
    ;;
    (if (not (stringp doc-string))
	(error "timu: need string."))

    (setcar menu doc-string)
    (set menu-symbol menu)
    ))

;;}}}

;;{{{ code: menu

;;; ........................................................ menu func ...


;;; ----------------------------------------------------------------------
;;; - This is only simple help. You can't resize the window etc...
;;;
(defun timu-help (menu-sym)
  "Show menu help of MENU-SYM.
The help commands are:
  n or space   = next
  p or del     = previous
  q            = end help"
  (let* ((msg     "Help: space or n = next, backspace or p = prev, q = quit")
         (oframe  (selected-frame))
         (buffer  "*help*")
         (docs    (documentation-property
                   menu-sym 'variable-documentation))
         ch
         )
    (cond
     ((stringp docs)
      (unwind-protect                   ;make sure the help buffer is deleted
          (save-excursion
            (save-window-excursion
              ;;  We have to sabe the source window confi above
              ;;
              (use-non-dedicated-frame)

              ;; now we may be in another frame; save it's configuration
              ;; too
              ;;
              (save-window-excursion
                (with-output-to-temp-buffer buffer (princ docs))
                (select-window  (get-buffer-window buffer))

                ;;  This is simplest way to resize help window
                (balance-windows)

                (while (not (memq
			     (setq ch (read-char-safe-until msg))
			     '(?q ?\e)))
                  (cond
                   ;;  127  = backspace in windowed
                   ;;
                   ((char-in-list-case ch '(?p ?P ?\177 ?\b))
                    (safe-do (scroll-down 5)))

                   ((char-in-list-case ch '(?n ?N ?\ ))
                    (safe-do (scroll-up 5))
                    )))

		;; save-excursion; after unwind
		)))

	(if (and (not (null oframe))
                 (framep oframe))
            (if (framep (setq oframe (raise-frame oframe)))
                (select-frame oframe)))
        (kill-buffer buffer)
        ))
     (t
      (message "Sorry, no help defined.")
      (sleep-for 1)
      (message "")
      ))

    (discard-input)
    ))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun timu-menu (menu-symbol &optional pfx-arg)
  "The menuing engine.

Input:
  MENU-SYMBOL           variable symbol containing menu items
  PFX-ARG               the prefix arg user ppossibly passed to menu

References:
  `timu-:menu-sample'   Shows how the menu is constructed.
  `timu-:prefix-arg'    Copy of current prefix arg"
  (let* ((var           menu-symbol)
         (m             (eval var))     ;menu content
         (loop          t)
         (current-prefix-arg  pfx-arg)  ;set for menu functions

         prompt flag
         alist
	 ch
	 elt
         eval-form
         )
    (setq timu-:prefix-arg pfx-arg)

    (while loop
      (setq prompt      (eval (nth 0 m))
            alist       (nth 1 m))

      ;;  Safe reading. Eg. moving the mouse confuses read-char

      (setq ch (read-char-safe-until prompt))

      (setq eval-form nil)              ;clear this always
      ;; .................................................. what ch ? ...
      (cond
       ((char-in-list-case ch '(?q ?Q ?\e))	;quit
        (setq loop nil))

       ((char= ch ?? )		;handle help
        (timu-help var))

       ((setq elt (char-assq ch alist))
        (setq elt (cdr elt))

        ;; ................................. new menu or call function ...
        (cond
         ((symbolp elt)
          (if (not (boundp elt))
              (error (format "Menu variable does not exist: %s" elt))
            ;;  replace with another menu
            (setq var elt
                  m   (eval elt))
            ))
         ;; ..................................................... list ...
         ((listp elt)

          ;;  See if there is flag.
          (cond
           ((eq 2 (length elt))
            (setq flag t)
            (setq elt (nth 1 elt)))
           (t
            (setq flag nil)
            (setq elt (nth 0 elt))))

          (cond
           ((fboundp (car elt))         ;is first element function ?
            (setq eval-form elt)
            (setq loop flag))
           (t
            (error "Menu structure error %s" elt)
            ))
	  )

	 ))
       (t
        ;;  ch not found from list, keep looping
        (sleep-for-count 2000)          ;flash the echo area
        ))

      (message "")                      ;clear echo area

      (if (not (null eval-form))
          (eval eval-form))
      )))



;;; ----------------------------------------------------------------------
;;; - This is user function
;;;
(defun timu-menu-default (&optional arg)
   "Call echo area menu with prefix ARG.
Plese read the documentation of variable `timu-:menu-sample' to see
the structure of menu.

Menu pointed by `timu-:menu' is used and PREFIX-ARG is passed to menu engine
'timu-:menu'.

References:
  `timu-:menu-sample'   variable. Shows how the menu is constructed."
   (interactive "P")
   (timu-menu timu-:menu arg))



;;}}}

(provide   'tinymenu)
(run-hooks 'timu-:load-hook)

;;; tinymenu.el ends here
