;; @(#) tinyrmail.el -- RMAIL additions, pgp, mime labels

;; @(#) $Id: tinyrmail.el,v 1.37 1997/06/27 18:14:23 jaalto NotReleased $
;; @(#) $Keywords: mail $
;; $KnownCompatibility: 19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of GNU emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Jun 4 1996
;;
;; To get information on this program use ident(1) or do M-x tirm-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyrmail|Jari Aalto|jari.aalto@poboxes.com|
;; RMAIL add-ins, eg. label summary AND, detect incoming PGP, MIME mail|
;; 27-Jun-1997|$Revision: 1.37 $|~/misc/tinyrmail.el.Z|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install


;; ....................................................... &t-install ...
;;  Put this file on your Emacs-Lisp load path, add following into your
;;  ~/.emacs startup file. Rip code with with tinylib.el/ti::m-pkg-rip-magic
;;
;;* _
;;*     (require 'tinyrmail)
;;
;;  or prefer this; your .emacs loads up much quicker
;;
;;* _
;;* (autoload 'tirm-rmail-summary-by-labels-and "tinyrmail" t t)
;;* (autoload 'tirm-install			"tinyrmail" t t)
;;* (add-hook 'rmail-mode-hook			'tirm-install)
;;
;;  You can also install the attached PGP signed file.
;;  Before you do this, you need commands 'pgp' and 'tar' along
;;  Emacs exec-path
;;
;;      M-x tirm-install-files
;;
;;  After you have unpacked the anti-spam text file, set these variables
;;
;;	(setq tirm-:anti-spam-no-site-regexp "YOUR-DOMAIN-REGEXP")
;;	(setq tirm-:anti-spam-file           "~/txt/spam.txt")
;;
;;  If you have any questions, use this function
;;
;;      M-x tirm-submit-bug-report       ,send bug report

;;}}}

;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tirm-" in front of
;;   every function & variable. It stands for '(ti)ny
;; - variable names contain letters 'tirm-:'

;;  Preface, overview of features
;;
;;	o   Detect PGP, MIME mail and label incoming messages accordingly.
;;	    User can add more checking functions and labels to incoming email
;;	    messages
;;	o   New label summary cmd with AND eg. finding {pgp,v} verified pgp
;;	o   Flag incoming mail as deleted by regexp.
;;	o   "S" command for Spam message reply.
;;	o   Commands to fix your RMAIL messages.
;;	o   advice: "n" and "p" do not to auto display msg in Summary buffer
;;	o   advice: mouse click in Summary does not automatically update msg
;;	o   advice: `rmail-ignored-headers' now reformats old messages too.
;;
;;  Description
;;
;;	This little package offers some autmatic detection of PGP
;;	MIME  mails: It attaches labels to your incoming mails.
;;	There is also new summary function, which enables you to
;;	make a query by ANDing the labels in your RMAIL.
;;
;;	This means, that you can now classify your message, like this:
;;
;;    	  BASE
;;    	  SUBSET-IDENTIFIER
;;                MINOR-IDENTIFIER
;;                   NOTE-IDENTIFIER
;;
;;       Eg. For PGP mails I have
;;
;;          {pgp}
;;          {pgp,v}         -- verified signature
;;          {pgp,u}         -- not verified
;;          {pgp,v,e}       -- verified and encrypted
;;
;;	The normail rmail's summary function gives you the OR summary, which
;;	would mean, that if you wanted symmary by {pgp,v}, it would give
;;	you all mail that has either {v} or {pgp} somewhere. Well, this
;;	summary is not suitable if you use one CHAR to denote attributes
;;	of your base-identifiers (multichar)
;;
;;  Automatic deletion of incoming mail
;;
;;	There is default function to mark messages as deleted according
;;	to regexp. Please configure this variable to suit your needs:
;;
;;	    tirm-:delete-regexp
;;
;;	If you want more personal control whether the mail
;;	should be deleted or not, please remove the default delete function
;;	and add your own:
;;
;;	    (add-hook 'tirm-:load-hook 'my-tirm-:load-hook)
;;
;;	    (defun my-tirm-:load-hook ()
;;	     "Cancel some default settings and modify parameters."
;;	    (remove-hook 'tirm-:get-new-mail-hook 'tirm-delete-function)
;;	    (add-hook    'tirm-:get-new-mail-hook 'my-rmail-delete-function)
;;	    )
;;
;;
;;	    (defun my-rmail-delete-function ()
;;	     ...)
;;
;;  Replying to spam messages
;;
;;	The spamming has become a very serious promlem in private
;;	email communication. Not only the spammers send mail to newsgroups,
;;	but they are harrashing private mailboxes as well.
;;
;;	With this package you can send reply to a spammer and complaint
;;	to all postmasters that have allowed the spam mail to travel through
;;	their networks.
;;
;;  New commands in RMAIL
;;
;;	Refer to function tirm-define-default-keys for exact setup.
;;	Currently the only new command added is
;;
;;	    "L"	tirm-rmail-summary-by-labels-and
;;	    "S" tirm-spam-reply
;;
;;  Fixing RMAIL format
;;
;;	Sometimes you may get following error after you have hit "g"
;;	to get new mail: "Cannot convert to babyl". The reason for
;;	this behavior is still not quite clear to me, but the cause
;;	is in the incoming message that does not have
;;
;;	    From
;;
;;	Field at the beginning of message. I have seen even some garbage
;;	Prepended to field so that it looked like
;;
;;	    m?From
;;
;;	What have to start editing the RMAIL file directly to fix its
;;	format. Change the mode to text-mode, run M-x widen and search the
;;	last message that rmail was not able to read. You will easily find the
;;	point where "**** EOOH" markers do not appear any more.
;;
;;	Now starts the fixing part to make rmail happy again:
;;
;;	o   Make sure From line is left flushed. Edit if needed and put
;;	    lines in their right places.
;;	o   Select all individual message's headers at a time.
;;	o   Call function tirm-fix-make-rmail-message-header
;;	    which you should propably bound to some convenient key.
;;	    The ESC-z combination is propably free for temporary use.
;;	    (local-set-key "\ez" 'tirm-fix-make-rmail-message-header)
;;
;;	After you have converted all headers to rmail format, you can
;;	start rmail again with command
;;
;;	    M-x rmail-mode
;;
;;	If you made any mistakes, rmail will let you know and you have to
;;	repeat the header fixing again. (possibly removing the prevous
;;	EOOOH markers and reconverting them). We aren't quite finished
;;	yet. You see, on error, rmail leaves the read mail into your home
;;	directory. Please check that
;;
;;	    ~/.newmail-USERNAME
;;
;;	file doesn't contain any new message that aren't already in your RMAIL
;;	buffer. If there is only old message, delete that file. Now we
;;	have finished and you can again use "g" to get new mail.
;;
;;  Standard Rmail distribution changes
;;
;;	This package changes the standard Rmail distribution sligtly and here
;;	summary. If you want to disable these features or only use some of
;;	them, you have to put separate configuration to your .emacs.
;;	To disable forms:
;;
;;	    (setq tirm-:load-hook '(tirm-install))
;;
;;	To disable advices, you do
;;
;;	    (setq tirm-:load-hook '(tirm-install my-tirm-install))
;;
;;	    (defun my-tirm-install ()
;;	      (ti::m-ad-control
;;		'(rmail-show-message
;;		  rmail-summary-enable
;;		  rmail-summary-next-msg
;;		  )
;;		 "^tirm"
;;		 'disable
;;		 ))
;;
;;
;;	`tirm-:forms-rmail'
;;
;;	o   Every time RMAIL package is loaded these forms are executed.
;;	o   These define some keybindings to summary buffer
;;	    that I have found appropriate. Mouse-2 selects message
;;	    (and does not yank as the original). RET key also selects message.
;;	o   The post command hook is cleared so that you can search regexp
;;	    in summary buffer. Normally moving a cursor would move the
;;	    current message too.
;;	o   The "q" quit key is too easily pressed and I have removed it
;;	    alltogether. If I really want to quit RMAIL, I usually
;;	    quit Emacs too.
;;
;;	Advices:
;;
;;	*rmail-show-message* active
;;
;;	The message's headers are now always reformatted. If you change
;;	variable `rmail-ignored-headers', the old messages are not affected
;;	until you "t"oggle headers. This advice does it for you
;;	automatically every time you select message. This advice slows
;;	message displaying a bit, but for me, it isn't very noticeable.
;;	You can very well turn this off if you dont' change content of
;;	`rmail-ignored-headers'.
;;
;;	*rmail-summary-enable* active
;;
;;	This replaces whole function. The original function did automatic
;;	message update whenever you moved around summary buffer. Now you
;;	can keep summary buffer search separated from the current
;;	message displayed.
;;
;;	*rmail-summary-next-msg* active
;;
;;	Same as above.
;;

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun	27	1997	[jari]		19.28	v1.31-1.37	NotReleased
;; - checkdoc 1.42 remarks corrected. Defcustom support. lib 'm' changes.
;; - Added advice code from tinyad.el.
;;
;; May	14	1997	[jari]		19.28	v1.30		NotReleased
;; - Internal changes.
;; - Changed all non standard boolean tests to use when and unless
;;
;; Mar	27	1997	[jari]		19.28	v1.28-1.29	NotReleased
;; - Wrote 'Misc commands, fixing RMAIL format' and added rmail header
;;   fixing command. Supports compressed anti-spam file.
;;
;; Mar	25	1997	[jari]		19.28	v1.24-1.27	NotReleased
;; - Moved one function into lib 'mt'.
;; - Added spam reply command "S" and PGP tar block to the end of file.
;;
;; Feb	28	1997	[jari]		19.28	v1.23		NotReleased
;; - Renamed all variables to use -: convention. tirm-:label-table
;;   now accepts lisp form and not just function.
;;
;; Jan	30	1997	[jari]		19.28	v1.21-1.22	NotReleased
;; - text changes
;;
;; Sep	25	1996	[jari]		19.28	v1.19-1.20	NotReleased
;; - Textual chnages to documentation. Rearranged loading, uses more
;;   autoloads.
;;
;; Sep	22	1996	[jari]		19.28	v1.18		NotReleased
;; - Added automatic message deletion function to filter incoming mail.
;;   Wrote section 'Automatic deletion of incoming mail'.
;;   delete-regexp			:+ user var
;;   tirm-delete-function	:+
;;
;; Sep	19	1996	[jari]		19.28	v1.17		NotReleased
;; - Rewrote tirm-rmail-get-new-mail-function. Now it attaches labels
;;   from new user variable tirm-:label-table. Removed PGP hooks.
;; - Now detects MIME mail too.
;;   tirm-rmail-get-new-mail-function	:!
;;   tirm-:label-table			:+
;;   tirm-:get-new-mail-hook		:+  use this general hook now
;;   tirm-get-new-mail-pgp-hook		:-
;;   tirm-get-new-mail-no-pgp-hook	:-
;;
;; Sep	9	1996	[jari]		19.28	v1.16		NotReleased
;; - Some minor changes to installation and to documentation. It seems
;;   that I haven't had a need to touch the basic setup for a long time.
;;
;; Jun	28	1996	[jari]		19.28	v1.14-1.15	NotReleased
;; - Intermediate release. The tinypgp.el is not required in this
;;   release. I may decide to put some pgp function into lib 'mt'.
;; - Removed Maicrypt.el dependency.
;;
;; Jun	12	1996	[jari]		19.28	v1.13		NotReleased
;; - Corrected PGP mail detection. It gave false hits for citated PGP.
;;
;; Jun	9	1996	[jari]		19.28	v1.12		NotReleased
;; - Now it handles M-x RMAIL command and can find out what is the
;;   first message, if there is any.
;;   tirm-rmail-new-message-ptr		:!
;;   tirm-rmail-get-new-mail-function	:!
;;
;; Jun	9	1996	[jari]		19.28	v1.11		NotReleased
;; - Corrected intallation instructions. Now runs hooks when
;;   new RMAIL messages are examined.
;; - Known problem: When calling RMAIL for the first time, the PGP
;;   messages are not detected. I'm working on that..
;;   tirm-get-new-mail-pgp-hook		:+
;;   tirm-get-new-mail-no-pgp-hook	:+
;;   tirm-install-hooks			:! corrected
;;   tirm-install-advices		:! corrected.
;;
;; Jun	6	1996	[jari]		19.28	v1.9-1.10	NotReleased
;; - Separated the PGP functions to separate package: tinypgp.el and
;;   left RMAIL specific tasks here.
;;
;; Jun	5	1996	[jari]		19.28	v1.8		NotReleased
;; - Found a serious bug in MC, the PGP string in there did not have
;;   start anchor ^ and this wrecked the MC for posts that had embedded
;;   --BEGIN PGP messages. This was also problem in this package.
;; - Currently MC's variables are fixed here in separate function,
;;   later it should be removed.
;; - Added sections 'How the conversion goes' and
;;   'Hiding the PGP signature automatically'
;;   tirm-mc-fix-vars			:+
;;   tirm-cnv				:+
;;   tirm-re				:+
;;
;; Jun	5	1996	[jari]		19.28	v1.7		NotReleased
;; - Commented the debug calls.
;;
;; Jun	5	1996	[jari]		19.28	v1.6		NotReleased
;; - A minor mistake. The MC seems to use mc-gnus-verify-signature
;;   which changes buffer to *Article*. Can't let that happen because
;;   the X-headers are stripped in separate buffer, where verification
;;   should also happend
;; - Added defadvice to take control to this package.
;;
;; Jun	5	1996	[jari]		19.28	v1.5		NotReleased
;; - Added tirm-pgp-allowed-p function to check if we can use this
;;   packages PGP function in buffer. They are mostly valid only
;;   in Mail and News.
;; - Corrected the PGP restore C-c / /, it didn't quite
;;   restore the original contants.
;;
;; Jun	5	1996	[jari]		19.28	v1.4		NotReleased
;; - Changed the commands. now it can install to MC minor modes
;;   directly. Added function that removes siging from message:
;;   You can correct the message.
;; - Added dynamic compilation and swept away all compiler errors.
;;
;; Jun	4	1996	[jari]		19.28	v1.4		NotReleased
;; - Now tested interaction with mailcrypt 3.5beta, the headers
;;   work just great!
;;
;; Jun	4	1996	[jari]		19.28	v1.3		NotReleased
;; - Lots of changes, too many to list. Started tests with MC.
;;
;; Jun	4	1996	[jari]		19.28	v1.2		NotReleased
;; - Complete package. The pgp part is not integrated to mailcrypt.el
;;   yet. Receiving pgp messages and labelling them automatically works.
;; - Converting to/from pgp headers works fine in normal
;;   buffer. Haven't fully tested in GNUS or in RMAIL.
;;
;; Jun	4	1996	[jari]		19.28	v1.1		Created

;;}}}

;;; Code:

;;{{{ setup: variables

;;; ......................................................... &require ...

(require  'rmail)			;Uses macros from there
(require  'tinylibm)
(require  'tinylibmt)


(eval-and-compile
  (autoload 'rmail-new-summary			"rmailsum")
  )

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinyRmail tirm-: mail
  "Additional features to RMAIL.
Overview of features

	o   Detect PGP, MIME mail and label incoming messages accordingly.
	    User can add more checking functions and labels to incoming email
	    messages
	o   New label summary cmd with AND, eg. {pgp,v} for verified pgp mails.
"
  )

;;; .......................................................... &v-bind ...

;;; ....................................................... &v-private ...

(defvar tirm-:rmail-info-list  nil
  "Values of saved message counters before we get new mail.")


(defconst tirm-:forms-rmail
  '(progn
     (when (boundp 'rmail-summary-mode-map)
       ;;  mouse-2 is paste, move it to select a buffer.
       ;;  See the rmail advices.
       (define-key rmail-summary-mode-map [down-mouse-2]
	 'rmail-summary-goto-msg)

       (define-key rmail-summary-mode-map [mouse-2]
	 'rmail-summary-goto-msg)

       ;;  Enter selects a message too
       (define-key rmail-summary-mode-map "\C-m" 'rmail-summary-goto-msg)

       ;; rmailsum.el makes this buffer local, loop all rmail summary
       ;; buffers and remove function from post-command-hook.
       ;;
       (ti::b-buffer-list
	'(eq major-mode 'rmail-summary-mode)
	(not 'temp-buffers)
	(not 'exclude)
	'(progn
	   (remove-hook 'post-command-hook 'rmail-summary-rmail-update)
	   ))

       ;; disable "quit", it's too risky. I want to be in RMAIL,
       ;; and only there hit the "q" key.
       ;;

       (define-key rmail-summary-mode-map "q" 'ignore)
       )

     (when (boundp 'rmail-mode-map)
       (define-key rmail-mode-map "q"
	 '(lambda ()
	    "Confirm quit."
	    (interactive)
	    (if (y-or-n-p "Really quit RMAIL ")
		(rmail-quit)
		))))
     )
  "Additional forms to `after-load-alist'.
Set this variable to '(progn) if you want to disable these features.")



;;; ......................................................... &v-hooks ...

(defcustom tirm-:load-hook '(tirm-install tirm-install-forms)
  "*Hook that is run when package is loaded."
  :type 'hook :group 'TinyRmail
  )

(defcustom tirm-:rmail-get-new-mail-before-hook nil
  "*Additional hook added by advice in package tinyrmail.el.
Hook run just before new mail is fetched.
Contain default function `tirm-rmail-get-new-mail-before-function',
which saves the Rmail message pointers before getting new mail."
  :type 'hook :group 'TinyRmail
  )


(defcustom tirm-:get-new-mail-hook  nil
  "*Hook run inside each _new_ mail message.
The default function `tirm-delete-function' reads variable
`tirm-:delete-regexp' and marks buffer as deleted if the regexp
matches message contents."
  :type 'hook :group 'TinyRmail
  )


;;; ........................................................ &v-public ...
;;; User configurable


(defcustom tirm-:delete-regexp
  (concat
   "make.*money"
   "\\|this is your chance.*money"
   )
  "*Mark messge deleted if this regexp match.
If this regexp is nil, no mail is marked as deleted.
This variable is efective only if `tirm-delete-function' is
installed into `tirm-:get-new-mail-hook'."
  :type '(string :tag "Regexp") :group 'TinyRmail
  )


(defcustom tirm-:anti-spam-no-site-regexp  nil
  "*Regexp to drop out site names in the Received header list.

You set this variable to match your local domain names, so that the
spam complaints to CC postmasters doesn't go there."
  :type '(string :tag "Regexp") :group 'TinyRmail
  )


(defcustom tirm-:label-table
  '((ti::mt-pgp-p   "pgp")
    (ti::mt-mime-p  "mime")
    )
  "*Labels to attach to new RMAIL messages.
Format is

  '((CHECK-FUNCTION STRING-OR-SYMBOL) (F S) ..).

The STRING-OR-SYMBOL may be either \"string\" or variable name
'lisp-var, where its `symbol-value' is used.

The CHECK-FUNCTION is run without arguments inside every new
message and it should return. This can also be a lisp form if
the elt is not function symbol.

  nil         ,if no action should be taken
  t           ,if the STRING-OR-SYMBOL should be used for labelling
  string      ,that string is used for labelling."
  :type '(repeat
	  (list
	   (function :tag "Check function")
	   (choice
	    :inline t
	    (string :tag "String Label")
	    (symbol :tag "Var Symbol")
	    )))
  :group 'TinyRmail
  )



(defcustom tirm-:anti-spam-file  nil
  "`tirm-spam-reply' prepend this text before inserting original message.
It should contain educational text to teach about proper email
etiquette and explain why spamming is very intrucive way to
reach possible customers.

The default text for this file can be found from tinyrmail.el

Note: compressed file support:

  If the filename in this varaible is FILE.txt, then we automatically
  serch for extension .txt.gz and .txt.Z also if no plain .txt
  file is found."
  :type 'file :group 'TinyRmail
  )



;;}}}
;;{{{ version

;;; ......................................................... &version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinyrmail.el"
 "tirm"
 tirm-:version-id
 "$Id: tinyrmail.el,v 1.37 1997/06/27 18:14:23 jaalto NotReleased $"
 '(tirm-:version-id
   )))

;;}}}

;;; ########################################################### &Funcs ###

;;{{{ Installation

;;; ----------------------------------------------------------------------
;;;
;;;###autoload (autoload 'tirm-install-files "tinyrmail" t t)
(ti::m-fmacro-install-pgp-tar tirm-install-files "tinyrmail.el")



;;; ----------------------------------------------------------------------
;;;
(defun tirm-install-forms  ()
  "Some other things to do to get all installed.
See source code for
better explanation."
  (interactive)

  (when (boundp 'rmail-summary-mode-map)
    (eval tirm-:forms-rmail))		;run it immediately

   (cond
    ((not (fboundp 'eval-after-load))
     (load "rmailsum")
     (load "rmail")
     (eval tirm-:forms-rmail)
     )
    ((fboundp 'eval-after-load)
     ;;  Quiet XEmacs 19.14 compiler who says this function doesn't exist
     ;;
     (sfuncall 'eval-after-load "rmailsum"  tirm-:forms-rmail)
     (sfuncall 'eval-after-load "rmail"     tirm-:forms-rmail)
     )))


;;; ----------------------------------------------------------------------
;;; - If more commnds are added, I make this a separate minor mode...
;;;
;;;
(defun tirm-define-default-keys  ()
  "Define keys to various maps."
  (interactive)
  (let* (
	 )
    ;; Making summaries by ANDING labels.
    ;;
    (when (boundp 'rmail-mode-map)
      (define-key rmail-mode-map "L" 'tirm-rmail-summary-by-labels-and)
      (define-key rmail-mode-map "S" 'tirm-spam-reply)
      )

    ;;  This is not loaded, that's why symbol-value to shut up byte
    ;;  compiler.
    (when (boundp 'rmail-summary-mode-map)
      (define-key
	(symbol-value 'rmail-summary-mode-map)
	"L" 'tirm-rmail-summary-by-labels-and)
      )

  ))

;;; ----------------------------------------------------------------------
;;;
(defun tirm-install-advices  (&optional remove verb)
  "Install advices. Optionally REMOVE advices. VERB."
  (interactive "P")
  (ti::m-ad-control
   '(rmail-get-new-mail
     )
   "^tirm-" remove
   (or verb (interactive-p))
   ))



;;; ----------------------------------------------------------------------
;;;
(defun tirm-install (&optional remove)
  "Install package hooks. Optionally REMOVE installation.
Can't restore changes to keymaps."
  (interactive "P")
  (let* ((f (if remove 'remove-hook 'add-hook))
	 )
    ;; Set up RMAIL for PGP
    ;;
    (funcall f 'rmail-get-new-mail-hook 'tirm-rmail-get-new-mail-function)
    (funcall f 'tirm-:rmail-get-new-mail-before-hook
	       'tirm-rmail-get-new-mail-before-function)

    ;; New commands
    ;;
    (funcall f 'rmail-mode-hook		'tirm-define-default-keys)
    (funcall f 'rmail-summary-mode-hook 'tirm-define-default-keys)
    (funcall f 'gnus-article-mode-hook  'tirm-define-default-keys)
    (tirm-define-default-keys)		;Install immediately too

    (tirm-install-advices remove)
    ))



;;}}}
;;{{{ rmail, labels

;;; .................................................... &rmail-labels ...


;;; ----------------------------------------------------------------------
;;; see rmailsum.el
;;;
;;;###autoload
(defun tirm-rmail-summary-by-labels-and (labels)
  "Display a summary of all messages with one or more LABELS.
LABELS should be a string containing the desired labels, separated by commas.
This summary is prduced by _ANDING_ the labels."
  (interactive "s(AND) Labels to summarize by: ")
  (if (string= labels "")
      (setq labels (or rmail-last-multi-labels
                       (error "No label specified"))))
  (setq rmail-last-multi-labels labels)
  (rmail-new-summary (concat "labels " labels)
                     (list 'rmail-summary-by-labels labels)
                     'tirm-rmail-message-labels-and-p
		     ;; convert to list of label string
		     ;;
		     (ti::s-get-blocks labels "[ ,]+")
		     ))

;;; ----------------------------------------------------------------------
;;;
(defun tirm-rmail-message-labels-and-p (msg labels)
  "Check and condition in MSG nbr with LABELS LIST."
  (let* ((copy	labels)	;since labels list vanishes in loop
	 (i	0)
	 elt
	 )
    (save-excursion
      (save-restriction
	(widen)
	(goto-char (rmail-msgbeg msg))
	(forward-char 3)

	(list-loop
	    labels elt
	  ;; May look like this:
	  ;;
	  ;;    1,, pgp, v,
	  ;;
	  (if (looking-at (concat ".* " elt ","))
	      (inc i))
	  )))
    ;;  Must have as many hits as labels passed to function
    (eq (length copy) i)
    ))


;;}}}
;;{{{ rmail, new message


;;; .................................................... &new-messages ...


;;; ----------------------------------------------------------------------
;;;
(defun tirm-delete-function  ()
  "Mark messages as deleted if it find regexp `tirm-:delete-regexp'.
This function is in `tirm-:get-new-mail-hook'."
  (PMIN)
  (if (and (stringp tirm-:delete-regexp)
	   (re-search-forward tirm-:delete-regexp nil t))
      (rmail-delete-message)))


;;; ----------------------------------------------------------------------
;;;
(defsubst tirm-rmail-new-message-ptr ()
  "Return first new message NBR.
Function must be called only after the \"g\" key, in `rmail-get-new-mail'."
  (if (and tirm-:rmail-info-list
	   (not (eq (car tirm-:rmail-info-list)
		    rmail-total-messages))
	   (integerp (car tirm-:rmail-info-list))
	   )
      (1+ (car tirm-:rmail-info-list))
    ;;  Whan you first hit M-x RMAIL, this tells you the first message
    (rmail-first-unseen-message)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tirm-rmail-get-new-mail-function  ()
  "Loop over every incoming mail message and do labelling."
  (let* ((table                 tirm-:label-table)
	 (rmail-current-message rmail-current-message)
	 nbr
	 list elt func label stat
	 )
    ;; Is there new mail, maybe some of them are not read yet?
    ;;
    (setq nbr (tirm-rmail-new-message-ptr))

    (when nbr
      (while (< nbr (1+ rmail-total-messages))
	(ti::mt-rmail-do-message-macro nbr nil
	  (setq rmail-current-message nbr)

	  (setq list table)
	  (list-loop list elt
            ;; .......................................... handle table ...
	    (setq func  (nth 0 elt)
		  label (nth 1 elt)
		  stat  (if (symbolp func)
			    (funcall func)
			  (eval func))
		  )

            ;; .............................................. evaluate ...
	    (if (symbolp label)
		(setq label (symbol-value label)))

            ;; ............................................. add label ...
	    (cond
	     ((stringp stat)
	      (rmail-add-label stat))

	     ((and stat (stringp label))
	      (rmail-add-label label))

	     ((and stat (not (stringp label)))
	      (error "Label is not a string %s %s" label table)
	      ))
	    )
	  (run-hooks 'tirm-:get-new-mail-hook)
	  )
	(inc nbr))

      )))


;;; ----------------------------------------------------------------------
;;;
(defun tirm-rmail-get-new-mail-before-function  ()
  "Reset some values before getting mail."
  (setq
   tirm-:rmail-info-list
   (list
    rmail-total-messages
    rmail-current-message
    rmail-message-vector
    rmail-deleted-vector
    rmail-summary-vector
    )))


;;; ----------------------------------------------------------------------
;;;
(defadvice rmail-get-new-mail  (before tirm-hook act)
  "Run hook 'tirm-:rmail-get-new-mail-before-hook'."
  (run-hooks 'tirm-:rmail-get-new-mail-before-hook))


;;}}}
;;{{{ Spam reply

;;; ............................................................ &spam ...

;;; ----------------------------------------------------------------------
;;;
(defun tirm-spam-reply ()
  "Reply to spam message by sending 'remove' request and complaint.

o  Send 'remove' request to the From address.
o  Copy the current message with original headers
o  Parse all received sites from the message and CC
   appropriate postmasters
o  Insert custom anti-spam text from file.

References:
  `tirm-:anti-spam-file'
  `tirm-:anti-spam-no-site-regexp'"
  (interactive)
  (let* ((orig-msg  (ti::mt-rmail-copy-message))
	 (file      tirm-:anti-spam-file)
	 (re        tirm-:anti-spam-no-site-regexp)
	 mail-archive-file-name
	 from
	 subj
	 list
	 elt
         )

    ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. checkings . .
    (if (null tirm-:anti-spam-no-site-regexp)
	(error "Please set tirm-:anti-spam-no-site-regexp."))

    (cond
     ((not (stringp file))
      (error "Please set tirm-:anti-spam-file."))

     ((file-exists-p (setq elt (concat file ".gz")))
      (require 'jka-compr) (setq file elt))

     ((file-exists-p (setq elt (concat file ".Z")))
      (require 'jka-compr) (setq file elt))

     ((not (file-exists-p file))
      (error "tirm-:anti-spam-file does not exist %s"
	     tirm-:anti-spam-file)
      ))

    ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. . get travel sites . .
    (ti::mt-rmail-do-message-macro rmail-current-message 'orig
      (setq list (ti::mt-parse-received re)))

    (if (null list)			;Didn't find any!!
	(error "Your were not in RMAIL message buffer."))

    (setq from (ti::mt-get-field-1 "from")
	  subj (ti::mt-get-field-1 "subject")
	  )

    (mail)

    ;;  We add two spaces, so that possible TinyMail turns off.
    ;;
    (ti::mt-kill-field "To" (concat "  " from))
    (ti::mt-kill-field "Subject" (format " ABUSE (was: %s)" subj))

    ;;  Add "postmaster" fields
    ;;
    (list-loop list elt
      (ti::mt-add-to-field-string
       "CC" (concat "postmaster@" elt)))

    (ti::mt-kill-field "fcc")
    (ti::mt-text-start 'move)

    (insert-file-contents file)
    (PMAX)
    (insert "\n" orig-msg)
    (ti::mt-text-start 'move)

    (if (y-or-n-p (format "Send REMOVE request to %s?" from))
	(ti::mt-sendmail-macro from "remove" 'send))


    (message "TIRM Anti-spam: When you're ready, send this mail.")
    ))

;;}}}
;;{{{ Advice

;;; .......................................................... &advice ...


;;; ----------------------------------------------------------------------
;;; (ad-unadvise 'rmail-show-message)
;;;
(defadvice rmail-show-message  (before tirm act)
  "Reformat message.
If you change the `rmail-ignored-headers' it won't affect the current
messages unless you hit 't' to toggle headers. This advice reformats
message every time the message is shown."
  ;;   We do}t want expunge to call use, only direct
  ;;   show message command.
  (when (interactive-p)
    (safe-widen
      (rmail-maybe-set-message-counters)
      (narrow-to-region (rmail-msgbeg (ad-get-arg 0)) (point-max))
      (let ((buffer-read-only nil))
	(goto-char (point-min))
	(forward-line 1)
	;;  Convert 1 --> 0, otherwise format command barfs.
	;;
	(delete-char 1) (insert "0")

	(forward-line 1)
	(let ((case-fold-search t))
	  (while (looking-at "Summary-Line:\\|Mail-From:")
	    (forward-line 1)))
	(insert "*** EOOH ***\n")
	(forward-char -1)
	(search-forward "\n*** EOOH ***\n")
	(forward-line -1)
	(let ((temp (point)))
	  (and (search-forward "\n\n" nil t)
	       (delete-region temp (point))))
	(goto-char (point-min))
	(search-forward "\n*** EOOH ***\n")

	(rmail-reformat-message (point-min) (point-max))
	))))

;;; ----------------------------------------------------------------------
;;; - Copy from rmailsum.el
;;; - This would normally cause automatic update by mouse click, disable it
;;; - I want to select message with RETURN or mouse-2. This way I can
;;;   move around the buffer and leave the message in RMAIL untouched.
;;;
(defadvice rmail-summary-enable (around tirm act)
  "Replace function.
Disable automatic update when mouse - 1 is pressed or cursor is moved.
You can browse the summary buffer more freely and keep the
selected message in RMAIL."
  (use-local-map rmail-summary-mode-map)
  ;; (add-hook 'post-command-hook 'rmail-summary-rmail-update)
  (setq revert-buffer-function 'rmail-update-summary)
  )



;;; ----------------------------------------------------------------------
;;; - Copy from rmailsum.el.
;;; - I hate when I can't browse forward without getting
;;;   the Summary. Grr...
;;;
(defadvice rmail-summary-next-msg (around tirm act)
  "Replace function. Disbale automatic showing of summary buffer."
  (forward-line 0)
  (and (> number 0) (end-of-line))
  (let ((count (if (< number 0) (- number) number))
	(search (if (> number 0) 're-search-forward 're-search-backward))
	(non-del-msg-found nil))
    (while (and (> count 0) (setq non-del-msg-found
				  (or (funcall search "^....[^D]" nil t)
                                      non-del-msg-found)))
      (setq count (1- count))))
  (beginning-of-line)

;;; this does automatic update, "p", "n" and mouse click
;;;  (display-buffer rmail-buffer)
  )

;;}}}
;;{{{ Fixing RMAIL messages

;;; ............................................................. &fix ...


;;; ----------------------------------------------------------------------
;;; - When you run RMAIL over FCC'd file, and afterwards add more to that
;;;   FCC mail, the file may become corrupt so that RMAIL can't read all
;;;   messages in it.
;;; - This little function, when header region is selected, converts
;;;   the headers to Rmail, so that summary can be used.
;;; - I don't understand why my 'From ' field goes totally wrong...
;;;
;;;
(defun tirm-fix-make-rmail-message-header (beg end)
  "Fix RMAIL header in BEG END.
To use this function you must do this.

1. Be in RMAIL buffer
2. Change mode to text with \\[text-mode]
3. run \\[widen]
4. Select message's full headers
5. Call this function

After the call, the appropriate RMAIL message format for headers has been
created."
  (interactive "r")
  (let* (;; START and END headers strings
	 ;;
	 (s-h (concat  (char-to-string ?\037) "\f\n1,,\n")) ;start header
	 (e-h "*** EOOH ***\n")
	 blk line
	 from date
	 rmail-lines
	 )
    (keep-lower-order beg end)

    ;;  These lines are show in the real rmail message, rest are hidden.
    ;;
    (setq rmail-lines
	  (ti::b-grep-lines
	   "^To:\\|^From:\\|^date:\\|^Subject:" beg end))


    (setq blk (buffer-substring beg end))
    (kill-region beg end)

    (goto-char beg)
    (insert s-h blk "\n" e-h)
    (goto-char beg) (forward-line 2)

    (if (null
	 ;; Is this corrupted From line ?
	 (looking-at "From\\( [a-zA-Z]+ \\)\\([FSMTWS].*\\)"))
	nil
      (setq from (buffer-substring (match-beginning 1) (match-end 1)))
      (setq date (buffer-substring (match-beginning 2) (match-end 2)))
      (kill-line)

      (setq line (concat "Date: " date "\n" "From:" from))
      (insert line)

      (if (null (re-search-forward (regexp-quote "***")))
	  (message "Not found [***]")
	(forward-line 1)
	(setq beg (point))
	(insert blk)
	(goto-char beg)
	(kill-line)
	(insert line)
	(re-search-forward "^Subject")
	(forward-line)
	))

    ;; If there is no babyl at all we may want to insert the RMAIL headers
    ;;
    (when (y-or-n-p "insert Rmail headers too?")
      (if (null (re-search-forward (regexp-quote "***")))
	  (message "Can't find ***")
	(forward-line 1)
	(insert (mapconcat 'concat rmail-lines "\n") "\n")
	))
    ))


;;}}}

;;{{{ example

;;; ......................................................... &example ...
;;
;; Here is sample text file that you can use for anti-spam replies
;; To unpack this call M-x tirm-install-files
;;
;; file pkg.tar:
;; rw-r--r-- 240/200   2588 Nov 24 22:10 1996 spam.txt
;;
;; -----BEGIN PGP MESSAGE-----
;; Version: 2.6.3ia
;; Comment: Base64 signed. File: pkg.tar uncompresses to approx. 10K
;;
;; owHtV8+LHEUY3SA5dIuecvEg1kHYXZjp/ZHdDcRD2CQmbtiEJZslBBGp7q6ZqWx1
;; VaeqeieTP0BUBK968OJFPejVHwf15EUQFUUEb4J4CXjx4CX6vqrp2Y2gJ4Mo/R12
;; 6Or68X3vve9V70tzew8dnzu5cevEpx/d/fbiicfXjx1beejY73uPvLD21dd3zz55
;; 7/XjH/904/o9/d7nT+zku2/8NrmkH/v1zV8++GZx9f0f3vni5f1PXn3xlZ/VWz++
;; dvvLz75/+/m73/U/fDR/d+Hh/LjzpdRzCFfzKvO3/dwDjGXExtpa+D25sRx/V+Iv
;; Yn35JN5trK5trKyun1rewPjK6qnl9bnlB5lUG43z3OLIm5wrb/56nq7c3+4zLWb2
;; +x+JNB15X59eWipMldVWVkILn+FhiZSxL/VwKW8mwuZizK3IRr5S6bMXMclyJe+I
;; kh0I66TRWZY9l6abSgrNPVaxC41vrGDngK+pMCdNk8sGA7KqjfVcezXpMV5iuZeO
;; Fkh9wJ08EGrCFpzQJY35Ef66NPGGOexitOgxy/1IWLzimpnBQFiayJmTXtCgxx8x
;; YQXeDoVPE6kHxlbIyWg2sKZaZNIxrirjPGu0pPS5wplW4FCPgjAPO6TJFp4sgcGu
;; mNyUE6bkvnBMeqStSyYHbGKaWQWC5ZM0IcyqmLhgV4SnKfNKhTyxhtbxCJFgA25Z
;; BUTSpBamViJWFPb03vICB19DoWKe0rWCIwNOe6bJZnumxbuzihf7SjpMP2/Ccsyl
;; gsbAmJ4tA5s110jPWGbskGt5J+IBWHNBBUt/hm2xGlgYHRaP+IFgPE1so0TfDPp+
;; 1FT5U5gzlihHCyTAoAtUNAkcBWjp6fCwwMXuzuZl16KRsa35ihXInUuNQ7TxKA2k
;; Zmm6a9iYFkBkrJowU1OC7gxUE0AI48p4ByhR2cSFSonkQwIicazlbZoBLaSDDJ2o
;; hHORDFIa1U8ralM3KiIidZoMo7p7KNAHhlCT832BZYXHMmR73VhVXpelYNdFHnNk
;; QVFxK8t4XVvDixGzcjjySGBMsgvL2GxdUvOhcBnbntYVoZMQGUmNXTrHdoTWAn2y
;; 6RXYlEWaXBWFsaXrsV2NM5SIYqwIeUN94ULBrWBaLY9FnrELoB99UoFgxivTQB44
;; tAL+kxmalIKAOMfSj+LSXLgwDw8VCBwAFbWPOl3o8lZrxu6LMk1Ko+eD6ELn7sd+
;; xIz5kpHSjrQ7sQNEQGWbLHG9pQmYsEFQ4NDyeiQLF4p0SLlEljXPpZKecPLG0Cso
;; m9iVxDzSRRNPWyqoNScCtUdHUcahwbAPdQYaVliSMiyCHTGKHs6KE9tm1QZYiniW
;; UBI9zoPd0NDQTlgFYRGZoeFLwFJIEZSS3KAOBP7RoMYBoCGypsol0WOFYCUpejwS
;; UcFajF2bmmOlUMSI9CTXikMY1qDXKGVMnJ3cQ2vH3dIE6TWevGrQ2ibcRlpAtA8l
;; KjHkOcDBDpQxJ+ILobB9mgS/mG6ZsYUtD4fxVh5IrmBHFmaOt9kiORNrqOcGaRKX
;; 5yTiYK3ADs4bwL7VhKIheFcTyawfPIJcAaIkcqZOfGgKYCtWXPZwQhPlJUlOyHQg
;; xrPqAO41WbsABGTdFAXSHjSKNIwLp0DKhDvBfjpN2TQuwpSnl4UJO3IHQ7fgNGAy
;; 1cMzwf6wXKk+cpdDTeom2XjMF3QKUUr9yy5T/vfdalYM4cfYZjLlPnZ6bCqj+0qC
;; EKlLcbtt9QkfRSWjsaBiuosZYexAJuXqWkWBYdM4THREdwUEySapP6UG2FYGQiGn
;; XkyXzGAI5H2AqW5yJYt43YbcaO+M7TTh0mR7V7fTZIGXpY02Ocufpi2SWsNARnhw
;; ut17bSfhjiOnuO8+JzmHe4IaifwiZMylIglE0z8vXWEFdRxHD50LvMkCvO05+gxh
;; O9Fl6NOBZDMdpfYYWtPUhxcjvh2I0SPMTw0KfdQPCdwk6WoBa0AXURcgP1kQHeTA
;; m9g9bBkda8SxruLwNF/jeiWywr0PkY0Yp2N9xq3PjkwB3rm8c3SEPqJqCNU6jBak
;; iiIjE7/VCPenlQvISfNKRIUvwiUaVZJRQOol27zGNre32w8OKozESGZP3KkGrd/a
;; Nyhw9NUVfTfAjDrNfLiC6eMLdR1iNnPLllncWWWDDw+0y9Nwuo7//z////5/IF10
;; 0UUXXXTRRRdddNFFF1100UUXXXTRRRdddNFFF138k/EH
;; =iNSX
;; -----END PGP MESSAGE-----

;;}}}

(add-hook 'tirm-:get-new-mail-hook 'tirm-delete-function)

(provide   'tinyrmail)
(run-hooks 'tirm-:load-hook)

;;; tinyrmail.el ends here
