;; @(#) tinytfo.el -- Document layout tool for 'Technical text format'

;; @(#) $Id: tinytfo.el,v 1.28 1997/06/27 21:14:26 jaalto NotReleased $
;; @(#) $Keywords: wp $
;;
;; $KnownCompatibility: 19.28+ XEmacs19.14+ $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1997 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Jan 31 1997
;;
;; To get information on this program use ident(1) or do M-x titf-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinytfo.el|Jari Aalto|jari.aalto@poboxes.com|
;; 'Technical text format', outline, html code markup, text2html conversion|
;; 28-Jun-1997|$Revision: 1.28 $|~/misc/|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file. Rip code with with tinylib.el/ti::m-pkg-rip-magic
;;
;;*     (require 'tinytfo)
;;* _
;;
;;
;; or use this, your .emacs starts faster, prefered method
;;
;;* (autoload 'titf-mode "tinytfo" t t)
;;* _
;;* ;;  Make prefix key C-z which is closer to your pinky
;;* ;;  I put minor modes under after 'm' map. The C-cmf
;;* ;;  in my setup calls folding.el::foldig-mode, that's why 'F' here.
;;* ;;
;;* (require 'tinylibm)
;;* (use-prefix-key "\C-z")
;;* (global-set-key   "\C-zmF" 'titf-mode)
;;
;;
;; If you feel that you have to redefine some binding to suit your
;; keyboard better, please do add similar setup to your emacs
;;
;;* ;;  Make sure the default bindings are run first and my hook runs
;;* ;;  after that, That's why the 'append parameter.
;;* ;;
;;* (add-hook 'titf-:mode-define-keys-hook  'titf-mode-define-keys)
;;* (add-hook 'titf-:mode-define-keys-hook  'my-titf-mode-define-keys 'append)
;;* _
;;* _
;;* (defun my-titf-mode-define-keys ()
;;*   (let ((map titf-:mode-map)
;;*     (p   titf-:mode-prefix-key)
;;*     )
;;*     ;; I use different keyboard nowadays...
;;*     ;;
;;*     (define-key map (concat p "]")  'titf-mark-word-sample)
;;*     (define-key map (concat p "[")  'ignore)
;;*     (define-key map (concat p "{")  'titf-mark-word-emp)
;;*     (define-key map (concat p "}")  'titf-mark-word-strong)
;;*     (define-key map [f10] 'titf-indent-paragraph-text-and-fill)
;;*     (define-key map [f11] 'titf-indent-paragraph-quote-and-fill)
;;*     (define-key map [f12] 'titf-indent-paragraph-sample)
;;*     ))
;;
;; You can also install the atached PGP signed tar file.
;; Before you do this, you need commands 'pgp' and 'tar' along
;; Emacs exec-path
;;
;;      M-x titf-install-files
;;
;; If you have any questions or feedback, use this function
;;
;;      M-x titf-submit-bug-report

;;}}}

;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "titf-" in front of
;;   every function & variable. It stands for '(ti)ny (t)echnical (fo)ormat
;; - variable names contain letters 'titf-:'

;;  Preface, Jan 1997
;;
;;      Late in the your 1996 I decided to put several text documents
;;      available to my University's ftp site. However I tried to
;;      find some decent text2html converter, so that I could let
;;      people read them in nicer format. I didn't find any suitable
;;      program to do the conversion and that made me code a little
;;      perl script which generates quite nice html page from plain
;;      text file. For example the documentation in this lisp file
;;      can be very easily converted into html page.
;;
;;      In the end of pacakge you find PGP signed tar file
;;      that contains additional progrmas for making html
;;      pages out of 'Techical format" text files.
;;
;;  Overview of features
;;
;;      The text layout you write
;;
;;      o   You write text in rigid format called 'Technical'
;;      o   There are only two heading levels, one at column 0, and
;;          another at column 4. NO OTHER SUB-HEADINGS SUPPORTED.
;;      o   Text is written in column 8
;;      o   Each column has different meaning how text is intepreted
;;          into html with t2html.pls perl script.
;;      o   The full 'Technical text format' is described in the
;;          function description of `titf-mode'.
;;
;;      This package
;;
;;      o   Shows text with outline style: you can open and close
;;          headings.
;;      o   Provides commands to move among headings easily.
;;      o   Capitalizes heading with one command.
;;      o   Numbers headings with one command.
;;      o   Text is untabified in regular intervals.
;;      o   On-line help in 19.30+. It assist you writing the text
;;          by displaying message in echo area: how the text is intepreted by
;;          t2html.pls program. (how the text looks in html)
;;      o   Offers functions to mark text with special text MARKERS that
;;          produce <STRONG> or <EMP> in the html output.
;;
;;  What is 'Technical text format'?
;;
;;      In short: it is list of text writing and formatting rules.
;;      And you're looking at it right now in this document too.
;;
;;      This pacakge offers minor for text files and helps you to
;;      maintain correct lasyout. You can even converted file into html
;;      very easily with the attched perl script in the end of source
;;      code file. You don't need to know a shred about the html language
;;      itself. And it is much easier to update text files, than deal
;;      with html itself. When you have text ready, you just feed it to
;;      the t2html.pls perl script and it gives you nicely formatted html
;;      page. Writing html *home* pages is different story, but putting
;;      some text document available in html format is made possible with
;;      this pacakge.
;;
;;      In the other hand, while you may not be interested in html, you
;;      could still consider writing your documents in 'Technical format'
;;      -- with word *technical* I refer to the layout of the text, which
;;      is very _rigid_. In order to use facilities in this package,
;;      eg. heading hiding/showing, the headings must be placed in
;;      columns 0 and 4 and the first word must be in *uppercase*.  The
;;      actual text you write starts at column 8.
;;
;;      If you decide write text like this, you become accustomed to the
;;      layout very quickly and it also helps keeping your documents in
;;      the same format.
;;
;;      All in all, this package was primarily designed to help writing
;;      text documents for t2html.pls and viewing document in *outline*
;;      styled selective display. Please refer to mode description for
;;      full details of the text layout format.
;;
;;  Post command note
;;
;;      While you type text in the buffer, the post command activates at
;;      regular intervals to untabifies whole buffer. The untabify is
;;      done because it makes formatting text easier and when you print
;;      the text file, you can be sure that the output is the same in any
;;      buffer.
;;
;;      If you have Emacs 19.30+ there is additional help feature
;;      available to you. When you sit still in some column position for
;;      few seconds, the column description will be shown automatically.
;;      That should help you keeping informaed of the text layout.
;;
;;  How do you write text
;;
;;      This package turns on two minor modes: tit-mode, that handles
;;      your TAB key movements and titf-mode, the Technical formatting
;;      minor mode.
;;
;;      If you're uncertain about how the column will be treated in html
;;      output, call following function. If you have 19.30+ this is not
;;      necessary, see note about post command above.
;;
;;          \ RET   titf-column-info-display
;;
;;      Normal text you write as usual, but if you want to mark regions
;;      as "quotations" or "code examples" there is appropriate indent
;;      commands
;;
;;          \ /     titf-indent-region-text
;;          \ :     titf-indent-region-strong
;;          \ '     titf-indent-region-quote
;;          \ ;     titf-indent-region-sample
;;
;;          \ t     titf-indent-paragraph-text
;;          \ l     titf-indent-paragraph-text-and-fill
;;	    \ q	    titf-indent-paragraph-quote-and-fill
;;	    \ s	    titf-indent-paragraph-sample
;;
;;      There is no functions for bullet creation, because you can write
;;      them easily by hand. Remember to use some filling mode to fill
;;      bullet text nicely (filladapt.el). Bullets look like this
;;
;;          o   This is bullet..
;;              and cont'd line here
;;          o   Yet another buller
;;              and cont'd line here
;;
;;      The tit-mode will advance your tab key by 4 every time, so the
;;      text in the bullets go to the right column (12). Remeber also to
;;      keep the tit-return-key-toggle on, that allows continuing
;;      indented lined from the same indent level when you press return.
;;
;;      Minor mode key vindings are for HP keyboard and you propably want
;;      to move them somewhere else in your keyboard.
;;
;;  Heading writing and handling
;;
;;      You can only use 2 heading levels, which normally suffices.
;;      Sorry, there is no support for more deeper headings. You start
;;      headings with big letters or number at column 0 or 4. Here is
;;      some specialized commands for headings.
;;
;;      This one converts first character of each heading to
;;      uppercase. This fixes mistakenly left lowercase letters.
;;
;;          \ f     titf-heading-fix
;;
;;          before command:
;;          heading
;;              heading
;;
;;          after command:
;;          Heading
;;              Heading
;;
;;      You can (re)number heading easily with following command. If
;;      there is no number in the line, one is added to the beginning of
;;      heading.  And if you have added new heading somewhere in the
;;      middle of text, just call this function and it renumbers all
;;      headings again.  Running commadn with *prefix* *arg* removes
;;      numbering.
;;
;;          \ 0     titf-heading-numbering
;;
;;          before command:
;;          heading
;;              heading
;;          heading
;;              heading
;;
;;          after command:
;;          1.0 heading
;;              1.1 heading
;;          2.0 heading
;;              2.1 heading
;;
;;      One note about renumbering. Some people write heading number so
;;      that they are closed with parenthesis. This style is not
;;      recommended with technical format style and when you do renumber,
;;      those parenthesis will be removed. The parenthesis style is not
;;      supported because it is visually more clearer if you just write
;;      plain numbers without extra parenthesis.
;;
;;          before command:
;;          1.0) heading
;;              1.1) heading
;;
;;          after command:
;;          1.0 heading
;;              1.1 heading
;;
;;
;;  About table of contents (index)
;;
;;      When you write text, you don't write the table of contents, but
;;      the headings. After you have finished writing, you normally
;;      follow these steps:
;;
;;          \ 0     titf-heading-numbering
;;                  If you don't already have numbers.
;;          \ f a   titf-fix-all
;;
;;      The fix command will reformat and renumber all headings as well
;;      as create the table of contents to the heading that has name
;;      "Table of contents." If there is no such heading, progrmam
;;      notifies that it couldn't update table of contents.
;;
;;      If you want manually create the index (table of contentns), then
;;      there are couple of commands:
;;
;;          \ i             titf-toc
;;          \ mouse-1       titf-toc-mouse
;;
;;  Word about key definitions when mode is turned on
;;
;;      When the minor mode is active it overrides some commonly used
;;      keybindings and moves their original function under Control key.
;;      For example:
;;
;;          original PgUp    --> Now Control-PgUp
;;          original PgDown  --> Now Control-PgDown
;;
;;          PgUp             --> Moving heading backward
;;          DownUp           --> Moving heading forward
;;
;;      If you are using X envinronment or your emacs recognizes mouse,
;;      then there is one handy binding that opens or closes heading
;;      levels when you click over them.
;;
;;          [mouse-3]       titf-mouse-context-sensitive
;;
;;      If you press this mouse button anywhere else than over the
;;      headings, it'll call original binding. This feature is similar as
;;      what is used in folding.el
;;
;;  Code note: about the outline layout
;;
;;      Speed was my primary goal when I added the outline code. But that
;;      came with a price: I would have liked that the Heading 0 were
;;      separated by at least one empty space so that the buffer would
;;      look visually better.
;;
;;          heading1-1
;;              heading1-1
;;              heading1-2
;;                                      << empty space here
;;          heading2-1
;;              heading2-1
;;              heading2-2
;;
;;      But that would have required adding some extra code to do bound
;;      checking every time the block is collapsed/opened. Currently I'm
;;      not going to add that checking because it would reduce speed and
;;      the check would cause unnecessary complexity to current code
;;      flow.
;;
;;      If I have time, I may take a look at that more. For now, please
;;      try to bear the spartan look of the outlined headings.
;;
;;      Please do send me a patch that would correct the "spartan" look,
;;      if you decide to hack my code. Also check that the performance
;;      isn't decreased much with elp.el.

;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Jun	 27	1997	[jari]		19.28	v1.26-1.28	NotReleased
;; - Changed some keybindings.
;; - Added paragraph handling commands.
;;
;; Jun   18     1997    [jari]          19.28   v1.23-1.25      NotReleased
;; - Checkdoc 1.42 clear. defcustom support. Rewrote macros (much better
;;   format now). Updated shell/perl scrips
;; - Corrected function create macros (accidentally I had commented the
;;   body out
;; - Fixed "two space" sentence end. Updater PGP tar block.
;;
;; May   14     1997    [jari]          19.28   v1.19-1.22      NotReleased
;; - now uses new minor mode wizard.
;; - Changed all non standard boolean tests to use when and unless
;;
;; Feb   24     1997    [jari]          19.28   v1.15-1.18      NotReleased
;; - Added new user function titf-heading-numbering, binding  "\0".
;; - Wrote more to documentation, fixed heading numbering functions.
;; - Added more examples to installation instructions.
;;
;; Feb   23     1997    [jari]          19.28   v1.11-1.15      NotReleased
;; - Now uses 'fmacros' to create sililar function.
;; - If region is over 30 lines; ask confirmation to prevent accidents.
;;
;; Feb   21     1997    [jari]          19.28   v1.11-1.13      NotReleased
;; - Corrected the titf-show-buffer function. Oly opened all levels
;;   below.
;; - Started using function create macros from 'm' lib.
;; - Renamed all variables to have '-:'
;;
;; Feb   21     1997    [jari]          19.28   v1.9-1.10       NotReleased
;; - minor fixes in installation functions and in interactive argument
;;   specs. Added "*" so that read-only buffers are recognized.
;; - Keymap Prefix variable name changed.
;;
;; Feb   16     1997    [jari]          19.28   v1.5-1.8        NotReleased
;; - Added PGP signed tar file and installation function for it. Now
;;   all the needed programs ship with this module. You can start writing
;;   your html pages instantly.
;; - Added paragraph moving commands. Added titf-heading-fix user command.
;; - Rewrote the installation function and added new PGP tar (1.24)
;;
;; Feb   3      1997    [jari]          19.28   v1.4            NotReleased
;; - Small fixes to the info-display, the PgUp/Down is now case sensitive
;;   as it should have been in the first place. Word marked with '=='
;;   now means <SAMPLE>
;;
;; Feb   2      1997    [jari]          19.28   v1.3            NotReleased
;; - Rewrote the ouline code because it took 10 minutes to fold
;;   96K text buffers. Yuck! After code rearrange it too 10 secs,
;;   but that was still too much. After using elp.el to spot time
;;   hoggers, I uses direct subst-char-in-region functions calls in
;;   appropriate places and reduced functions calls. Now it takes 2 secs.
;; - Added Mode description from perl script
;; - Added word marking commands. This seems pretty complete package
;;   already.
;;
;; Feb   1      1997    [jari]          19.28   v1.2            NotReleased
;; - Outline code is now working.
;;
;; Jan  31      1997    [jari]          19.28   v1.1            Created


;;}}}

;;; Code:

;;{{{ compile

;;; ......................................................... &require ...

(require 'tinylibm)
(require 'easymenu)

(eval-and-compile
  (defvar   tit-mode            nil)
  (defvar   tit-:div-factor     nil)
  (autoload 'tit-mode           "tinytab" t t)
  (autoload 'imenu--mouse-menu "imenu" t t)
  )

(eval-when-compile (use-dynamic-compilation))

(defgroup-tiny TinyTfo titf-: wp
    "Minor mode for writing text in 'Technical text format'.
  In nutshell

      o   You write text in rigid format called 'Technical'
      o   There are only two heading levels, one at column 0, and
          another at column 4. NO OTHER SUB-HEADINGS SUPPORTED.
      o   Text is written in column 8
      o   Each column has different meaning how text is intepreted
          into html with t2html.pls perl script.
      o   The full 'Technical text format' is described in the
          function description of `titf-mode'"
    )

;;}}}
;;{{{ setup: hooks, private


;;; .......................................................... &v-mode ...
;;;###autoload (autoload 'titf-install-mode "tinytfo" t t)
;;;###autoload (autoload 'titf-mode "tinytfo" t t)


(eval-and-compile

(ti::m-fmacro-minor-mode-wizard
 "titf-" " Tf" "\\" "Tf" 'TinyTfo "titf-:"              ;1-6

 "Minor mode for writing and editing text in technical format.
The text layout presented in this minor mode is mnt to be
feed to t2html.pls perl script which generates html version.

The perl code is included in source file and can be unpacked with
command titf-install-files.

Mode description:
\\{titf-:mode-map}

The following documentation is from the Perl tool's help page.


 DESCRIPTION:

    This is simple text to html converter. Unlike other tools, this
    tries to minimize the use of text tags to format the document,
    The basic idea is to rely on indentation level, and I call
    this layout 'Technical format'.

--//-- decription start

0123456789 123456789 123456789 123456789 123456789
HEADER 1 HERE

 emphatised text at column 1,2,3

    This is heading 2 at column 4, started with big letter

        Standard text starts at column 8, you can
        *emphatize* text or make it _strong_ and show
        variable name like =ThisVariableSample=
        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt

       strong text is between columns 5, 6, 7
       \"Special EMP text in column 7 starts with double quote\"

        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt

         strong text at columns 9 and 11

          Column 10 has quotation text
          Column 10 has quotation text
          Column 10 has quotation text

            Column 12 is reserved for code examples
            Column 12 is reserved for code examples
            All text here are surrounded by SAMP codes

    Header Section 2, at column 4 again

        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt

        o   Bullet 1 txt txt txt txt txt txt txt txt
            ,txt txt txt txt txt txt txt txt

            Notice that previous paragraph ends to P-comma code,
            it tells this paragraph to continue in bullet
            mode, otherwise this column at 12 would be
            intepreted as SAMPLE code.

        o   Bullet 2, text starts at column 12
        o   Bullet 3. Bullets are adviced to keep together
        o   Bullet 4. Bullets are adviced to keep together

        .   This is ordered list nbr 1, text starts at column 12
        .   This is ordered list nbr 2
        .   This is ordered list nbr 3

        .This line uses BR code, notice the DOT-code at beginning
        .This line uses BR code
        .This line uses BR code

       \"This is emphatized text starting at column 7\"
        .And this text is put after the previous line with BR code
       \"This starts as separate line just below previous one, EM\"
        .And continues again as usual with BR code

        See the document #URL-BASE/document.txt, where #URL-BASE
        tag is substituted with -base switch contents.

        Make this email address clickable <foo@site.com>
        Do not make this email address clickable -<bar@site.com>,
        because it may only example and not real address.

HEADER 2 HERE

        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt
        txt txt txt txt txt txt txt txt txt txt txt txt

--//-- decription end

    That's it, there is the whole layout described above.
    More formally the the rules of text formatting are these:

    About headers
    o   There are only _two_ heading levels in this style.
        Heading columns are 1 and 4 and the heading must start with
        big letter or number
    o   In column 4, if the text starts with small letter, that line
        is interpreted as <STRONG>
    o   The heading level 1 uses <HR> code to mark big sections.
    o   The headings are gathered and the index jump block is built.
        The NAME reference consists of first 4 sequential words
        from the heading name. Make sure your heading are uniquely named,
        otherwise there will be same NAME references in the generated html.
        Spaces are converted into underscore when joining the words.

    About text placement
    o   Text at column 1 is undefined if it doesn't start with
        big letter or number to indicate Heading 1.
    o   Text between colums 1-3 is marked with <EM>
    o   Column 4 is reserved for heading level 2
    o   Text between colums 5-7 is marked with <STRONG>
    o   Text in column 7 is <EM> if the first character is double quote.
    o   Column 10 is reserved for <EM> text. If yuu want to quote
        someone's words or reference text, place the text in this column.
    o   Text in colums 9,11 are marked with <STRONG>

    Column 8 for text and special codes
    o   Column 8 is reserved for normal text

    Column 12 is special
    o   Column 12 is treated specially: block is started with <PRE> and
        lines are marked as <SAMP></SAMP>. Wwhen the last text at _column_
        12 is found, the block is closed with </PRE>
        Note follwing example

            txt txt txt     ;evenly started block, fine, do it like this
            txt txt

            txt txt txt txt ;No! can't terminate the /pre, because last
            txt txt txt txt ;column is not at 12
                txt txt txt txt

    Other text markings
    o   If there is dot-code, '.', and immediately non-whitespace, then
        <BR> code is added to the end of line.

        .This text contains BR code
        While these two line are joined together
        in normal html document.

    o   If there is ',' then the <P> code is not inserted if
        the previous line is empty. If you use both '.' and ',' they must
        be in order '.,'

        The P-comma-code works differently if it is used in bullet

    o   Special text markings:
            _this_          is intepreted as <STRONG>this</STRONG>
            *this*          is intepreted as <EM>this</EM>
            =this=          is intepreted as <SAMPLE>this</SAMPLE>

    Special #-commands
    o   #REF command is used for refering to NAME inside current document.
        Example:

        #REF how_to_profile ;(Note:) profiling;
             (1)             (2)

        1.  The NAME reference in current document, a single word
            You can get NAME list by enabling -pref option

        2.  The clickable text delimited by ; characters.

    o   #URL-BASE is substituted with the command line -base URL
        reference: only directory part is used from the -base.
        It allows you to refer to documents local to current site.

            base = http://this.com/dir1/dir2/text.html
            #URL-BASE/next.html
            -->
            http://this.com/dir1/dir2/next.html

    http and email highlighting control
    o   All http and ftp references as well as <foo\@site.com> email
        addeses are marked clickable. Email must have surrounding <>
        characters to be recognized.
    o   If you don't want to make some reference clickable, preceed it
        with dash character.

            <foo@com> This is made clickable
            -<foo@com> This is not clickable
            http://this.com is clickable
            -http://this.com is not clickable

    lists and bullets
    o   The bulletin table is contructed if there is 'o' at
        column 8 and 3 spaces after it, so that text starts at column 12.
        Bulleted lined must be kept together, no spaces between bullet
        blocks.
    o   The ordered list is started with '.' and contructed like bullet.

    About line breaks
    o   All line breaks are visible in your document, do not use more
        tan one line break to separate text from ewach other.
    o   Very important is that there is only _one_ line break
        after headers 1 and 2

 WHEN DOCUMENT DOESN'T FORMAT RIGHT

    Believe or not, the most common error is wrong spacing. Remember,
    Keeep _one_ empty line between headers and text, between body text
    and bullet. Check those first.

    Second common error is that you have put text on wrong column.
    Remember that text column is at 8.

    Third, _check_ that the block ends evenly, expecially when it's inside
    columns 1-7 or 12.

    Headings start with _big_ letter.

 CONTACT:

    <jari.aalto@poboxes.com>

"

 "Technical text format"

 (progn
   ;;  We reinstall every time, because we read some key definitions
   ;;  dynamically from current/global map
   ;;
   (cond
    (titf-mode
     (titf-install-mode)
     (put 'indent-tabs-mode 'titf indent-tabs-mode)
     (setq selective-display t
           selective-display-ellipses t
           indent-tabs-mode nil
           )
     (put 'tit-mode 'titf tit-mode)      ;Save previous state
     (unless tit-mode (tit-mode 1))      ;Turn on this mode too
     (setq tit-:div-factor 4)
     (titf-install-hooks)

     ;;  Single space ends sentence.
     ;;
     (make-local-variable 'sentence-end-double-space)
     (setq sentence-end-double-space t)

     ;;  Make sure RETURN key continues indent.
     ;;
     (local-set-key "\C-m" 'tit-auto-indent)
     )
    (t


     (setq indent-tabs-mode (get 'indent-tabs-mode 'titf)) ;Restore
     (setq selective-display nil)
     (titf-install-hooks 'uninstall)

     (if (and tit-mode  ;;Still on? (User could have shut it)
              (null (get 'tit-mode 'titf)));;Was is originally on?
         (tit-mode 0))

     (titf-show-buffer)
     )))

 "Techical text writing menu."
 (list
  titf-:mode-easymenu-name
  ["Hide buffer"            titf-hide-buffer            t]
  ["Show buffer"            titf-show-buffer            t]
  ["hide heading"           titf-hide                   t]
  ["Show heading"           titf-show                   t]
  ["Show/hide toggle"       titf-show-toggle            t]
  "----"
  ["Heading 1 backward"     titf-heading-backward-0     t]
  ["Heading 1 forward"      titf-heading-forward-0      t]
  ["Heading 2 backward"     titf-heading-backward-any   t]
  ["Heading 2 forward"      titf-heading-forward-any    t]
  ["Heading numbering"      titf-heading-numbering      t]
  ["Heading fix 1st chars"  titf-heading-fix            t]
  ["Paragraph forward"      titf-forward-paragraph      t]
  ["Paragraph backward"     titf-backward-paragraph     t]
  "----"
  ["Indent region strong"   titf-indent-region-strong   t]
  ["Indent region sample"   titf-indent-region-sample   t]
  ["Indent region quote"    titf-indent-region-quote    t]
  ["Indent region text"     titf-indent-region-text     t]
  ["Indent paragraph text"  titf-indent-paragraph-text  t]
  ["Indent paragraph text and fill"  titf-indent-paragraph-text-and-fill  t]
  ["Indent paragraph Quote"   titf-indent-paragraph-quote  t]
  ["Indent paragraph Quote and fill" titf-indent-paragraph-quote-and-fill t]
  ["Indent paragraph Sample"  titf-indent-paragraph-sample t]
  ["Indent paragraph zero"  titf-indent-paragraph-zero t]
  "----"
  ["Mark word strong"       titf-mark-word-strong       t]
  ["Mark word sample"       titf-mark-word-sample       t]
  ["Mark word emphatised"   titf-mark-word-emp          t]
  ["Unmark word"            titf-unmark-word            t]
  "----"
  ["Index create"               titf-toc                    t]
  ["Index popup"                titf-toc-mouse      t]
  ["Selective display copy" ti::m-selective-display-copy-to t]
  ["Selective display print" ti::m-selective-display-print t]
  "----"
  ["Untabify buffer"        titf-untabify-buffer        t]
  ["Column info"            titf-column-info-display    t]
  ["Reinstall post-hook (col info)"      titf-install-hooks t]
  ["Exit and do cleanup"        titf-exit                   t]
  )

 (progn
   (let* (vec
          )

   (tinyck-maybe-activate 'emacs)

    ;;  These bindings are based on HP keyboard, os
    ;;  Don't wonder if they look very weir in your keyboard.
    ;;  The idea is that there is some key just above return,
    ;;  in HP this is the "\" that is defualt prefix.
    ;;
    ;;  then you select all other commands close to that key.
    ;;  Please send me mapping of some other keyboard and
    ;;  I record the alternative keys here.
    ;;
    (define-key map (concat p "p") 'titf-heading-backward-0)
    (define-key map (concat p "n") 'titf-heading-forward-0)
    (define-key map (concat p "u") 'titf-untabify-buffer)
    (define-key map (concat p "0") 'titf-heading-numbering)

    (define-key map (concat p p)   'titf-show-toggle)
    (define-key map (concat p "-") 'titf-hide)
    (define-key map (concat p "=") 'titf-show)

    (define-key map (concat p "T") 'titf-toc)

    ;;  mouse binding "Prefix + mouse-1"
    ;;
    (setq vec (make-vector-keybinding p 'mouse-1))
    (define-key map vec 'titf-toc-mouse)

    ;;  These are closet to default "\" key, can be accessed with thumb
    ;;
    (define-key map (concat p ",") 'titf-hide-buffer)
    (define-key map (concat p ".") 'titf-show-buffer)
    ;;  But user may remember better these
    ;;
    (define-key map (concat p ">") 'titf-hide-buffer)
    (define-key map (concat p "<") 'titf-show-buffer)

    (define-key map (concat p ":") 'titf-indent-region-strong)
    (define-key map (concat p ";") 'titf-indent-region-sample)
    (define-key map (concat p "'") 'titf-indent-region-quote)
    (define-key map (concat p "/") 'titf-indent-region-text)


    (define-key map (concat p "t") 'titf-indent-paragraph-text)
    (define-key map (concat p "l") 'titf-indent-paragraph-text-and-fill)
    (define-key map (concat p "q") 'titf-indent-paragraph-quote)
    (define-key map (concat p "w") 'titf-indent-paragraph-quote-and-fill)
    (define-key map (concat p "s") 'titf-indent-paragraph-sample)
    (define-key map (concat p "z") 'titf-indent-paragraph-zero)

    (define-key map (concat p "f")  nil)
    (define-key map (concat p "fa") 'titf-fix-all)
    (define-key map (concat p "fh") 'titf-heading-fix)
    (define-key map (concat p "fn") 'titf-heading-fix-newlines)

    (define-key map (concat p "P") 'ti::m-selective-display-print)
    (define-key map (concat p "c") 'ti::m-selective-display-copy-to)

    ;;  Don't know yet what to put to "}"
    ;;
    (define-key map (concat p "{")  'titf-mark-word-sample)
    (define-key map (concat p "}")  'ignore)
    (define-key map (concat p "]")  'titf-mark-word-emp)
    (define-key map (concat p "[")  'titf-mark-word-strong)
    (define-key map (concat p " ")  'titf-unmark-word)

    (define-key map (concat p "\C-m")  'titf-column-info-display)
    (define-key map (concat p "h")     'titf-install-hooks)
    (define-key map (concat p "x")     'titf-exit)

    ;;  Original PgUp and down keys --> move under Conrol key
    ;;
    (copy-key-definition map [C-prior]  [prior])
    (copy-key-definition map [C-next]   [next])
    (define-key map [prior]         'titf-heading-backward-any)
    (define-key map [next]          'titf-heading-forward-any)
    (define-key map [S-prior]       'titf-heading-backward-0)
    (define-key map [S-next]        'titf-heading-forward-0)

    ;;  The 'home' and 'end' keys
    (copy-key-definition map [C-end]      [end])
    (copy-key-definition map [C-home]     [home])
    (copy-key-definition map [C-select]   [select])
    (define-key map [home]          'titf-backward-paragraph)
    (define-key map [select]        'titf-forward-paragraph)
    (define-key map [end]           'titf-forward-paragraph)

    (define-key map [mouse-3]       'titf-mouse-context-sensitive)


    (tinyck-maybe-activate 'emacs 'disable)
    ))
 )

)


;;; ......................................................... &v-hooks ...

(defcustom titf-:load-hook nil
  "*Hook that is run when package is loaded."
  :type 'hook :group 'TinyTfo
  )

(defcustom titf-:move-paragraph-hook  nil
  "*Hook run with arguments BEG END of region that was moved."
  :type 'hook :group 'TinyTfo
  )

(defcustom titf-:fix-all-hook  nil
  "*Hook run when `titf-fix-all' function is called."
  :type 'hook :group 'TinyTfo
  )

;;; ....................................................... &v-private ...
;;; Private variables

(defconst titf-:factor 4
  "The indent factor. DO NOT CHANGE THIS. It is hard coded to 4.")

(defvar titf-:counter  nil
  "Post command counter.")
(make-variable-buffer-local 'titf-:counter)

(defvar titf-:counter-reset  100
  "After how many keypresses to untabify buffer.")

(defvar titf-:idle-timer-elt  nil
  "In 19.34+ this hold the idle timer elt activated when mode is turned on.")
(make-variable-buffer-local 'titf-:idle-timer-elt)

(defvar titf-:buffer-heading  "*titf-headings*"
  "List of gatehered Headings from buffer.")

(defconst titf-:column-table
  '(
    ;;  First the most common positions.
    ;;
    (0   ((nil "Heading 0")))
    (4   ((nil "Heading 1")))
    (8   ((nil "Standard text")))

    ;; Then special positions
    (5   ((nil "Strong")))
    (6   ((nil "Strong")))

    (7   (("\"" "Emphatised")
          (nil  "Strong")
          ))

    (8   (("o "  "Bullet, reguler")
          ("\. " "Bullet, numbered")
          ("\.[^ \t]" "Single line, <BR> added to the end of line")
          (",[^ \t]"  "Line continues in next chapter. <P> code not added")
          ))

    (9   ((nil "Strong")))
    (10  ((nil "Quotation, emphatised")))
    (11  ((nil "Strong.")))
    (12  ((",[^ \t]" "Bullet continues in next chapter. <P> code not added")
          (nil       "Sample")
          ))
    )
  "Column positions and their properties. DO NOT TOUCH THIS!

Format:
 '((COL ((REGEXP-OR-NIL EXPLANATION-STRING)
         (REGEXP-OR-NIL EXPLANATION-STRING)
         ..
         ))
   (C   ((R E) (R E) ..))
   )")

;;}}}
;;{{{ setup: user config

;;; ........................................................ &v-public ...
;;; User configurable

(defcustom titf-:topic-charset  "[A-Z0-9]"
  "*Heading characterset. This charset is case sensitive.
If there is these characters immediately after indent, then this line
is heading."
  :type '(string :tag "Charset regexp") :group 'TinyTfo
  )


;;}}}
;;{{{ version

;;; ....................................................... &v-version ...

(eval-and-compile
(ti::m-fmacro-version-bug-report
 "tinytfo.el"
 "titf"
 titf-:version-id
 "$Id: tinytfo.el,v 1.28 1997/06/27 21:14:26 jaalto NotReleased $"
 '(titf-:version-id

   titf-:load-hook
   titf-:mode-define-keys-hook
   titf-:mode-hook
   titf-mode
   titf-:mode-map
   titf-:counter
   titf-:counter-reset
   titf-:idle-timer-elt
   titf-:mode-name
   titf-:mode-prefix-key
   titf-:topic-charset
   titf-:factor
   titf-:column-table
   )))


;;}}}

;;; ########################################################### &Funcs ###

;;{{{ mode install

;;; ......................................................... &install ...


;;; ----------------------------------------------------------------------
;;;
;;;###autoload (autoload 'titf-install-files "tinytfo" t t)
(ti::m-fmacro-install-pgp-tar titf-install-files  "tinytfo.el")


;;; ----------------------------------------------------------------------
;;;
(defun titf-install-hooks  (&optional remove)
  "Install hook to mode or REMOVE.
You have to call this command by hand if you have pressed too much
\\[keyboard-quit] which disables all functions in `post-command-hook'."
  (interactive "P")
  (let* ((hook  (if (boundp 'post-command-idle-hook)
                       'post-command-idle-hook
                     'post-command-hook))
         )

    (cond
     (remove
      (remove-hook hook 'titf-post-command)
      (if titf-:idle-timer-elt
          (sfuncall 'cancel-timer titf-:idle-timer-elt))
      )
     (t
      (add-hook-local hook 'titf-post-command)

      ;;  If not running, install this "echo area help"
      ;;  In other emacs releases it is handled in titf-post-command
      ;;
      (if (fboundp 'run-with-idle-timer)
          (if (null titf-:idle-timer-elt)
              (setq titf-:idle-timer-elt
                    (sfuncall 'run-with-idle-timer 2
                              'repeat 'titf-idle-command))))
      ))))

;;}}}
;;{{{ macros, defsubst

;;; ...................................................... &macro-func ...
;;; These macros create functions
;;;
;;; For some unknown reason the ByteCompiler doesn't see thse
;;; function in the followed macros unless the functions are wrapped
;;; inside eval-and-compile FORM.

(eval-and-compile

;;; ----------------------------------------------------------------------
;;;
(defun titf-fmacro-indent-region-1  (func doc col msg)
  "Use `titf-fmacro-indent-region' with FUNC DOC COL MSG."
  (let* ((sym (intern (symbol-name (` (, func)))))
         )
    (`
     (defun (, sym)  (beg end &optional verb)
       (, doc)
       (interactive "*r")
       (VERB)
       (titf-move-paragraph-to-column
        beg end (, col)
        (if verb (, msg))
        ))
     )))


;;; ----------------------------------------------------------------------
;;;
(defun titf-fmacro-mark-word-1  (func doc char)
  "Use `titf-fmacro-mark-word' with FUNC DOC CHAR."
  (let* ((sym (intern (symbol-name (` (, func)))))
         )
    (`
     (defun (, sym) ()
       (, doc)
       (interactive "*")
       (ti::m-surround-with-char (, char))
       (skip-chars-forward " \n\t\f")
       )
     )))



;;; ----------------------------------------------------------------------
;;;
(defun titf-fmacro-indent-paragraph-1  (func doc col msg &rest body)
  "Use `titf-fmacro-mark-word' with FUNC DOC COL MSG BODY."
  (let* ((sym (intern (symbol-name (` (, func)))))
         )
    (`
     (defun (, sym)  (&optional verb)
       (, doc)
       (interactive "*")
       (let* ((empty-line (var-nil-p (read-full-line)))
	      beg
	      end
	      )
	 (VERB)
	 (save-excursion
	   (if (not empty-line)
	       (or (re-search-backward "^[ \t]*$" nil t)
		   (PMIN)))
	   (setq beg (point))
	   )
	 (if empty-line
	     (skip-chars-forward " \t\n"))
	 (or (re-search-forward "^[ \t]*$" nil t) (PMAX))
	 (setq end (point))
	 (titf-move-paragraph-to-column
	  beg end (, col)
	  (if verb (, msg))
	  'noask)

	 (,@ body)
	 )))))

;;; --++-- --++-- --++-- --++-- --++-- --++-- --++-- --++-- - eval end --
)

;;; ----------------------------------------------------------------------
;;;
(defmacro titf-fmacro-indent-region  (func doc col msg)
  "Create indent function FUNC with DOC COL and MSG.
Created function arguments: (beg end &optional verb)"
  (` (, (titf-fmacro-indent-region-1
         func doc  col msg
         ))))

;;; ----------------------------------------------------------------------
;;;
(defmacro titf-fmacro-mark-word  (func doc char)
  "Create word marking function FUNC with DOC and CHAR.
Created function arguments: ()"
  (` (, (titf-fmacro-mark-word-1
         func doc char
         ))))

;;; ----------------------------------------------------------------------
;;;
(defmacro titf-fmacro-indent-paragraph (func doc col msg &optional body)
  "Create word marking function FUNC with DOC and CHAR.
Created function arguments: ()"
  (` (, (titf-fmacro-indent-paragraph-1
         func doc col msg body
         ))))

;;; ........................................................... &macro ...
;;; These are conventional macros

;;; ----------------------------------------------------------------------
;;;
(put 'titf-paragraph-macro 'lisp-indent-hook 0)
(defmacro titf-paragraph-macro (&rest body)
  "Set paragraph values locally while executing BODY."
  (`
   (let* (
          (sentence-end "[.?!][]\"')}]*[ \n]+")
          (paragraph-start "^[|:> \t]*$")
          (paragraph-separate paragraph-start)
          )
     (,@ body)
     )))


;;; ----------------------------------------------------------------------
;;;
(put 'titf-heading-macro  'lisp-indent-hook 0)
(defmacro titf-heading-macro (&rest body)
  "Map over every heading. The point sits at the beginning of heading text.
The BODY must move the point so that next heading can be found."
  (`
   (let* ((RE-search  (concat (titf-regexp) "\\|" (titf-regexp 1)))
          )
     (save-excursion
       (PMIN)
       (while (re-search-forward RE-search nil t)
         (,@ body)
         )))))

;;; ----------------------------------------------------------------------
;;;
(put 'titf-level-macro 'lisp-indent-hook 0)
(defmacro titf-level-macro (&rest body)
  "Search begin point of current heading level or signal error.
You can refer to variable 'level' and 'beg' and 'end' in the BODY.
The point is at start of level."
  (`
   (let* ((level  (titf-level-number))
          beg
          end
          )

     ;; .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. ..  beg positon . .
     (unless level
       (if (titf-heading-backward-any)
           (setq level (titf-level-number))
         (error "Can't find begin point")
         ))
     (setq beg (point)
           end (titf-block-end))
     (goto-char beg)
     (,@ body)
     )))

;;; ----------------------------------------------------------------------
;;;
(defsubst titf-indent (&optional level)
  "Return indent space string at LEVEL."
  (make-string (* titf-:factor (or level 0)) ?\ ))

;;; ----------------------------------------------------------------------
;;;
(defsubst titf-regexp (&optional level)
  "Return indent regexp string at LEVEL."
  ;;  control character are not counted, like ^L page mark
  ;;
  (concat "^" (titf-indent level) titf-:topic-charset))


;;; ----------------------------------------------------------------------
;;;
(defsubst titf-level-p  (&optional level)
  "Check if line is LEVEL."
  (let* (case-fold-search               ;case sensitive match
         )
    (save-excursion
      (beginning-of-line)
      (looking-at (titf-regexp level))
      )))

;;; ----------------------------------------------------------------------
;;;
(defsubst titf-level-number ()
  "Check current level on this line."
  (cond
   ((titf-level-p 0) 0)
   ((titf-level-p 1) 1)
   ))



;;; ----------------------------------------------------------------------
;;;
(defsubst titf-block-end ()
  "Return text block end."
  (save-excursion
    (if (null (titf-heading-forward-any))
        (point-max)
      (beginning-of-line)
      (1- (point))
      )))


;;; ----------------------------------------------------------------------
;;;
(defsubst titf-heading-number-regexp  (&optional no-grouping)
  "Return heading number regexp: match 'N.n '  or  'N.n) '.
If NO-GROUPING is non-nil, the regexp will not have regexp group operator."
  (if no-grouping
      "[0-9]+\\.[0-9]+)?[ \t]+"
    "\\([0-9]+\\.[0-9]+)?[ \t]+\\)"
    ))

;;; ----------------------------------------------------------------------
;;;
(defsubst titf-headings-numbered-p  ()
  "Check if first heading is numbered."
  (save-excursion
    (PMIN)
    (re-search-forward (titf-heading-number-regexp) nil t)
    ))

;;; ----------------------------------------------------------------------
;;;
(defsubst titf-heading-string  (major minor)
  "Return heading string. 'MAJOR.MINOR '."
  (concat major "." minor " "))


;;; ----------------------------------------------------------------------
;;;
(defun titf-goto-non-space  ()
  "Goto first non-whitespace of bol."
  (interactive)
  (beginning-of-line)
  (if (re-search-forward "^[ \t]+" (EOLP) t)
      (goto-char (match-end 0))))


;;; ----------------------------------------------------------------------
;;;
(defsubst titf-heading-same-p (level heading-regexp)
  "Check if heading LEVEL is identical to HEADING-REGEXP.
After the regexp there must be non whitespace, which starts the heading
name."
  (save-match-data
    (save-excursion
      (beginning-of-line)
      (looking-at (concat
                   "^"
                   (titf-indent level)
                   heading-regexp

                   ;;  After the spaces there must be NON-space to start
                   ;;  the heading name
                   ;;
                   "[^ \t]"
                   ))
      )))

;;}}}

;;{{{ misc:

;;; ............................................................ &misc ...

;;; ----------------------------------------------------------------------
;;;
(defun titf-untabify-buffer ()
  "Untabify whole buffer."
  (interactive)
  (untabify (point-min) (point-max)))

;;; ----------------------------------------------------------------------
;;;
(defun titf-column-info  ()
  "Return column intepretation."
  (interactive)
  (let* ((col (current-column))
         (elt (assq col titf-:column-table))
         list
         re
         ret
         )
    (when elt
      (setq list (nth 1 elt))

      (list-loop list elt
        (setq re (car elt))
        (if (not (stringp re))  ;Stop there
            (setq list nil)
          (if (looking-at re)           ;found match ?
              (setq list nil))  ;Yes, stop there
          ))

      (when elt
        (setq ret (nth 1 elt)))
      )
    ret
    ))

;;; ----------------------------------------------------------------------
;;;
(defun titf-column-info-display  ()
  "Display column info.
This function displays information about the column only and the text
that follows it. It does not look around the text, so it will fail
to detext text insoe bulletins or inside any other complex text.
E.g.

        o   *Bullet here...
            Continues here

Cursor at [*], which is column 12, will just tell you that the column
is used for code exsample and marked with <SAMPLE>."
  (interactive)
  (let ((string (titf-column-info))
        (col    (current-column))
        )
    (if string
        (message "%s: %s (col %d)" titf-:mode-name string col)
      (message "%s: Nothing special at column %d" titf-:mode-name col))
    ))

;;}}}
;;{{{ headings

;;; ........................................................ &headings ...

;;; ----------------------------------------------------------------------
;;;
(defun titf-heading-positions (&optional point and-heading-names)
  "Read all headings position forward.

Input:
 POINT              start point of search, defaults to current point.
 AND-HEADING-NAMES  see return value.

Return:
 '(pos pos ..)
 '((heading pos) (heading pos) ..) If AND-HEADING-NAMES is non-nil
 nil"
  (let* ((re (concat (titf-regexp) "\\|" (titf-regexp 1)))
           heading
           list
           )
    (save-excursion
      (if point (goto-char point))
      (while (re-search-forward re nil t)
        (setq point (BOLPP))
        (cond
         (and-heading-names
          ;;  Dont read trailing spaces.
          ;;
          (end-of-line)
          (if (re-search-backward "[^ \t]+" point t)
              ;; Search 'eats' one character include it too.
              ;;
              (setq heading (buffer-substring point (1+ (point))))
            (setq heading (buffer-substring point (point))))

          (push (cons  heading (point)) list)
          )
         (t
          (push (point) list)
          (end-of-line)
          ))
        ))

    ;;  Add point max there too. (due to hide region)
    ;;
    (if (and list (null and-heading-names))
        (push (point-max) list))

    (nreverse list)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun titf-heading-fix-newlines (&optional verb)
  "Search all headings and remove extra newlines. VERB.

Return:
  nil       ,nothing fixed
  nbr       ,this many fixed."
  (interactive)
    (let  ((mark   (point-marker))
           (fix    0)
           beg
           )
      (VERB)
      (titf-heading-macro
        (move-marker mark (point))
        ;;  Delete previous newlines
        ;;
        (forward-line -1) (setq beg (point))

        (unless (bobp)
          (skip-chars-backward "\n\t ")
          (forward-line 1)

          (unless (eq (point) beg)
            (inc fix)
            (delete-region beg (point))
            ))

        ;;  Delete following newlines
        ;;
        (goto-char (marker-position mark))
        (forward-line 1) (setq beg (point))

        (unless (eobp)
          (skip-chars-forward "\n\t ")
          (forward-line -1)

          (unless (eq (point) beg)
            (inc fix)
            (delete-region beg (point))
            ))

        )
      (setq mark nil)                   ;kill marker

      (if verb
          (if (> fix 0)
              (message "%s: fixed %d places around headings"
                       titf-:mode-name fix)
            (message "%s: no newline fixes." titf-:mode-name)))

      (if (> fix 0) fix)                ;anything to return?
      ))


;;; ----------------------------------------------------------------------
;;;
(defun titf-heading-fix  (&optional verb)
  "Search all headings and convert first letter to uppercase if needed. VERB."
  (interactive "P")
  (let* ((re  (concat                   ;Search lower letters only
               "^" (titf-heading-number-regexp) "?"
               "[a-z]"

               "\\|^" (titf-indent 1)
               (titf-heading-number-regexp) "?"
               "[a-z]"
               ))
         (count  0)
         case-fold-search
         )
    (VERB)
    (save-excursion
      (PMIN)
      (while (re-search-forward re nil t)
        (backward-char 1)
        (capitalize-word 1)
        (inc count)
        ))
    (if verb
        (message "%s: Fixed %d headings" titf-:mode-name count))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun titf-heading-numbering  (&optional remove verb)
  "Number heading levels 1 and 2, old numbering is replaced.
Optionally REMOVE numbering. VERB."
  (interactive "P")
  (let* ((re-no-nbr (titf-regexp 1))
         (re-nbr    (titf-heading-number-regexp))
         (re1       (concat "^" (titf-indent 0) re-nbr))
         (re2       (concat "^" (titf-indent 1) re-nbr))
         (c1        0)                  ;counters
         (c2        0)
         (count     0)
         str
         )
    (VERB)

    (titf-heading-macro
      (inc count)
      (beginning-of-line)

      (cond
       (remove
        (if (or (looking-at re1)
                (looking-at re2))
            (kill-match 1)
          ))

       ((looking-at re1)                ;heading 1
        (setq c2 0) (inc c1)
        (setq str (titf-heading-string c1 c2))

        ;;  Only change if different, this prevents buffer modify flag
        ;;  change
        ;;
        (if (not (titf-heading-same-p 0 str))
            (kill-match 1 str))
        )

       ((looking-at re2)
        (inc c2)
        (setq str (titf-heading-string c1 c2))
        (if (not (titf-heading-same-p 1 str))  (kill-match 1 str))
        )

       ((looking-at re-no-nbr)          ;Level 2
        (goto-char (match-end 0))
        (backward-char 1)
        (inc c2)


        (insert (titf-heading-string c1 c2))
        )

       ((looking-at "^")                ;must be level 1
        (setq c2 0)     (inc c1)
        (insert (titf-heading-string c1 c2))
        )

       )
      (end-of-line)
      )

    (if verb
        (message "%s: Numbered %d headings" titf-:mode-name count))
    ))


;;; ----------------------------------------------------------------------
;;;
(defun titf-fix-all  (&optional verb)
  "Exit and fix Headers, untabify buffer and do other things. VERB."
  (interactive)
  (VERB)

  ;;  when you fill paragraphs, the default Emacs has nasty habits to
  ;;  end the sentece to two spaces, which is some idiot way to separate
  ;;  two sentences (the history says that the typewriter fonts were so bad
  ;;  that the typists needed to add two spaces these).
  ;;
  ;;  Fix them to have only standard space.
  ;;
  (save-excursion
    (PMIN)
    (while (re-search-forward "[a-zA-Z]+[!?.,]\\(  +\\)[a-zA-Z]+" nil t)
      (kill-match 1 " ")))


  (titf-heading-fix)
  (titf-untabify-buffer)
  (titf-heading-fix-newlines)
  (when (titf-headings-numbered-p)
    (titf-heading-numbering))


  (run-hooks titf-:fix-all-hook)

  (if verb
      (message "%s: Fixing done." titf-:mode-name))
  )



;;}}}

;;{{{ misc: post/idle functions

;;; ................................................... &post-commands ...

;;; ----------------------------------------------------------------------
;;;
(defun titf-idle-command  ()
  "Diplay column info when idle."
  (titf-column-info-display)

  ;; also make sure the post-command is active, too many C-g's
  ;; drop it away...
  (or (memq post-command-hook 'titf-post-comman)
      (add-hook-local 'post-command-hook 'titf-post-comman))
  )

;;; ----------------------------------------------------------------------
;;;
(defun titf-post-command  ()
  "Untabify buffer in regular intervals."

  (if (not (integerp titf-:counter))
      (setq titf-:counter 0))

  (when (eq (inc titf-:counter) titf-:counter-reset)
    (untabify (point-min) (point-max))
    (setq titf-:counter 0))
  )



;;}}}
;;{{{ text: formatting with codes

;;; ...................................................... &formatting ...


;;; ----------------------------------------------------------------------
;;;
(defun titf-move-paragraph-to-column (beg end col &optional msg noask)
  "If region BEG END is big, ask confirmation for COL move with MSG NOASK."
  (let* ((lines (+ (count-char-in-region beg end ?\r)
                   (count-char-in-region beg end ?\n)))
         )

    ;;  We use markers, ebacsue the points have moved after the
    ;;  call to ti::m-move-paragraph-to-column. We must pass moved BEG end
    ;;  to hook functions.
    ;;
    (keep-lower-order beg end)
    (save-excursion
      (goto-char beg) (setq beg (point-marker))
      (goto-char end) (setq end (point-marker))
      )

    (when (or noask
              (< lines 31)
              (and
               (> lines 30)
               (y-or-n-p (format "%s: really move %d lines? "
                                 titf-:mode-name lines))
               ))
      (ti::m-move-paragraph-to-column
       (marker-position beg) (marker-position end)
       col)

      (if (and (null noask) (< col 12))
          (let ((fill-prefix (make-string col ?\ )))
            (call-interactively 'fill-paragraph)))

      (if msg (message msg))
      (run-hook-with-args 'titf-:move-paragraph-hook
                          (marker-position beg) (marker-position end))
      (setq beg nil end nil)            ;Kill markers
      )))


;;; ----------------------------------------------------------------------
;;;
(titf-fmacro-indent-region
 titf-indent-region-text
 "Move selected region to STRONG html code position."
 8 (format "%s: Regular text, column 8" titf-:mode-name)
 )

(titf-fmacro-indent-region
 titf-indent-region-strong
 "Move selected region to STRONG html code position."
 9 (format "%s: Strong, column 9"  titf-:mode-name)
 )

(titf-fmacro-indent-region
 titf-indent-region-quote
 "Move selected region to EMPHATISED (quoted text) html code position."
 10 (format "%s: Quotation, column 10" titf-:mode-name)
 )

(titf-fmacro-indent-region
 titf-indent-region-sample
 "Move selected region to SAMPLE (example code) html code position."
 12 (format "%s: Sample, column 12" titf-:mode-name)
 )

;;; ----------------------------------------------------------------------
;;;
(titf-fmacro-indent-paragraph
 titf-indent-paragraph-zero
 "Move paragraph to column 0."
 0 (format "%s: Zero, column 0" titf-:mode-name)
 )

(titf-fmacro-indent-paragraph
 titf-indent-paragraph-text
 "Move paragraph to text position."
 8 (format "%s: text, column 8" titf-:mode-name)
 (progn
   (titf-forward-paragraph)
   ))

(titf-fmacro-indent-paragraph
 titf-indent-paragraph-text-and-fill
 "Move paragraph to text position and fill."
 8 (format "%s: text, column 8" titf-:mode-name)
 (let* ((fill-prefix (or (strmatget "^[ \t]+" 0 (read-full-line)) ""))
	)
   (fill-paragraph nil)
   (titf-forward-paragraph)
   ))

(titf-fmacro-indent-paragraph
 titf-indent-paragraph-quote
 "Move paragraph to Quotation position."
 10 (format "%s: Quotation, column 10" titf-:mode-name)
 )

(titf-fmacro-indent-paragraph
 titf-indent-paragraph-quote-and-fill
 "Move paragraph to Quotation position and fill."
 10 (format "%s: Quotation, column 10" titf-:mode-name)
 (let* ((fill-prefix (or (strmatget "^[ \t]+" 0 (read-full-line)) ""))
	)
   (fill-paragraph nil)
   (titf-forward-paragraph)
   ))


(titf-fmacro-indent-paragraph
 titf-indent-paragraph-sample
 "Move paragraph to Sample code position."
 12 (format "%s: Sample, column 12" titf-:mode-name)
 (progn
   (titf-forward-paragraph)
   ))


;;; ----------------------------------------------------------------------
;;;
(titf-fmacro-mark-word
 titf-mark-word-sample
 "Put 'EMP' code aroound word and move forward."
 ?=
 )

(titf-fmacro-mark-word
 titf-mark-word-emp
 "Put 'EMP' code aroound word and move forward."
 ?*
 )

(titf-fmacro-mark-word
 titf-mark-word-strong
 "Put 'STRONG' code aroound word and move forward."
 ?_
 )

;;; ----------------------------------------------------------------------
;;;
(defun titf-unmark-word ()
  "Remove 'STRONG' and 'EMP' mark from word and move forward."
  (interactive)
  (let* (beg
         end
         word
         )
    (unless (looking-at "[ \t\f]")
      (skip-chars-backward "^ \n\t\f")
      (setq beg (point))
      (skip-chars-forward "^ \n\t\f")
      (setq end (point))
      (setq word (buffer-substring beg end))
      (when (string-match "[_*=]+\\(.*\\)[_*=]+$" word)
        (setq word (read-match 1 word))
        (delete-region beg end)
        (insert word)
        )
      (skip-chars-forward " \n\t\f")
      )))

;;}}}

;;{{{ movement

;;; ........................................................ &movement ...

;;; ----------------------------------------------------------------------
;;;
(defun titf-backward-paragraph  ()
  "Like `titf-forward-paragraph' but goes backward."
  (interactive)
  (titf-paragraph-macro
    (backward-paragraph)
    (backward-paragraph)
    (skip-chars-forward " \t\n")
    (titf-goto-non-space)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun titf-forward-paragraph  ()
  "Like `forward-paragraph' but keep cursor at the beginning of text."
  (interactive)
  (titf-paragraph-macro
    (forward-paragraph)
    (skip-chars-forward " \t\n")
    (titf-goto-non-space)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun titf-heading-forward-0 ()
  "Forward."
  (interactive) (titf-forward 0))

(defun titf-heading-forward-1 ()
  "Forward."
  (interactive) (titf-forward 1))

(defun titf-heading-forward-any ()
  "Forward."
  (interactive) (titf-forward 1 nil 'any))

(defun titf-heading-backward-0 ()
  "Backward."
  (interactive) (titf-forward 0 t))

(defun titf-heading-backward-1 ()
  "Backward."
  (interactive) (titf-forward 1 t))

(defun titf-heading-backward-any ()
  "Backward."
  (interactive) (titf-forward 1 t 'any))

;;; ----------------------------------------------------------------------
;;;
(defun titf-forward (level &optional back any)
  "Go to next heading. Optionally searching LEVEL or BACK or ANY level.

Return:
 nbr    ,point if moved
 nil"
  (let* ((re (if any
                 (concat (titf-regexp) "\\|" (titf-regexp 1))
               (titf-regexp level)))
         case-fold-search               ;case sensitive
         point
         )
    (if back
        (when (re-search-backward re nil t)
          (skip-chars-forward " \t") (point))
      (cond
       ((bolp)                          ;Startt seaching from next char
        (save-excursion
          (forward-char 1)
          (if (re-search-forward re nil t)
              (setq point (point))))
        (if point                       ;If search ok, then move
            (goto-char (1- point)))
        )
       (t
        (when (re-search-forward re nil t)
          (backward-char 1) (point))
        )))

    ))

;;}}}
;;{{{ outline control: show/hide

;;; ......................................................... &outline ...

;;; ----------------------------------------------------------------------
;;;
(defun titf-hide-region  (beg end &optional show)
  "Hide region BEG END with selective display. Optionally SHOW.
Point is END after function finishes."
  (let (buffer-read-only
        (ch1   (if show ?\r ?\n))
        (ch2   (if show ?\n ?\r))
        )
    (subst-char-in-region beg end ch1 ch2)
    (set-buffer-modified-p nil)
    (goto-char (max beg end))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun titf-mouse-context-sensitive  (event)
  "If `mouse-point' points indent 0 or 1 line, then hide/show level. EVENT.
In other places call original fucntion."
  (interactive "e")

  (mouse-set-point event)
  (beginning-of-line)
  (cond
   ((or (titf-level-p 0)
        (titf-level-p 1))
    (titf-show-toggle)
    )
   (t
    (ti::xe-mouse-call-original 'titf-mode event)
    )))

;;; ----------------------------------------------------------------------
;;;
(defun titf-show-buffer ()
  "Remove selective display codes from buffer."
  (interactive)
  (save-excursion
    (titf-hide-region (point-min) (point-max) 'show)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun titf-hide-buffer ()
  "Hide whole buffer."
  (interactive)
  (let* (list
         post-command-hook
         beg end
         )
    (save-line-col nil nil
      (setq list (titf-heading-positions (point-min)))
      (while list
        (setq beg (car list)  end (or (nth 1 list) (point-max)))
        (setq list (cdr list))
        (subst-char-in-region beg (1- end) ?\n ?\r)
        )
      (set-buffer-modified-p nil)
      )))

;;; ----------------------------------------------------------------------
;;;
(defun titf-show-toggle ()
  "Open/close level 1. Does't touch level 0."
  (interactive)
  (let* ((point  (point))
         )
    (if (looking-at ".*\r")
        ;; This level is already collapsed, open it.
        ;;
        (titf-show)
      (titf-hide))
    (goto-char point)
    (titf-goto-non-space)
    ))

;;; ----------------------------------------------------------------------
;;;
(defun titf-show ()
  "Show current level."
  (interactive)
  (titf-hide 'show))


;;; ----------------------------------------------------------------------
;;;
(defun titf-hide (&optional show)
  "Hide current level. Optionally SHOW."
  (interactive "P")
  (let* ((ok t)
         (point  (point))
         )
    (titf-level-macro
      (cond
       ((eq 1 level)
        (titf-hide-region (point) (titf-block-end) show))
       (t
        (titf-hide-region (point) end show)
        (setq ok (titf-heading-forward-any))
        (while (and ok
                    ;;  Until next level 0 ...
                    (not (eq 0 (titf-level-number)))
                    )
          (setq beg (point))
          (cond
           ((setq ok (titf-heading-forward-any))
            (beginning-of-line) (backward-char 1)

            (if show
                (subst-char-in-region beg (point) ?\r ?\n)
              (subst-char-in-region beg (point) ?\n ?\r))

            (forward-char 1)
            )
           (t
            (titf-hide-region beg (point-max))
            )))

          (set-buffer-modified-p nil)
        (goto-char point)
        )))
    ))

;;}}}
;;{{{ misc: toc, exit

;;; ............................................................. &toc ...



;;; ----------------------------------------------------------------------
;;;
(defun titf-toc-mouse  (event)
  "Create heading selection x-popup with mouse EVENT."
  (interactive "e")
  (titf-toc event))

;;; ----------------------------------------------------------------------
;;;
(defun titf-toc  (&optional arg verb)
  "Create table of contents.

Input ARG can be:
 nil        ,create plain index to separate buffer.
 C - u      ,create occur jump index
 other      ,must be mouse event, create x-popup jump index.

VERB enables verbose messages."
  (interactive "P")
  (let* ((hlist     (titf-heading-positions (point-min) 'strings))
         (buffer    titf-:buffer-heading)
         elt
         )
    (VERB)

    (if (null hlist)
        (if verb (message "titf: No headings found"))
      (cond
       ((null arg)
        (setq buffer (use-tmp-buffer buffer 'clear))
        (job-buffer buffer
          (list-loop hlist elt

            (setq elt (car elt))
            (if (string-match "^[ \t]+" elt)
                (insert elt "\n")       ;sub heading...
              (insert "\n" elt "\n"))
            ))

        (when verb
          (pop-to-buffer buffer)
          (PMIN)
          )
        )
       ((equal arg '(4))
        (occur-do-macro (concat (titf-regexp) "\\|" (titf-regexp 1))))

       (t
        ;; the ARG must be mouse-event
        ;;
        (if (null (ti::xe-window-system))
            (message "Sorry, Requires X to use X-popup")
          (when (setq elt
                      (if (< (length hlist)  20)
                          (ti::xe-popup
                           (mapcar 'cdr hlist)
                           arg nil "headings")
                        ;; It's too long to be displayed in one x-widget.
                        ;; Use imenu
                        (car-safe (imenu--mouse-menu hlist arg))))
            (PMIN)
            (re-search-forward elt)
            )))
       ))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun titf-exit  ()
  "Exit and do Header fixing, untabifying and other things."
  (titf-fix-all)
  (titf-mode 0)
  )


;;}}}
;;{{{ Examples

;;; ........................................................ &Examples ...
;; Here is 3 files that you propably want to use in addition to this
;; elisp package.
;;
;; t2h  shell script that is used to control t2html.pls,
;;      it defines all the needed options for my documents. Use it
;;      as a reference to make your own shell script
;;
;; t2html.pls
;;      v1.30 t2html.pls script
;;
;; ema-doc.pls
;;      Reads documentation from elisp packages that start with
;;      'preface' and end to ';;; Change log:'. You can feed the
;;      output to t2html.pls. Your emacs pacakage docuemtations
;;      is now available in html too.
;;
;; r-xr-xr-x 240/222   5628 May 29 22:43 1997 t2h
;; r-xr-xr-x 240/222  52461 Jun  8 22:50 1997 t2html.pls
;; r-xr-xr-x 240/222   4827 May 29 22:49 1997 ema-doc.pls
;;
;; -----BEGIN PGP MESSAGE-----
;; Version: 2.6.3ia
;; Comment: Base64 signed. File: pkg.tar uncompresses to approx. 71K
;;
;; owHsWUuPI1lWrga1UCYSCAmJQail205PZ2aRdvqRzqxyPaZcWa5qM/kaZ1Z3DT2t
;; 6rDj2o7KeHXciHSaVgPLgS0Clkhs2I0QS4YF/AEWIHYICWm27NAsZgPfOffGw4+s
;; qummkZCwuu3KiHvPPefc8/jOOX906/kvvnur+Vd//sO//9v/+Jdnv/5e6513Ln7r
;; F279uPqPLyabf/re2j/8+6/98K+b//Sb//Wg+uLe33zvkz/5i38b/+t/vvvPOz/7
;; yY+/9Ze/+rOf3vnJFx/92Xu/8cD94+ZPP/uV7e0//KUf/ejv/uBHt9751i8P3lWx
;; 7fi38Ikbk1vf9KeGT6vV4t/mfk3/Nvf5t1arNw4O9m7V9pt7zWZrv96i9/VGs4Hf
;; b5wzfBIVWxGOfGVZbhzcvC7whp7l3/xeC1PLfv+PfDbe3x04/u5QTURltL6xviGE
;; 6D3pnlz0nvYOOxe90xN6uPZoa2NbiA8vjo/E0yjwYyF9W8SBgPnEnlsNXSXUMHLC
;; uCqOrUsp6KkIrbFUvD2MgnFkeaJNG/B3+UkwdOw21s5Eoy7qd+8eiHqjXT8Qv2NF
;; jujQVYgyLTzEYdYwpsX3X+Fdla/pURgMgmupqsPAe6gX9mwmvnMl6tV9prhba+02
;; 7uKf7b39dqsh9A2LkyDuS1daStq008j85PTw+THEzkW+mDhK4D81ka5rpBPxxIqF
;; LUeOL5Ww8DyeSOFLaYNYaEFEGctIiVEQgUJBOaGMoA+jhenEGU7EMPCvZBQrEctr
;; 0AyGiSd9/On4YJI3MhtnZhM4GeJAnOP4oO+6wdTxxwI26cuIV0IK8W2cmckiqlXx
;; yXnvovtp9j59tZaeJxwb387IgTzDCSQg/s1i2ro2tHwxkKLkBji+JIJIeI5SONoo
;; SebHZQpgtQWe1BSHrBLWnGfFkJx0tumMNrF/iGsgkfFsQdX3aNlMKFzVMBYqlEMw
;; Ocz0ZFSMb9CMwU5VEC/EMp1OR9hS4YUVO4EvlBNL6F1Gku6FXqZ0pg6OhIBhJMG/
;; tAt0cM9W4tINBUSQ7djxrGiW7yat0AFVY0bdk496/dMTVvFHnX6v8/ioe86a8gNf
;; ruODVR8TH0Q0TGJRZAYLnZHYEqWyFY2vPilv0M+nJfHgQab/bdrgr9P1wDaS3HC0
;; 3bAaeamYBCre0U89cOxE0GMQzfRW/lYyFknkrj0QI8eV7fKHp8fd3fg6XpeukuYI
;; eIsYO1fS3xGJkqlKirpdJkj/FCAah+1dRJZqElvVkbMbJoNdpeCF1jqihzNa18HG
;; d2LHche8pxBYoBSiG1hJPAkiMFvKY0RJvwqJD4VXW6J8W2xDy0aNda27SI4XNNfv
;; Pnt+1OmLJLRh8Tvml1ZASNKGqi7IZQ4RdEjpVUmUJP4fh/Q1pK9L+hrhmFS4eRbg
;; ul+VBSFKMW659HpWYidkbiJ/5pXE61lbh4alBU+EfZHOUoLQXHpG97jTOyJlrw66
;; OTP5tZTNFZXWiOkgcsawEDdbOEjiWBtGqQwjMaZQpWvecB3/Ur2Mg5dZGHzp+KXi
;; 1tBsJTeae+GvfBE7sSv1i/zh5dRem3tg2TZEX1g10muM7qtf8SOkZw0VbV/XpHLH
;; 1gaB9yPr88wm1vTRWhj4QlyvkieupXwbKRPfuTakabd+XxC2y6+edr6naZRSC5pT
;; lz69cilnitVfusfLUuP4mpIb+WGNLL0mdvbsLE9xN2jEWPAqhfCrlQqhYwpKka6j
;; wmW1PL/oiApji00lLhx/dgZy0t0Rz2mrVhqxGFrDSwItX0lvX1ttIgJ5x32N0Wjn
;; XqUhvKhM3IKOtC8y05btOf4jrKharmOpqi/j0pIiLTjQzAsSZbhAFkc2Dgm0DFzp
;; qTcoVW/aEXEUJFivJkFAWVksnPrzq3WVGkBU3agGJaOrlWro+HYkp6Jre4FvG3E4
;; zKUKqD8iJVQNDRnpQPf1FdXzVRwlQx1jObkh56866isa3ZIjUba4wY/8WQVv1Upf
;; Yucx3pB6Qua1nOvfYAR6Kx0CdBO4KZGVNAqxt+Ij81fgxZ+XFm/+6/nTOAG6NUF4
;; ZQxerSXWw0oNTQBqqaopRhutVJJXFeLHW+hJ70Yitj2kf4Q4VBQ+wfvcWgydQpr7
;; eW2Bkv9KGVM60NGyITB/zB7WoD604cpKq1Nce4HtWaG4BlbzqIrxASzx/q1kzslp
;; armAYdEIU+lWST9CRfiWcYJBzw3CM5k3CE9rVufZFaLxYiyLXyfcwu2tkm8Y2PKt
;; b3d4s4BM5w0C0hpzrSqeufKtBO0ubHWpGP5qsoaX4xsRyJKs4c2yEp03iJrGouLN
;; vF7OdMc0cvhOhxM5vAStuLpa2Jsu7n8AT4kAckevAQavVuvGYOzXhjKUaYLxN0G2
;; NyjmKdZmlSRifMg9nlwdKcBnMP5G97y+Af2Ow8r1xH4914T7XlToW8WImlZkv/lO
;; Aa1eVID8kK8tG+pMd1LqDgOF8noFnhPiJOByLC2D5bUTF4tDlup9Uf7OKJNGDicB
;; 8Eb/2XlbvBKRADoV40sxHgocPh4JStCCsJwGtqV7TPQGdSGGvU8VSUpdi/m0d9TV
;; OimP2pEBMy8u1rJn0iyM5EiYh1R1lUfZ40nxMdFj4zXbSOKez+/TtkC2l96dJvH8
;; uwKB9TVud1Wm4ve5uVjog/1gnTkUooK4MXLt4oMBvmv5g7JBCPkKtq5SmW+4tJY/
;; t/TztPrMX0imk2I8fPJX3iXvgWkVN7BHl8r0s/A41o/jhce+fuzPPbZQwIsyabjw
;; lK58+am+HXpKD9fKRu3ZgoeibLQNrWatI9PgIugw49vAO2N1upOT7ilYFH6o4dIF
;; wghGehO9ze/mm+sxv7b/X9/bb9K7/ebe/j6+9w+o/9/cb/1///9/47Px/m6iol1u
;; GbKzkutyM5Nb/uXzxKOOZ1uY7v+544WIhdSzZstJ29goYcSTIsxXOmgSLq5Sn93Q
;; e94/amvTfk1/sNgbKux9y1mAWf1dOZsGka1wHIejAreKWsL5Snr9kYyIr71qc1+0
;; qrDPwnszWjB+whOGZs2MGPZ3a3doxNCqtevNbMSgBwviiy++mFdKNm44I44MJ0qo
;; JAwD6NA2b+nD4wczMaDOeJAgWkKgSzF14gkqhVm6n/yZJaTqgeJCZZpRUVg8nKAA
;; QxnkBzERtJOh5O3KGfvUU7donBNFQaTEFtPZY0qt7bSp3RkESZxNL1xrhj/nOJWQ
;; VHz55ZcCpgIcgfcAwjJr67MhWIhb1JOapeFHlEaBi6ydksHVRIWa0Qw7aA1196Ur
;; tgrNmiKkIz7nTArf1aGyq0lSVZLNKkGVrXYhFX5g5jK2dqn/Ppa6/+5asQSsm5dJ
;; x1nLVYFghKsnGG4QXOLvS5kjOvFBbI3pj/QCZhkZEnosfUljFVsMZob7oyIgpfJx
;; 4HE3Sg+GzKkZkazDr+vCJEq96vvQM41nXiVeKCDUVEqf5RnAnXEPZCgZlcM4citJ
;; yDzyv+1g6lM9pqvOIZIJd58DJK/SAFLyXZbmiWj2jyvX4HrgVVCqwvWr4hyXTRzC
;; GCEmXFxkXp0q9Wniw+SyYRNszdK6JxCZJqR5m3p28lz0D8/1pq36dmaBJHMYOdAI
;; 7nWUuC4sTQ+C2BvIOA0ZEKYh3RWChUVdKX2HdCs8RKMtopzNO7v9th7GlTMCJBmb
;; ANueRwdzV4S6OIg3HqluCprEnr49HB8msR4Ost1ktHj4RYXEoy0Yz3ZVXABDGyKq
;; XRD+20wSQaL/7KRz3C08Z1Xkz/UA5eSif/rk+WE6uDSLz3jESRg3irUbsm+bOSbH
;; ENP2z4aDFepjrRX8Qve1IHBxQUVN1i4CcwG0SlmcEnh4lw9KAxwtLZrRra3lx7Pj
;; b41ohFwBItk2DH/88cfi8PTkonN4waO3ghx0A5OY/Xo6nVIXEbqfArVTvN81sX/3
;; O3k+YHP2rStnDIPOyOB53llgmdIA4AUwB23YBTNKx8Ld88N+72x+KHzu0eXOjXTn
;; h7k8vcltjQrIGMh9Yw1WR2MRW1wArPkOjel0OE2NcSsLAfI6dC2H/GkK1E8xFfuj
;; xNeXsC3MkNjYIw6fCVy4485YfjkaUVfmSrozPUbujTgsg6afTh4pBxTYta5QIJGX
;; kG9wrtc8YTPt5DnpyOEIl2cmYemjjSNWhTlnSkklRgxbm1hXqDEtfzgT2BBCPMVD
;; ZNI6VKgC3zL3oTWxw1UOH8IDV32Mo9MXuR0WGZEE02b39FDQ5RGVFE9a0qkI4tIN
;; SFZMJlPxooLCfegF1cx/YH7IQW4wJc+dawzkUV77QR4BYag8ca5mrkVBHoaLKG8g
;; wNfoBXzQH6qjgDMOkyrjjwWAQgGsL68chgeMVopwJccrBrBgdUUc4/rjoE1lZ8zZ
;; b2opcZ+Lp4em3FAxRVTOmLBMYY0A+4SeWWAx6wNlnUSuq4rHyXgHbhFRs4CADU74
;; iCwFGcOh5jkjjnxB8Vz68hIXtbErdWuCSTt+Gm70+Tsi8GHuOoGDNeaJmB4FiQ8O
;; emSVnJB5e/U2Ah5lYhyrvcxzlFlgGiADSYNRdoqqOExZ01ouqLNxN1Vna1f/u723
;; 367dnVNnx7YlNXN5uu38Hqwo8bmpu5NFOrJHJMt7oK2tjkGCHDu+D2Njpik90Mgs
;; FV6nSZDPuBPeDIFW67RkJ6ELK44NKmT8WzL3REGUPC/t91QLbHImEJUrDg0c2FNs
;; 6SfeANh+SQV3UhXs7dbqqNvvtFt77b07cyrQcXLkEDjnjF6v1tPeC7X8EbC4sWY7
;; tv9lrMEtwuMEyu8hPJnoTWFq5ES4YMMLIsE0N5yiGL6cmqSG2p/gFFnSzKAkFcqh
;; M5qZkY9l27BnZexZm67BWVUUyVcOUhTD9hw9MBU6eyBFAP1EibvKOA5SzTR3SUuN
;; ZrtRa9f2VhhHkV0egfgDSNeheEbHTGAfFJ602FhPqiN94T2FOGaajILAADV0sMgf
;; Sgb7/mZMOSm2jN1oGiLxXRKaI3h+uvYuxe6i1dmXxI+dcUezGbKNfFJTEMNULxyc
;; dYDkXrLoeDBL0pn0x0i9E/F5EgBni882cb++HqEYGRyl+acz2GnVzBsECGUaauf4
;; jezX+JHSrD6FedmpVUVaQzusP8cfugkx+Mnmp2I4sSKYS0TRPli+tP3CpbXw73q7
;; cafdXOXRKQZKSys9sjEnMaBtTAB/JwUgRE+p/bKhEWkGhLQAH0fQCkjEXJqxvn+b
;; tJFFBNqexQHKNUXEPBcKNiubfBtk2UTnovOMgS7BLdqiQzVxTMGA+cnMBi56FpFG
;; UGjOTB2Al0qHYxVT+p/AOV1yUGlzseLK1AZnaYXKKXlAhQiuCbHoku5tWd+tVN+N
;; 3fod1ner1W7N67tXTERZPjAyaPcli2ADp9h/T+cDzgVIw8wab0UhQxlJ3x+zTWVc
;; yBCKfUxIjZ+YtPEYolgt6tcWt18+YGKoongf1aUc1zjzEEnQbyLXI7Sdnhx9X/vV
;; SDhAI4aPJNQ0T2Cfkfw8cSCFaJgCEzfk+ITMaAQIS6YHrao4JS1MHdxHioZsR6pl
;; ne4VdAr9NhrtJmLPfOChlEch5AsEMzgagBlFOLZIeS2HOisvUW4uUUa8b9TnKOdW
;; CFHo5/aDl0va4pVZ6BYb/e7TgiqfmMBlFIOIbQ0JmCIBEFxclrhR4Iulb9cbixIr
;; TkKM9hAD/LFcolLPqdRI0n0K2PVVVDQBxRBVbD54sEn6jC3/EjBbjAPiHAB7SFZG
;; 9RXA08MdNgIKR/fPO8dnD5ka2czmk8SU9bpvvamtSAd+AK/H/Yc8Y1uWujbPb+2g
;; vVdbTL3a1plFU57xvI7ug66fQjWNYa8cO8H9M05YPKd+t3AOaboJB23vtVbFRM4P
;; JlnkXuUHfgX2HbhXnD7n01Qa+k2NhGSl2xvU84kCr4gTEJrIXxjok+mnfkUVAV1L
;; SlgxJeIl0WBfJWNcF1VlSUSQSphcV+R3Sb91DW32d+uwKkrmd9q1/UX9DlDsoHY8
;; 46EfZ1rHT3hIeCNqrB8U6JI+DxgYHMzRvdCpgsqszpMn/e75OVXwsWRDoWBGMisZ
;; WllH6cN+aiZCsJpew8H+PAc1+MveYpbT+XQYJGQ0CEDa8gfJGGbfhZOWtCWlXksJ
;; aRi4ieeLg+UDW/mBbEJ7BJSb8yIfslPZotw/PH/Ze7KpAGASshcDF6iTgIPmHiDD
;; XZFGEBCx0svqA26rtsQWsqmMwsDNkHB5u2CusKE0LXsEkBzTkBxSBqMARAAvlP4q
;; Fe4tSYRouLcK3aGA536gNVAZZlki10zJ1Xc5frfae3Dnxg1gkeNl2naCBDnmQ8Zw
;; bJlBWNaHbgcsi9AonNlkgIqQV68v2eF0ElBbJ2s6EnQxCYgiFCf5SAbR2PJR3xAK
;; AksoPuQ4iHSCMoS0j5hrNuAVhpxpRZccbNsTRm+DhEIpOa2+WSuKLO4bc8cDHGGV
;; WqpffMVTxWy0WNGtIMsUYrAIajXQnNGWsOexrrFQplCLjGJ25HA5Qptsyd1x/LVx
;; fnbUu2ArJJRguYW2i3Y67gmNUMskumVMLjpDcOBJCd1GiqoPXeDYkWMgXaKoV5Hp
;; YCDjeEWhVa/nt0URuUEJrl1bZSFHvZPvwtzIUftyyqjycRLHIJMGCI074rS3faZv
;; RvcyswjGDWQd3uAbPiVOCjfcvDFKZ5W2RU91vTCeXWDJDjzpUh6hrPyQRsDpUWkp
;; QlDjKapecd773W5mwWli1xd+ftw5OtKVeVqQmPitAx0lWpv6fksqqhVUdIcTFZxo
;; oeIiO0RUldcFg75yXI6i58+Oj3YIpbEd4sH9DsJq9+lDcf+o95Cgw/3dzkONJyfI
;; 0VhZRLvZLixOd6ab5tsIho8NFP2Vx53zbhbRdawADTiG0b1u3DIm2Aq2dQakUnpZ
;; /rvz4lNUR76qr7CQUqWUli2wZ8efOAPCpqYxUATW9H4gua2m+xAZlNfmrqtF03sH
;; +4wxlCmGCkk+T/AQ0xQccKGsBaP1aGJuXn+eB4DQJpjr0ZSpBKUaWiFZ6Q8eLbYA
;; ThAgM296gZslSzTHUBUaw0DxDx3IFB1AdhiYxoDlajlRJlq6TuQYixBEDuHp4GZG
;; iAgOYovZ+WAc3xM6tXSPxe3bvOn8on968ky8fKlxFUO6FNK6aQlP0HbpJu8UbvKA
;; q6L6DX2lrCeAwjrOKvAr3GapWjImVFh+3L3ocGhgZ1cStfCEynLuL5tUqA0uDbIU
;; 3zR21QYjVVtUvEv8bxeBHxLxiAjnjSNqZg8Sx41X4D1t/lqLNG8iqW0aOznKVOyi
;; d36a9sK5jw/neJVAxM9o3WeLCjsoKAzgBniqCei+SmGlnVIe7V6C7Euhp1d0G2fa
;; z2jwNgiuZPaXpPC22G5zfFAgPKTnZnMxnZsDxgoZCcO+9TA2K0IQ+Zww0chkyQD2
;; C/K0ODXXAO1r8101LsWLsEyOKSRp1KT5uH/2UNeUullGj85MatLc6Jw4KuSCfhdJ
;; b0QGgXAsdXXID2l0bLnUWZmJs4J7cgdfa0O3+BaFaS0IU2/XWovYpjjlZlSRwoQ4
;; QGEK9MJDTjugCEGQLx0scF/e9ChZonq1Sb1xsi8Ch9wODXyaSuFCba16XAnPVEgu
;; nXNM54lOohjBlIz1OXHu81rqx+xVsBqOZ1bapjPOR8Fbj3/zsVmaJuZgGVtChHCa
;; DyZi6qlkCU5UYzOaWOhgppNk5mlR23sFbRskCVeY1zbnD5HQ3MqKCFS3uTcBH938
;; b+qstLltY8t+tn5FB5JNKSFBU7YTW6JU1gJHrGgrinJeJk70QBKkYIEADYCmNbLm
;; t89degO42HJcU/NYZZMCum/f7r7bOZc1zyYQbCcxhSgsXE22F5SvcIf7lx2dwbkN
;; kA5nwKNV1za4xnz289aLInY8IduRUVw1IgtId1bI5taLIhCTSWKJlMaMlE04G7v3
;; 0WKSXqRyErVo4Pn9/T38f3d3p0dkUJSO8VXt2z/iCUlBESvZpKtkY4vgbgV7RauF
;; riM9kc+xG5qLYZR0LVo6k0PoaxJn1+EgX3/dOj2oQpaJwny9IirVNe/07V3l5I9z
;; r3183Nqv3G9soPtLjxczr8X2il7z5BZD+0c0vCjspn56W6W4HAcBGobklE1NbTRW
;; Wa8C80Zh1nPHUQWX5eUuL7w2rBR8EA5fhQOL0ry1cTqkBoH87AjHtLecbbEG8uCh
;; GrbNkz6LwmdHNOQmJLLER5W1Vr/UK6P2mNUd080xVkqcirWKXOGt176AOwE563S0
;; gs5Wyd/Y+HPzLx4Je1zXqz55YqZuiLt7MftZZQquhm3DCLNlWoWCr5aM5QZUxwgO
;; omk63e9ej5NugqEX/HHXkUpCcakObZCPt+r1XuZOct8dhPXxpFvPMtiXL79cPAfH
;; 0lktxEqDqG9T+PWRt3cIO8aT2theuV9R7iT9ibBPHePkP3YmhlHKmfgv9qMo6fnR
;; ulgbZUOxAZq8vtq29Ieon0F8mF7fUjifYthOA+yp0uTzduu0syUlcWVz0YEttUWz
;; 6Z29QcuyvZSGveV8tGUMRVkovT7Afkwv39LXybNUPZ0RXuRMZWrNGJIc4kJxGVPF
;; mWDGxh5GRDRkyN6fI+DGWZCPwxGGEkaYVOuyRCz+YIDMbvhaZaWqVANgn5+FPRH2
;; Ax8VIoIBEiOcEiInUJ4KtgjycFSlVNCCtBRFrAHuIvJvsfyrdILedRzCO7lcBQ61
;; VqvX4b76AbVFUBAVnCsrTxubz56/+PmXl6/EV/5aYesSDXHktT04RajTAGaEmUr0
;; hpNqVDerz4rnrLpom9aw51XWRjG33XAI+0RAznPxc6HSMy1BwzNLxMuqytN6xo9S
;; r/8OfuRJBLJuCEBeAQxN4uEV85RYJapZut0Yc6Mbxu+g7m/l8wvCWjt6PBYLD/j3
;; fz5PTeQNy0oM6aN8igwSH18mXlTFz1XxixrtXDAqA1x1rqs3RTSq06e7stlCZ+U/
;; 7liKB6PtKROvyDQaDWskhBC26qdUK+OW2Sdx7nceZo0zIzcF1cuAm7CoZuIIMJXC
;; //94zp5CSVT3YlxWFSlzQNhCYQDO+h1xk+CCSz+xWbV9WvhDKPb/Ay0igX8MNiDE
;; fe1q+Kl+vdEJbBbkYS9ghkZjJk31YRuCcgD2OKDap2OvFgRAGMuDKMo4+JuZSAMz
;; n4icm2Ij7JkjlMW5jBqbJEDeG2gDNjMl4NQNigvGeQCaEtbKyBiOPUluzjs8MIf5
;; sbqxOW/4M1f+kgCv/zHsMT1yEwTYLR4GqPC8qc+/cqqe6wqTkJK0b6icuJtC0vqy
;; 3ksFfOWwZ5Y+NIjgHFQNmdjnrhIV+mwkgTg863C7C0nKAEr9WFX7SyR88b0O+EpN
;; nTTtTFvqNDlG7B6BbixXZGIZQwXiD3KskmwygFamhFHSjBdWZ53p7hrPIPKpG0QA
;; 9G1eoQp5qaSEMvmM4w6KmmTUXbUXNeVEEBSqMMMK19Uj5AGqkgPQb0148YfMkXez
;; PMwnunapQRUXKJoXlQJJmeUgJ1iB0IkFBcZXU7yiOUgAW2RhHhCw0DMPE8J9oy8K
;; qDW7fmpJMGFDMUshisHSEupLmQk030cMsRQKeqtqb1NVe2r//9/D+WzFCyFV1aJQ
;; FIO151XDwOW65SbLaMhyMLOLwQ5ZSZdmniDrRlU1clY4B/+lE2TfVa3PRTf7TEoD
;; smCLF97rouRrSpucehNSR6VbupCrfJpc6TqZ6v0MA3nOjnIbSV3wcyRHqaoFhTTo
;; Kp9bfBiPGaEvMc2MlmqMQlfbWCEzta11a8VWkc6MZSE6ks0zG8oyqpzR0Hv1ChhZ
;; qCWsk0eTWfpd+wyKW4ZtUKBqHrV3uWRBfOWnN6RvJtkmd54APoahT3dreEHuP72f
;; jMaiC+D0hspgpMrNcaKQIm+OLpxBxKbrHYQpHOFzWP3DBNwa62PqMej5gzQZFc4c
;; QYTLPp9NmEtM9UvUchKHIAtuHUf2jafayRmtY4qscxekICopUftsHIEmKPuSNdWx
;; DrsdfCgK0cp2MVZ2adZDo4Y4F4v3SRirJiTtzLXtlu59HIEwYlP1yZcwH3WnYsVt
;; gc0QO86dyK80P+6P9UNsZ2sbb7jFJQsQJhON2jNcGU1ExeKmd2Is7EDVpeV6uGh2
;; m0sXeVH7ZWaRGVMuwyWYgJoo92Er0q0ufG2DKLesMQCEssokDq/DFa2BuJ1MNDFP
;; IvTZYp8N0iXEOu6FwVRj2Tk1juk+jVurOMPKu/M3Beio2miQQc09Ccm3MDAnhRld
;; IdaWuJJRRGmnL2c2GlOotSBRAdao1mH5wOhlji2uQK8Y3W4Zty+wDc3ztreLyulj
;; o96NvT2KWFDv7jbr9OWK38lf8NgiP8s1erzic7vSoliXAWIoSjZGh16UZFqFOuqg
;; JwE0wCwTRVO0S5mgi+jBTnf42cb2DIQRtTNapirQB6tgXuiDxGVozqokaLF0/Ld9
;; mvyA3EoFd5qOwhj9EndTh5BuGlZ4FMvlSJ/gXhhjjcKEOZNYszMm3riFl95glDc5
;; ymqk9ZO8xqVzxa0wTRaOILKGoDFG2SSuTa+hMKIOMN2IIYya+yrVYCuS2i6yOQuV
;; A8Z/tAu3XLWTTljo+dibKtfdv19jI4lqAAF5XbVsAgq1tEAJ1oCPSaMnClIXoytz
;; t1upVmgPuu2o1Mfj5Q4rxWBTRs0U5arRyqEkmRBv2YX8gydIBygXuaUKwqolUVdC
;; NTCyWjHngllUwlZGLBB8buBqwgGFnhzugbMCJoqMcpFCqWaXioAqXPlWwViu0Jiv
;; zN+yzJitMnAceK4Vp9XnR3z14zIREGl5usol6rODj3eWrk4IWa3Of/AG1eZWa7KV
;; Z4x5te290Q0+dUADFbgpOSdcAMD1hnC4vUmKh2pZitLI48CxZW6GhF8n06s8uRqn
;; CTU5t9cx3GwJ/hsW2NguuuR6Y6P49+aGkdhw51ZNmClKeoEzCmx6RlxaFNb4Q7dY
;; c5ZEILkL4CQGRIObrsG5DkRChbxZfVOubrAPmUs/iMJRmDNptW1SLeEZfdAK0s2F
;; cegnhTYr4zqYoxfXu93iur0fYg80SW+RizHGrWtCkmBup5VjLzqZZuR0xPYPuPJR
;; J5ZxD4UIHXmYhOmKsZr02oFoQd0mNDYEfXXQpoH/bdapTqCetT3N4N147vtarWju
;; i+Wb+StqIIUNhqfX4fA6gn85w5Q4TxOTsJFw1OMH8G0VteBB0wDeoycVETFL1tph
;; pM6K+Vrbgys80oKwz7X/0TCaqE7TbNHYCB43xDa4y2QYIw/i2pEX76qfYCJUFRfh
;; cSy0bNCg1q9isO0FGONM5CQDq9QqZtHSlfJ+aauKlhn5fUts8abmDMfwP3906R65
;; Epkrds7IolgaHhE4wvvjEF6AtfIZFpXknVRYgglMepyWrESWQK4xR6S6OiT4mcgk
;; giFeCWF7ljDMXETUGTdjYjBgL+6zIcD13gTjXCdf5NnUGqrgL9GmVE+V4GaB0StX
;; lSp5WvulAozlqkDEwIoCTBfq1Zus4BrWc7Luj2EW0jHGDCRNaO0zN4SpewRozmKp
;; YuTLLElosppjY74CA1Qw9XvXjDrNJt8GEMzC0ThJczR2Ykm4e8mXxnQFyL+yFjB+
;; SbclyQ7JSmzCtn8/8k7F4dnB5Yl32oEf3sVppSPenLVP9jqi3fr1SHZm9yGKQ1VL
;; qDBhqgY2h9AJ4jLSOeDIhJ6m1MDB+wOndkUb4CmCSAbTvwVIAbOSVOdIVeU1K/WI
;; JCBApA0g6d+alo0xcFccXAdQwufXUL8zmJPuewEhg268qB6dGcYNij9jBaRhCCtu
;; Qy2lu2XdpmSGJy9d3U1NAVBc9VCVK30tEl1Q74ABQRXggwJAjPFDor6odlgxrob4
;; +Rc8avQd3dwhMsVgdnEFYP1KonUcdnmx96vkBNfG6ZBapn9Ckv6LVMfygl82OXFn
;; HG9rPhtI5cQVbxJwfDT3CZxnikW2MmqeWYNcTgfj/er965znHQbo0RKljfy8d43B
;; fBykEYTgIezXLQQy++OBfYSmzL2Mw091yBA9hGgRbteF5cqN5PJHXyFxKDgRLWhV
;; 3N3dkZ2sivv7exeLmoUiiikEG9DSnjCBzRI6C+WUMII6sUHULwxTaUGh7wGGLgrM
;; Q7VxXnnhQpCFy4SoDlFs4gunWvbzNx4RXPPf5rCCD4SE7V5G+WPgeD/Mej6o3lfx
;; swZ7x280NVdi4UNF6VN5izbJsSNJwyEgVwkpaAbLCB7hl3ey1zp+RNUkgrlerpn1
;; AIymUIHItbuTHBeEQkqrug+PsIWbiGFC4DEZS15Jm/WiXYJZoiTMhTFyNgxOMTN1
;; WWgJ1Wkdxl/QQcM9ky7gCngRLST+gpCY6uo5AqQELEKLEi7BHbF+Jeo9YeqLzJWu
;; BSwc7ClHJgHf48kuPBluwej5igoFN5Shqja6oYGVPbEvDiieiJMg9yHL3zKNKVpx
;; L5r0A/1E7MFYGHxAve6FSyvHoUoewEQW+GkPm7NDtEkXQj5n0oUCbqbl9ZgxlwUN
;; OACn7oUCMExxB9iPb+kk9KbZxwHW6sG06UPyUwq4Zt998xAcEnFkdeGSI/+TaDx9
;; +pTqU9jkFw5BqoOOVkM62wj6XTETZcKaqWjIplgXsXc2as8XKsSMKZ8jTh1DBWoj
;; K07i5OrZ4th/lEzBE6HYY+ZhnGRcTRXzcz5NFkp4L4sFmoEXE3yCOCEbQcTMKx3x
;; yUIxunuL1yPjfs9HCGO0AKebpFi7LZRCNG2irjlnt0Cl0gkJhwOeYNNloQC44Jp4
;; 9cp1MaTksDX0UthAADugo1W1Ub74RLA7QuVt6YpLgMb+nEcBBotJFthb4FoSogIU
;; QCnkvV4YZJpYXCiL1iuQIh9Dn8SusvJXVJKEi/NamnQREcgbRUkWQY5yGg3ZA15y
;; EYFqW2BTFZwmHgYqhVPchBtZciCmO6TLOHXyXCYwbsWNZJPue/CZJWkaAoqBqDEC
;; T/uQfeIcCh4bd9NHj8hR9V3iKajrZIetFtyM9ooJ+9GjR1jfgiNhUTCFzUN06AcD
;; fxLlGqLJ9kolw+FOw30KRp+niVOmjMCTJlSlgkhnlce4OEeFH1g7BIBBqFoxzubs
;; uYRRx4/zuhNzmGQieBWTjBdJkUOIM+25cCYw5ZJLQ1iFymVZntH5l7RFLEcbWm24
;; jSu5I3LZolG74vQMUI13cHYCcOfQO1SHT3xWyTNgE7kdJtct8Gh5JAe6jeVVF3Y2
;; +2Iy5h4F8Ui0Bep/jpMQ6SW20SVFqqwrCe0qlDtGLZm0zXLQJnWXZ080C1lRg8TF
;; boCfxwZG5KMxFXS7XKkZfmneR7V4zf3QEWld2R2oklleSnNVyDckL+hLNY2uJr+x
;; plGnXKprpBLc3SkkebyMPMwV2UlO5o/HkJTpRk+DHIrkcUC15yCFc2BJH8Fa2cIg
;; CWaoYBgPsNNAGURW011xut8uKHoYdCdDLlqOvbfesYvdKQhEY2QF2AygKoeKojtZ
;; BlTkmnKd6/UgGm98rk0yfxjwAFaNnOcaXgooVwLZm6nhg1p2rfSn90iJZ8RvA3xC
;; npDrGzg57+wNTbun/8kEhCPE7q5YG2VDBwtt1ZzGB9tGyVW4llsxmMQ9usApgB7w
;; mwi2RaKCT1A5NLZX7ldWCOKtMIppYXikex/iC/effcQTOAH4BgNYrX2XzwpkDVDS
;; Q/KD2rjiboV3e+hdHLRb553W2al8IiS5AWfU8f7VMX0tgCF9P/eRTwUxVU7XppOW
;; 6flwLQ4iA4dqIcn4IdmATBkTMERI8wQ5rXV6ftnRItbIroWocqFaGNn2Opdto+3T
;; akPSkBjh1sXaQGyIHeGsRWHXtfbsbBdGXdGo11fbxclpkNOLp/I5PdgRDTwOnARY
;; 1XHE58/4e+d/RP3vd9lPa59xt2v10JpDJvKdr++IQ//Su2sHlIEYcEJ4kukCo6P8
;; ybSFerPsDogl408Vfy+/hU/ah6rl1bnVh74kF11yX7zJr7ssXse+LlhpHS7lz73a
;; fz2tvfqrLjbo8Z0OSnLODjoxh4ggyuQscff8Xs50YeriuZvbMrqsmMcUEr7rdTNt
;; veS6H353lmfRmybc4i6kTLQZRGGjpB8OQsmUG+bzq+982cXydmYutip+xHUedr/i
;; 7uX9+p+J+9eGuHt2v+7+tLHspnHWWoNcN3G251ynNcB1tk2NwWe1I9bmXfhMEiCS
;; q4rM6/e1BMp4F8SgLQ/bULPmgaxnaTh6oM0OLjGYToJtrPgWi10q84hmEBcBovR+
;; 4C6zg9fCfKq0INF1K4utwWypaA9iDVStwpdPXHAVbgCrnCJPsYZME7xCToy/xvwV
;; i9LAUZD7SIVU+edvkvgxg7TV2Uq8Bn+AGZB5xIY0hoen9NINUDYnnVDsjtbAaf5w
;; eHbQ+ePc43Hnl/vHrQPxzqnV678fPKvXDzuH4qhzciyeuZv1unf6ztl9FzvCtSWA
;; oeCzh30sCXjJkzj8BMncz9cbG5QtIF3H9AfWQ7LRMYnRQPxhRstZEh6+fFmHd/nr
;; 9dUNTdxukSV8WeyshAPJ3G4pIxLNNeJ1d+dLK+zim/ZR0AEr8+PW6W90TMRUKcrM
;; 73/0AZbA7zSZZgGzaoHY37vwACcMYwy8Welu3+WA/YIZvKFpbMPG4pWNQuw3wQoI
;; O2YkKSHuvG1aI8GadpechG15aJi7zSNv73DeDHtkp9U59nbZm5t1/stMsUfSeRy1
;; vTc77xzydLJ4GqtHSa8cT7Jr8T38VWB0sLEQ+SsLEuLE6+zxdY7T5CP4hXDK4x3s
;; h0iuodiKQQHXeT7eqten0ylhWMQu2EKoa6haB9zXnaR1rI7qKJuxLs238p6BLFYw
;; m8l7uKKjOG6n8Pic9c8MBU4QqsDlAq6DP6BUa8H2siC4wXJ+L8r9gqS3kCJ8MmCI
;; C2jlfr+PRFNCR2CJn8R9xYPNtD9QUJhnQTRwRYtpF+5sCE3bUysb8S1gaexmAupF
;; UI/JqVqQgwtMrxNExP5Q42hwK3SIH8xQ/QuNZ51MpwhYwSKbdONIlYANqr28w/CQ
;; H5yddrzTDtqmfQkclTcKVcLslWEqmrkuo0d5ZYuvf+eI0sooa96q3wA+ZY8xYi/Q
;; lr9S1E58+FAvKOhweGUPVto5MuiiZnUhVVsmRPY3SY4Rohigj2tvvfYFFDkkb5E4
;; irdt7+2OMwLsoKOQQ1EEnoVRnmxxEuBYUt+wIogU9WTk3wQHyQgtdN3Zv+x0zk6h
;; zHrTOm1hlSUuOnvtjrNBIUZ79p6K6TEU6hDXexjXieZSzGTIzZe+SQrfEBf8KKKZ
;; VlTQFvaDeGKB3HWuh6CwWWxnvNXjML45AnnrjmoOOlWHJvMmF827TKN2MFhXdZdz
;; Xm4tOvAQ3i2U4sz3lHn7yB+0jzzpyS3kX78FVGjf790UOrUP2sLSHcQP2gF2Ch6+
;; hRgUOrV7sw+/geLLd3GzbiV1OfS1ZjW+P7rxIEs8HOcuxCRfAB6wWgmGwskBEDWo
;; A3lf+CJKdy4y7QPSqtqVB3z1Q0Iab0wRIsSvEBdaF2fi5c9PGwgJ4r4PSQ2TmGi8
;; evWi1mjUnv5CxdvR0dbJibUjWgKW/jd9V356/Eft8aj2uC8eH209HlX+zRvoXSdj
;; qY6w10VdIh+WSQsi4akNPuCiz8twggh7qGqCNISUgKGM+OhuFPaoI2IxvaR3Lxkj
;; YYBR15JCZDHGv3GQjsIMed+vra0tKbLc3ZPXIsuvvmnXQuiCiPuRQ62zQApPr2TU
;; EJB9Ap7/RZUsKWbf1DyOAQ1BjYHS4iQXYwB7IZ6PrmScuVI+TBIQgKMzLubD7BvO
;; xZT9Y+6k+tHUv4Vrg2fU41M7xg5XVDgYSwqUbHB8GdZ3mIlM5+RBsEt2MRKoIor1
;; r2yjDYj1zRPXWSZlbn3PfKbPfR1j0dwHywI4RTR77tfQeBmLrbKLPbgUgJNxEK+/
;; gTiLbyFKfv4M0sF+1gZb4sDHNhGOEH/S+79sxogdSLxPwni9UqmK5ptdO8b2IoTJ
;; bxaF1kKJIRwA/pcnUO8I7/RQ7B+fHfxmIvYMTVA/b3u7SwFqAeyJpneyi1EVjAyj
;; W7MOf0sYVR4tCYRlwmfg4VIDKUDE9u4XYLU9eu/wsO39L29W35i2kfT/tj/FRuhs
;; sAEZp722NtBgmyS+s42Lcds743ACBOgiJEUSsf3Evs/+zMyupF0hiJukpY0Rq53Z
;; 2dl5+c3s1VVTe8ZsOoy8krCYUxBm9UlJLj/2kcnMoAAaWha5koc1Hg9+Aq3Gd7E+
;; GKSb3v5luFEMhPjnsO/EzRSrX/W6nYs3zQTXJpeddUO80jI6ATmPus0vaq9kOMW9
;; FREekgYJ8n/2iX748AjY44DVW7xI1zBXak38WzdaTc7r8RlcRJ/mgG05kWDGOFQH
;; ngKr82/kuzWNUtYKH0ROrK/D5pg9PhCpsK+vFWSVhnK0IFunEZtnYtHK26POyb8U
;; RSpvqVeyoomRZ7Ecbn17vEWXoag01NjafjJhpVCqrFPrp14l2bd0vzTxILcFaqf4
;; +qp9xd6cdY5aZwnf61A0qaeONzSdMInW3GECbxqYEN/h12hmhSo/tWn9ygyCWJ1l
;; vKdE64aIN/TGDywtluAfhfIvwIuytlTMuINrr8OImTNNDhWmbqX976LafUBRIS8A
;; Xeu697bThQdqlmU62zQPO2XwHkrVQa9zKZ4gTfwqHi/av/dyyM7/2f7Xb53uyRVM
;; O8dzX5qiVMdMyl2n7ti6h61jcQDxqwxP4LZXrJRLgqeTyyoG4cWlvWIQkJjhvNgR
;; cODODFymnWGb6oBBlROCos0A+IbOIvBLvLLhTiNfzMwxa1cY7uAbO1P73gfL75nD
;; cL0rvcZscQmp4Ptk6G+McgNWHlguHhwgl3W2Htt5AtPQ0kHF5sNKu0ay1badCp+x
;; bCe/5sG7Oj2C6Vgnxzgl/t1gP4rp2FKEI3GWYFcNwgg6S2i8K96860e3O6ViP9ot
;; Gcs2WmNVVtQAuN4vvYs/xWTpHSja3Wk0K+r7pZXTK6wYz6qVQPkxdam0TGNYCoJ7
;; 5fwJZa9c+D/nWo93uZo88kbmdI2l6NGDj3e/ZYyxH01nQZbSbacRWF8AKFFn4Kmh
;; M6+zJVobqyB3utqq5I1lqmyUq8wXVwwMsnjaxTuD9I8TtRhfwHTs2j3yQ/jWZ8Ab
;; KV/QfEA1YptAfMq4K8k1A6a5nmtp0qkASownI2RcOi96U/eo4Ws6Td7DoYPBfpDn
;; fkH6SneYOQqQtMxFKov1lRPhQ8BFw9bWi7SewvFDqWvHJ1ofhLBZl4eTzQDFG1z0
;; FiEdHSm5meWE1joaJEGKR7Hy05/jjqIy+2saUdKCWS+5j7KHASPKlTFI3QdtV5kG
;; VfU9+2EPx9VqOkKq1RdyOIMj2NU8KvFFoARZ/xTFE6z469ROy+WAudVJz2bcWRDO
;; +djjc+wyRlGH8V4hNZZK6jGG0YNjDcviwWK8W8iW61KcBisnE9HrZBfbW/KqlEiL
;; C0rtUKZPi8nlVkQsRqYZ0WsdAcrsvAb9nrR/V247uH4Za41GXjCG4I/dntC3QBGY
;; P67OW2dndOsGxbNr3VEt4DnpPXfCwHOhhF6A9gNshLLXHbzrqdQ0GeJnCnUeDPhV
;; Efb3n1nT5Zg8cnpbawLccSgfinIBgGQdBWFXp/9ukzRNBKV1AwebDOq2WvMx4xp1
;; atb8ct3ptZv59Vwu+Ed7aaAxNfYOZaREtpc94tiqgIAw9o1u30rNfELGVK9SfDbe
;; 9cNdQ0hA9PburtS3AqsVdpUZAqQFDACQAX3JkF5TzEle98OdYnWnBF+6sopj4yQ6
;; MoU7Ga7xju3etCr/3qv8dGuwFGalu+SCEDVo9fqMV89ATKpqsL1DdSotp/UjwApN
;; LfNu4eCOk7FMUpFWAu/IrPRieSU6KmnwaXNTYVNtKPM1VKdwy7xGSjUz27HXtVvU
;; 2XI+LOifhGE8ac3HfN7LcX+NJJhWnz9bhBjRMFLeZS6aJAdIb5ryHBz7Jqk3rW4V
;; SpKsj1ztixNNgvTSlbEqk5DmhG1tsS2/FUAZqekTKCp3YJbahMkpJ79tErz2sWv1
;; llopLazoPpsMvzINLi2YwSCD/GRIEQk0OPACewpTkuQg4lGzqQ94BU66jQMYzW4g
;; V+U0ki4CLifKST42OO5cX/Ta3cSjCxu/zSyXYnawGCFAntJtTshTkLuYDyHRAJh4
;; YHPvo1UWRFces6NtGPcCCzYxxS4uJi8H6hP8voN8FvK0dIf8wbAwvWG6QSyvtJ42
;; 8G+tVt1j1yFOcsz5cGzi5Y9Jdy6VCgu9Od5EeYvpLAolKsBR+UQS/9C4wTi526/i
;; d/V21zB4fHrd6b5p9waomMHF9fkRaOwQ7ZH3QvDKQtY5Bd4EreNGBtcXp78s55cB
;; 2YKqbi0XjxfE96U9es8mdhBGrFb5XugOU5hoAeJ5pERKKtg0ijeVcvXwYLt/B1sM
;; d7/oV2lNEolNrZZJAaQP9oXLf/GCz17vmSv8kQW+Ccs/ymRTNSyak74OOTwx5BEY
;; 0GGEPe9ToLaDz6LAtB10pNA3R1ao8DMGxlQ6jNBAxGL0z/Rav71moQL3f8e7s4KR
;; GVoyYCbFvIjdiektgmKf9METBL8sH5s3tWuV75jlYgzACGOP8Up28vBz1v8kVg1W
;; WyOdOR4jqs7zTB51JVbS9i+7pxe9g+zcer3deb15ow9u2Q2F5dvNY9OlC2z0bpSd
;; +zWIDsE2pGibxEPe2yizoTUyF6CpiK5+TSeAAPCA6B/2GjzwNj6qIkRLQVOoskt+
;; T408QSXBJnAy53iQeEnM81AI8RPITLpFfmBjDy9iZ+ZHwSeOPELC6uZmy4mswIVA
;; +tGCA0RGNBtEjxYQGyEm814Kq6AMlYVrfwAqVMATqSXN+gcMtyACWYgYQOQmkZbG
;; ttoRjBMY6l1JX4M/p0nFFfSZVuEqeIByiRCNH+r5TRaOE8dttRc1I/dOZ6pnT/E+
;; wltSTpzSORacgqATjPlQrVq9X4lQkFKy9jK3JN7jRLFWAhi6QuBqyUAXsXhZbKUs
;; RMvFM6Igx/ZaeuuhX+ItBKBCnmFJATDS7rYvjtsHgq2Ecji1+mlwkMpbb2BiBaxd
;; y0wTX9grkyvrU7DciCwX3IpdWFE4Mn30lvkcCmUAOUDAuLdhje0xD38k5HcAe4AW
;; V+SHw/uFLJybjgO6sMnp6MjA5ZRqWwiPVVFaBfehDO5DHYwvRSHM95sGRXHeDYxd
;; ObVrxCs6Fdpngb1aXOAHSztR7fc1oWhNosyhwJq+74pThwcuMwzW1bWXKFOsI2+o
;; uW4/XyLdfp50++sk2ozX/Pax5HfHjD5/4dALTDd00CAJ0I5mZmCOIgzQ4HMeIwYj
;; b7z+ahaytLXO7cnHadJKH4+lXV+bQOrfMrbMuX+I6Z+PNI2taST9rhtbjvy7rxlb
;; HxYeH5Ijt1zmnXiUxFBGqDLuI4LxdKkME77wFNjW2OvN8H7EYvGhnHj086/ofvKl
;; lvSJIRCyM/U1cbOru6G2uL7BKrqGf/bp8SX++S7bAfUD62OZf+3Dt2vx2wY7vISR
;; t+MgO38IScmKemIaohNSTGaWHeIMvDQOT0PO74gI+WNwDJYplkEjzROqPfejByER
;; PcvZYsYDQYYsmvtpI28083xuiIeSnA0yImFOuAyO0OYJEVvUUEtau3FVxYEGW/rU
;; sZ60zGA0o5Cu0NB7iwTHA2suvcTcCYjZDCJM1IHsiEIi+oBYcNI3/NhZZf+WCk76
;; AXHwUNpJ7vyaMl80y/hG8+bvyvN3a6zOO5+xA/JzjenocBuysTTSk05bc+mJE5U4
;; u0Z6+jBTxFXZvNJmpbQAJu3TkGOKYqynUkYlqU644UiiKIPUN0101/gf76EmzVPJ
;; 9CQOyiDnEGtzmYOdHncDziIxq+of/cQwA1v4IZJL1iL6uEW2Wyr2qzfvWD+q3mJZ
;; BYopMIhkTDvp9DTq/nPgDsXB1HZdhPbehKB6NpsqRyYVkRBRpB3pypt9+c2+9Ia3
;; McIaXoZBMIWCpQizy6xWOpSdomthW4jkox6SxzMYRnL+4VjvqCvOWrRU8M3TkiLK
;; pIdyrIelj1DMJalleffcMP/s3We3dpm3M5FBtcJ196xy1Lpqa5QItKmdpEX0ijgR
;; Y8xLguAXWFpqcltQUaSmBv/eggE6Fisg9Bb2uGSGWM/jhH64Ky4jDvGb/hgJattA
;; TZIGSXOkMUxR+PTykPIUPH0H29igg5IqWGwfDuHnMPScBcAfqeQdzSyoj+2Il6FA
;; itSIYiL/wLi7u6tikK6C5AVONLIGFHhhYmGDy6EV4JtfOryAzfRxK9WdfgFkh2LD
;; mACnxxmyO8x2U2BDVbWgoFPGTZXgDWyJ74afNMKUG6C57bv4vS++X/LvwW2cgZ6+
;; +hzZFmIB6RjR7vlVBaQgqNZjhY7H1hjAIz9eMlQ7WkQwIDRPXVwXSv6YEXYkIPE6
;; OB4q6QsTcRq+72Y2WE2RzctoBMWbd5NZX6s3b8FLUZ+GQa7ad293H/lRSQMTID1I
;; f5eQQ5lJdoSbI6OBZ+5pNX0fXQ1wffu8qWUORX9Z1l9COKhi4SPeo519E13jSraT
;; 5zLnOJ5YKtSKdxZzLdA3143o1YjqkR9IwmK+cCLbd3AcmMAxBVYYWtgRT6DvM3Sv
;; oeYQYBdJmaj+V8lTCZE4TtAkzQ741eIqTWooTuQdgLbL+n5WpS+/kUoxCo1CWaXd
;; 4yv23nqg/lI2+vBdFvv66RicVi+pW8pE5Bruqn7V63Yu3jT1/bohHmkT+stM/KWa
;; CzMnBzcE3uYLcB9wHese22RwmtSkNCcRNmxmOO5DkWZ7booM7fxDS0CTADj8l8D9
;; OxIo+rrIDnBzMeZ4NA9DpNAsxViQTJYVOBPt5a2lKyqRfSRV84y4pXbIcFounFfr
;; feMd+1TbLz+Bod4aaikO0fiFfnR9dtbuDc47J20+EOtRnpuVnuQi6MqEcPL1gDio
;; c2zTQCZx7CgCBwpnXhDBAQ8fwIsBpeR0twVhJ7rj1122i11l0V/DtrPnsbFl+SqB
;; KhUapoAIpKC/S6pMLbd+1Tq/bOoDsFh80HK1Vn7JtbZ8eGLrKrDhjNHwB9yJ85j+
;; 8NSnitzI0WoehyUG35d/EFKt5hA75CBxyFxZfiJGj7jVWsrzK5nW9j7Paq2OiuzT
;; j0+Y4hgHnstMRAWVaP8rI6SIR3JJIEDhCfb99tMWKSHLA7TENNrMEeQqXrR0TZlO
;; Bk5y+bS0M0J0mCyuz+LDV8c7ME71UsKyqbieCBb8wzFz34UEdXbalOpCmbUsOWo0
;; W32mM7Pa4JSsc7GiilB2/vioLlUiXaSF4GpVGKt0YTxDGWhtmb0rFNJsfgeWtlOS
;; F2rYK/CpeIq2Fy6YbwbmNDD9GaA6K8SrHCy8TEhSl1R6lZeoCQJCfoN8R81w3sjO
;; htxsXFPPaS8vqPUjPAh+5K6kssxtWizHMchpuwsMs3FKRmP+XHAVy9Qvm+oim5uy
;; h/4xE3r9eoUNpZzyQ8k6095bWidTFHJFPDNIjC1+ZJi2KPnjMWMYkqIGZ3g64WcK
;; /4+gvpq77McypCTf90IstaYLxwxE58qycJtS3wtNAfsK1FYImOWOsbuQ2FgWbiQ9
;; KTWyJCoQCT3u/NBPARSUXL8EBdIzExaLyBrbJmOPahYC3pc4VGZedMc1MvYWQ0dK
;; 5wVFDmZcdtuPmG4NW2UPCgPkHrCZGRJzP/Bm9tBGsLAIURPck1bwjT02GcxaSiHW
;; MWEKc4gNEpL3zoyv96iYICiK69suSBOJ7Uls4hIDMM2dt3DGYsMJGiU4A/JKgmZ6
;; E7Vmc53WlbI1NXJyNX2Q4x1p/ARWaznHSikonMmRjctmDmvFsgjvqMlrRWqHM14l
;; bMJkTdSXpQJOWV9lbPBbp3sCSQ5OEyCeJ5SNb3bwzQ69seb+zIzs/7PGSt2xPdjm
;; h+xj38KCIA+lHdTvCEIZNga3m9s4NJqBveH5ShXkLL4MQp9Em0xui5TLoY3fhB05
;; nuezlyyy51Yo16ZQgQbvF37cMXCtMIql3BjsNHAPjZ0BXcHwvzv0t5FcymDXCCrZ
;; hEqoZqO3AAuDUiCwR6YLQk4dO5yx/6DdVyYLdxRBDbUt1c/8jis0kRkh62rMKIld
;; 7kdA3zbVyGbE2nNzhP4/Wsxh2KQXsCnlcizUijdQDzdvS/3/8CIZCuP+NhaKHHDv
;; x4A76bxx1MepbkADt4KMnqVuFxwC6JbUSiUi+bTwu7kZjWbWuErtqgnEzaJuN/YQ
;; ytQZtsLs3d3S5gYwycg4SGUckIxLhSzJmSXbScl2RA2MJPDFcuc30vmNZVUImqek
;; VFazH28h5zhKFd3tqJvxEU2kuOSOT1zytQII3pBHMJ7iWPVrPmzLDEQLfevUtSPb
;; dMDZ4ED1sXc1sye8k49YTD/pHMcyN5jeO+2dIXzRsT+Gtx/Xvbedrui16Od4JZjM
;; Pf9n+1/oDldM+YirJQhuvUGvcykm4y8IGL/Gzxft33sx20tcLCbe47QZLKW/7nTf
;; wAheUg0urs+P2t0rUdSK7o/e6r759Wbvll9QVIySOAxeb9KeEVeK/R8qIhfEBOHz
;; IfY+PIisEW8DJkoj+J3/KTAw65GVNrKsj6azICesJsUKxFY/ALdmGuYRtQgBmcdD
;; HcVetcDYGi7S1IVQjYudAirBvEAzYQaH0idypBdlHKxm6omOiF9y1BLTLM1wEUUZ
;; sviYP0fn59AJg1hP6OYQCutZR2iGVpawddVOiZLxsW0B/oTpSavLZNiRZdT/uSGy
;; W8ZVyVn8j801Q1dgAL4YXHfPeDtRmUoNYaOc9LpVOceWkxHzpH22fmtAMnHGKlWK
;; uDAXnliYf0RCmHgO5sMqQEpMbSFPFu8tyw85yIXqIrQo/YhUoTIUAmnvigVW+rn/
;; Cf97FD+e8D8tX0pvlN0YRJq1G8seWPu8dfoZXczfZ2h4iPqDNEkgW0uH9lBZuPaH
;; DPlF6xzO/uL0F7nhkUPqDoMMZX5Q28vnAZhokqG/5FF6xaLL8/n0NVvMOnecDtaQ
;; fOQkGyL26L/CJk6x5eAHU/Ig/bhz0Wsd9yjcbVj39gpGFWVpxwyjVbEWPwUoBKAa
;; Dizm+WS65WFgme+Zt4hyuc+KluOXfn6sLIqhObVKPxv2ilBbgJLDSTuYWwucny8y
;; TqyEM0UD9ZNWr9UUO106GkvaIgWeaxegrjd1EQuz8M4GnHSgVsEJUOA44dy03a/F
;; B3SH4xI8SPLPC8gvE2rwC7+TbknEAAa39sWvn7bp9/YT9oxoE5AwLfcj49Mgc048
;; r/8qBOSO95Aq/OFZkFdbmLEP2Cv8SnPhq9BZBHi7U28exlUFlsfYZ4a6E4AxVZwW
;; lScYp+ULEkFcZ/tUepJwiSHe0Eq3VEFarreYziD0+YuIIGqoHUqXIFczKi+g5rSn
;; LhpZ6M05lA2ZORp5AVUYsNPAmlr3PvtZLjHUbUwDyy+yFwaGUaMcj5dQ54nC4dWh
;; SrXVxmuVcc8chkW2I2hiCXUMGO3u4LhzfdFrd9OQodvpM6HsZLksOiVUDsuceGfw
;; JO5gkixWJkRelhaOCf1FOAOuZgAVOWdSUq+QtsgL8PrjxBuB6DCTC56A3a8Etthx
;; IcOFfQ4G7YuTwWCz8MIY2q4xgnqqMtnkRf3pSfuid/r69LjVg3CEgxuvioUSY297
;; 52fsNVSmEXVv4BCjfazbqr4DJesosP2oyq9GqJrzwftDIgfrmwbmnB0gAfzWYYf2
;; +ID9Y+GyvR9Z7aeffmD7+wff/cj+YQY2a5kO8NZxIjbyoBTFyfX/wruqie9e+d7Q
;; u4f6DJykySeejg8kacofWa36co84G3t/N2iRg+/3Dmov2X+JBbtAOrFjyK/X57Dp
;; dMNxpRhCoHLE3nj656YHxsxvtqldBJaIfaw59tEQNgTAQVKNbwUOi3UQF+FQgQaR
;; wBFx5SkqVyIkMS4FEdassCCsA8EHYInj3aEbzU3XtQKaiRb0N1gz2QscNbu5Ou21
;; b5P38auNeD1m4yWUPbFhP1T0o/xiMpJuYL0NsE5zPFhew7bd3KYoIpRkpcslCiC1
;; odMnbQQBnKiUJZ1t25NtoB+ZIRXrvIMlq/oQpz0AuHIAXrHQt0Yg5CjRk1Ax/AWe
;; EUI0aoKgyEz0wv6/Pat/ahtJ2vnV/ismii/gXSwbDKRiNtlwxMnyHhDKQC57IaFk
;; e2wrWJJXI8X4Ntm//X26ZySNwXxc2K2rq4qrguP56Omv6el+ui9VQiA0lfIUT6F3
;; GUuyC01mdKY+joSAyBDAP8rsgg7s7KEiUezmI8le7CMNnBW7SSu6TNFuhAi/i9Ka
;; Vfx2u7O7/fe99hFrKoxCSTeY6nyukUhgYoixmX5E2Doe4T7hb4M4CojUyhy24vX7
;; MQI4/Z/EYCyIwvGUNLuLZ4GsSc4TFAwqYmzuhXEW3yFH88YMgQRFdltNoMI1u1NB
;; bfz5feURfX1wCPDOPKPK0L+BWQ7TwqU1UywvLxWjSCUrBas+HvEkimc20gIFpfG4
;; 9IwfqVbllzf77XpykZQpCzBHHED0oY+nbYWAmsxYttWvEuQwDKLJpFVHxHPTxHMH
;; fn2SdutKISp4ZcK/BmUdBHXtf+leWyFGq1ZEXpqMohjMOkXscvSUTq8whVfiB7wk
;; 5UyNq1p3eAIvaa7Tfn2yt90R6aSPu7hivg1IzU+2e0kuc4igQ5xPjnAk/g0n9KdH
;; f87pzwDHZMLNs4Cg8q0sCOEksLJzMyso6ZmbOJwFjriZtTI0LD3ECPgX6SwjWC3n
;; ZxTarhjNOyXiJYr9IQw/zhdSva3t7VRge2Nhl6z3CK/vuTpLorP8opz5oWNvnZit
;; dG/nJsKFE4mfoOrVAE4+eD7tl+YGcIUh0aVVA73mfm3e/I2HroumjTh8fVjEAht8
;; 44usHcDYx7hASbOkheQply5eKZPHSE/HoJy7QC5JdbIc+2ri6FWWKk6Ot0WNX/Ql
;; JY79cHYIcnK8Ik5oqy6xicWJ1zunVMHJk0hb2yCEU2rncqbYeiYpzlzm3mpD3EVK
;; P9aZ0UIVadddpCFM1EZjS0faJZlprx/44QuscL2x7yk3lIlzRZEe/GgWRKkyXFCn
;; CJURJQvdsQzULUrVm1ZEEuv21CiK6DUUl079RrVecRW67dd4Sjgj2EIt9BZ2D2Pv
;; zNbzYPstYhJ5gvHHZvtALtxu3a8CcXBsse7vLMMUKZOar/9sHcnFCmIVLFTOCJkS
;; Jcr2VdL6JIGVdTluVFHb2k2JRIDIjfurOL/wQ5XEqcapDB0rlP2nbkBxe6GMGR3o
;; 6KoPMH/MHtag5KAyUGl1iosg6gfeRFxQG5pS45BAtXB4J5kLcppaIeDE9r9MukXS
;; D1BkLJB+ofCD64VnMrcIT2vEwPvtTqLxYixLbhLukvUWyUe9vTtbt3e9gEznFgG5
;; j6jNqpLZWN5J0PalrRpW+CZZJ+fDRaIulnVyvaxE5xZRzdWcs8zNcmY7prGf6Ear
;; 7J2DVuIuFvY6w/0JyYKIqJK44dX7tFg3Jo+6MZQhwxacY1E+cotiXmFtXgQgyE8Y
;; NijUkSVxnHDdej0vFjMNNmoXo/7NXFNS865Gf1WCqOnF/dttirzhXQ1pDV5trw91
;; ZjvpAZ9ECpXRgmRFiINI90lMBUN4p53Xs1QPReXnQS6N7I0ihv2OWuKTiAVSLzE8
;; F8OewOHDgaC3WVCiorM2Z4uJXqMuxLCHlHVm1LWYr3b32lonlUEr1mPtd8elfEya
;; hbEcCDNImXVlkA+P7GGix85rtpHEuyHPZxVdvpfm3qTJ/JxFoFxiDKU2FX8wXmWB
;; K6e6Jy2E6e7YA138bRQDFZMhFCvYu5wKW9gpFeOeHs8qjGIiOOcJ+I89ytfWqdDX
;; peFEDyeXhkM9HM4NUyNNVEiN1ijZ9eqoNgGN0mCpYnSbL3guKkalUF1e2htohPID
;; DQBjzriWrrSzPZbb4IsK4jbSiGigN323/xX7U7CGpYihB3/Vp4HPxsYGfzc3G/q7
;; ucnfjcbqarPZfNDYbK43mxubT+j/jdXmk7W1B42/jCPrkyLyxThSg6vXr4uCXuCF
;; 189rYRr59//I59HDeqriOiNc7KDkrowKMnJeOUoDgg5bcJb23u7RoajVRMcvUoi5
;; coYguGPcytAnvEyDnARWG1onnb2W9rsboCwb77D23hFON6v/IWfTKO4rHMfXj4p1
;; /ULqklmD2IpYLjbRyrcyJhbX3eam2HAbDWuacHrrtjBQv2Zw+o362lPG6ZutxnqO
;; 00dJR44lgmNfI/bcy3tpK8zAr4fEo2FICZVOJlGcyL6ZpQ+j+gaIJ8BZt6qi+FxM
;; /WSEWmmW7adgxzJT/URBszbNqeheI2wmNQQLgv20J3m7jOMoVsu8dZ03b1QzeHi7
;; G6VJ3gcYezPquNrMSYhGvR4BX0HyhHlk/zIHyDll8BDHJ16czLJwLJxBNEaqkpGB
;; UeK8UpZZ24DWEE4ux2LZgl/sPJb4nPMr/HV7qu+mqask+1aqoJ06pMIX/FwmXp2c
;; dSg1Xjz2Eolcdl4m/e54YxWZLiD3AsZRdI7f57JIY8XjxBvSj0zns5wMCT2UoaQG
;; RV90Z4b7PTsLp5q5GzC+pFss5tScSI5I62I4Zaw7GrviV+iZGh2f0mAiINRUypDl
;; 6eI+ww7kGzmVnSQe19IJ88j/70fTkIpQXWr38LgyWhrhMXe6kJJt6cwT0ezv1y7A
;; dTeooT6XceKKIxibOIT/QUzcc5Ff7Uypr9IQXpa3bWKcqnVPmXP2QM/71OuDE9HZ
;; OdKblleruQeSzNxzJAh9kI7H8DQdbfgCkHMaMiBM7a7PiBge4UzahmQVbkfpAJX3
;; Ddudlm5rVXICJBm7APteQAczCkSQOoJOQKqjTgaxp62H4ydpotts7Dc5LW4jUfX0
;; YhnOU3XFMQoHQ0S1LOH/ppsj4rDz+mB7v22NsyqK8Xu1VsXjTk/tRey5TKqCH5dj
;; HGmiIz/7HFoo4NkRrwh5JuZhMZ4Hadpbsm8bd0XsvDk43t45XiG8C3O1z1lAos4N
;; m9PEMBbYMqlmz2JjNWNjrd5oIlN60lpvtBqrFhtZFyQ2m/JutI7C2TRSuHRCU7Vv
;; /4jHTIVIlFXazWj7/0ZUNIXTy/bRTmf3kJu0PJIXVEeIQMNx1PUsJzVL+CsN1cgf
;; JMsvdg92VoSajP1keUksrVTaB29/X9r/9bDd2dvb/fvS12qViiTd48V7c3labJkq
;; rjKJh4QxlkylbkxtKtTKlxImVrfM2rftzhF4psYIHy34bFHBrTzbfVmtvl/7sJXX
;; Z2ZUiMePi61V8ftXcfXzSGRpsYJrhD0Zr+C9rEUTc7BxFapcref+dO69z1imcEMf
;; rL1bWuFYPGcHaaZB6j9m+Ou8Z6UK8aFOWPg93cr6sFsxYeNRnK4ti0qghiiFn4kX
;; Z1slVEsI4Qr3YzqacYibSu7U5pDvYWf34LhlSOgL99NP7TevyNq2h/L8W30VW4UT
;; ZJ7D0yYfa+Wm0rvsPIXLtUsAEj3jGk6kfhw/PHDYPpd3Zfut06FcScZ+TWSlTJ2C
;; Cqc89HRQjocKa2sLsqGI83pSc0FkUf5x/OEYzIu2xM7IC6FDCnMZu1IsTfTWpSyt
;; 4mOEvJiMPT+kcD2KppzS0JHZo61If0PkaOxJ1BOWF75KJB7EPPOKkVNQ9LM70lou
;; JDXjaAr5B4mM+U1yc34sTQUoTvBbKbG8+tRtrlZJdbUgTGo0O0hD1s4SsqGghtDp
;; D2YmhTB1sEKqYJ68ZJ6yr0QRpL1eBEWSpBCjSK90p8/gQsoV+xG48QPKTT24jqnJ
;; B36MYVKyTvfsY5iCth2L0pXQErxmPNMZFCWH8M0SJ9jv3r0jzhBPCYKCWizKfal6
;; sc9N1cJw5F6X9EqJAQFDdNJAavtbFTdfYXgeKb3Qk8mCleB1oH9ytP26PR8txXuU
;; 3x+015bLuDM8+9W6SY4Qz5/zjXTmAjEGtuwwQpczs5yYesrkufo4gr4o+l6KKdl6
;; 9WeHlEmPoBvzSpV2uCpKjOJ7Iy+Grcg/I6yE7mixO/d+7R4cnhzPjbw5OZ4fykLV
;; oMod7bHfdfWxJgrr+UpvtLpCsJBKTDwz+se4ePaHUPVl94dq/fSksnraruudDk3y
;; FueKxvZxdcvlewLNj1ErhpwdPS7edDDGYGRm4Bfbnddv6fWo0H/eNz4I+Ztwap8d
;; US1rxVo+ciWW5uHzNDT6yNyARSrDS1EoLYufnufkKGt4LRP7euRR2UJh62L5fc2r
;; /Xu79q9G7emH6tyvH09x1U7Vj7Ua/pBiQZ12/k6YW+9ObkAAXIVFeCYea4suV1ZX
;; KmvVLXM3cl7qJsLWfQFtiroOwziZAjEGs7Nz+bUAxa5qufQ7Q1aZGunc05B1Vvqq
;; gULxErU2ChjTkAm4YOJIjNpM128ov7jWosjEW/AkzBucAtKxruVYQC5+Kfvva+LN
;; nDIVR3ghVxBpeh6qEC77+NVFYa03ciVhXjCVHbi1owkQprLwCKveBedKLIFJ+PUS
;; y0I/4N5LvDMk+lBVHVS/YOLUPXW/fBRfsPoLFtUp4yup+setrR/f//BBnNXrW9Zv
;; 61ddCFrMrOx7iMuKS0w8GwWL+PyiOwWrFPkpr9ATHz9+hFoUgj40jBnjPTr2rpsJ
;; rU1t1o9E6+fl02mVr7QorCsqZxQj+G4L1/YqAbdyhZMbnGBVcoXM1fLbX25slc/O
;; 2gcvz87K3+1/B/v/9xHI75/vn++f/9an6BDrhm/WY54bMN2n+VUDvSbrDbv3+nAP
;; m6KuJkZ4fZbZq/JcC9iLkWA/w9G6XZz3mTVLgmXhKTe5SPQwyaMnHDomJSRPcpkj
;; KVtw9KqkUMXJ8baoif/zYh+11TGejEOQI3zWAqCJRVPpKCdPzFgz3CZlQgSwEMJq
;; ww6mPXl/taEA4mYGqW2xiuJwFizWECZqo7GlI90wZKa9fuCHL7DCRcLrKTeUiXNF
;; kV4YYUWUKsMFEgPCPKiY6o5loG5Rqt60IpI4SgnyGkVRQgXopVO/Ua1XXAW+dJ2n
;; hDOCwNRCb2H3mAcw1HzFeYuYRJ6xerN9IBdut+5XjVLbGlz0N8cW6/7OMkx9VNHa
;; WRbpSC5WEKtgoXJGUSAJC7evktYnCaysy3GjitrWbqroA7lC91djQ36oWx6UQRk6
;; xgu+xQ2G59fImNGBjq76QNFuwpquH1J2prQ6xUUQ9QNvIi5QHwWU+oaU7oXDO8lc
;; kNPUCgEntv9l0i2SfhCFyQLpFwo/uF54JnOL8LRGDLzf7iQaL8ay5CbhLllvkXy9
;; qC/vbN3e9QIynVsEpDXGrCqZjeWdBG1f2sqFxrfJOjkfLhJ1sayT62UlOreImoGY
;; tmVuljPbMY19tmlvJHvnoJW4i4W9znB/QrIgIsgd3/DqfVqsG9MAuDGUDSAETHiu
;; KB+5RTGvsLbvx7jwUTzjRheRKNSRGOJ4KOXt1/NiMdNgo3Yx6t/MNSU172r0N0Nr
;; b7cp8oZ3NaQ1eLW5qs92MvgaKW/sLEhWhDiIGAMVQ/+zDFcYqLIRQZbqoaj8PMil
;; kb1RJJztzuujlvgkYoHUSwzPxbAncPhwIOhtFpSo6KzN2WKi16gLMewhZZ0ZdS3m
;; q929ttZJZdCK9Vj73XEpH5NmYSwHwgym8bheGeTDI3uY6LHzmm0k8W7I87+82W/X
;; YYx8L829SZP5OYtAuZT1jf6oI9rXLTT6VIMeQtQQNwbjvj3Qxd9GMVAxGUKxgr3L
;; qbCFnVIx7ulxnc7ZE8E5T8B/7FG+tk6Fvi4NJ3o4uTQc6uFwbphAZAJvByNrlOx6
;; dVSbgEZpsFQxus0XPBcVo1INSP2TkKas10D5wYxVzgAIuxb9auV7LLfBF6FCbaQR
;; pgNd/m7/K/a3WsB/WY3ZwGdjY4O/m5sN/d3c5O9GY3W12Ww+aGw215vNjc0n9P/G
;; avPJ2tqDxl/GkfVJqd2HIz9xh/n6dVHQC7zw+nktTCP//h/5PHpYT1Vc5y4MOyi5
;; a/kRLg435SpHaRAQVglnae/tHh1Sf67jFynEfP8ticQxbmXog5ppMIpKTuuks9fS
;; fkcPbateR9mVJp478OuTtFvXD7T54utj7TX9Zr/fEj99wivmsrleTKJudCGV24uC
;; 59bqf8gZ9YYVjuPrR8W6fiF1yaxBaupvW5to5VsZE4vrbnNTbLiNhjW9i6Ot27Ly
;; Way6a2L16dMn9cZGfe0p/tvaaLYa60K7Et7KpCPHEsGxT2RMZ+qlrTAeFeKQeDQM
;; KaHSCfVYZd/M0ud4hBCIp3kYe4HKGtUQ8VxM/WSEWmmW7adgxzJnXejaNKeisLg3
;; gs1QCIZRknU+ebuM4yhWy7x1nTdvVF3DwXY3SpPseDH2Zvg5x5yEaIRSMyyuBOaR
;; /ZuGMLWaKWUgbHzixcksC8fCIazbycnAKHFeKUvTF7Xx8GULfrHzWOJzzq/w1+2p
;; vpumrpLsW6mCduqQCl/wc5l4dXLWISVJ4H7sJRK57LxM+t3xCJvX/QPqiIhxFJ3r
;; 7nWexorHie4ZZDqf5WRI6KEMZexRR6Q7M9zv2Vk49yQCxpe4KZ2dmhPJ7pjQxTA3
;; Z1Biu+JX6Jla3p/SYCIg1FTKkOXp4j7DDuQbOZWdJB7X0gnzyP/vR9OQilBdavfw
;; uCbUxojwmDtdSMm2dOaJaPb3axfguhvUUJ/LOHHFEYxNHML/ICbuucivdqbUV2kI
;; L0O9gVUDH2YQntY9Zc7ZAz3vU68PTkRn50hvWl6t5h5IMutWIOw6SMdjeJqONnwB
;; yDkNGRCGG4nPiBge4UzahmQV+GYWoMTuy/bB8e6r3Xanpbs2lZwAScYuwL4X0MGM
;; AmEv94JIdVPQJPa09XD8JE0Un8t+k9PCYb0RVU8vluE8VVcco3AwRFTLEv5vTBJx
;; ofP6YHu/bY2zKorxe3W4xeNOT/0/
;; =nx41
;; -----END PGP MESSAGE-----

;;}}}

(add-hook 'titf-:mode-define-keys-hook  'titf-mode-define-keys)

(provide   'titf)
(run-hooks 'titf-:load-hook)

;;; tinytfo.el ends here
